//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "Global2.h"
#include "CfgSourceDlgs.h"
#include "PrintDlgs2.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TCfgSourceDlg *CfgSourceDlg;
//---------------------------------------------------------------------------

bool __fastcall EditConfigSource(TForm *Owner, const AnsiString &nym, TStrings *spec)
{
   CfgSourceDlg = new TCfgSourceDlg(Owner, nym, spec);
   CfgSourceDlg->ShowModal();
   bool ok = CfgSourceDlg->ModalResult == mrOk;
   delete CfgSourceDlg;
   return ok;
}
//---------------------------------------------------------------------------

bool __fastcall EditConfigSource(TForm *Owner, const AnsiString &Nym, TTreeView *TreeView)
{
   AnsiString blank = "    ";
   TStringList *spec = new TStringList();
   TTreeNode *node = TreeView->Items->GetFirstNode();
   while (node != NULL)
   {
      if (node->Text.Pos("Reply-Block:") == 1 && spec->Count > 0)
         spec->Add("");
      spec->Add(blank.SubString(1, node->Level - 1) + node->Text);
      node = node->GetNext();
   }

   bool ok = EditConfigSource(Owner, Nym, spec) && spec->Count > 1;
   if (!ok && spec->Count == 0)
      TreeView->Items->Clear();

   if (ok)
   {
      TTreeNode *cnode, *pnode = NULL;
      TreeView->Items->Clear();
      Snooze(300);

      for (int i = 0; i < spec->Count; i++)
      {
         AnsiString line = spec->Strings[i].Trim();
         if (line.IsEmpty())
         {
            pnode = NULL;
            continue;
         }

         if (line.Pos("Reply-Block:") == 1)
         {
            if (pnode)
               pnode->Expand(false);
            pnode = TreeView->Items->AddChild(NULL, line);
            continue;
         }

         if (line.Pos(":") == 0)
         {
            if (!pnode)
               pnode = TreeView->Items->AddChild(NULL, "Reply-Block:");
            cnode = TreeView->Items->AddChild(pnode, line);
            continue;
         }

         TreeView->Items->AddChild(cnode, line);
      }

      if (!pnode->Expanded)
         pnode->Expand(false);
   }

   return ok;
}
//---------------------------------------------------------------------------

__fastcall TCfgSourceDlg::TCfgSourceDlg(TComponent* Owner)
        : TForm(Owner)
{
}
//---------------------------------------------------------------------------

__fastcall TCfgSourceDlg::TCfgSourceDlg(TComponent* Owner, const AnsiString &Nym,
   TStrings *Spec) : TForm(Owner)
{
   nym = Nym;
   if (!nym.IsEmpty())
      Caption = Caption + " -- " + nym;

   spec = Spec;
   Editor->Clear();
   Editor->Lines->AddStrings(spec);

   TColor color;
   LoadFonts(AppIni(), NymIni, color, Font, NULL, Editor->Font);
   Editor->Color = color;

   Editor->Modified = false;
   Editor->SelStart = 0;
   OpenDialog1->InitialDir = AppDir();
   SaveDialog1->InitialDir = AppDir();
}
//---------------------------------------------------------------------------

void __fastcall TCfgSourceDlg::FormCloseQuery(TObject *, bool &CanClose)
{
   if (Editor->Modified)
   {
      if (ModalResult == mrCancel)
      {
         switch (MsgBox(this,
            "Your configuration is modified.    \n\n"
            "Save your changes?", MT_CONFIRM,
             MB_YESNOCANCEL))
         {
            case IDCANCEL: CanClose = false;
            case IDNO: return;
         }
      }

      spec->Clear();
      spec->AddStrings(Editor->Lines);
      ModalResult = mrOk;
      return;
   }

   ModalResult = mrCancel;
}
//---------------------------------------------------------------------------

void __fastcall TCfgSourceDlg::FormResize(TObject *)
{
   SetEditRect(Editor);
}
//---------------------------------------------------------------------------

void __fastcall TCfgSourceDlg::FormPaint(TObject *)
{
   SetEditRect(Editor);
}
//---------------------------------------------------------------------------

void __fastcall TCfgSourceDlg::Cut1Click(TObject *)
{
   Editor->CutToClipboard();
}
//---------------------------------------------------------------------------

void __fastcall TCfgSourceDlg::Copy1Click(TObject *)
{
   Editor->CopyToClipboard();
}
//---------------------------------------------------------------------------

void __fastcall TCfgSourceDlg::Paste1Click(TObject *)
{
   Editor->SelText = Clipboard()->AsText;
 //Editor->PasteFromClipboard();
}
//---------------------------------------------------------------------------

void __fastcall TCfgSourceDlg::Savetofile1Click(TObject *)
{
   if (SaveDialog1->Execute())
   {
      Editor->Lines->SaveToFile(SaveDialog1->FileName);
   }
}
//---------------------------------------------------------------------------

void __fastcall TCfgSourceDlg::Loadfromfile1Click(TObject *)
{
   if (OpenDialog1->Execute())
   {
      Editor->Lines->LoadFromFile(OpenDialog1->FileName);
      Editor->Modified = true;
   }
}
//---------------------------------------------------------------------------

void __fastcall TCfgSourceDlg::Insert1Click(TObject *)
{
   if (OpenDialog1->Execute())
   {
      TStringList *lines = new TStringList();
      lines->LoadFromFile(OpenDialog1->FileName);
      Editor->SelText = lines->Text;
      delete lines;
   }
}
//---------------------------------------------------------------------------

void __fastcall TCfgSourceDlg::PopupMenu1Popup(TObject *)
{
   bool hassel = Editor->SelLength > 0;
   Cut1->Enabled = hassel;
   Copy1->Enabled = hassel;
   Paste1->Enabled = Clipboard()->HasFormat(CF_TEXT);
}
//---------------------------------------------------------------------------

void __fastcall TCfgSourceDlg::Selectall1Click(TObject *)
{
   Editor->SelectAll();
}
//---------------------------------------------------------------------------

void __fastcall TCfgSourceDlg::Print1Click(TObject *)
{
   if (MsgBox(this, "Print reply block source?", MT_CONFIRM, MB_OKCANCEL) == IDOK)
      PrintDocument(this, "QuickSilver Reply Blocks", Editor->Lines, Editor->Font);
}
//---------------------------------------------------------------------------
