//---------------------------------------------------------------------------
#pragma hdrstop

#include "Global2.h"
#include "NymConfigs.h"
#include "NymGenRBDlgs.h"
#include "PGPsdk.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
TNymConfig *NymConfig;
//---------------------------------------------------------------------------

__fastcall TNymConfig::TNymConfig(TForm *owner, TPGPsdk *pgp, TNymProfile *Profile)
   : TObject()
{
   Owner = owner;
   profile = Profile;
   PGP = pgp;
}
//---------------------------------------------------------------------------

__fastcall TNymConfig::~TNymConfig()
{
}
//---------------------------------------------------------------------------

AnsiString __fastcall TNymConfig::CreateMessagePGPHeader(const AnsiString &PGPKeyID,
   const AnsiString &Nymserver)
{
   assert(Nymserver != "");

   AnsiString enckeyid = PGP->GetKeyID(PGPNymserverRing(), Nymserver);
   assert(enckeyid != "");

   return "sign=" + PGPKeyID + "; encrypt=" + enckeyid + ";";
}
//---------------------------------------------------------------------------

void __fastcall TNymConfig::PlaceMessageHeader(TStrings *Message, const AnsiString &Header,
   const AnsiString &Value)
{
//look(Message, "header: " +Header +"                value: "+ Value);
   assert(Message != NULL && Message->Count > 0 && Header != "" && Value != "");

   for (int i = 0; i <= Message->Count; i++)
   {
      if (i == Message->Count)
      {
         Message->Insert(i, Header + Value);
         break;
      }

      AnsiString line = Message->Strings[i].Trim();
      if (line.IsEmpty())
      {
         Message->Insert(i, Header + Value);
         break;
      }

      if (line.Pos(Header.Trim()) == 1)
      {
         Message->Strings[i] = Header + Value;
         break;
      }
   }
}
//---------------------------------------------------------------------------

void __fastcall TNymConfig::CopyTemplateToMessage(TStrings *Message, TStrings *Template)
{
   assert(Message != NULL && Template != NULL && Template->Count > 0);

   Message->Clear();
   for (int i = 0; i < Template->Count; i++)
   {
      AnsiString line = Template->Strings[i].Trim();
      if (line.IsEmpty())
         break;

      Message->Add(line);
   }
}
//---------------------------------------------------------------------------

bool __fastcall TNymConfig::GetNymserverKey(const AnsiString &Nymserver, TStrings *Key)
{
   assert(Nymserver != "" && Key != NULL);

   Key->Clear();
   TStringList *keys = new TStringList();
   if (PGP->GetKeys(PGPNymserverRing(), keys) || PGP->GetKeys(PGPRemailerRing(), keys))
   {
      for (int i = 0; i < keys->Count; i++)
      {
         AnsiString value, line = keys->Strings[i];
         if (line.Pos(Nymserver) > 0)
         {
            int p = 0; while ((value = GetTok(p, line, '\255')) != "")
            {
               Key->Add(value);
               p++;
            }
         }
      }
   }

   delete keys;
   return Key->Count > 0;
}

//---------------------------------------------------------------------------
// Nym Config Messages
//---------------------------------------------------------------------------

bool __fastcall TNymConfig::CreateNymCreateMessage(TStrings *Template,
   bool preview)
{
   assert(Template != NULL && Template->Count > 0);

   if (SecureModeMsg(Owner))
      return false;

   AnsiString nymserver = profile->NymServer;
   TStringList *blocks = new TStringList();
   profile->GetReplyBlocks(blocks);

   if (!GenerateReplyBlocks(Owner, blocks, PGP, preview))
   {
      //MsgBox(Owner,
      //   "Sorry, I've failed to generate your reply block(s)   ",
      //   "Bad Day In Black Rock",
      //    MB_OK
      //);

      delete blocks;
      return false;
   }

   TStringList *message = new TStringList();

   CopyTemplateToMessage(message, Template);

   PlaceMessageHeader(message, "To: ", "config@" + nymserver);
   PlaceMessageHeader(message, "Pgp: ", CreateMessagePGPHeader(profile->PublicKeyID, nymserver));
   message->Insert(0, "Nym-Log: create nym message");

   TStringList *pubkey = new TStringList();
   profile->GetPublicKey(pubkey);
   pubkey->Delete(0);

   message->Add("");
   message->Add("Config:");
   message->Add("From: " + profile->UserId);
   message->Add("Nym-Commands: create? " + profile->NymCommands);
   message->Add("Public-Key:");
   message->AddStrings(pubkey);
   message->AddStrings(blocks);
   message->Add("~~");

   Global->DB->PutMailWaiting(message);
   PostMessage(HWND_BROADCAST, Global->WM_QSAPPMESSAGE, QSMSGID_MAILWAITING, NULL);

   delete message;

   delete pubkey;
   delete blocks;

   return true;
}
//---------------------------------------------------------------------------

bool __fastcall TNymConfig::CreateDeleteMessage(TStrings *Template)
{
   assert(Template != NULL && Template->Count > 0);

   if (SecureModeMsg(Owner))
      return false;

   AnsiString nymserver = profile->NymServer;

   TStringList *message = new TStringList();

   CopyTemplateToMessage(message, Template);
   PlaceMessageHeader(message, "To: ", "config@" + nymserver);
   PlaceMessageHeader(message, "Pgp: ", CreateMessagePGPHeader(profile->PublicKeyID, nymserver));
   message->Insert(0, "Nym-Log: delete nym message");

   message->Add("");
   message->Add("Config:");
   message->Add("From: " + profile->UserId);
   message->Add("Nym-Commands: delete");
   message->Add("~~");

   Global->DB->PutMailWaiting(message);
   PostMessage(HWND_BROADCAST, Global->WM_QSAPPMESSAGE, QSMSGID_MAILWAITING, NULL);

   delete message;

   return true;
}
//---------------------------------------------------------------------------

bool __fastcall TNymConfig::CreateModifyMessage(TStrings *Template, bool preview,
   bool commands, bool key, bool replyblock, AnsiString SignKey)
{
   assert(Template != NULL && Template->Count > 0);

   if (SecureModeMsg(Owner))
      return false;

   AnsiString nymserver = profile->NymServer;
   TStringList *blocks = new TStringList();
   TStringList *pubkey = new TStringList();

   TStringList *message = new TStringList();

   if (replyblock)
   {
      profile->GetReplyBlocks(blocks);
      if (!GenerateReplyBlocks(Owner, blocks, PGP, preview))
      {
         //MsgBox(Owner,
         //   "Sorry, I've failed to generate your reply block(s)   ",
         //   "Bad Day In Black Rock",
         //    MB_OK);
         //delete blocks;
         return false;
      }
   }

   CopyTemplateToMessage(message, Template);
   if (SignKey.IsEmpty())
      SignKey = profile->PublicKeyID;

   PlaceMessageHeader(message, "To: ", "config@" + nymserver);
   PlaceMessageHeader(message, "Pgp: ", CreateMessagePGPHeader(SignKey, nymserver));
   message->Insert(0, "Nym-Log: modify nym message");
   message->Add("");
   message->Add("Config:");
   message->Add("From: " + profile->UserId);

   if (commands)
   {
      message->Add("Nym-Commands: create? " + profile->NymCommands);
   }

   if (key)
   {
      profile->GetPublicKey(pubkey);
      pubkey->Delete(0);

      message->Add("Public-Key:");
      message->AddStrings(pubkey);
   }

   if (replyblock)
   {
      message->AddStrings(blocks);
   }

   message->Add("~~");

   Global->DB->PutMailWaiting(message);
   PostMessage(HWND_BROADCAST, Global->WM_QSAPPMESSAGE, QSMSGID_MAILWAITING, NULL);

   delete message;

   delete pubkey;
   delete blocks;

   return true;
}
//---------------------------------------------------------------------------

bool __fastcall TNymConfig::CreateReplayMessage(TStrings *Template)
{
   assert(Template != NULL && Template->Count > 0);

   if (SecureModeMsg(Owner))
      return false;

   AnsiString ToHdr;

   TStringList *replay = new TStringList();
   TStringList *message = new TStringList();

   profile->GetReplay(replay);
   while (replay->Count > 0)
   {
      AnsiString hdr = replay->Strings[0];
      if (hdr.IsEmpty())
         break;

      if (hdr.Pos("To: ") == 1)
         ToHdr = hdr.SubString(5, 256);

      replay->Delete(0);
   }

   CopyTemplateToMessage(message, Template);
   PlaceMessageHeader(message, "To: ", ToHdr);

   // remove the pgp header is included in template
   for (int i = 0; i < message->Count; i++)
   {
      if (message->Strings[i].Pos("Pgp:") == 1)
      {
         message->Delete(i);
         break;
      }
   }

   message->AddStrings(replay);
   message->Add("~~");

   Global->DB->PutMailWaiting(message);
   PostMessage(HWND_BROADCAST, Global->WM_QSAPPMESSAGE, QSMSGID_MAILWAITING, NULL);

   delete message;
   delete replay;
   return true;
}

//---------------------------------------------------------------------------

bool __fastcall TNymConfig::CreateFromNymMessage(TStrings *Template)
{
   assert(Template != NULL && Template->Count > 0);

   if (SecureModeMsg(Owner))
      return false;

   TStringList *message = new TStringList();
   message->AddStrings(Template);
   int p = message->Count;

   for (int i = 0; i < message->Count; i++)
   {
      AnsiString hdr = message->Strings[i].Trim();
      if (hdr.IsEmpty())
      {
         p = i;
         break;
      }

      if (hdr.Pos("From: ") == 1)
      {
         p = i + 1;
         break;
      }
   }

   message->Insert(p, "Nym: " + profile->Nym);

   Global->DB->PutMailWaiting(message);
   PostMessage(HWND_BROADCAST, Global->WM_QSAPPMESSAGE, QSMSGID_MAILWAITING, NULL);

   delete message;

   return true;
}
//---------------------------------------------------------------------------
