//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop
//#include <dateutils.hpp>

#include "Global2.h"
#include "NymDlgs.h"
#include "GenNymCfgDlgs.h"
#include "..\print\PrintCnfrmDlgs.h"
#include "NymProfiles.h"
#include "RBWizDlgs.h"
#include "RBEditDlgs.h"
#include "CfgSourceDlgs.h"
#include "NymGenRBDlgs.h"
#include "nymconfigs.h"
#include "NymGenCfgNowDlgs.h"
#include "NymGenModNowDlgs.h"
#include "NymGetTmplDlgs.h"
#include "NewNymDlgs.h"
#include "FullAutoDlgs.h"
#include "NymKeyDlgs.h"
#include "ValidNymKeys.h"
#include "UpdateStatsdlgs.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TNymDlg *NymDlg;
//---------------------------------------------------------------------------

bool __fastcall OpenNymDialog(TForm *Owner, AnsiString &Nym, TPGPsdk *PGP)
{
   assert(Owner != NULL && Nym != "");

   if (Nym == "<new nym>")
   {
      int chainlength;
      bool replyblock = false;
      AnsiString pgpkey, address;

      if (GetNewNymValues(Owner, PGP, pgpkey, replyblock, address, chainlength))
      {
         return CreateNym(Owner, PGP, pgpkey, replyblock, address, chainlength);
      }

      return false;
   }

   return ModifyNym(Owner, Nym, PGP);
}

//---------------------------------------------------------------------------

bool __fastcall CreateNym(TForm *Owner, TPGPsdk *PGP, const AnsiString &pgpkey,
   bool ReplyBlock, const AnsiString &Address, int ChainLength)
{
   assert(Owner != NULL && PGP != NULL);

   if (SecureModeMsg(Owner))
      return false;

   NymDlg = new TNymDlg(Owner, PGP);

   AnsiString defnymserver = Global->DB->GetIni("defnymserver", "hod.aarg.net");
   NymDlg->NymServers->ItemIndex = NymDlg->NymServers->Items->IndexOf(defnymserver);

   AnsiString keyid = GetTok(1, pgpkey, '\255');
   NymDlg->PGPkey->Text = pgpkey;
   AnsiString nym = GetEAddress(pgpkey);

   if (!NymDlg->NymExists(nym) && NymDlg->IsNymDomain(nym))
   {
      AnsiString name, address, domain, userid, key = NymDlg->PGPkey->Text.SubString(12, 256);
      address = GetEAddress(key);
      int x = address.Pos("@");
      domain = address.SubString(x + 1, 256);
      userid = address.SubString(1, x - 1);
      name = key.SubString(1, key.Pos("<") - 1).Trim();
      NymDlg->UserID->Text = userid;
      NymDlg->NymServers->ItemIndex = NymDlg->NymServers->Items->IndexOf(domain);
      NymDlg->Name->Text = name;
   }

   NymDlg->PageControl1->ActivePage = NymDlg->TabSheet1;
   NymDlg->ActiveControl = NymDlg->UserID;
   NymDlg->ModifyNymName->Visible = false;
   NymDlg->RetiredBox->Enabled = false;
   NymDlg->Caption = " Create Nym";

   if (ReplyBlock)
   {
      NymDlg->ChainLength = ChainLength;
      NymDlg->Address = Address;
      NymDlg->Timer2->Enabled = true;
   }

   NymDlg->ShowModal();

   bool ok = NymDlg->ModalResult == mrOk; // || NymDlg->SavedToProfile && NymDlg->Virgin;
   delete NymDlg;

   return ok;
}

//---------------------------------------------------------------------------

bool __fastcall ModifyNym(TForm *Owner, AnsiString &Nym, TPGPsdk *PGP)
{
   assert(Owner != NULL && Nym != "" && PGP != NULL);

   if (SecureModeMsg(Owner))
      return false;

   NymDlg = new TNymDlg(Owner, PGP);
   NymDlg->Caption = " Modify Nym";
   NymDlg->PageControl1->ActivePage = NymDlg->TabSheet1;
   NymDlg->ActiveControl = NymDlg->PGPkey;

   NymDlg->ModifyNymName->Caption = GetTok(0, Nym, '\255');
   NymDlg->ModifyNymName->Top = 53;
   NymDlg->UserID->Hide();
   NymDlg->NymServers->Hide();
   NymDlg->Label7->Hide();
   NymDlg->Label1->Hide();
   NymDlg->KeyImportBtn->Enabled = true;

   NymDlg->LoadNym(Nym);
   NymDlg->ActiveControl = NymDlg->Name;

   NymDlg->ShowModal();

   Nym = NymDlg->Profile->NymIndexStr;
   bool ok = NymDlg->ModalResult == mrOk;
   delete NymDlg;

   return ok;
}

//---------------------------------------------------------------------------

AnsiString __fastcall GetNymLine(const AnsiString &label, TStrings *lines)
{
   assert(label != "" && lines != NULL && lines->Count > 0);

   AnsiString line;
   int i = 0; while (i < lines->Count)
   {
      line = lines->Strings[i];
      if (line.Pos(label) == 1)
         return line.SubString(label.Length() + 1, 256);
      i++;
   }
   return "";
}
//---------------------------------------------------------------------------

void __fastcall GetNymServers(TStrings *servers)
{
   assert(servers != NULL);

   servers->Clear();
   AnsiString server, serverfile = AppDir() + NymserverFile;
   if (FileExists(serverfile))
   {
      TStringList *lines = new TStringList();
      lines->LoadFromFile(serverfile);

      for (int i = 0; i < lines->Count; i++)
      {
         AnsiString line = lines->Strings[i];

         if (line.Pos("$nymserver{") == 1)
         {
            server = GetTok(0, GetTok(1, line, '@'), '>');
            if (servers->IndexOf(server) < 0)
               servers->Add(server);
            continue;
         }

         //if (line.Pos("-----BEGIN PGP") == 1)
         //   break;
      }

      delete lines;
   }
}

//---------------------------------------------------------------------------
// TNymDlg
//---------------------------------------------------------------------------

__fastcall TNymDlg::TNymDlg(TForm* Owner)
        : TForm(Owner)
{
}
//---------------------------------------------------------------------------

__fastcall TNymDlg::TNymDlg(TForm* Owner, TPGPsdk *pgp)
        : TForm(Owner)
{
   PGP = pgp;

   Profile = new TNymProfile();
   ReplayPad->Clear();
   NotePad->Clear();
   LogPad->Clear();
   History->Clear();
   UserID->Clear();
   Name->Clear();

   TColor color;
   LoadFonts(AppIni(), NymIni, color, Font, UserID->Font, History->Font);

   NymServers->Font->Assign(UserID->Font);
   Name->Font->Assign(UserID->Font);
   PGPkey->Font->Assign(UserID->Font);
   ReplyBlocks->Font->Assign(UserID->Font);

   ReplayPad->Font->Assign(History->Font);
   LogPad->Font->Assign(History->Font);
   NotePad->Font->Assign(History->Font);

   UserID->Color = color;
   NymServers->Color = color;
   Name->Color = color;
 //PGPkey->Color = color;
   ReplyBlocks->Color = color;
   History->Color = color;
   ReplayPad->Color = color;
   LogPad->Color = color;
   NotePad->Color = color;
   KeyImportBtn->Font->Color = Font->Color;

   ModifyNymName->Font->Color = Font->Color;
   RetiredBox->Font->Color = Font->Color;

   OrgOptions = Global->DB->GetIni("defnymoptions",
      "+acksend -signsend +cryptrecv -fixedsize -disable -fingerkey +nobcc");

   OptAck->Checked = OrgOptions.Pos("+acksend") > 0;
   OptSign->Checked = OrgOptions.Pos("+signsend") > 0;
   OptCrypt->Checked = OrgOptions.Pos("+cryptrecv") > 0;
   OptDisable->Checked = OrgOptions.Pos("+disable") > 0;
   OptFixed->Checked = false;
   OptFixed->Enabled = false;
   OptFinger->Checked = OrgOptions.Pos("+fingerkey") > 0;
   OptBcc->Checked = OrgOptions.Pos("+nobcc") > 0;

   OrgOptions = GetOptions();

   History->Modified = false;
   ReplayPad->Modified = false;
   LogPad->Modified = false;
   NotePad->Modified = false;

   PublicKey = new TStringList();
   SecretKey = new TStringList();
   OrgBlocks = new TStringList();

   OrgNym     = "";
   OrgUserID  = "";
   OrgServer  = "";
   OrgPgpKey  = "";
   OrgName    = "";
   OrgRetired = false;

   GetNymServers(NymServers->Items);

   KeyImportBtn->Height = PGPkey->Height + 2;
   ModifyNymName->Caption = "";

   SavedToProfile = false;
   Virgin = true;

   MessageBtn->Font->Color = Font->Color;
}
//---------------------------------------------------------------------------

__fastcall TNymDlg::~TNymDlg()
{
   delete OrgBlocks;
   delete SecretKey;
   delete PublicKey;
   delete Profile;
}
//---------------------------------------------------------------------------

void __fastcall TNymDlg::FormCloseQuery(TObject *Sender, bool &CanClose)
{
   if (Modified() || SavedToProfile)
   {
      if (ModalResult == mrCancel)
      {
         switch (MsgBox(this, "Your nym's profile is modified. Save your changes?",
            MT_CONFIRM, MB_YESNOCANCEL))
         {
            case IDCANCEL: CanClose = false;
            case IDNO: return;
         }
      }

      if (UserID->Text.IsEmpty())
      {
         MsgBox(this,
            "You must give your nym a user ID before I can give save it for you.",
             MT_INFO, MB_OK);
         CanClose = false;
         return;
      }

      SaveNym();
   }
}
//---------------------------------------------------------------------------

bool __fastcall TNymDlg::NymExists(const AnsiString &Nym)
{
   bool exists = false;
   TStringList *nyms = new TStringList();
   Global->DB->GetNyms(nyms);

   for (int i = 0; i < nyms->Count; i++)
   {
      AnsiString nym = GetEAddress(GetTok(0, nyms->Strings[i], '\255'));
      if (nym == Nym)
      {
         exists = true;
         break;
      }
   }

   delete nyms;
   return exists;
}
//---------------------------------------------------------------------------

bool __fastcall TNymDlg::IsNymDomain(const AnsiString &Nym)
{
   AnsiString domain = GetTok(1, Nym, '@');
   return NymServers->Items->IndexOf(domain) >= 0;
}
//---------------------------------------------------------------------------

void __fastcall TNymDlg::EnableProfile(int pos)
{
   switch (pos)
   {
      case 0: Prof0->Enabled = true; break;
      case 1: Prof1->Enabled = true; break;
      case 2: Prof2->Enabled = true; break;
      case 3: Prof3->Enabled = true; break;
      case 4: Prof4->Enabled = true; break;
      case 5: Prof5->Enabled = true; break;
      case 6: Prof6->Enabled = true; break;
      case 7: Prof7->Enabled = true; break;
      case 8: Prof8->Enabled = true; break;
      case 9: Prof9->Enabled = true; break;
   }
}
//---------------------------------------------------------------------------

void __fastcall TNymDlg::LoadFromProfile()
{
   // History
   Profile->GetHistory(0, History->Lines);
   assert(History->Lines->Count > 0);

   Profile->GetPublicKey(PublicKey);
   Profile->GetSecretKey(SecretKey);

   int cnt = Profile->CountHistories();
   for (int i = 0; i < cnt; i++)
      EnableProfile(i);

   History->SelStart = 0;
   History->Perform(EM_SCROLLCARET, 0, 0);
   History->Modified = false;

   // Replay
   TStringList *replay = new TStringList();
   Profile->GetReplay(ReplayPad->Lines);
   if (replay->Count > 1)
   {
      ReplayPad->Lines->AddStrings(replay);
      ReplayPad->SelStart = 0;
      ReplayPad->Perform(EM_SCROLLCARET, 0, 0);
      ReplaySendBtn->Enabled = true;//ReplayPad->Lines->Count > 2;
   }
   ReplayPad->Modified = false;
   delete replay;

   // Log
   Profile->GetLog(LogPad->Lines);
   assert(LogPad->Lines->Count > 0);
   LogDate->Caption = LogPad->Lines->Strings[0];
   LogPad->Lines->Delete(0);
   LogPad->SelStart = 0;
   LogPad->Perform(EM_SCROLLCARET, 0, 0);
   LogPad->Modified = false;

   // Notes
   Profile->GetNotes(NotePad->Lines);
   assert(NotePad->Lines->Count > 0);
   NoteDate->Caption = "Modified: " + NotePad->Lines->Strings[0].SubString(8, 20);
   NoteDate->Visible = true;
   NotePad->Lines->Delete(0);
   NotePad->SelStart = 0;
   NotePad->Perform(EM_SCROLLCARET, 0, 0);
   NotePad->Modified = false;

   SetHistoryBtns();
   SetReplayBtns();
   SetNoteBtns();
   SetLogBtns();
}
//---------------------------------------------------------------------------

void __fastcall TNymDlg::SaveToProfile()
{
   TNymProfile *newprofile = new TNymProfile();

   AnsiString datetime = FormatDateTime("yyyy-mm-dd hh:nn", Now());

   newprofile->Add((RetiredBox->Checked ? "Retired: " : "Date: ") + datetime);
   newprofile->Add("Nym: " + UserID->Text + "@" + NymServers->Text);
   newprofile->Add("Name: " + Name->Text);
   newprofile->Add("Commands: " + GetOptions());
   newprofile->Add("PgpKey: " + PGPkey->Text);
   newprofile->Add("");

   TStringList *data = new TStringList();

   GetReplyBlocks(data);
   newprofile->AddStrings(data);
   OrgBlocks->Text = data->Text;
   newprofile->Add("");
   AnsiString keyID = GetTok(0, PGPkey->Text, ' ');

   if (PGPkey->Text != OrgPgpKey) // || PublicKey->Count < 6 || SecretKey->Count < 6)
   {
      PublicKey->Clear();
      SecretKey->Clear();

      TStringList *exports = new TStringList();
      PGP->ExportKey(PGPPublicRing(), PGPSecretRing(), keyID, true, exports);

      bool ispublic = false, issecret = false;

      for (int i = 0; i < exports->Count; i++)
      {
         AnsiString line = exports->Strings[i];
         if (line.Pos("-----BEGIN PGP PUBLIC KEY BLOCK-----") == 1)
         {
            issecret = false;
            ispublic = true;
         }
         else
         if (line.Pos("-----BEGIN PGP PRIVATE KEY BLOCK-----") == 1)
         {
            ispublic = false;
            issecret = true;
         }

         if (ispublic)
         {
            PublicKey->Add(line);
         }
         else
         if (issecret)
         {
            SecretKey->Add(line);
         }

         if (line.Pos("-----END PGP PUBLIC KEY BLOCK-----") == 1)
         {
            ispublic = false;
         }
         else
         if (line.Pos("-----END PGP PRIVATE KEY BLOCK-----") == 1)
         {
            issecret = false;
         }
      }

      PublicKey->Insert(0, "Public Key: " + keyID);
      SecretKey->Insert(0, "Secret Key: " + keyID);

      delete exports;

      OrgPgpKey = PGPkey->Text;
      OrgUserID = UserID->Text;
      OrgName = Name->Text;
   }

   newprofile->AddStrings(PublicKey);
   newprofile->Add("");

   newprofile->AddStrings(SecretKey);
   newprofile->Add("");

   for (int i = 0; i < 9; i++)
   {
      Profile->GetHistory(i, data);
      if (data->Count == 0)
         break;

      newprofile->AddStrings(data);
      newprofile->Add("");
   }

   TrimNullsAfter(ReplayPad->Lines);
   if (ReplayPad->Lines->Count > 0)
      newprofile->AddStrings(ReplayPad->Lines);
   else
      newprofile->Add("Sent:");
   newprofile->Add("");
   ReplayPad->Modified = false;

   TrimNullsAfter(LogPad->Lines);
   newprofile->Add("Log: " + datetime);
   GetUnWrappedText(LogPad, data);
   newprofile->AddStrings(data);
   newprofile->Add("");
   LogPad->Modified = false;

   TrimNullsAfter(NotePad->Lines);
   if (NotePad->Modified) newprofile->Add("Notes: " + datetime);
   else newprofile->Add("Notes: " + NoteDate->Caption.SubString(11, 32));
   GetUnWrappedText(NotePad, data);
   newprofile->AddStrings(data);
   newprofile->Add("");
   NotePad->Modified = false;

   delete data;

   Profile->Clear();
   Profile->AddStrings(newprofile);
//look(Profile);
   SavedToProfile = true;
}
//---------------------------------------------------------------------------

int __fastcall TNymDlg::LoadNym(const AnsiString &Nym)
{
   assert(Nym != "");

   Virgin = false;
   Global->DB->GetNym(Nym, Profile);

   OrgNym     = Nym;
   OrgUserID  = Profile->UserId;
   OrgServer  = Profile->NymServer;
   OrgPgpKey  = Profile->PGPKey;
   OrgName    = Profile->Name;
   OrgRetired = Profile->Retired;

   Profile->GetPublicKey(PublicKey);
   Profile->GetSecretKey(SecretKey);

   Name->Text = OrgName;
   UserID->Text = OrgUserID;

   RetiredBox->Checked = OrgRetired;
   Caption = Caption + " -- " + OrgUserID + "@" + OrgServer;

   int idx = NymServers->Items->IndexOf(OrgServer);
   if (idx < 0)
   {
      NymServers->Items->Add(OrgServer);
      idx = NymServers->Items->IndexOf(OrgServer);
   }
   NymServers->ItemIndex = idx;

   PGPkey->Text = Profile->PGPKey;

   OrgOptions = Profile->GetOptions();
   SetOptions(OrgOptions);

   Profile->GetReplyBlocks(OrgBlocks);
   if (OrgBlocks->Count > 1)
   {
      TTreeNode *node, *cnode;
      for (int i = 0; i < OrgBlocks->Count; i++)
      {
         AnsiString line = OrgBlocks->Strings[i].Trim();
         if (line.IsEmpty())
            continue;

         if (line.Pos("Reply-Block:") > 0)
         {
            node = ReplyBlocks->Items->AddChild(NULL, line);
            continue;
         }

         if (line.Pos(":") == 0)
         {
            cnode = ReplyBlocks->Items->AddChild(node, line);
            continue;
         }

         ReplyBlocks->Items->AddChild(cnode, line);
      }

      node = ReplyBlocks->Items->GetFirstNode();
      node->Expand(false);
      while ((node = node->GetNext()) != NULL)
      {
         if (node->Level == 0)
            node->Expand(false);
      }

      ReplyBlocks->Items->GetFirstNode()->MakeVisible();
   }

   if (OrgRetired)
   {
      ActiveControl = RetiredBox;
      RetiredBox->Checked = true;
   }

   History->SelStart = 0;
   return Profile->Count;
}
//---------------------------------------------------------------------------

void __fastcall TNymDlg::SaveNym()
{
   if (SecureModeMsg(this))
      return;

   if (Modified())
      SaveToProfile();

   Global->DB->DelNymByName(Profile->Nym);
   Global->DB->PutNym(Profile->NymIndexStr, Profile);
}
//---------------------------------------------------------------------------

int __fastcall TNymDlg::GetReplyBlocks(TStrings *blocks)
{
   assert(blocks != NULL);// && blocks->Count > 0);

   blocks->Clear();
   if (ReplyBlocks->Items->Count == 0)
   {
      blocks->Add("Reply-Block:");
      return 0;
   }

   AnsiString spc = "    ";
   TTreeNode *node = ReplyBlocks->Items->GetFirstNode();
   blocks->Add(node->Text);

   while ((node = node->GetNext()) != NULL)
   {
      AnsiString line = node->Text;

      if (line.Pos("Reply-Block:") == 1)
         blocks->Add("");

      blocks->Add(spc.SubString(1, node->Level - 1) + line);
   }

   return blocks->Count;
}
//---------------------------------------------------------------------------

AnsiString __fastcall TNymDlg::GetOptions()
{
   AnsiString opts;

   opts += OptAck->Checked ? "+acksend" : "-acksend";
   opts += OptSign->Checked ? " +signsend" : " -signsend";
   opts += OptCrypt->Checked ? " +cryptrecv" : " -cryptrecv";
   opts += OptFixed->Checked ? " +fixedsize" : " -fixedsize";
   opts += OptDisable->Checked ? " +disable" : " -disable";
   opts += OptBcc->Checked ? " +nobcc" : " -nobcc";
   opts += OptFinger->Checked ? " +fingerkey" : " -fingerkey";

   return opts;
}
//---------------------------------------------------------------------------

void __fastcall TNymDlg::SetOptions(const AnsiString &opts)
{
   assert (opts != "");

   OptAck->Checked = opts.Pos("+acksend") > 0;
   OptSign->Checked = opts.Pos("+signsend") > 0;
   OptCrypt->Checked = opts.Pos("+cryptrecv") > 0;
   OptDisable->Checked = opts.Pos("+disable") > 0;
   OptFixed->Checked = false;
   OptFinger->Checked = opts.Pos("+fingerkey") > 0;
   OptBcc->Checked = opts.Pos("+nobcc") > 0;
}
//---------------------------------------------------------------------------

void __fastcall TNymDlg::Prof0Click(TObject *Sender)
{
   History->Clear();
   History->Lines->BeginUpdate();

   if (Sender == Prof0) Profile->GetHistory(0, History->Lines); else
   if (Sender == Prof1) Profile->GetHistory(1, History->Lines); else
   if (Sender == Prof2) Profile->GetHistory(2, History->Lines); else
   if (Sender == Prof3) Profile->GetHistory(3, History->Lines); else
   if (Sender == Prof4) Profile->GetHistory(4, History->Lines); else
   if (Sender == Prof5) Profile->GetHistory(5, History->Lines); else
   if (Sender == Prof6) Profile->GetHistory(6, History->Lines); else
   if (Sender == Prof7) Profile->GetHistory(7, History->Lines); else
   if (Sender == Prof8) Profile->GetHistory(8, History->Lines); else
   if (Sender == Prof9) Profile->GetHistory(9, History->Lines); else
   if (Sender == Prof10)
   {
      Snooze(300);
      History->Lines->AddStrings(Profile);
   }

   History->Lines->EndUpdate();
   History->Modified = false;
}
//---------------------------------------------------------------------------

void __fastcall TNymDlg::CopyBtnClick(TObject *Sender)
{
   Clipboard()->AsText = History->SelText;
}
//---------------------------------------------------------------------------

void __fastcall TNymDlg::Timer1Timer(TObject *Sender)
{
   Timer1->Enabled = false;
   if (Profile->Count)
      LoadFromProfile();
}
//---------------------------------------------------------------------------

void __fastcall TNymDlg::FormShow(TObject *Sender)
{
   Timer1->Enabled = true;
}

//---------------------------------------------------------------------------
// Note Page
//---------------------------------------------------------------------------

void __fastcall TNymDlg::NoteMenuPopup(TObject *Sender)
{
   NoteCutItem->Enabled = NotePad->SelLength > 0;
   NoteCopyItem->Enabled = NotePad->SelLength > 0;
   NotePasteItem->Enabled = Clipboard()->HasFormat(CF_TEXT);
   NoteSelectAllItem->Enabled = NotePad->Lines->Text.Length() > 0;
   NotePrintItem->Enabled = NotePad->Lines->Text.Length() > 0;
}
//---------------------------------------------------------------------------

void __fastcall TNymDlg::SetNoteBtns()
{
   NoteCutBtn->Enabled = NotePad->SelLength > 0;
   NoteCopyBtn->Enabled = NotePad->SelLength > 0;
   NotePasteBtn->Enabled = Clipboard()->HasFormat(CF_TEXT);
   NotePrintBtn->Enabled = NotePad->Lines->Text.Length() > 0;
}
//---------------------------------------------------------------------------

void __fastcall TNymDlg::NotePadSelectionChange(TObject *Sender)
{
   SetNoteBtns();
}
//---------------------------------------------------------------------------

void __fastcall TNymDlg::NoteCutItemClick(TObject *Sender)
{
   NotePad->CutToClipboard();
}
//---------------------------------------------------------------------------

void __fastcall TNymDlg::NoteCopyItemClick(TObject *Sender)
{
   NotePad->CopyToClipboard();
}
//---------------------------------------------------------------------------

void __fastcall TNymDlg::NotePasteItemClick(TObject *Sender)
{
   NotePad->PasteFromClipboard();
}
//---------------------------------------------------------------------------

void __fastcall TNymDlg::NoteSelectAllItemClick(TObject *Sender)
{
   NotePad->SelectAll();
}
//---------------------------------------------------------------------------

void __fastcall TNymDlg::NotePrintItemClick(TObject *Sender)
{
   AnsiString nym = UserID->Text + "@" + NymServers->Text;
   CheckPrintDocumentNow(this, "Print " + nym + "'s notes now?", "NOTES", nym, NotePad);
}

//---------------------------------------------------------------------------
// History Page
//---------------------------------------------------------------------------

void __fastcall TNymDlg::HistoryMenuPopup(TObject *Sender)
{
   HistCopyItem->Enabled = History->SelLength > 0;
   HistSelectAllItem->Enabled = History->Lines->Text.Length() > 0;
   HistPrintItem->Enabled = HistSelectAllItem->Enabled;
}
//---------------------------------------------------------------------------

void __fastcall TNymDlg::SetHistoryBtns()
{
   HistCopyBtn->Enabled = History->SelLength > 0;
   HistPrintBtn->Enabled = History->Lines->Text.Length() > 0;
}
//---------------------------------------------------------------------------

void __fastcall TNymDlg::HistorySelectionChange(TObject *Sender)
{
   SetHistoryBtns();
}
//---------------------------------------------------------------------------

void __fastcall TNymDlg::HistCopyItemClick(TObject *Sender)
{
   History->CopyToClipboard();
}
//---------------------------------------------------------------------------

void __fastcall TNymDlg::HistSelectAllItemClick(TObject *Sender)
{
   History->SelectAll();
}
//---------------------------------------------------------------------------

void __fastcall TNymDlg::HistPrintItemClick(TObject *Sender)
{
   AnsiString nym = UserID->Text + "@" + NymServers->Text;
   CheckPrintDocumentNow(this, "Print " + nym + "'s profile now?", "PROFILE", nym, History);
}

//---------------------------------------------------------------------------
// Replay Page
//---------------------------------------------------------------------------

void __fastcall TNymDlg::ReplayMenuPopup(TObject *Sender)
{
   ReplayCopyItem->Enabled = ReplayPad->SelLength > 0;
   ReplaySelectAllItem->Enabled = ReplayPad->Lines->Text.Length() > 0;
   ReplayPrintItem->Enabled = ReplaySelectAllItem->Enabled;
}
//---------------------------------------------------------------------------

void __fastcall TNymDlg::SetReplayBtns()
{
   ReplayCopyBtn->Enabled = ReplayPad->SelLength > 0;
   ReplayPrintBtn->Enabled = ReplayPad->Lines->Text.Length() > 0;
   ReplaySendBtn->Enabled = ReplayPrintBtn->Enabled;
}
//---------------------------------------------------------------------------

void __fastcall TNymDlg::ReplayPadSelectionChange(TObject *Sender)
{
   SetReplayBtns();
}
//---------------------------------------------------------------------------

void __fastcall TNymDlg::ReplayCopyItemClick(TObject *Sender)
{
   ReplayPad->CopyToClipboard();
}
//---------------------------------------------------------------------------

void __fastcall TNymDlg::ReplaySelectAllItemClick(TObject *Sender)
{
   ReplayPad->SelectAll();
}
//---------------------------------------------------------------------------

void __fastcall TNymDlg::ReplayPrintItemClick(TObject *Sender)
{
   AnsiString nym = UserID->Text + "@" + NymServers->Text;
   CheckPrintDocumentNow(this, "Print " + nym + "'s message now?", "REPLAY", nym, ReplayPad);
}

//---------------------------------------------------------------------------
// Log Page
//---------------------------------------------------------------------------

void __fastcall TNymDlg::LogMenuPopup(TObject *Sender)
{
   LogCopyItem->Enabled = LogPad->SelLength > 0;
   LogSelectAllItem->Enabled = LogPad->Lines->Text.Length() > 0;
   LogPrintBtn->Enabled = LogPad->Lines->Text.Length() > 0;
}
//---------------------------------------------------------------------------

void __fastcall TNymDlg::SetLogBtns()
{
   LogCopyBtn->Enabled = LogPad->SelLength > 0;
   LogPrintBtn->Enabled = LogPad->Lines->Text.Length() > 0;
}
//---------------------------------------------------------------------------

void __fastcall TNymDlg::LogPadSelectionChange(TObject *Sender)
{
   SetLogBtns();
}
//---------------------------------------------------------------------------

void __fastcall TNymDlg::LogCopyItemClick(TObject *Sender)
{
   LogPad->CopyToClipboard();
}
//---------------------------------------------------------------------------

void __fastcall TNymDlg::LogSelectAllItemClick(TObject *Sender)
{
   LogPad->SelectAll();
}
//---------------------------------------------------------------------------

void __fastcall TNymDlg::LogPrintItemClick(TObject *Sender)
{
   AnsiString nym = UserID->Text + "@" + NymServers->Text;
   CheckPrintDocumentNow(this, "Print " + nym + "'s log now?", "LOG", nym, LogPad);
}

//---------------------------------------------------------------------------
// Other stuff
//---------------------------------------------------------------------------

bool __fastcall TNymDlg::Modified()
{
/*sm(GetOptions() + "  " + OrgOptions + "\n" +
   Name->Text + "  " + OrgName + "\n" +
   UserID->Text + "  " + OrgUserID + "\n" +
   IntToStr(RetiredBox->Checked) + "  " + IntToStr(OrgRetired) + "\n" +
   PGPkey->Text + "\n" + OrgPgpKey + "\n" +
   IntToStr(ReplyBlocksModified()));*/

   if (History->Modified) return true;
   if (ReplayPad->Modified) return true;
   if (LogPad->Modified) return true;
   if (NotePad->Modified) return true;
   if (GetOptions() != OrgOptions) return true;
   if (Name->Text != OrgName) return true;
   if (UserID->Text != OrgUserID) return true;
   if (RetiredBox->Checked != OrgRetired) return true;
   if (PGPkey->Text != OrgPgpKey) return true;
   if (ReplyBlocksModified()) return true;

   return false;
}
//---------------------------------------------------------------------------

bool __fastcall TNymDlg::ReplyBlocksModified()
{
   TStringList *NewBlocks = new TStringList();

   GetReplyBlocks(NewBlocks);
   bool mod = NewBlocks->Text != OrgBlocks->Text;

   delete NewBlocks;

   return mod;
}
//---------------------------------------------------------------------------

void __fastcall TNymDlg::NewRBBtnClick(TObject *Sender)
{
   if (UserID->Text.IsEmpty())
   {
      MsgBox(this,
        "You need to give your nym a userid before creating a reply block.",
         MT_INFO, MB_OK);
      ActiveControl = UserID;
      return;
   }

   TTreeNode *xnode, *node, *cnode;
   TStringList *replyblock = new TStringList();

   if (RunReplyBlockWizard(this, Nym(), replyblock))
   {
      node = ReplyBlocks->Items->AddChild(NULL, replyblock->Strings[0]);

      for (int i = 1; i < replyblock->Count; i++)
      {
         AnsiString line = replyblock->Strings[i];
         switch (line.Length() - line.Trim().Length())
         {
            case 1: cnode = ReplyBlocks->Items->AddChild(node, line.Trim()); break;
            case 2: ReplyBlocks->Items->AddChild(cnode, line.Trim()); break;
         }
      }

      ReplyBlocks->FullCollapse();
      node = ReplyBlocks->Items->GetFirstNode();
      while (node != NULL)
      {
         if (node->Level == 0)
         {
          //node->Expand(false);
            xnode = node;
         }
         node = node->GetNext();
      }
      xnode->Expand(false);
    //xnode->MakeVisible();

      ReplyBlocks->TopItem = xnode;
   }
   ModRBBtn->Enabled = false;
   DelRBBtn->Enabled = false;

   delete replyblock;
}
//---------------------------------------------------------------------------

void __fastcall TNymDlg::ModRBBtnClick(TObject *Sender)
{
   TTreeNode *node, *cnode, *xnode;
   TStringList *replyblock = new TStringList();

   int x = 1;
   AnsiString blank = "     ";
   xnode = node = ReplyBlocks->Selected;
 //node = xnode;

   while (xnode != NULL)
   {
      if (x > 1 && xnode->Level == 0)
         break;

      replyblock->Add(blank.SubString(1, xnode->Level) + xnode->Text);
      xnode = xnode->GetNext();
      x++;
   }

   if (ModifyReplyBlockWizard(this, Nym(), replyblock))
   {
      node->DeleteChildren();
      node->Text = replyblock->Strings[0];
    //node = ReplyBlocks->Items->AddChild(NULL, replyblock->Strings[0]);

      for (int i = 1; i < replyblock->Count; i++)
      {
         AnsiString line = replyblock->Strings[i];
         switch (line.Length() - line.Trim().Length())
         {
            case 1: cnode = ReplyBlocks->Items->AddChild(node, line.Trim()); break;
            case 2: ReplyBlocks->Items->AddChild(cnode, line.Trim()); break;
         }
      }

      ReplyBlocks->FullCollapse();
      node->Expand(true);
      node->MakeVisible();
   }
   DelRBBtn->Enabled = false;
   ModRBBtn->Enabled = false;

   delete replyblock;
}
//---------------------------------------------------------------------------

void __fastcall TNymDlg::ReplyBlocksClick(TObject *Sender)
{
   ModRBBtn->Enabled = ReplyBlocks->Selected && ReplyBlocks->Selected->Level == 0;
   DelRBBtn->Enabled = ModRBBtn->Enabled;
}
//---------------------------------------------------------------------------

void __fastcall TNymDlg::EnableForm(bool enable)
{
   TColor color = enable ? WinColor : Color;
   Name->Color = color;
   Name->ReadOnly = enable;
 //PGPkey->Color = color;
 //PGPkey->Enabled = enable;
   KeyImportBtn->Enabled = enable;
   ReplyBlocks->Color = color;
   OptAck->Enabled = enable;
   OptSign->Enabled = enable;
   OptCrypt->Enabled = enable;
   OptDisable->Enabled = enable;
   OptFinger->Enabled = enable;
   OptFixed->Enabled = enable;
   OptBcc->Enabled = enable;

   NewBtn->Enabled = enable;
   ModRBBtn->Enabled = enable && ReplyBlocks->Selected != NULL;
   DelRBBtn->Enabled = enable && ReplyBlocks->Selected != NULL;

   History->Color = color;
   ReplayPad->Color = Color;
   LogPad->Color = Color;
   NotePad->Color = color;

   Caption = (enable ? " Modify Nym -- " : " Retired Nym -- ") + Nym();
}
//---------------------------------------------------------------------------

void __fastcall TNymDlg::RetiredBoxClick(TObject *Sender)
{
   if (RetiredBox->Checked)
   {
      RetiredBox->Caption = "&Retired Nym";
      EnableForm(false);
   }
   else
   {
      RetiredBox->Caption = "&Retire Nym";
      EnableForm(true);
   }

   ModifyNymName->Font->Color = Label6->Font->Color;
}
//---------------------------------------------------------------------------

void __fastcall TNymDlg::SourceBtnClick(TObject *Sender)
{
   EditConfigSource(this, Nym(), ReplyBlocks);
}
//---------------------------------------------------------------------------

void __fastcall TNymDlg::DelRBBtnClick(TObject *Sender)
{
   assert(ReplyBlocks->Selected && ReplyBlocks->Selected->Level == 0);

   if (MsgBox(this,
      "Delete the currently selected reply block?",
       MT_CONFIRM, MB_OKCANCEL) == IDOK)
   {
      ReplyBlocks->Selected->DeleteChildren();
      ReplyBlocks->Selected->Delete();
   }
}
//---------------------------------------------------------------------------

AnsiString __fastcall TNymDlg::Nym()
{
   AnsiString nym;

   if (!ModifyNymName->Caption.IsEmpty()) nym = ModifyNymName->Caption;
   else nym = UserID->Text + "@" + NymServers->Text;

   return nym;
}
//---------------------------------------------------------------------------

void __fastcall TNymDlg::MessageBtnClick(TObject *Sender)
{
   TPoint PT(0, 0); // + MessageBtn->Height);
   PT = MessageBtn->ClientToScreen(PT);
   PopupMenu1->Popup(PT.x, PT.y + MessageBtn->Height);
}
//---------------------------------------------------------------------------

bool __fastcall TNymDlg::CheckSaveB4Message()
{
   if (Modified())
   {
      if (MsgBox(this,
         "I'll need to save your profile before I can create a message.    \n"
         "Shall I save it?", MT_CONFIRM, MB_OKCANCEL) == IDCANCEL)
            return false;

      SaveNym();
   }

   return true;
}
//---------------------------------------------------------------------------

void __fastcall TNymDlg::PopNymCreateItemClick(TObject *Sender)
{
   if (ReplyBlocks->Items->Count == 0)
   {
      MsgBox(this,
         "You must define at least 1 reply block before creating a \n"
         "nym config message.",
          MT_INFO, MB_OK);
       return;
   }

   if (!CheckSaveB4Message())
      return;

   if (!CheckValidKeyForProfile(this, PGP, Profile))
   {
      return;
   }

   bool preview;
   TStringList *Template = new TStringList();

   if (GenNymCreateMessage(this, preview, Template))
   {
      TNymConfig *nymconfig = new TNymConfig(this, PGP, Profile);
      nymconfig->CreateNymCreateMessage(Template, preview);
      delete nymconfig;
   }

   delete Template;
}
//---------------------------------------------------------------------------

void __fastcall TNymDlg::PopNymDeleteItemClick(TObject *Sender)
{
   if (CheckNeedsRListUpdate())
      return;

   if (!CheckSaveB4Message())
      return;

   if (!CheckValidKeyForProfile(this, PGP, Profile))
   {
      return;
   }

   TStringList *Template = new TStringList();

   if (GenNymDeleteMessage(this, Template))
   {
      TNymConfig *nymconfig = new TNymConfig(this, PGP, Profile);
      nymconfig->CreateDeleteMessage(Template);
      delete nymconfig;
   }

   delete Template;
}
//---------------------------------------------------------------------------

void __fastcall TNymDlg::PopNymModifyItemClick(TObject *Sender)
{
   if (CheckNeedsRListUpdate())
      return;

   if (!CheckSaveB4Message())
      return;

   if (!CheckValidKeyForProfile(this, PGP, Profile))
   {
      return;
   }

   bool preview = false, cmds = false, rblk = false, key = false;
   TStringList *Template = new TStringList();

   if(GenNymModifyMessage(this, preview, Template, cmds, key, rblk, Profile))
   {
      TNymConfig *nymconfig = new TNymConfig(this, PGP, Profile);
      nymconfig->CreateModifyMessage(Template, preview, cmds, key, rblk,
         Profile->PublicKeyID);
      delete nymconfig;
   }

   delete Template;
}
//---------------------------------------------------------------------------

void __fastcall TNymDlg::PopReplayItemClick(TObject *Sender)
{
   if (!CheckSaveB4Message())
      return;

   if (!CheckValidKeyForProfile(this, PGP, Profile))
   {
      return;
   }

   TStringList *Template = new TStringList();

   if (GenNymReplayMessage((TForm*)Owner, Template))
   {
      TNymConfig *nymconfig = new TNymConfig((TForm*)Owner, PGP, Profile);
      nymconfig->CreateReplayMessage(Template);
      delete nymconfig;
   }

   delete Template;
}
//---------------------------------------------------------------------------

void __fastcall TNymDlg::PopFromNymItemClick(TObject *Sender)
{
   if (!CheckSaveB4Message())
      return;

   if (!CheckValidKeyForProfile(this, PGP, Profile))
   {
      return;
   }

   TStringList *Template = new TStringList();

   if (GenFromNymMessage((TForm*)Owner, Template))
   {
      TNymConfig *nymconfig = new TNymConfig((TForm*)Owner, PGP, Profile);
      nymconfig->CreateFromNymMessage(Template);
      delete nymconfig;
   }

   delete Template;
}
//---------------------------------------------------------------------------

void __fastcall TNymDlg::Timer2Timer(TObject *Sender)
{
   Timer2->Enabled = false;
   TStringList *replyblock = new TStringList();

   if (DoFullAutoReplyBlock(this, UserID->Text + "@" + NymServers->Text,
      replyblock, Address, ChainLength, Font))
   {
      TTreeNode *node, *cnode, *xnode;
      node = ReplyBlocks->Items->AddChild(NULL, replyblock->Strings[0]);

      for (int i = 1; i < replyblock->Count; i++)
      {
         AnsiString line = replyblock->Strings[i];
         switch (line.Length() - line.Trim().Length())
         {
            case 1: cnode = ReplyBlocks->Items->AddChild(node, line.Trim()); break;
            case 2: ReplyBlocks->Items->AddChild(cnode, line.Trim()); break;
         }
      }

      ReplyBlocks->FullCollapse();
      node = ReplyBlocks->Items->GetFirstNode();
      while (node != NULL)
      {
         if (node->Level == 0)
         {
            xnode = node;
         }
         node = node->GetNext();
      }
      xnode->Expand(false);
      ReplyBlocks->TopItem = xnode;

      //RBModified = true;
      if (MsgBox(this,
         "This nym profile I have just created for you is complete and ready    \n"
         "to use. There is no need to modify it in any way. You'll need to send a    \n"
         "message to the nym server to create the nym. Shall I make one up    \n"
         "right now?    ",
          MT_CONFIRM, MB_YESNO) == IDYES)
      {
         Snooze(300);
         PopNymCreateItem->Click();
      }
   }

   delete replyblock;
}
//---------------------------------------------------------------------------

void __fastcall TNymDlg::KeyImportBtnClick(TObject *Sender)
{
   setwincursors(this, crHourGlass);

   AnsiString newkey = PGPkey->Text;
   if (GetNewNymKey(this, PGP, newkey, NymServers->Text))
   {
      AnsiString OldKey, NewKey;
      OldKey = GetTok(0, PGPkey->Text, ' ');
      PGPkey->Text = newkey;
      SaveToProfile();
      NewKey = GetTok(0, newkey, ' ');

      bool preview = false, cmds = false, rblk = false, key = true;
      TStringList *Template = new TStringList();

      if(GenNymModifyMessage(this, preview, Template, cmds, key, rblk, Profile,
         NewKey, OldKey))
      {
         TNymConfig *nymconfig = new TNymConfig(this, PGP, Profile);
         nymconfig->CreateModifyMessage(Template, preview, cmds, key, rblk, OldKey);
         delete nymconfig;
      }

      delete Template;
   }

   setwincursors(this, crDefault);
}
//---------------------------------------------------------------------------

void __fastcall TNymDlg::PopupMenu1Popup(TObject *Sender)
{
   PopReplayItem->Enabled = ReplayPad->Lines->Count > 0;
}
//---------------------------------------------------------------------------

void __fastcall TNymDlg::RandomMouseMove(TObject *Sender,
      TShiftState Shift, int X, int Y)
{
   //static int x = 0; x++;
   //Caption = " " + IntToStr(x);
   PGP->RandomMouseMoved();
}
//---------------------------------------------------------------------------

