//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop
#include "Global2.h"
#include "NymKeyDlgs.h"
#include "ValidNymKeys.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TNymKeyDlg *NymKeyDlg;
//---------------------------------------------------------------------------

bool __fastcall GetNewNymKey(TForm *Owner, TPGPsdk *PGP, AnsiString &PGPKey,
   const AnsiString& Nymserver)
{
   NymKeyDlg = new TNymKeyDlg(Owner, PGP, PGPKey, Nymserver);
   NymKeyDlg->ActiveControl = NymKeyDlg->PGPKey;

   NymKeyDlg->ShowModal();
   bool ok = NymKeyDlg->ModalResult == mrOk;
   {
      PGPKey = NymKeyDlg->PGPKey->Text;
   }
   delete NymKeyDlg;

   return ok;
}
//---------------------------------------------------------------------------
__fastcall TNymKeyDlg::TNymKeyDlg(TComponent* Owner)
        : TForm(Owner)
{
}
//---------------------------------------------------------------------------

__fastcall TNymKeyDlg::TNymKeyDlg(TComponent* Owner, TPGPsdk *Pgp, const AnsiString &PGPkey,
   const AnsiString &NymServer) : TForm(Owner)
{
   PGP = Pgp;
   PGPKey->ItemHeight = abs(PGPKey->Font->Height) + 5;
   Nymserver = NymServer;
   ThePGPKey = PGPkey;
   LoadPGPKeys();

   TColor color;
   LoadFonts(AppIni(), NymIni, color, Font, PGPKey->Font, NULL);
   PGPKey->Color = color;
}
//---------------------------------------------------------------------------

//AnsiString __fastcall TNymKeyDlg::NymServer()
//{
//   return GetTok(1, GetEAddress(ThePGPKey), '@');
//}
//---------------------------------------------------------------------------

void __fastcall TNymKeyDlg::PGPKeyDrawItem(TWinControl *Control, int Index,
      TRect &Rect, TOwnerDrawState State)
{
   TCanvas *canvas = PGPKey->Canvas;
   AnsiString keyid, name, key = PGPKey->Items->Strings[Index];
   if (Index == 0)
   {
      canvas->TextRect(Rect, Rect.Left + 2, Rect.Top + 1, key);
      return;
   }

   keyid = key.SubString(1, 10);
   name = key.SubString(12, 256);

   AnsiString orgname = canvas->Font->Name;
   canvas->Font->Name = "Courier New";
   int idwidth = canvas->TextWidth(keyid);

   canvas->TextRect(Rect, Rect.Left + 2, Rect.Top + 1, keyid);
   canvas->Font->Name = orgname;
   Rect.Left += idwidth + 12;
   canvas->TextRect(Rect, Rect.Left, Rect.Top + 1, name);
}
//---------------------------------------------------------------------------

void __fastcall TNymKeyDlg::FormCloseQuery(TObject *Sender, bool &CanClose)
{
   if (ModalResult == mrOk)
   {
      if (PGPKey->ItemIndex == 0)
      {
         MsgBox(this, "You need to select a key or click Cancel!    ",
            MT_INFO, MB_OK);
         CanClose = false;
         return;
      }
   }
}
//---------------------------------------------------------------------------

AnsiString __fastcall TNymKeyDlg::KeyAlgo(const AnsiString &Key)
{
   return GetTok(4, Key, '\255');
}
//---------------------------------------------------------------------------

int __fastcall TNymKeyDlg::KeyBits(const AnsiString &Key)
{
   return GetTok(3, Key, '\255').ToIntDef(2049);
}
//---------------------------------------------------------------------------

bool __fastcall TNymKeyDlg::KeySecret(const AnsiString &Key)
{
   return GetTok(6, Key, '\255') == "1";
}
//---------------------------------------------------------------------------

AnsiString __fastcall TNymKeyDlg::KeyID(const AnsiString &Key)
{
   return GetTok(1, Key, '\255');
}
//---------------------------------------------------------------------------

AnsiString __fastcall TNymKeyDlg::KeyUserID(const AnsiString &Key)
{
   return GetTok(0, Key, '\255');
}
//---------------------------------------------------------------------------

void __fastcall TNymKeyDlg::LoadPGPKeys()
{
   TStringList *keys = new TStringList();
   keys->Sorted = true;

   PGP->GetKeysLV(PGPPublicRing(), PGPSecretRing(), keys);

   for (int i = 0; i < keys->Count; i++)
   {
      AnsiString key = keys->Strings[i];
      if (!ValidKeyForNymserver(this, key, Nymserver))
         continue;

      if (!KeySecret(key))
         continue;

      if (KeyAlgo(key).Pos("RSA legacy") == 1 && KeyBits(key) > 2048)
         continue;

      AnsiString addkey = KeyID(key) + " " + KeyUserID(key);

      if (addkey != ThePGPKey)
         PGPKey->Items->Add(addkey);
   }

   PGPKey->Items->Insert(0, "select here ---------------------------------------------");
   PGPKey->ItemIndex = 0;

   delete keys;
}
//---------------------------------------------------------------------------

/*bool __fastcall TNymKeyDlg::HasCap(const AnsiString &cap)
{
   return caps.Pos(" " + cap + " ") > 0;
}
//---------------------------------------------------------------------------

AnsiString __fastcall TNymKeyDlg::GetCaps(const AnsiString &nymserver)
{
   if (ThePGPKey.IsEmpty())
      return "";

   AnsiString thecaps, nymserverfile = AppDir() + "nymserver.asc"; //NymserverFile;
   if (FileExists(nymserverfile))
   {
      TStringList *Caps = new TStringList();
      Caps->LoadFromFile(nymserverfile);

      for (int i = 0; i < Caps->Count; i++)
      {
         AnsiString line = Caps->Strings[i];
         if (line.Pos("-----BEGIN PGP ") == 1)
            break;

         if (line.Pos("$nymserver{") == 1 && line.Pos(nymserver))
         {
            thecaps = GetTok(3, line, '\"') + " ";
            break;
         }
      }
      delete Caps;
   }

   return thecaps;
}
//---------------------------------------------------------------------------

bool __fastcall TNymKeyDlg::ValidKeyForNymserver(const AnsiString &Key)
{
   AnsiString keyalgo = KeyAlgo(Key);
   int keybits = KeyBits(Key);

   if (HasCap("newnym"))
   {
      if (keyalgo.Pos("RSA legacy") == 0 || keybits > 2048)
         return false;
   }
   return true;
}*/
//---------------------------------------------------------------------------
