//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "Global2.h"
#include "RBWizDlgs.h"
#include "RBRemDlgs.h"
#include "RBLegWins.h"
#include "RBCapWins.h"
#include "RBBrkWins.h"
#include "RBProbDlgs.h"
#include "RModDlgs.h"
#include "NymAdvDlgs.h"
#include "CfgSourceDlgs.h"
#include "NymProbDlgs.h"
#include "RBHelperDlgs.h"
#include "pgpsdk.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TRBWizardDlg *RBWizardDlg;
//---------------------------------------------------------------------------

bool __fastcall RunReplyBlockWizard(TForm *Owner, const AnsiString &Nym, TStrings *ReplyBlock)
{
   if (SecureModeMsg(Owner))
      return false;

   RBWizardDlg = new TRBWizardDlg(Owner);
   RBWizardDlg->ReplyBlock->AddStrings(ReplyBlock);
   RBWizardDlg->Nym = Nym;

   RBWizardDlg->ShowModal();
   bool ok = RBWizardDlg->ModalResult == mrOk;
   if (ok)
   {
      ReplyBlock->Clear();
      ReplyBlock->AddStrings(RBWizardDlg->ReplyBlock);
   }

   delete RBWizardDlg;
   return ok;
}
//---------------------------------------------------------------------------

bool __fastcall FullAutoReplyBlockWizard(TForm *Owner, const AnsiString &Nym, TStrings *ReplyBlock,
   const AnsiString &EmailAddress, int ChainLength)
{
   if (SecureModeMsg(Owner))
      return false;

   RBWizardDlg = new TRBWizardDlg(Owner);
   RBWizardDlg->ReplyBlock->AddStrings(ReplyBlock);
   RBWizardDlg->Nym = Nym;
   RBWizardDlg->EmailAddress->Checked = true;
   RBWizardDlg->Destination->Text = EmailAddress;
   RBWizardDlg->autochainlength = ChainLength;

   bool ok = RBWizardDlg->AutoGenReplyBlock(ChainLength);
   if (ok)
   {
      ReplyBlock->Clear();
      ReplyBlock->AddStrings(RBWizardDlg->ReplyBlock);
   }

   delete RBWizardDlg;
   return true;
}
//---------------------------------------------------------------------------

bool __fastcall ModifyReplyBlockWizard(TForm *Owner, const AnsiString &Nym, TStrings *ReplyBlock)
{
   if (SecureModeMsg(Owner))
      return false;

   RBWizardDlg = new TRBWizardDlg(Owner);
   RBWizardDlg->ReplyBlock->AddStrings(ReplyBlock);
   RBWizardDlg->HelperBtn->Enabled = false;
   RBWizardDlg->modify = true;
   RBWizardDlg->Nym = Nym;
   if (!Nym.IsEmpty())
      RBWizardDlg->Caption = RBWizardDlg->Caption + " -- " + Nym;

   RBWizardDlg->ShowModal();
   bool ok = RBWizardDlg->ModalResult == mrOk;
   if (ok)
   {
      ReplyBlock->Clear();
      ReplyBlock->AddStrings(RBWizardDlg->ReplyBlock);
   }

   delete RBWizardDlg;
   return ok;
}
//---------------------------------------------------------------------------

bool __fastcall FinalHeader(const AnsiString &line)
{
   AnsiString finals = "Subject,Newsgroups,Followup-To,From,Cc,Bcc,Reply-To";
   return finals.Pos(GetTok(0, line, ':')) > 0;
}

//---------------------------------------------------------------------------
// RBWizDlgs
//---------------------------------------------------------------------------

__fastcall TRBWizardDlg::TRBWizardDlg(TComponent* Owner)
        : TForm(Owner)
{
   PageControl->ActivePage = LengthTab;
   ActiveControl = Destination;
   mlist = new TStringList();
   rlist = new TStringList();
   brokenchains = new TStringList();
   capstrings = new TStringList();

   AnsiString line, rlisttxt = MixDir() + "rlist.txt";
   if (FileExists(rlisttxt))
   {
      rlist->LoadFromFile(rlisttxt);
      GetBrokenCpunkChains(rlist, brokenchains);
      GetCapStrings(rlist, capstrings);
   }

   ReplyBlock = new TReplyBlock();
                  // emailaddress, newsgroup, email?, post?, esub? chain length
   lastdefs = Global->DB->GetIni("nymlastrbdefaults", ",,1,1,1,3");
   bool email = GetTok(2, lastdefs, ',') == "1";

   if (email) EmailAddress->Checked = true;
   else Newsgroup->Checked = true;

   emailaddr = GetTok(0, lastdefs, ',');
   newsgroup = GetTok(1, lastdefs, ',');

   if (newsgroup.IsEmpty())
      newsgroup = "alt.anonymous.messages";

   Destination->Text = email ? emailaddr : newsgroup;

   PostDirect->Checked = GetTok(3, lastdefs, ',') == "1";
   PostDirect->Enabled = !email;

   Esub->Checked = GetTok(4, lastdefs, ',') == "1";
   Esub->Enabled = !email;

   AnsiString m2nfile = AppDir() + Mail2newsFile;
   if (FileExists(m2nfile))
   {
      M2nGateways->Items->LoadFromFile(m2nfile);
   }

   HelperBtn->Enabled = EmailAddress->Checked || PostDirect->Checked;

   TColor color;
   LoadFonts(AppIni(), NymIni, color, Font, ListBox1->Font, NULL);
   Destination->Font->Assign(ListBox1->Font);
   RemSelBox->Font->Assign(ListBox1->Font);
   Chain->Font->Assign(ListBox1->Font);
   M2nGateways->Font->Assign(ListBox1->Font);
   UseGateways->Font->Assign(ListBox1->Font);
   RBView->Font->Assign(ListBox1->Font);
   Label9->Font->Color = ListBox1->Font->Color;
   Label2->Font->Color = ListBox1->Font->Color;
   Label7->Font->Color = ListBox1->Font->Color;
   Label6->Font->Color = ListBox1->Font->Color;
}
//---------------------------------------------------------------------------

void __fastcall TRBWizardDlg::FormCloseQuery(TObject *Owner, bool &CanClose)
{
   if (SecureModeMsg(this))
   {
      CanClose = false;
      return;
   }

   if (ModalResult == mrOk || !Visible)
   {  // copy treeview to strings
      int x = 1;
      AnsiString blank = "     ";
      TTreeNode *node = RBView->Items->GetFirstNode();
      while (node != NULL)
      {
         ReplyBlock->Add(blank.SubString(1, node->Level) + node->Text);
         node = node->GetNext();
         x++;
      }

      bool email = EmailAddress->Checked;
      lastdefs = Destination->Text;
      if (email)
         lastdefs = Destination->Text + "," + newsgroup;
      else
         lastdefs = emailaddr + "," + Destination->Text;

      lastdefs += "," + email ? "1" : "0";
      lastdefs += "," + PostDirect->Checked ? "1" : "0";
      lastdefs += "," + Esub->Checked ? "1" : "0";

      Global->DB->PutIni("nymlastrbdefaults", lastdefs);
   }

   bool email = EmailAddress->Checked;
   AnsiString lastdefs = Destination->Text;
   if (email)
      lastdefs = Destination->Text + "," + newsgroup;
   else
      lastdefs = emailaddr + "," + Destination->Text;

   lastdefs += email ? ",1" : ",0";
   lastdefs += PostDirect->Checked ? ",1" : ",0";
   lastdefs += Esub->Checked ? ",1," : ",0,";
   //lastdefs += Length->Text;

   Global->DB->PutIni("nymlastrbdefaults", lastdefs);
}
//---------------------------------------------------------------------------

__fastcall TRBWizardDlg::~TRBWizardDlg()
{
   delete capstrings;
   delete brokenchains;
   delete rlist;
   delete mlist;
}
//---------------------------------------------------------------------------

void __fastcall TRBWizardDlg::GetBrokenCpunkChains(TStrings *Rlist, TStrings *Broken)
{
   Broken->Clear();

   bool saving = false;
   for (int i = 0; i < Rlist->Count; i++)
   {
      AnsiString line = Rlist->Strings[i];
      if (line.Pos("Broken type-I remailer chains:") == 1)
      {
         saving = true;
         continue;
      }

      if (saving && line.Trim().IsEmpty())
         break;

      if (saving)
      {
         int pos;
         while ((pos = line.Pos("(")) > 0) line.Delete(pos, 1);
         while ((pos = line.Pos(")")) > 0) line.Delete(pos,1);
         while ((pos = line.Pos(" ")) > 0) line[pos] = ',';

         Broken->Add(line);
      }
   }
}
//---------------------------------------------------------------------------

void __fastcall TRBWizardDlg::GetCapStrings(TStrings *Rlist, TStrings *CapStrings)
{
   // get remailer capstrings first so GetCaps for nymservers below works
   for (int i = 0; i < Rlist->Count; i++)
   {
      AnsiString line = Rlist->Strings[i].Trim();
      if (line.Pos("$remailer{") == 1)
         CapStrings->Add(line);
   }
   // construct nymserver caps and combine
   AnsiString nymsrvfile = AppDir() + "nymserver.asc";
   if (FileExists(nymsrvfile))
   {
      TStringList *nymsrv = new TStringList();
      nymsrv->LoadFromFile(nymsrvfile);

      for (int i = 0; i < nymsrv->Count; i++)
      {
         AnsiString line = nymsrv->Strings[i].Trim();
       //if (line.Pos("-----BEGIN PGP PUBLIC KEY BLOCK-----") == 1)
       //   break;

         if (line.Pos("$nymserver{") == 1)
         {
            AnsiString caps, remailer;
            int p = line.Pos(";");
            if (p > 0)
            {
               remailer = line.SubString(p + 1, 64).Trim();
            }
            caps = GetTok(0, line, ';').Trim();
            if (!remailer.IsEmpty())
               caps += " " + GetCaps(remailer);
            CapStrings->Add(caps);
         }
      }
      delete nymsrv;
   }
//look(CapStrings);
}
//---------------------------------------------------------------------------

AnsiString __fastcall TRBWizardDlg::GetCaps(AnsiString Remailer)
{
   AnsiString caps, line;
   int p = Remailer.Pos(" {");
   if (p > 0)
      Remailer = Remailer.SubString(1, p - 2);

   for (int i = 0; i < capstrings->Count; i++)
   {
      line = capstrings->Strings[i];

             // remailer                      nymserver address in string
      if (GetTok(1, line, '"') == Remailer || line.Pos("@" + Remailer) > 0)
      {
         caps = line;
         break;
      }
   }

   return caps;
}
//---------------------------------------------------------------------------

bool __fastcall TRBWizardDlg::RemailerIsBroken(const AnsiString &remailer,
   const AnsiString &rembefore, const AnsiString &remafter, int &X)
{
   if (IsBroken(remailer, "*") && IsBroken("*", remailer))
   {
      // everything is broken from this and it is NOT the last remailer
      X = 2;
      return true;
   }
   if (IsBroken(remailer, "*") && !remafter.IsEmpty())
   {
      // everything is broken from this and it is NOT the last remailer
      X = 1;
      return true;
   }

   if (IsBroken("*", remailer) && !rembefore.IsEmpty())
   {
      // everything is broken to this and it is NOT first
      X = -1;
      return true;
   }

   if (IsBroken(rembefore, remailer))
   {
      // broken with the one before
      X = -1;
      return true;
   }

   if (IsBroken(remailer, remafter))
   {
      // broken with the one after
      X = 1;
      return true;
   }

   return false;
}
//---------------------------------------------------------------------------

bool __fastcall TRBWizardDlg::IsBroken(const AnsiString &Remailer1, const AnsiString &Remailer2)
{
   return brokenchains->IndexOf(Remailer1 + "," + Remailer2) >= 0;
}
//---------------------------------------------------------------------------

void __fastcall TRBWizardDlg::NextBtnClick(TObject *)
{
   if (SecureModeMsg(this))
      return;

   forward = true;;
   int idxafter, idxbefore = PageControl->ActivePageIndex, cnt = PageControl->PageCount;
   if (PageControl->ActivePage == ReviewTab)
   {
      ModalResult = mrOk;
      return;
   }

   if (PageControl->ActivePageIndex == 1)
   {
      if (EmailAddress->Checked || PostDirect->Checked)
         PageControl->SelectNextPage(true, false);
   }

   PageControl->SelectNextPage(true, false);
   idxafter = PageControl->ActivePageIndex;
   if (idxafter == idxbefore)
      return;

   BackBtn->Enabled = idxafter > 0;
   NextBtn->Caption = idxafter == cnt - 1 ? "&Finish" : "&Next";
   CancelBtn->Caption = idxafter == 0 ? "Close" : "Cancel";
}
//---------------------------------------------------------------------------

void __fastcall TRBWizardDlg::BackBtnClick(TObject *)
{
   if (SecureModeMsg(this))
      return;

   forward = false;
   int idx = PageControl->ActivePageIndex, cnt = PageControl->PageCount;
   if (idx > 0)
      idx--;

   BackBtn->Enabled = idx > 0;
   NextBtn->Enabled = idx <  cnt - 1;
   NextBtn->Caption = idx == cnt - 1 ? "&Finish" : "&Next";
   CancelBtn->Caption = idx == 0 ? "Close" : "Cancel";

   if (PageControl->ActivePage == ReviewTab)
      LoadSelectFromReview();
   if (PageControl->ActivePage == SelectTab)
      LoadLengthFromSelect();

   if (PageControl->ActivePageIndex == 3)
   {
      if (EmailAddress->Checked || PostDirect->Checked)
         PageControl->SelectNextPage(false, false);
   }

   PageControl->SelectNextPage(false, false);
   if (NextBtn->Caption == "&Finish")
      ActiveControl = NextBtn;
   else
      NextBtn->ModalResult = 0;
}
//---------------------------------------------------------------------------

/*AnsiString __fastcall TRBWizardDlg::BackEndRemailer(const AnsiString &NymServer)
{
   AnsiString bakrem;
   for (int i = 0; i < capstrings->Count; i++)
   {
      AnsiString caps = capstrings->Strings[i];
      if (caps.Pos("$nymserver{") == 1 && caps.Pos(NymServer) > 0)
      {
         bakrem = GetTok(5, caps, '\"');
      }
   }
   return bakrem;
}*/
//---------------------------------------------------------------------------

AnsiString __fastcall TRBWizardDlg::BackEndRemailer(const AnsiString &NymServer)
{
   AnsiString bakrem;
//look(capstrings, NymServer);
   for (int i = 0; i < capstrings->Count; i++)
   {
      AnsiString caps = capstrings->Strings[i];

      if (caps.Pos("$nymserver{") == 1)
      {
         AnsiString server = GetTok(0, GetTok(1, caps, '@'), '>');

         if (server == NymServer)
         {
            int p = caps.Pos("$remailer{");
            if (p > 0)
            {
               bakrem = GetTok(1, caps.SubString(p, 64), '"');
            }
         }
      }
   }
   return bakrem;
}
//---------------------------------------------------------------------------

void __fastcall TRBWizardDlg::LoadSelectFromLength()
{
   if (Chain->Items->Count == 0)
   {
      AnsiString nymserver = GetTok(1, Nym, '@');

      //TStringList *directives = new TStringList();
      //AddEkAndLatent(nymserver, directives);

      Chain->Items->AddObject(nymserver + " {" + BackEndRemailer(nymserver) + "}", NULL);
      Chain->Items->AddObject(Destination->Text, NULL);
      Chain->ItemIndex = 1;
   }
   RemSelBox->ItemIndex = 0;
   LoadRemailers();
}
//---------------------------------------------------------------------------

void __fastcall TRBWizardDlg::LoadLengthFromSelect()
{
   Destination->Text = Chain->Items->Strings[Chain->Items->Count - 1];
   if (Destination->Text.Pos("@") > 0) EmailAddress->Checked = true;
   else Newsgroup->Checked = true;
}
//---------------------------------------------------------------------------

void __fastcall TRBWizardDlg::LoadSelectFromReview()
{
   ChainClear();

   bool ispostdirect = false;
   TTreeNode *node = RBView->Items->GetFirstNode();
 //TStringList *kids = NULL;
   AnsiString directive, text;

   while (node != NULL)
   {
      if (node->Level == 1)
      {
          /*if (node->Count > 0)
          {
             kids = new TStringList();
             for (int i = 0; i < node->Count; i++)
             {
                directive = node->Item[i]->Text;
                if (directive.Pos("Anon-") == 1)
                   continue;

                kids->Add(directive);
             }
          }*/
          Chain->Items->AddObject(node->Text, NULL);//kids);
          //kids = NULL;
      }

      if (node->Text.Pos("Anon-Post-To: ") > 0)
         ispostdirect = true;

      text = node->Text;  // use last node text after exit while
      node = node->GetNext();
   }

   bool ispost = text.Pos("@") == 0;

   Chain->ItemIndex = Chain->Items->Count - 1;

   if (RemSelBox->Text.IsEmpty())
      LoadRemailersAll();

   PostDirect->Checked = ispostdirect;
   EmailAddress->Checked = !ispost;
   Newsgroup->Checked = !EmailAddress->Checked;
   Destination->Text = text;
}
//---------------------------------------------------------------------------

void __fastcall TRBWizardDlg::ProcessKidsForReview(TTreeNode *node, int position)
{
   assert(position >= 0 && position < Chain->Items->Count - 1);

   TStrings *kids = (TStrings*)Chain->Items->Objects[position];
   assert(kids != NULL);

   AnsiString remailer, nextLeg;
   remailer = Chain->Items->Strings[position];
   nextLeg = Chain->Items->Strings[position + 1];

   if (nextLeg == TheDestination() && Newsgroup->Checked && PostDirect->Checked && HasCap(remailer, "post"))
      RBView->Items->AddChild(node, "Anon-Post-To: " + nextLeg);
   else
      RBView->Items->AddChild(node, "Anon-To: " + nextLeg);

   for (int k = 0; k < kids->Count; k++)
   {
      AnsiString kid = kids->Strings[k];

      // make sure to change anon-to for addition or deletion of remailer
      if (kid.Pos("Anon-To: ") == 1)
      {
         //RBView->Items->AddChild(node, "Anon-To: " + nextLeg);
         continue;
      }

      // make sure to change anon-to for addition of remailer
      if (kid.Pos("Anon-Post-To: ") == 1 && k < kids->Count - 2)
      {
         //RBView->Items->AddChild(node, "Anon-To: " + nextLeg);
         continue;
      }

      // final headers can only be last in chain
      if (FinalHeader(kid) && nextLeg != LastInChain())
      {
         TStrings *Kids = (TStrings*)Chain->Items->Objects[Chain->Items->Count - 2];
         if (!Kids)
         {
            Kids = new TStringList();
            Chain->Items->Objects[Chain->Items->Count - 2] = Kids;
         }

         Kids->Add(kid);
         continue;
      }

      RBView->Items->AddChild(node, kid);
   }
}
//---------------------------------------------------------------------------

void __fastcall TRBWizardDlg::LoadReviewFromSelect()
{
   TTreeNode *node, *cnode;
   RBView->Items->Clear();
   RBView->Items->BeginUpdate();
   AnsiString latenttime = Global->DB->GetIni("deflatenttime", "+1:00r");

   node = RBView->Items->AddChild(NULL, "Reply-Block: " + Probability);
   for (int i = 0; i < Chain->Items->Count - 1; i++)
   {
      TStrings *kids;
      AnsiString remailer = Chain->Items->Strings[i];
      cnode = RBView->Items->AddChild(node, remailer);

      int pos = remailer.Pos(" ");
      if (pos > 0)
         remailer = remailer.SubString(1, pos - 1);

      // check for existing children
      kids = (TStrings*)Chain->Items->Objects[i];
      if (kids)
      {
         ProcessKidsForReview(cnode, i);
         continue;
      }

      if (i < Chain->Items->Count - 2)
      {
         RBView->Items->AddChild(cnode, "Anon-To: " + Chain->Items->Strings[i + 1]);
      }

      if (i == Chain->Items->Count - 2)
      {
         if (Newsgroup->Checked)
         {
            if (PostDirect->Checked)
            {
               RBView->Items->AddChild(cnode, "Anon-Post-To: " + Destination->Text);
               RBView->Items->AddChild(cnode, "Subject: new mail " + IntToStr(GetTickCount()));
               if (Esub->Checked)
               {
                  RBView->Items->AddChild(cnode, "Encrypt-Subject: " + PGP->GetEncryptKey(ekLength));
               }
            }
            else
            {
               RBView->Items->AddChild(cnode, "Anon-To: " + GetMail2NewsGateways());
               RBView->Items->AddChild(cnode, "Newsgroups: " + Destination->Text);
               RBView->Items->AddChild(cnode, "Subject: new mail " + IntToStr(GetTickCount()));
               if (Esub->Checked)
               {
                  RBView->Items->AddChild(cnode, "Encrypt-Subject: " + PGP->GetEncryptKey(ekLength));
               }
            }
         }
         else
         {
            RBView->Items->AddChild(cnode, "Anon-To: " + Destination->Text);
         }
      }

      Sleep(10);

      //if (HasCap(remailer, "ekx") && !HasCap(remailer, "ek"))
      //   RBView->Items->AddChild(cnode, "Encrypt-3DES: " + GetEncryptKey());

      if (HasCap(remailer, "ek"))
         RBView->Items->AddChild(cnode, "Encrypt-Key: " + PGP->GetEncryptKey(ekLength));
      if (HasCap(remailer, "latent") && !latenttime.IsEmpty())
         RBView->Items->AddChild(cnode, "Latent-Time: " + latenttime);
   }

   RBView->Items->AddChild(node, Destination->Text);

   //RBView->FullExpand();
   node->MakeVisible();
   node->Expand(false);
   RBView->Items->EndUpdate();

   //ModRemailerBtn->Enabled = false;
}
//---------------------------------------------------------------------------

/*void __fastcall TRBWizardDlg::LoadReviewFromSelect()
{
   TTreeNode *node, *cnode;
   RBView->Items->Clear();
   RBView->Items->BeginUpdate();
   AnsiString latenttime = Global->DB->GetIni("deflatenttime", "+1:00r");

   node = RBView->Items->AddChild(NULL, "Reply-Block: " + Probability);
   for (int i = 0; i < Chain->Items->Count - 1; i++)
   {
      TStrings *kids;
      AnsiString remailer = Chain->Items->Strings[i];
      cnode = RBView->Items->AddChild(node, remailer);

      int pos = remailer.Pos(" ");
      if (pos > 0)
         remailer = remailer.SubString(1, pos - 1);

      // check for existing children
      kids = (TStrings*)Chain->Items->Objects[i];
      if (kids)
      {
         ProcessKidsForReview(cnode, i);
         continue;
      }

      if (i < Chain->Items->Count - 2)
      {
         RBView->Items->AddChild(cnode, "Anon-To: " + Chain->Items->Strings[i + 1]);
      }

      if (i == Chain->Items->Count - 2)
      {
         if (Newsgroup->Checked)
         {
            if (PostDirect->Checked)
            {
               RBView->Items->AddChild(cnode, "Anon-Post-To: " + Destination->Text);
               RBView->Items->AddChild(cnode, "Subject: mail for " + Nym);
               if (Esub->Checked)
               {
                  RBView->Items->AddChild(cnode, "Encrypt-Subject: " + PGP->GetEncryptKey(ekLength));
               }
            }
            else
            {
               RBView->Items->AddChild(cnode, "Anon-To: " + GetMail2NewsGateways());
               RBView->Items->AddChild(cnode, "Subject: mail for " + Nym);
               if (Esub->Checked)
               {
                  RBView->Items->AddChild(cnode, "Encrypt-Subject: " + PGP->GetEncryptKey(ekLength));
               }
            }
         }
         else
         {
            RBView->Items->AddChild(cnode, "Anon-To: " + Destination->Text);
         }
      }

      Sleep(10);

      //if (HasCap(remailer, "ekx") && !HasCap(remailer, "ek"))
      //   RBView->Items->AddChild(cnode, "Encrypt-3DES: " + GetEncryptKey());

      if (HasCap(remailer, "ek"))
         RBView->Items->AddChild(cnode, "Encrypt-Key: " + PGP->GetEncryptKey(ekLength));
      if (HasCap(remailer, "latent") && !latenttime.IsEmpty())
         RBView->Items->AddChild(cnode, "Latent-Time: " + latenttime);
   }

   RBView->Items->AddChild(node, Destination->Text);

   //RBView->FullExpand();
   node->MakeVisible();
   node->Expand(false);
   RBView->Items->EndUpdate();

   //ModRemailerBtn->Enabled = false;
}*/
//---------------------------------------------------------------------------

void __fastcall TRBWizardDlg::ChainClear()
{
   for (int i = 0; i < Chain->Items->Count; i++)
      delete Chain->Items->Objects[i];
   Chain->Clear();
}
//---------------------------------------------------------------------------

/*int  __fastcall TRBWizardDlg::ChainLength()
{
   assert(Length->Text.ToIntDef(0) >= 0 && Length->Text.ToIntDef(0) <= 20);
   return Length->Text.ToInt();
}*/
//---------------------------------------------------------------------------

void __fastcall TRBWizardDlg::ChainClick(TObject *)
{
   if (Chain->ItemIndex == 0)
      Chain->ItemIndex = 1;

   DelRemBtn->Enabled = Chain->ItemIndex > 0 && Chain->ItemIndex < Chain->Items->Count - 1;
   LoadRemailers();
}
//---------------------------------------------------------------------------

AnsiString __fastcall TRBWizardDlg::LastInChain()
{
   assert(Chain->Items->Count > 1);

   return GetRemailerAt(Chain->Items->Count - 2);
}
//---------------------------------------------------------------------------

AnsiString __fastcall TRBWizardDlg::TheDestination()
{
   assert(Chain->Items->Count > 0);

   return Chain->Items->Strings[Chain->Items->Count - 1];
}
//---------------------------------------------------------------------------

void __fastcall TRBWizardDlg::NewsgroupClick(TObject *)
{
   emailaddr = Destination->Text;
   Destination->Text = "alt.anonymous.messages";
   ActiveControl = Destination;
   PostDirect->Enabled = true;
   Esub->Enabled = true;
   HelperBtn->Enabled = PostDirect->Checked;
}
//---------------------------------------------------------------------------

void __fastcall TRBWizardDlg::EmailAddressClick(TObject *)
{
   //newsgroup = Destination->Text;
   Destination->Text = emailaddr;
   ActiveControl = Destination;
   PostDirect->Enabled = false;
   Esub->Enabled = false;
   HelperBtn->Enabled = true;
}
//---------------------------------------------------------------------------

void __fastcall TRBWizardDlg::PageControlChanging(TObject *Sender,
      bool &AllowChange)
{
   // Destination Page
   if (((TPageControl *)Sender)->ActivePage == LengthTab)
   {
      if (EmailAddress->Checked)
      {
         if (Destination->Text.Pos("@") == 0)
         {
            MsgBox(this,
               "Your email address is not valid.",
               "Error", MB_OK);
            AllowChange = false;
            return;
         }
      }
      else
      {
         if (Destination->Text.Pos(".") == 0)
         {
            MsgBox(this,
               "Your newsgroup name is not valid.",
               "Error", MB_OK);
            AllowChange = false;
            return;
         }
      }

      LoadSelectFromLength();
   }

   // Select Remailers Page
   if (((TPageControl *)Sender)->ActivePage == SelectTab)
   {
      AnsiString final = Chain->Items->Strings[Chain->Items->Count - 2];
      if (HasCap(final, "middle") && !(PostDirect->Checked && HasCap(final, "post")))
      {
         if (forward && MsgBox(this,
            "The exit remailer of your chain--" + final + "--is a middleman! It cannot deliver   \n"
            "your message to the final destination. You can leave this remailer where   \n"
            "it is, but " + final + " will select an exit remailer at random and forward the message    \n"
            "to there. This gives you less control of the exact path your message takes.    \n"
            "I advise against it.\n\nContinue with this remailer?",
             MT_INFO, MB_OKCANCEL) == IDCANCEL)
         {
            AllowChange = false;
            return;
         }
      }

      AnsiString remailer = LastInChain();

      if (forward & Newsgroup->Checked && PostDirect->Checked && Esub->Checked)
      {
         if (!HasCap(remailer, "post") && !HasCap(remailer, "esub"))
         {
            MsgBox(this,
               "The options you selected require your final remailer support both   \n"
               "'post' and 'esub' directives. Yours doesn't support either.    \n",
                MT_INFO, MB_OK);
            AllowChange = false;
            return;
         }

         if (!HasCap(remailer, "post"))
         {
            MsgBox(this,
               "The options you selected require your final remailer support the    \n"
               "'post' directive. Your's doesn't support that directive.    \n",
                MT_INFO, MB_OK);
            AllowChange = false;
            return;
         }

         if (!HasCap(remailer, "esub"))
         {
            MsgBox(this,
               "The options you selected require your final remailer support the    \n"
               "'esub' directive. Your's doesn't support that directive.    \n",
                MT_INFO, MB_OK);
            AllowChange = false;
            return;
         }

      }

      if (forward && Newsgroup->Checked && PostDirect->Checked)
      {
         if (!HasCap(remailer, "post"))
         {
            MsgBox(this,
               "The options you selected require your final remailer support the    \n"
               "'post' directive. Your's doesn't support that directive.    \n",
                MT_INFO, MB_OK);
            AllowChange = false;
            return;
         }
      }

      if (forward && Newsgroup->Checked && Esub->Checked)
      {
         if (!HasCap(remailer, "esub"))
         {
            MsgBox(this,
               "The options you selected require your final remailer support the    \n"
               "'esub' directive. Your's doesn't support that directive.    \n",
                MT_INFO, MB_OK);
            AllowChange = false;
            return;
         }
      }

      // check for broken links
      int i = 0; while (i < Chain->Items->Count - 2)
      {
         AnsiString rem1 = GetRemailerAt(i), rem2 = GetRemailerAt(i + 1);
         if (IsBroken(rem1, rem2))
         {
            MsgBox(this,
               "Sorry, your chain link from '" + rem1 + "' to '" + rem2 + "' is broken.",
                MT_INFO, MB_OK);
            AllowChange = false;
            return;
         }
         i++;
      }

      if (forward && (!Newsgroup->Checked || PostDirect->Checked))
         LoadReviewFromSelect();
   }

   // Mail2news Page
   if (((TPageControl *)Sender)->ActivePage == Mail2newsTab)
   {
      if ((Newsgroup->Checked && !PostDirect->Checked) && forward && UseGateways->Items->Count == 0)
      {
         MsgBox(this,
            "Sorry, you must select at least 1 mail2news gateway before continuing.    ",
            MT_INFO, MB_OK);
         AllowChange = false;
         return;
      }

      if (forward)
      {
         LoadReviewFromSelect();
      }
   }

   // Review Page
   if (((TPageControl *)Sender)->ActivePage == ReviewTab)
   {
   }
}
//---------------------------------------------------------------------------

AnsiString __fastcall TRBWizardDlg::GetMail2NewsGateways()
{
   TStrings *items = UseGateways->Items;
   assert(items->Count > 0);
   AnsiString recips;

   for (int i = 0; i < items->Count; i++)
   {
      if (i > 0)
         recips += ", ";

      recips += items->Strings[i];
   }

   return recips;
}
//---------------------------------------------------------------------------

/*AnsiString __fastcall TRBWizardDlg::GetMail2NewsGateways()
{
   AnsiString m2nfile = AppDir() + "mail2news.txt";
   TStringList *mail2news = new TStringList();

   if (FileExists(m2nfile))
      mail2news->LoadFromFile(m2nfile);

   int i = 0; while (i < mail2news->Count)
   {
      if (mail2news->Strings[i].Pos("_nospam") == 0) i++;
      else mail2news->Delete(i);
   }

   AnsiString recips;
   for (i = 0; i < mail2news->Count; i++)
   {
      if (!recips.IsEmpty())
         recips += ", ";

      recips += mail2news->Strings[i];
   }
   delete mail2news;

   return recips;
}*/
//---------------------------------------------------------------------------

void __fastcall TRBWizardDlg::ListBox1DrawItem(TWinControl *, int Index,
   TRect &Rect, TOwnerDrawState)
{
   TCanvas *canvas = ListBox1->Canvas;
   AnsiString spec, line = ListBox1->Items->Strings[Index];
   AnsiString orgname = ListBox1->Font->Name, name = GetTok(0, line, ' ');
   int orgsize = ListBox1->Font->Size;

   if (!HasCap(name, "latent") || !HasCap(name, "ek") || line.Pos("broken") > 0)
      canvas->Font->Color = clGrayText;

   bool broke = line.Pos("broken") > 0;
   if (broke)
      canvas->Font->Style = TFontStyles() << fsStrikeOut;

   canvas->FillRect(Rect);
   canvas->TextOut(Rect.Left + 3, Rect.Top, line.SubString(1, 9).Trim());

   if (broke)
      canvas->Font->Style = TFontStyles();

   canvas->Font->Name = "Courier New";
   canvas->Font->Size = 8;
   canvas->TextOut(Rect.Left + 63, Rect.Top, line.SubString(43, 12));
   canvas->Font->Name = orgname;
   canvas->Font->Size = orgsize;
   spec = line.SubString(56, 9);
   canvas->TextOut(Rect.Left + 195 - canvas->TextWidth(spec), Rect.Top, spec);
   spec = line.SubString(65, 7);
   canvas->TextOut(Rect.Left + 245 - canvas->TextWidth(spec), Rect.Top, spec);
}
//---------------------------------------------------------------------------

void __fastcall TRBWizardDlg::ListBox1Click(TObject *)
{
   CopyBtn->Enabled = ListBox1->ItemIndex >= 0;
   Hint = " " + GetCurrentCaps();

   AnsiString remailer = GetTok(0, ListBox1->Items->Strings[ListBox1->ItemIndex], ' ');
   if (RBCapWin != NULL)
   {
      OpenCapsToolWindow(this, remailer, Hint, Left + CapBtn->Left + PageControl->Left,
         Top + PageControl->Top + CapBtn->Top + 25);
   }
}
//---------------------------------------------------------------------------

AnsiString __fastcall TRBWizardDlg::GetRemailerFromDomain(const AnsiString &Domain)
{
   AnsiString remailer, line;
   for (int i = 0; i < rlist->Count; i++)
   {
      line = rlist->Strings[i];
      if (line.Pos("$remailer{") != 1)
          continue;

      if (GetTok(3, line, '"').Pos(Domain) > 0)
      {
         remailer = GetTok(1, line, '"');
         break;
      }
   }
   return remailer;
}
//---------------------------------------------------------------------------

bool __fastcall TRBWizardDlg::HasCap(const AnsiString &Remailer, const AnsiString &Cap)
{
   bool has = false;

   AnsiString caps = GetCaps(Remailer);

   if (caps.Pos(" " + Cap + " ") > 0)
      has = true;

   return has;
}
//---------------------------------------------------------------------------

AnsiString __fastcall TRBWizardDlg::FixBrokenLine(AnsiString &line, int p)
{
   AnsiString broken;
   if (p <  0) broken = "<broken     "; else
   if (p == 1) broken = " broken>    "; else
   broken = "<broken>    ";

   AnsiString fixed = line.SubString(1, 42) + broken + line.SubString(55, 32);
   line = fixed;
   return line;
}
//---------------------------------------------------------------------------

void __fastcall TRBWizardDlg::GetRemailersBeforeAndAfter(AnsiString &RemBefore,
   AnsiString &RemAfter)
{
   int idx = Chain->ItemIndex;

   RemBefore = "";
   RemAfter = "";

   RemBefore = GetRemailerAt(idx - 1);
   RemAfter = GetRemailerAt(idx);
}
//---------------------------------------------------------------------------

bool __fastcall TRBWizardDlg::RemailerPrecedesOrFollowsSelf(const AnsiString &Remailer,
   const AnsiString &RemBefore, const AnsiString &RemAfter)
{
   return Remailer == RemBefore || Remailer == RemAfter;
}
//---------------------------------------------------------------------------

AnsiString __fastcall TRBWizardDlg::GetRemailerAt(int Position)
{
   AnsiString remailer;

   if (Position == 0)
   {
      remailer = GetTok(0, GetTok(1, Chain->Items->Strings[0], '{'), '}');
   }
   else
   {
      remailer = Chain->Items->Strings[Position];
   }

   return remailer;
}
//---------------------------------------------------------------------------

/*void __fastcall TRBWizardDlg::AllRemailersClick(TObject *Sender)
{
   TRadioButton *button = dynamic_cast<TRadioButton*>(Sender);
   if (!button)
      return;

   if (button->Caption.Pos("All") > 0)
   {
      LoadRemailersAll();
      return;
   }

   if (button->Caption.Pos("Post") > 0)
   {
      LoadRemailersPostOnly();
      return;
   }

   if (button->Caption.Pos("Exit") > 0)
   {
      LoadRemailersExitOnly();
      return;
   }

   if (button->Caption.Pos("E&sub") > 0)
   {
      LoadRemailersEsubOnly();
      return;
   }
}*/
//---------------------------------------------------------------------------

void __fastcall TRBWizardDlg::LoadRemailers()
{
   if (RemSelBox->Text == "All remailers")
      LoadRemailersAll();
   else
   if (RemSelBox->Text == "Posting remailers")
      LoadRemailersPostOnly();
   else
   if (RemSelBox->Text == "Posting & Esub remailers")
      LoadRemailersPostEsubOnly();
   else
   if (RemSelBox->Text == "Exit & Esub remailers")
      LoadRemailersExitEsubOnly();
   else
   if (RemSelBox->Text == "Exit remailers")
      LoadRemailersExitOnly();
   else
   if (RemSelBox->Text == "Esub remailers")
      LoadRemailersEsubOnly();
}
//---------------------------------------------------------------------------

void __fastcall TRBWizardDlg::LoadRemailersAll()
{
   AnsiString rembefore, remafter;
   GetRemailersBeforeAndAfter(rembefore, remafter);

   ListBox1->Clear();
   Snooze(50);
   for (int i = 0; i < rlist->Count; i++)
   {
      AnsiString line = rlist->Strings[i];
      if (line.Pos("%") > 0)
      {
         AnsiString remailer = GetTok(0, line, ' ');

         if (RemailerPrecedesOrFollowsSelf(remailer, rembefore, remafter))
            continue;
         if (remailer == rembefore)
            continue;

         int p;
         if (RemailerIsBroken(remailer, rembefore, remafter, p))
            FixBrokenLine(line, p);

         ListBox1->Items->Add(line);
      }
   }

   Label11->Caption = IntToStr(ListBox1->Items->Count) + " remailers";

   if (RemSelBox->Text != "All remailers")
      RemSelBox->ItemIndex = RemSelBox->Items->IndexOf("All remailers");

   CopyBtn->Enabled = false;
}
//---------------------------------------------------------------------------

void __fastcall TRBWizardDlg::LoadRemailersExitOnly()
{
   AnsiString rembefore, remafter;
   GetRemailersBeforeAndAfter(rembefore, remafter);

   ListBox1->Clear();
   for (int i = 0; i < rlist->Count; i++)
   {
      AnsiString line = rlist->Strings[i];
      if (line.Pos("%") > 0)
      {
         AnsiString remailer = GetTok(0, line, ' ');

         if (!HasCap(remailer, "middle"))
         {
            if (RemailerPrecedesOrFollowsSelf(remailer, rembefore, remafter))
               continue;
            if(remailer == rembefore)
               continue;

            int p;
            if (RemailerIsBroken(remailer, rembefore, remafter, p))
               FixBrokenLine(line, p);

            ListBox1->Items->Add(line);
         }
      }
   }

   Label11->Caption = IntToStr(ListBox1->Items->Count) + " remailers";

   if (RemSelBox->Text != "Exit remailers")
      RemSelBox->ItemIndex = RemSelBox->Items->IndexOf("Exit remailers");

   CopyBtn->Enabled = false;
}
//---------------------------------------------------------------------------

void __fastcall TRBWizardDlg::LoadRemailersPostOnly()
{
   AnsiString rembefore, remafter;
   GetRemailersBeforeAndAfter(rembefore, remafter);

   ListBox1->Clear();
   for (int i = 0; i < rlist->Count; i++)
   {
      AnsiString line = rlist->Strings[i];
      if (line.Pos("%") > 0)
      {
         AnsiString remailer = GetTok(0, line, ' ');
         if (HasCap(remailer, "post"))
         {
            if (RemailerPrecedesOrFollowsSelf(remailer, rembefore, remafter))
               continue;
            if(remailer == rembefore)
               continue;

            int p;
            if (RemailerIsBroken(remailer, rembefore, remafter, p))
               FixBrokenLine(line, p);

            ListBox1->Items->Add(line);
         }
      }
   }

   Label11->Caption = IntToStr(ListBox1->Items->Count) + " remailers";

   if (RemSelBox->Text != "Posting remailers")
      RemSelBox->ItemIndex = RemSelBox->Items->IndexOf("Posting remailers");

   CopyBtn->Enabled = false;
}
//---------------------------------------------------------------------------

void __fastcall TRBWizardDlg::LoadRemailersPostEsubOnly()
{
   AnsiString rembefore, remafter;
   GetRemailersBeforeAndAfter(rembefore, remafter);

   ListBox1->Clear();
   for (int i = 0; i < rlist->Count; i++)
   {
      AnsiString line = rlist->Strings[i];
      if (line.Pos("%") > 0)
      {
         AnsiString remailer = GetTok(0, line, ' ');
         if (HasCap(remailer, "post") && HasCap(remailer, "esub"))
         {
            if (RemailerPrecedesOrFollowsSelf(remailer, rembefore, remafter))
               continue;
            if(remailer == rembefore)
               continue;

            int p;
            if (RemailerIsBroken(remailer, rembefore, remafter, p))
               FixBrokenLine(line, p);

            ListBox1->Items->Add(line);
         }
      }
   }

   Label11->Caption = IntToStr(ListBox1->Items->Count) + " remailers";

   if (RemSelBox->Text != "Posting & Esub remailers")
      RemSelBox->ItemIndex = RemSelBox->Items->IndexOf("Posting & Esub remailers");

   CopyBtn->Enabled = false;
}
//---------------------------------------------------------------------------

void __fastcall TRBWizardDlg::LoadRemailersExitEsubOnly()
{
   AnsiString rembefore, remafter;
   GetRemailersBeforeAndAfter(rembefore, remafter);

   ListBox1->Clear();
   for (int i = 0; i < rlist->Count; i++)
   {
      AnsiString line = rlist->Strings[i];
      if (line.Pos("%") > 0)
      {
         AnsiString remailer = GetTok(0, line, ' ');
         if (!HasCap(remailer, "middle") && HasCap(remailer, "esub"))
         {
            if (RemailerPrecedesOrFollowsSelf(remailer, rembefore, remafter))
               continue;
            if(remailer == rembefore)
               continue;

            int p;
            if (RemailerIsBroken(remailer, rembefore, remafter, p))
               FixBrokenLine(line, p);

            ListBox1->Items->Add(line);
         }
      }
   }

   Label11->Caption = IntToStr(ListBox1->Items->Count) + " remailers";

   if (RemSelBox->Text != "Exit & Esub remailers")
      RemSelBox->ItemIndex = RemSelBox->Items->IndexOf("Exit & Esub remailers");

   CopyBtn->Enabled = false;
}
//---------------------------------------------------------------------------

void __fastcall TRBWizardDlg::LoadRemailersEsubOnly()
{
   AnsiString rembefore, remafter;
   GetRemailersBeforeAndAfter(rembefore, remafter);

   ListBox1->Clear();
   for (int i = 0; i < rlist->Count; i++)
   {
      AnsiString line = rlist->Strings[i];
      if (line.Pos("%") > 0)
      {
         AnsiString remailer = GetTok(0, line, ' ');

         if (HasCap(remailer, "esub"))
         {
            if (RemailerPrecedesOrFollowsSelf(remailer, rembefore, remafter))
               continue;
            if(remailer == rembefore)
               continue;

            int p;
            if (RemailerIsBroken(remailer, rembefore, remafter, p))
               FixBrokenLine(line, p);

            ListBox1->Items->Add(line);
         }
      }
   }

   Label11->Caption = IntToStr(ListBox1->Items->Count) + " remailers";

   CopyBtn->Enabled = false;
}
//---------------------------------------------------------------------------

void __fastcall TRBWizardDlg::RBViewClick(TObject *)
{
  ModBtn->Enabled =
     RBView->Selected &&
     RBView->Selected->Level == 1 &&
     RBView->Selected->Index != RBView->Selected->Parent->Count - 1;
  AdvBtn->Enabled = ModBtn->Enabled;
}
//---------------------------------------------------------------------------

void __fastcall TRBWizardDlg::AddEkAndLatent(const AnsiString &remailer, TStrings *directives)
{
   AnsiString latenttime = Global->DB->GetIni("deflatenttime", "+1:00r");

   if (HasCap(remailer, "ek"))
      directives->Add("Encrypt-Key: " + PGP->GetEncryptKey(ekLength));

   if (HasCap(remailer, "latent") && !latenttime.IsEmpty())
      directives->Add("Latent-Time: " + latenttime);
}
//---------------------------------------------------------------------------

void __fastcall TRBWizardDlg::CopyBtnClick(TObject *)
{
   assert(ListBox1->ItemIndex >= 0);

   AnsiString remailer, item = ListBox1->Items->Strings[ListBox1->ItemIndex];
   if (item.Pos("broken") > 0)
   {
      MessageBeep(MB_ICONHAND);
      return;
   }
   remailer = GetTok(0, item, ' ');
   if (Chain->ItemIndex < 0)
      Chain->ItemIndex = Chain->Items->Count - 1;
   int idx = Chain->ItemIndex;
   if (idx == 0) idx++;

   if (Chain->Items->IndexOf(remailer) > 0 || GetRemailerAt(0) == remailer)
   {
      if (MsgBox(this,
         "This remailer is already in your chain. You can use it again, but   \n"
         "it's a bad idea. My advise is to choose another remailer.    \n\n"
         "Continue with this one?",
          MT_INFO, MB_OKCANCEL) == IDCANCEL)
      {
         return;
      }
   }

   //TStringList *directives = new TStringList();
   //AddEkAndLatent(remailer, directives);

   Chain->Items->InsertObject(idx, remailer, NULL);

   LoadRemailers();

   CopyBtn->Enabled = false;
   NextBtn->Enabled = true;
}
//---------------------------------------------------------------------------

void __fastcall TRBWizardDlg::DelRemBtnClick(TObject *)
{
   assert(Chain->ItemIndex > 0);

   int idx = Chain->ItemIndex, cnt = Chain->Items->Count;
   if (idx > 1 && idx < cnt - 1)
   {
      AnsiString rembefore, remailer, remafter;
      rembefore = GetRemailerAt(idx - 1);
      remafter = GetRemailerAt(idx + 1);
      remailer = GetRemailerAt(idx);

      if (brokenchains->IndexOf(rembefore + "," + remafter) >= 0)
      {
         if (MsgBox(this,
            "There is a problem with deleting " + remailer + ". Links between the remailer    \n"
            "before it--" + rembefore + "--and the remailer after it--" + remafter + "--are broken. You can      \n"
            "delete " + remailer + ", but you will have to correct the broken link before I    \n"
            "can let you advance from this page.\n\nDelete " + remailer + " now?",
             MT_INFO, MB_OKCANCEL) == IDCANCEL)
         {
            return;
         }
      }
   }

   delete Chain->Items->Objects[idx];
   Chain->Items->Delete(Chain->ItemIndex);

   if (idx == Chain->Items->Count)
      idx--;

   Chain->ItemIndex = idx;
   DelRemBtn->Enabled = idx >= 0;
}
//---------------------------------------------------------------------------

void __fastcall TRBWizardDlg::LegBtnClick(TObject *)
{
   OpenLegendToolWindow(this,
      Left + LegBtn->Left + PageControl->Left,
      Top + PageControl->Top + LegBtn->Top + 25
   );
}
//---------------------------------------------------------------------------

AnsiString __fastcall TRBWizardDlg::GetCurrentCaps()
{
   int idx = ListBox1->ItemIndex;
   if (idx < 0)
      return "";

   AnsiString caps, rem = GetTok(0, ListBox1->Items->Strings[idx], ' ').Trim();
   caps = GetCaps(rem).Trim();
   int p = caps.Pos(">");
   if (p > 0)
      caps = caps.SubString(p + 1, 256).Trim();
   caps = caps.SubString(1, caps.Length() - 3);
   return caps;
}
//---------------------------------------------------------------------------

void __fastcall TRBWizardDlg::CapBtnClick(TObject *)
{
   AnsiString remailer, caps;
   if (ListBox1->ItemIndex >= 0)
   {
      remailer = GetTok(0, ListBox1->Items->Strings[ListBox1->ItemIndex], ' ');
      caps = GetCurrentCaps();
   }
   else
   {
      remailer = "nothing selected";
      caps = " Select a remailer to view it's capstring.";
   }

   OpenCapsToolWindow(this, remailer, caps,
      Left + CapBtn->Left + PageControl->Left,
      Top + PageControl->Top + CapBtn->Top + 25
   );
}
//---------------------------------------------------------------------------

void __fastcall TRBWizardDlg::BrokeBtnClick(TObject *)
{
   OpenBrokenChainsWindow(this);
}
//---------------------------------------------------------------------------

void __fastcall TRBWizardDlg::ModBtnClick(TObject *)
{
   TTreeNode *Remailer = RBView->Selected;
   assert(Remailer != NULL);

   TStringList *spec = new TStringList();
   for (int i = 0; i < Remailer->Count; i++)
   {
      TTreeNode *node = Remailer->Item[i];
      spec->Add(node->Text);
   }

   if (ModifyRemailerSpec(this, spec, Newsgroup->Checked, Esub->Checked, GetCaps(Remailer->Text),
      (Remailer->Index == Remailer->Parent->Count - 2)))
   {
      Remailer->DeleteChildren();
      Snooze(300);

      TTreeView *TreeView = (TTreeView*)Remailer->TreeView;
      for (int i = 0; i < spec->Count; i++)
      {
         AnsiString line = spec->Strings[i].Trim();
         if (line.IsEmpty())
            continue;

         TreeView->Items->AddChild(Remailer, line);
      }

      Remailer->MakeVisible();
      Remailer->Item[Remailer->Count - 1]->MakeVisible();
   }

   delete spec;
}
//---------------------------------------------------------------------------

void __fastcall TRBWizardDlg::RemSelBoxChange(TObject *)
{
   if (RemSelBox->Text == "All remailers")
      LoadRemailersAll();
   else
   if (RemSelBox->Text == "Exit remailers")
      LoadRemailersExitOnly();
   else
   if (RemSelBox->Text == "Posting remailers")
      LoadRemailersPostOnly();
   else
   if (RemSelBox->Text == "Posting & Esub remailers")
      LoadRemailersPostEsubOnly();
   else
   if (RemSelBox->Text == "Esub remailers")
      LoadRemailersEsubOnly();
   else
   if (RemSelBox->Text == "Exit & Esub remailers")
      LoadRemailersExitEsubOnly();
}
//---------------------------------------------------------------------------

void __fastcall TRBWizardDlg::AdvBtnClick(TObject *)
{
   TTreeNode *Remailer = RBView->Selected;
   assert(Remailer != NULL);

   EditAdvancedRemailerDirectives(this, Remailer, GetCaps(Remailer->Text),
      (Newsgroup->Checked && PostDirect->Checked),
      (Remailer->Index == Remailer->Parent->Count - 2));
}
//---------------------------------------------------------------------------

void __fastcall TRBWizardDlg::SourceBtnClick(TObject *)
{
   EditConfigSource(this, Nym, RBView);
}
//---------------------------------------------------------------------------

void __fastcall TRBWizardDlg::FormShow(TObject *)
{
   if (modify)
   {
      AnsiString toline;
      TTreeNode *pnode, *cnode;
      assert(ReplyBlock->Count > 0);

      pnode = RBView->Items->AddChild(NULL, ReplyBlock->Strings[0]);
      for (int i = 1; i < ReplyBlock->Count; i++)
      {
         AnsiString line = ReplyBlock->Strings[i].Trim();
         if (line.IsEmpty())
            continue;

         if (line.Pos(":") == 0)
         {
            cnode = RBView->Items->AddChild(pnode, line);
            continue;
         }

         RBView->Items->AddChild(cnode, line);
      }

      LoadSelectFromReview();

      Destination->Text = cnode->Text;
      if (Destination->Text.Pos("@") == 0)
      {
         PostDirect->Checked = ReplyBlock->Text.Pos("Anon-Post-To: ") > 0;
         Esub->Checked = ReplyBlock->Text.Pos("Encrypt-Subject: ") > 0;
         Newsgroup->Checked = true;
      }

      pnode->Expand(false);

      PageControl->ActivePage = ReviewTab;
      NextBtn->Caption = "&Finish";
      BackBtn->Enabled = true;
      ReplyBlock->Clear();
      Label9->Caption = "Modify Reply Block";
      Label1->Caption = "Hi! I'm your handy dandy Reply Block Wizard. You can step "
         "backwards and modify remailer selections, destination and other options.";
   }
}
//---------------------------------------------------------------------------

void __fastcall TRBWizardDlg::ProbBtnClick(TObject *)
{
   TTreeNode *node = RBView->Items->GetFirstNode();
   AnsiString text = GetTok(1, node->Text, ' ').Trim();
   if (text == "Reply-Block:")
      text = "";

   if  (GetReplyBlockProbability(this, text))
   {
      if (!text.IsEmpty())
         node->Text = "Reply-Block: " + text;
      else
         node->Text = "Reply-Block:";
   }
}
//---------------------------------------------------------------------------

void __fastcall TRBWizardDlg::EnableSelect(bool enable)
{
   BackBtn->Enabled = enable;
   NextBtn->Enabled = enable;
   CancelBtn->Enabled = enable;
   RemSelBox->Enabled = enable;
   ListBox1->Enabled = enable;
   Chain->Enabled = enable;
   CapBtn->Enabled = enable;
   BrokeBtn->Enabled = enable;
   LegBtn->Enabled = enable;
}
//---------------------------------------------------------------------------

void __fastcall TRBWizardDlg::HelperBtnClick(TObject *)
{
   if (Destination->Text.IsEmpty())
   {
      MsgBox(this,
         "You need to specify a destination for your reply block before I can    "
         "create it for you.", MT_INFO, MB_OK);
      return;
   }

   ChainClear();
   RBView->Items->Clear();

   int rems = GetAutoReplyBlockSpec(this, EmailAddress->Checked);
   if (rems)
   {
      Snooze(300);
      AnsiString nymserver = GetTok(1, Nym, '@');

      //TStringList *directives = new TStringList();
      //AddEkAndLatent(nymserver, directives);

      Chain->Items->AddObject(nymserver + " {" + BackEndRemailer(nymserver) + "}", NULL);
      Chain->Items->AddObject(Destination->Text, NULL);
      Chain->ItemIndex = 1;

      CancelBtn->Caption = "Cancel";
      PageControl->ActivePage = SelectTab;
      EnableSelect(false);
      bool ok;

      if (EmailAddress->Checked)
      {
         ok = GenReplyBlockToEmail(rems);
      }
      else
      {
         ok = GenReplyBlockToGroup(rems);
      }

      EnableSelect(true);
      if (ok)
      {
         Snooze(300);
         LoadReviewFromSelect();
         PageControl->ActivePage = ReviewTab;
         NextBtn->Caption = "Finish";
      }
   }
}
//---------------------------------------------------------------------------

bool __fastcall TRBWizardDlg::AutoGenReplyBlock(int remailers)
{
   assert(remailers > 0 && remailers <= 20);

   AnsiString nymserver = GetTok(1, Nym, '@');

   //TStringList *directives = new TStringList();
   //AddEkAndLatent(nymserver, directives);

   Chain->Items->AddObject(nymserver + " {" + BackEndRemailer(nymserver) + "}", NULL);
   Chain->Items->AddObject(Destination->Text, NULL);
   Chain->ItemIndex = 1;

   bool ok = GenReplyBlockToEmail(remailers);
   if (ok)
   {
      LoadReviewFromSelect();
      FormCloseQuery(NULL, ok);
   }

   return ok;
}
//---------------------------------------------------------------------------

bool __fastcall TRBWizardDlg::GenReplyBlockToEmail(int Length)
{
   bool ok = true;
   AnsiString remailer;
   //assert(ListBox1->Items->Count > 0);

   ListBox1->ItemIndex = 0;
   for (int l = 0; l < Length; l++)
   {
      if (l == Length - 1) LoadRemailersExitOnly();
      else LoadRemailersAll();
      if (Visible)
         Snooze(300);

      remailer = GetNextRemailer();
      if (remailer.IsEmpty())
      {
         ok = false;
         break;
      }

      //TStringList *directives = new TStringList();
      //AddEkAndLatent(remailer, directives);

      Chain->Items->InsertObject(Chain->Items->Count - 1, remailer, NULL);
   }

   if (!ok)
   {
      MsgBox(this,
         "I failed to select your chain. There were not enough reliable   \n"
         "remailers! Sorry. Possibly you can hand select a few.",
          MT_ERROR, MB_OK
      );
   }

   return ok;
}
//---------------------------------------------------------------------------

AnsiString __fastcall TRBWizardDlg::GetNextRemailer()
{
   AnsiString backend, rem, remailer, item;
   backend = GetTok(0, GetTok(1, Chain->Items->Strings[0], '{'), '}');

   for (int i = 0; i < ListBox1->Items->Count; i++)
   {
      item = ListBox1->Items->Strings[i];
      rem = GetTok(0, item, ' ');
      ListBox1->ItemIndex = i;
      if (Visible)
         Snooze(300);

      if (rem == backend || Chain->Items->IndexOf(rem) >= 0)
         continue;

      if (!HasCap(rem, "ek") || !HasCap(rem, "latent"))
         continue;

      if (item.Pos("broken") > 0)
         continue;

      if (UpDayCount(rem) < 12)
         continue;

      remailer = rem;
      break;
   }

   return remailer;
}
//---------------------------------------------------------------------------

int __fastcall TRBWizardDlg::UpDayCount(const AnsiString &remailer)
{
   int updays = 0;

   for (int i = 0; i < ListBox1->Items->Count; i++)
   {
      AnsiString item = ListBox1->Items->Strings[i];

      if (item.Pos(remailer + " ") == 1)
      {
         for (int u = 43; u < 55; u++)
         {
            if (item[u] != ' ')
               updays++;
         }
      }
   }

   return updays;
}
//---------------------------------------------------------------------------

bool __fastcall TRBWizardDlg::GenReplyBlockToGroup(int Length)
{
   bool ok = true;
   AnsiString remailer;
   //ListBox1->ItemIndex = 0;
   //if (Length == 1)
   //   Snooze(300);

   for (int l = 0; l < Length; l++)
   {
      if (l == Length - 1)
      {
         if (PostDirect->Checked && Esub->Checked)
         {
            if (RemSelBox->Text != "Posting & Esub  remailers");
               LoadRemailersPostEsubOnly();
         }
         else
         if (Esub->Checked)
         {
            if (RemSelBox->Text != "Exit & Esub remailers")
               LoadRemailersExitEsubOnly();
         }
         else
         if (PostDirect->Checked)
         {
            if (RemSelBox->Text != "Posting remailers")
               LoadRemailersPostOnly();
         }
      }
      else LoadRemailersAll();
      Snooze(300);

      remailer = GetNextRemailer();
      if (remailer.IsEmpty())
      {
         ok = false;
         break;
      }

      //TStringList *directives = new TStringList();
      //AddEkAndLatent(remailer, directives);

      Chain->Items->InsertObject(Chain->Items->Count - 1, remailer, NULL);
   }

   if (!ok)
   {
      MsgBox(this,
         "I failed to select your chain. There were not enough    \n"
         "reliable remailers! Sorry. Possibly you can hand select    \n"
         "your chain",
          MT_ERROR,
          MB_OK
      );
   }

   return ok;
}
//---------------------------------------------------------------------------

void __fastcall TRBWizardDlg::PostDirectClick(TObject *Sender)
{
   HelperBtn->Enabled = PostDirect->Checked;
}
//---------------------------------------------------------------------------

void __fastcall TRBWizardDlg::M2nGatewaysDrawItem(TWinControl *Control,
      int Index, TRect &Rect, TOwnerDrawState State)
{
   TCanvas *canvas = M2nGateways->Canvas;
   AnsiString item = M2nGateways->Items->Strings[Index];
   canvas->FillRect(Rect);
   canvas->TextOut(Rect.Left + 3, Rect.Top + 1, item);
}
//---------------------------------------------------------------------------

void __fastcall TRBWizardDlg::UseGatewaysDrawItem(TWinControl *Control,
      int Index, TRect &Rect, TOwnerDrawState State)
{
   TCanvas *canvas = UseGateways->Canvas;
   AnsiString item = UseGateways->Items->Strings[Index];
   canvas->FillRect(Rect);
   canvas->TextOut(Rect.Left + 3, Rect.Top + 1, item);
}
//---------------------------------------------------------------------------

void __fastcall TRBWizardDlg::M2nGatewaysClick(TObject *Sender)
{
   CopyM2nBtn->Enabled = M2nGateways->ItemIndex >= 0;
}
//---------------------------------------------------------------------------

void __fastcall TRBWizardDlg::CopyM2nBtnClick(TObject *Sender)
{
   assert(M2nGateways->ItemIndex >= 0);
   UseGateways->Items->Add(M2nGateways->Items->Strings[M2nGateways->ItemIndex]);
   DelM2nBtn->Enabled = UseGateways->ItemIndex >= 0;
}
//---------------------------------------------------------------------------

void __fastcall TRBWizardDlg::UseGatewaysClick(TObject *Sender)
{
   DelM2nBtn->Enabled = UseGateways->ItemIndex >= 0;
}
//---------------------------------------------------------------------------

void __fastcall TRBWizardDlg::DelM2nBtnClick(TObject *Sender)
{
   assert(UseGateways->ItemIndex >= 0);
   UseGateways->Items->Delete(UseGateways->ItemIndex);
   DelM2nBtn->Enabled = UseGateways->ItemIndex >= 0;
}
//---------------------------------------------------------------------------

