//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "Global2.h"
#include "PGPnymsWins.h"
#include "PGPKeyFrames.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TPGPnymsWin *PGPnymsWin;
//const int FailedInit = -100000;
const AnsiString unknown = "unknown";
const AnsiString cypherpunk = "cypherpunk";
const AnsiString nymservers = "nymservers";
//---------------------------------------------------------------------------

void __fastcall OpenNymserverPGPKeyring(TForm *Owner, const AnsiString &pubring,
   const AnsiString &rlist)
{
   PGPnymsWin = new TPGPnymsWin(Owner, pubring, rlist);
   PGPnymsWin->ShowModal();
   delete PGPnymsWin;
}
//---------------------------------------------------------------------------

__fastcall TPGPnymsWin::TPGPnymsWin(TComponent* Owner)
   : TForm(Owner)
{
}
//---------------------------------------------------------------------------

__fastcall TPGPnymsWin::TPGPnymsWin(TComponent* Owner, const AnsiString &pgpPubRing,
   const AnsiString &nlistTxt)
   : TForm(Owner)
{
   remNlistTxt = nlistTxt;
   pgpPublicRing = pgpPubRing;
   Label1->Caption = pgpPublicRing;

   TIniFile *ini = new TIniFile(AppIni());
   LoadWindowPosition(this, ini, PgpKeysIni, "nymservers", ListView);
   LoadIniStuff(ini);
   delete ini;

   PGP = PGPInit(this);
   if (!PGP || !PGP->InitSuccess)
   {
      ShowMessage(PGP->GetLastErrorStr() + " Quiting!");
      Application->ShowMainForm = false;
      PostMessage(Handle, WM_QUIT, 0, 0);
      return;
   }

   ListView->Align = alClient;
   Rlist = new TStringList();
   Keys = new TStringList();

   if (FileExists(remNlistTxt))
      LoadRlist();

   if (FileExists(pgpPublicRing))
      LoadKeys();

   Keys->Sorted = true;
}
//---------------------------------------------------------------------------
#pragma argsused
void __fastcall TPGPnymsWin::FormCloseQuery(TObject *Sender, bool &CanClose)
{
   TIniFile *ini = new TIniFile(AppIni());
   SaveWindowPosition(this, ini, PgpKeysIni, inisection, ListView);
   delete ini;
}
//---------------------------------------------------------------------------

__fastcall TPGPremsWin::~TPGPnymsWin()
{
   delete Keys;
   delete Rlist;
   delete PGP;
}
//---------------------------------------------------------------------------

void __fastcall TPGPnymsWin::LoadIniStuff(TIniFile *ini)
{
   ExpertMode = ini->ReadBool("main", "expertmode", true);
   CoolBar->Visible = ini->ReadBool(PgpKeysExe, "toolbar", true);
   StatusBar->Visible = ini->ReadBool(PgpKeysExe, "statusbar", true);
   OkBtn->Visible = ini->ReadBool(Main, "exitleft", false);
   TB3->Visible = OkBtn->Visible;

   ListView->Color = (TColor)ini->ReadInteger("main", QSWORKSPACECOLOR, QSWORKSPACECOLORDEF);
   StrToFont(ini->ReadString("main", QSWINDOWFONT, QSWINDOWFONTDEF), ListView->Font);
   StrToFont(ini->ReadString("main", QSNAVBARFONT, QSNAVBARFONTDEF), Label1->Font);
   StrToFont(ini->ReadString("main", QSDIALOGFONT, QSDIALOGFONTDEF), Font);
   Label1->Top = (Panel1->Height - Label1->Height) / 2 - 1;
}
//---------------------------------------------------------------------------
#pragma argsused
void __fastcall TPGPnymsWin::Exit1Click(TObject *Sender)
{
   Close();
}
//---------------------------------------------------------------------------

void __fastcall TPGPnymsWin::LoadKeys()
{
   if (!PGP->GetKeysLV(pgpPublicRing, "", Keys))
      return;

   Keys->Sorted = true;
   for (int i = 0; i < Keys->Count; i++)
   {
      AnsiString userid, line = Keys->Strings[i];
      userid = GetTok(lvUserId, line, '\255');

      TListItem *Item = ListView->Items->Add();
      Item->Caption = LookupRlist(userid);

      Item->SubItems->Add(userid);
      Item->SubItems->Add(GetTok(lvKeyID,  line, '\255'));
      Item->SubItems->Add(GetTok(lvStatus, line, '\255'));
      Item->SubItems->Add(GetTok(lvBits,   line, '\255'));
      Item->SubItems->Add(GetTok(lvAlgo,   line, '\255'));
   }
}
//---------------------------------------------------------------------------
/* ownerdata version
void __fastcall TPGPremsWin::LoadKeys()
{
   PGP->GetPublicKeys(pgpPublicRing, Keys);

   int i = 0; while (i < Keys->Count)
   {
      AnsiString name, userid, line = Keys->Strings[i];
      userid = GetTok(lvUserId, line, '\255');
      name = LookupRlist(userid);
      Keys->Strings[i] = LookupRlist(userid) + "\255" + line;
      i++;
   }

   ListView->Items->Count = Keys->Count;
   Keys->Sorted = true;
}*/
//---------------------------------------------------------------------------

void __fastcall TPGPnymsWin::LoadRlist()
{
   if (FileExists(remRlistTxt))
   {
      Rlist->LoadFromFile(remRlistTxt);
      int i = 0; while (i < Rlist->Count)
      {
         if (Rlist->Strings[i].Pos("$remailer{") == 1 || Rlist->Strings[i].Pos("$nymserver{") == 1)
            i++;
         else
            Rlist->Delete(i);
      }
   }
}
//---------------------------------------------------------------------------

AnsiString __fastcall TPGPnymsWin::LookupNlist(AnsiString key)
{
   AnsiString name, line;

   for (int i = 0; i < Nlist->Count; i++)
   {
      line = Nlist->Strings[i];
      if (Nlist->Strings[i].Pos(GetEAddress(key)) > 0)
      {
         name = GetTok(1, line, '"');
         break;
      }
   }

   return name;
}
//---------------------------------------------------------------------------
// ownerdata
void __fastcall TPGPnymsWin::ListViewData(TObject *Sender, TListItem *Item)
{
   AnsiString userid, item = Keys->Strings[Item->Index];
   userid = GetTok(lvUserId, item, '\255');

   Item->Caption = GetTok(0, item, '\255'); // fields + 1 because of rem name added
   Item->SubItems->Add(GetTok(lvUserId + 1, item, '\255')); // to front
   Item->SubItems->Add(GetTok(lvKeyID + 1,  item, '\255'));
   Item->SubItems->Add(GetTok(lvStatus + 1, item, '\255'));
   Item->SubItems->Add(GetTok(lvBits + 1,   item, '\255'));
   Item->SubItems->Add(GetTok(lvAlgo + 1,   item, '\255'));
}
//---------------------------------------------------------------------------

void __fastcall TPGPnymsWin::ListViewAdvancedCustomDrawItem(TCustomListView *Sender,
   TListItem *Item, TCustomDrawState State, TCustomDrawStage Stage,
   bool &DefaultDraw)
{
   assert(Item->Index >= 0 && Item->Index < Keys->Count);

   AnsiString status = GetTok(lvStatus + 1, Keys->Strings[Item->Index], '\255');

   if (status == "revoked")
   {
      ListView->Canvas->Font->Style = TFontStyles() << fsItalic;
      ListView->Canvas->Font->Color  = clGrayText;
   }
   else
   if (status == "expired")
   {
      ListView->Canvas->Font->Style = TFontStyles() << fsItalic;
      ListView->Canvas->Font->Color  = clGrayText;
   }
   else
   if (status == "disabled")
   {
      ListView->Canvas->Font->Style = TFontStyles() << fsItalic;
      ListView->Canvas->Font->Color  = clGrayText;
   }
}
//---------------------------------------------------------------------------

void __fastcall TPGPnymsWin::Source1Click(TObject *Sender)
{
   Exec95("notepad.exe \"" + Label1->Caption + "\"");
}
//---------------------------------------------------------------------------

void __fastcall TPGPnymsWin::StatusBarResize(TObject *Sender)
{
   StatusBar->Panels->Items[0]->Width = StatusBar->Width - 160;
}
//---------------------------------------------------------------------------

void __fastcall TPGPnymsWin::Historytxt1Click(TObject *Sender)
{
   if (FileExists(AppDir() + "qspgpkeys.history.txt"))
   {
      Exec95("notepad.exe \"" + AppDir() + "qspgpkeys.history.txt\"");
   }
}
//---------------------------------------------------------------------------


