//---------------------------------------------------------------------------
#include <vcl\vcl.h>
#pragma hdrstop
#include <math.h>
#include <assert.h>
#include "PgpSdk6Calls.h"

HANDLE hSdkLib6;
PGPContextRef context;
PGPError err6;
//---------------------------------------------------------------------------
//  PGP utility
//---------------------------------------------------------------------------

bool __fastcall IsPgpError(PGPError err, const AnsiString &errmsg)
{
   if (IsPGPError(err))
   {
      char *buf = new char[256];

      PGPGetErrorString(err, 255, buf);
         Application->MessageBox(
         AnsiString(errmsg+"\nError: ("+AnsiString(err)+") "+AnsiString(buf)).c_str(),
        "PGP Error",
         MB_OK
      );

      delete buf;
      return true;
   }
   return false;
}

//---------------------------------------------------------------------------
// PGP Decode Callback
//---------------------------------------------------------------------------

PGPError DecodeEventHandler(PGPContextRef context, PGPEvent *event,
   PGPUserValue userValue)
{
   PGPSize useridsize;
   char useridstr[kPGPMaxUserIDSize];
   Tdecinfo6 *decinfo = (Tdecinfo6*)userValue;
   PGPKeyRef sigKey = kInvalidPGPKeyRef;
   PGPEventSignatureData *sig=&event->data.signatureData;
   AnsiString status, signer, sigtime;

   switch(event->type)
   {
      case kPGPEvent_DecryptionEvent:
         decinfo->decrypts++;
         break;
      case kPGPEvent_SignatureEvent:
      {
         bool known = false;

         char keyidstr[kPGPMaxKeyIDStringSize];
         err6 = PGPGetKeyIDString(&sig->signingKeyID, kPGPKeyIDString_Abbreviated, keyidstr);
         if (IsPGPError(err6))
            goto addsig;

         err6 = PGPGetKeyByKeyID(decinfo->keySet, &sig->signingKeyID, kPGPPublicKeyAlgorithm_Invalid, &sigKey);
         if (IsPGPError(err6))
            goto addsig;

         err6 = PGPGetPrimaryUserIDNameBuffer(sigKey, kPGPMaxKeyIDStringSize- 1, useridstr, &useridsize);
         if (IsPGPError(err6))
            goto addsig;

         known = true;

      addsig:

         if (known) signer = AnsiString(useridstr) + " (" + AnsiString(keyidstr) + ")";
         else signer = "Unknown, Key ID = " + AnsiString(keyidstr);

         sigtime = TimeT2SigTime(PGPGetStdTimeFromPGPTime(sig->creationTime));

         if (sig->verified) status = "good"; else
         if (signer.Pos("Unknown")==1) status = "unknown"; else
            status = "bad";

         if (decinfo->siginfos->Count) decinfo->siginfos->Add("");
         decinfo->siginfos->Add("*** PGP Signature Status: " + status);
         decinfo->siginfos->Add("*** Signer: " + signer);
         decinfo->siginfos->Add("*** Signed: " + sigtime);
         decinfo->siginfos->Add("*** Verified: " + FormatDateTime("m/dd/yy h:nn:ss ampm", Now()));
         decinfo->sigs++;

         break;
      }
   }

   return kPGPError_NoErr;
}
//---------------------------------------------------------------------------

AnsiString __fastcall TimeT2SigTime(time_t thetime)
{
   //Thu Oct 29 2:40:37 2001 (ctime)
   //10/29/01 2:40:37 PM     (sigtime)
   AnsiString mt,dy,yr,hr,mn,ampm="AM",timestr=ctime(&thetime);
   AnsiString months = "~~~JanFebMarAprMayJunJulAugSepOctNovDec";
   AnsiString mthstr = timestr.SubString(5, 3);
   mt = IntToStr(months.Pos(mthstr) / 3);
   dy = timestr.SubString(9, 2);
   yr = timestr.SubString(23, 2);
   mn = timestr.SubString(15, 5);
   hr = timestr.SubString(12, 2);
   while (hr.Length() && hr[1] == '0') hr = hr.SubString(2, 2);
   int h = hr.ToIntDef(-1);
   if (h > 12) { h -= 12; ampm = "PM"; }

   return mt + "/" + dy + "/" + yr + " " + AnsiString(h) + ":" + mn + " " + ampm;
}

//---------------------------------------------------------------------------
//  PGP SDK 1.7.8 Functions
//---------------------------------------------------------------------------

bool __fastcall IsPgpError(PGPError err6, char *errmsg)
{
   if (IsPGPError(err6))
   {
      ShowPGPError(err6, errmsg);
      return true;
   }
   return false;
}
//---------------------------------------------------------------------------

PGPError PGPGetErrorString(PGPError theError, PGPSize bufferSize,
   char *theString)
{
   PGPGETERRORSTRING geterrstr =
      (PGPGETERRORSTRING)GetProcAddress(hSdkLib6, "PGPGetErrorString");
   assert(geterrstr != NULL);

   return geterrstr(theError, bufferSize, theString);
}
//---------------------------------------------------------------------------

PGPError PGPsdkInit()
{
   PGPSDKINIT sdkinit = (PGPSDKINIT)GetProcAddress(hSdkLib6,"PGPsdkInit");
   assert(sdkinit != NULL);

   return sdkinit();
}
//---------------------------------------------------------------------------

PGPError PGPsdkCleanup()
{
   PGPSDKCLEANUP sdkcleanup =
      (PGPSDKCLEANUP)GetProcAddress(hSdkLib6, "PGPsdkCleanup");
   assert(sdkcleanup != NULL);

   return sdkcleanup();
}
//---------------------------------------------------------------------------

PGPError PGPNewContext(PGPUInt32 clientAPIVersion,
   PGPContextRef *newContext)
{
   PGPNEWCONTEXT newcontext =
      (PGPNEWCONTEXT)GetProcAddress(hSdkLib6,"PGPNewContext");
   assert(newcontext != NULL);

   return newcontext(clientAPIVersion, newContext);
}
//---------------------------------------------------------------------------

PGPError PGPFreeContext(PGPContextRef context)
{
   PGPFREECONTEXT freecontext =
      (PGPFREECONTEXT)GetProcAddress(hSdkLib6, "PGPFreeContext");
   assert(freecontext != NULL);

   return freecontext(context);
}
//---------------------------------------------------------------------------

PGPError PGPsdkLoadDefaultPrefs(PGPContextRef context)
{
   PGPSDKLOADDEFAULTPREFS loadprefs =
      (PGPSDKLOADDEFAULTPREFS)GetProcAddress(hSdkLib6,"PGPsdkLoadDefaultPrefs");
   assert(loadprefs != NULL);

   return loadprefs(context);
}
//---------------------------------------------------------------------------

PGPError PGPsdkPrefGetFileSpec(PGPContextRef context, PGPsdkPrefSelector selector,
   PGPFileSpecRef * outRef)
{
   PGPSDKPREFGETFILESPEC filespec =
      (PGPSDKPREFGETFILESPEC)GetProcAddress(hSdkLib6, "PGPsdkPrefGetFileSpec");
   assert(filespec != NULL);

   return filespec(context, selector, outRef);
}
//---------------------------------------------------------------------------

PGPError PGPGetFullPathFromFileSpec(PGPFileSpecRef fileRef,
   char **fullPathPtr)
{
   PGPGETFULLPATHFROMFILESPEC fullpath =
     (PGPGETFULLPATHFROMFILESPEC)GetProcAddress(hSdkLib6,"PGPGetFullPathFromFileSpec");
   assert(fullpath != NULL);

   return fullpath(fileRef, fullPathPtr);
}
//---------------------------------------------------------------------------

PGPError PGPGetSDKVersion(PGPUInt32 *version)
{
   PGPGETSDKVERSION sdkversion =
      (PGPGETSDKVERSION)GetProcAddress(hSdkLib6, "PGPGetSDKVersion");
   assert(sdkversion != NULL);

   return  sdkversion(version);
}
//---------------------------------------------------------------------------

PGPError PGPFreeFileSpec(PGPFileSpecRef fileRef)
{
   PGPFREEFILESPEC freespec =
      (PGPFREEFILESPEC)GetProcAddress(hSdkLib6,"PGPFreeFileSpec");
   return freespec(fileRef);
}
//---------------------------------------------------------------------------

PGPError PGPExportKeySet(PGPKeySetRef keys, PGPOptionListRef firstOpt,
   PGPOptionListRef secondOpt, PGPOptionListRef thirdOpt, PGPOptionListRef forthOpt,
   PGPOptionListRef fifthOpt)
{
   PGPEXPORTKEYSET exportset =
      (PGPEXPORTKEYSET)GetProcAddress(hSdkLib6,"PGPExportKeySet");
   assert(exportset != NULL);

   return exportset(keys, firstOpt, secondOpt, thirdOpt, forthOpt, fifthOpt);
}
//---------------------------------------------------------------------------

PGPOptionListRef PGPOExportFormat(PGPContextRef context,
   PGPExportFormat exportFormat)
{
   PGPOEXPORTFORMAT exportfmt =
     (PGPOEXPORTFORMAT)GetProcAddress(hSdkLib6,"PGPOExportFormat");

   return exportfmt(context, exportFormat);
}
//---------------------------------------------------------------------------

PGPOptionListRef PGPOExportPrivateKeys(PGPContextRef context, PGPBoolean exportKeys)
{
   PGPOEXPORTPRIVATEKEYS exportprivs =
      (PGPOEXPORTPRIVATEKEYS)GetProcAddress(hSdkLib6,"PGPOExportPrivateKeys");
   assert (exportprivs != NULL);

   return exportprivs(context, exportKeys);
}
//---------------------------------------------------------------------------

PGPOptionListRef PGPOVersionString(PGPContextRef context, char const *version)
{
   PGPOVERSIONSTRING verstr =
      (PGPOVERSIONSTRING)GetProcAddress(hSdkLib6,"PGPOVersionString");
   assert(verstr != NULL);

   return verstr(context, version);
}
//---------------------------------------------------------------------------

PGPOptionListRef PGPOLastOption(PGPContextRef context)
{
   PGPOLASTOPTION lastopt =
      (PGPOLASTOPTION)GetProcAddress(hSdkLib6,"PGPOLastOption");
   assert(lastopt != NULL);

   return lastopt(context);
}
//---------------------------------------------------------------------------

PGPOptionListRef PGPOAllocatedOutputBuffer(PGPContextRef context, void **buffer,
   PGPSize maximumBufferSize, PGPSize *actualBufferSize)
{
   PGPOALLOCATEDOUTPUTBUFFER allbuf =
      (PGPOALLOCATEDOUTPUTBUFFER)GetProcAddress(hSdkLib6,"PGPOAllocatedOutputBuffer");
   assert(allbuf != NULL);

   return allbuf(context, buffer, maximumBufferSize, actualBufferSize);
}
//---------------------------------------------------------------------------

PGPOptionListRef PGPORecursivelyDecode(PGPContextRef context,
   PGPBoolean recurse)
{
   PGPORECURSIVELYDECODE recur =
      (PGPORECURSIVELYDECODE)GetProcAddress(hSdkLib6,"PGPORecursivelyDecode");
   assert(recur != NULL);

   return recur(context,recurse);
}
//---------------------------------------------------------------------------

PGPOptionListRef PGPOPassphrase(PGPContextRef context, const char *passphrase)
{
   PGPOPASSPHRASE phrase = (PGPOPASSPHRASE)GetProcAddress(hSdkLib6,"PGPOPassphrase");
   assert(phrase != NULL);

   return phrase(context,passphrase);
}
//---------------------------------------------------------------------------

PGPError PGPFreeKeySet(PGPKeySetRef keys)
{
   PGPFREEKEYSET freeset = (PGPFREEKEYSET)GetProcAddress(hSdkLib6,"PGPFreeKeySet");
   assert(freeset != NULL);

   return freeset(keys);
}
//---------------------------------------------------------------------------

PGPError PGPFreeData(void *allocation)
{
   PGPFREEDATA freedat = (PGPFREEDATA)GetProcAddress(hSdkLib6,"PGPFreeData");
   assert(freedat != NULL);

   return freedat(allocation);
}
//---------------------------------------------------------------------------

PGPError PGPOpenDefaultKeyRings(PGPContextRef context, PGPKeyRingOpenFlags openFlags,
   PGPKeySetRef *keySet)
{
   PGPOPENDEFAULTKEYRINGS opendefs =
      (PGPOPENDEFAULTKEYRINGS)GetProcAddress(hSdkLib6,"PGPOpenDefaultKeyRings");
   assert(opendefs != NULL);

   return opendefs(context, openFlags, keySet);
}
//---------------------------------------------------------------------------

PGPError PGPNewKeyDisabledFilter(PGPContextRef context, PGPBoolean disabled,
   PGPFilterRef *outFilter)
{
   PGPNEWKEYDISABLEDFILTER filter =
      (PGPNEWKEYDISABLEDFILTER)GetProcAddress(hSdkLib6,"PGPNewKeyDisabledFilter");
   assert(filter != NULL);

   return filter(context, disabled, outFilter);
}
//---------------------------------------------------------------------------

PGPError PGPNewKeyRevokedFilter(PGPContextRef context, PGPBoolean revoked,
      PGPFilterRef *outFilter)
{
   PGPNEWKEYREVOKEDFILTER filter =
      (PGPNEWKEYREVOKEDFILTER)GetProcAddress(hSdkLib6,"PGPNewKeyRevokedFilter");
   assert(filter != NULL);

   return filter(context, revoked, outFilter);
}
//---------------------------------------------------------------------------

PGPError PGPFilterKeySet(PGPKeySetRef origSet, PGPFilterRef filter,
      PGPKeySetRef *resultSet)
{
   PGPFILTERKEYSET filterset =
      (PGPFILTERKEYSET)GetProcAddress(hSdkLib6,"PGPFilterKeySet");
   assert(filterset != NULL);

   return filterset(origSet, filter, resultSet);
}
//---------------------------------------------------------------------------

PGPError PGPOrderKeySet(PGPKeySetRef src, PGPKeyOrdering order, PGPKeyListRef *outRef )
{
   PGPORDERKEYSET orderset =
      (PGPORDERKEYSET)GetProcAddress(hSdkLib6,"PGPOrderKeySet");
   assert(orderset != NULL);

   return orderset(src, order, outRef);
}
//---------------------------------------------------------------------------

PGPError PGPNewKeyIter(PGPKeyListRef keys, PGPKeyIterRef *outRef)
{
   PGPNEWKEYITER iter =
      (PGPNEWKEYITER)GetProcAddress(hSdkLib6,"PGPNewKeyIter");
   assert(iter != NULL);

   return iter(keys, outRef);
}
//---------------------------------------------------------------------------

PGPError PGPGetKeyIDFromKey(PGPKeyRef key, PGPKeyID *id)
{
   PGPGETKEYIDFROMKEY getid =
      (PGPGETKEYIDFROMKEY)GetProcAddress(hSdkLib6,"PGPGetKeyIDFromKey");
   return getid(key, id);
}
//---------------------------------------------------------------------------

PGPError PGPGetHashAlgUsed(PGPKeyRef key, PGPHashAlgorithm *hashAlg )
{
   PGPGETHASHALGUSED gethash =
      (PGPGETHASHALGUSED)GetProcAddress(hSdkLib6,"PGPGetHashAlgUsed");
   assert(gethash != NULL);

   return gethash(key, hashAlg);
}
//---------------------------------------------------------------------------

PGPError PGPGetKeyIDFromString(const char *string, PGPKeyID *id)
{
   PGPGETKEYIDFROMSTRING getid =
      (PGPGETKEYIDFROMSTRING)GetProcAddress(hSdkLib6,"PGPGetKeyIDFromString");
   assert(getid != NULL);

   return getid(string, id);
}
//---------------------------------------------------------------------------

PGPError PGPGetKeyIDString(PGPKeyID const *ref, PGPKeyIDStringType type,
      char outString[kPGPMaxKeyIDStringSize])
{
   PGPGETKEYIDSTRING getstr =
      (PGPGETKEYIDSTRING)GetProcAddress(hSdkLib6,"PGPGetKeyIDString");
   assert(getstr != NULL);

   return getstr(ref, type, outString);
}
//---------------------------------------------------------------------------

PGPError PGPKeyIterNext(PGPKeyIterRef iter, PGPKeyRef *outRef)
{
   PGPKEYITERNEXT iternxt = (PGPKEYITERNEXT)GetProcAddress(hSdkLib6,"PGPKeyIterNext");
   assert(iternxt != NULL);

   return iternxt(iter, outRef);
}
//---------------------------------------------------------------------------

PGPError PGPFreeKeyIter(PGPKeyIterRef iter)
{
   PGPFREEKEYITER freeiter =
      (PGPFREEKEYITER)GetProcAddress(hSdkLib6,"PGPFreeKeyIter");
   assert(freeiter != NULL);

   return freeiter(iter);
}
//---------------------------------------------------------------------------

PGPError PGPFreeKeyList(PGPKeyListRef keys)
{
   PGPFREEKEYLIST freelist = (PGPFREEKEYLIST)GetProcAddress(hSdkLib6,"PGPFreeKeyList");
   assert(freelist != NULL);

   return freelist(keys);
}
//---------------------------------------------------------------------------

PGPError PGPFreeFilter(PGPFilterRef filter)
{
   PGPFREEFILTER freefilter = (PGPFREEFILTER)GetProcAddress(hSdkLib6,"PGPFreeFilter");
   assert(freefilter != NULL);

   return freefilter(filter);
}
//---------------------------------------------------------------------------

PGPError PGPNewFileSpecFromFullPath(PGPContextRef context, const char *path,
   PGPFileSpecRef *ref)
{
   PGPNEWFILESPECFROMFULLPATH getspec =
      (PGPNEWFILESPECFROMFULLPATH)GetProcAddress(hSdkLib6,"PGPNewFileSpecFromFullPath");
   assert(getspec != NULL);

   return getspec(context, path, ref);
}
//---------------------------------------------------------------------------

PGPError PGPOpenKeyRing(PGPContextRef context, PGPKeyRingOpenFlags openFlags,
      PGPFileSpecRef fileRef, PGPKeySetRef *keySet)
{
   PGPOPENKEYRING openring = (PGPOPENKEYRING)GetProcAddress(hSdkLib6,"PGPOpenKeyRing");
   assert(openring != NULL);

   return openring(context,openFlags,fileRef,keySet);
}
//---------------------------------------------------------------------------

PGPOptionListRef PGPOInputBuffer( PGPContextRef context, void const *buffer,
   PGPSize bufferSize)
{
   PGPOINPUTBUFFER inbuf = (PGPOINPUTBUFFER)GetProcAddress(hSdkLib6,"PGPOInputBuffer");
   assert(inbuf != NULL);

   return inbuf(context, buffer, bufferSize);
}
//---------------------------------------------------------------------------

PGPOptionListRef PGPOKeySetRef(PGPContextRef context, PGPKeySetRef keysetRef)
{
   PGPOKEYSETREF setref = (PGPOKEYSETREF)GetProcAddress(hSdkLib6,"PGPOKeySetRef");
   assert(setref != NULL);

   return setref(context, keysetRef);
}
//---------------------------------------------------------------------------

PGPError PGPOpenKeyRingPair(PGPContextRef context, PGPKeyRingOpenFlags openFlags,
   PGPFileSpecRef pubRef, PGPFileSpecRef secRef, PGPKeySetRef *keySet)
{
   PGPOPENKEYRINGPAIR openpair =
      (PGPOPENKEYRINGPAIR)GetProcAddress(hSdkLib6,"PGPOpenKeyRingPair");
   assert(openpair != NULL);

   return openpair(context, openFlags, pubRef, secRef, keySet);
}
//---------------------------------------------------------------------------

PGPError PGPGetKeyByKeyID(PGPKeySetRef keySet, PGPKeyID const *keyID,
      PGPPublicKeyAlgorithm pubKeyAlgorithm, PGPKeyRef *key )
{
   PGPGETKEYBYKEYID getkey =
      (PGPGETKEYBYKEYID)GetProcAddress(hSdkLib6,"PGPGetKeyByKeyID");
   assert(getkey != NULL);

   return getkey(keySet, keyID, pubKeyAlgorithm, key);
}
//---------------------------------------------------------------------------

PGPError PGPDecodeSig(PGPContextRef context, PGPOptionListRef firstOption,
   PGPOptionListRef secondOption, PGPOptionListRef thirdOption, PGPOptionListRef forthOption,
   PGPOptionListRef fifthOption)
{
   PGPDECODESIG decode = (PGPDECODESIG)GetProcAddress(hSdkLib6,"PGPDecode");
   assert(decode != NULL);

   return decode(context, firstOption, secondOption, thirdOption, forthOption,
         fifthOption);
}
//---------------------------------------------------------------------------

PGPError PGPDecode(PGPContextRef context, PGPOptionListRef opt1, PGPOptionListRef opt2,
   PGPOptionListRef opt3, PGPOptionListRef opt4, PGPOptionListRef opt5,
   PGPOptionListRef opt6, PGPOptionListRef opt7, PGPOptionListRef opt8,
   PGPOptionListRef opt9)
{
   PGPDECODE decode = (PGPDECODE)GetProcAddress(hSdkLib6,"PGPDecode");
   assert(decode != NULL);

   return decode(context, opt1, opt2, opt3, opt4, opt5, opt6, opt7, opt8, opt9);
}
//---------------------------------------------------------------------------

PGPError PGPEncode(PGPContextRef context, PGPOptionListRef opt1,
   PGPOptionListRef opt2, PGPOptionListRef opt3, PGPOptionListRef opt4,
   PGPOptionListRef opt5, PGPOptionListRef opt6, PGPOptionListRef opt7,
   PGPOptionListRef opt8, PGPOptionListRef opt9, PGPOptionListRef opt10,
   PGPOptionListRef opt11, PGPOptionListRef opt12, PGPOptionListRef opt13,
   PGPOptionListRef opt14, PGPOptionListRef opt15,
   PGPOptionListRef opt16)
{
   PGPENCODE encode = (PGPENCODE)GetProcAddress(hSdkLib6,"PGPEncode");
   assert(encode != NULL);

   return encode(context, opt1, opt2, opt3, opt4, opt5, opt6, opt7, opt8,
        opt9, opt10, opt11, opt12, opt13, opt14, opt15, opt16);
}
//---------------------------------------------------------------------------

PGPOptionListRef PGPOCompression(PGPContextRef context,
      PGPBoolean isCompressed )
{
   PGPOCOMPRESSION comp = (PGPOCOMPRESSION)GetProcAddress(hSdkLib6,"PGPOCompression");
   assert(comp != NULL);

   return comp(context, isCompressed);
}
//---------------------------------------------------------------------------

PGPOptionListRef PGPOClearSign(PGPContextRef context, PGPBoolean clearsign)
{
   PGPOCLEARSIGN sign = (PGPOCLEARSIGN)GetProcAddress(hSdkLib6,"PGPOClearSign");
   assert(sign != NULL);

   return sign(context, clearsign);
}
//---------------------------------------------------------------------------

PGPOptionListRef PGPOSignWithKey(PGPContextRef context, PGPKeyRef SignKey,
   PGPOptionListRef opt1, PGPOptionListRef opt2, PGPOptionListRef opt3)
{
   PGPOSIGNWITHKEY sign = (PGPOSIGNWITHKEY)GetProcAddress(hSdkLib6,"PGPOSignWithKey");
   assert(sign != NULL);

   return sign(context, SignKey, opt1, opt2, opt3);
}
//---------------------------------------------------------------------------

PGPError PGPNewKeySet(PGPContextRef context, PGPKeySetRef *keySet)
{
   PGPNEWKEYSET newset = (PGPNEWKEYSET)GetProcAddress(hSdkLib6,"PGPNewKeySet");
   assert(newset != NULL);

   return newset(context, keySet);
}
//---------------------------------------------------------------------------

PGPError PGPNewSingletonKeySet(PGPKeyRef key, PGPKeySetRef *keySet)
{
   PGPNEWSINGLETONKEYSET newset =
      (PGPNEWSINGLETONKEYSET)GetProcAddress(hSdkLib6,"PGPNewSingletonKeySet");
   assert(newset != NULL);

   return newset(key, keySet);
}
//---------------------------------------------------------------------------

PGPError PGPAddKeys(PGPKeySetRef keysToAdd, PGPKeySetRef keySet)
{
   PGPADDKEYS add = (PGPADDKEYS)GetProcAddress(hSdkLib6,"PGPAddKeys");
   assert(add != NULL);

   return add(keysToAdd, keySet);
}
//---------------------------------------------------------------------------

PGPOptionListRef PGPOArmorOutput(PGPContextRef Context,
   PGPBoolean armorOutput)
{
   PGPOARMOROUTPUT armor =
      (PGPOARMOROUTPUT)GetProcAddress(hSdkLib6,"PGPOArmorOutput");
   assert(armor != NULL);

   return armor(Context,armorOutput);
}
//---------------------------------------------------------------------------

PGPOptionListRef PGPOEncryptToKeySet(PGPContextRef Context,
   PGPKeySetRef keySet )
{
   PGPOENCRYPTTOKEYSET encset =
      (PGPOENCRYPTTOKEYSET)GetProcAddress(hSdkLib6,"PGPOEncryptToKeySet");
   assert(encset != NULL);

   return encset(Context,keySet);
}
//---------------------------------------------------------------------------

PGPOptionListRef PGPOConventionalEncrypt(PGPContextRef Context,
      PGPOptionListRef opt1, PGPOptionListRef opt2, PGPOptionListRef opt3)
{
   PGPOCONVENTIONALENCRYPT enccon =
      (PGPOCONVENTIONALENCRYPT)GetProcAddress(hSdkLib6,"PGPOConventionalEncrypt");
   assert(enccon != NULL);

   return enccon(Context,opt1,opt2,opt3);
}
//---------------------------------------------------------------------------

PGPOptionListRef PGPOOutputLineEndType(PGPContextRef context,
   PGPLineEndType lineEndType )
{
   PGPOOUTPUTLINEENDTYPE lend =
      (PGPOOUTPUTLINEENDTYPE)GetProcAddress(hSdkLib6,"PGPOOutputLineEndType");
   assert(lend != NULL);

   return lend(context,lineEndType);
}
//---------------------------------------------------------------------------

PGPOptionListRef PGPOForYourEyesOnly(PGPContextRef pgpContext,
   PGPBoolean forYourEyesOnly)
{
   PGPOFORYOUREYESONLY eyesonly =
      (PGPOFORYOUREYESONLY)GetProcAddress(hSdkLib6,"PGPOForYourEyesOnly");
   assert(eyesonly != NULL);

   return eyesonly(pgpContext,forYourEyesOnly);
}
//---------------------------------------------------------------------------

PGPOptionListRef PGPOHashAlgorithm(PGPContextRef pgpContext,
   PGPHashAlgorithm algID )
{
   PGPOHASHALGORITHM hashalg =
      (PGPOHASHALGORITHM)GetProcAddress(hSdkLib6,"PGPOHashAlgorithm");
   assert(hashalg != NULL);

   return hashalg(pgpContext,algID);
}
//---------------------------------------------------------------------------

PGPOptionListRef PGPODataIsASCII(PGPContextRef context,
   PGPBoolean dataIsASCII )
{
   PGPODDATAISSCII ascii = (PGPODDATAISSCII)GetProcAddress(hSdkLib6,"PGPODataIsASCII");
   assert(ascii != NULL);

   return ascii(context,dataIsASCII);
}
//---------------------------------------------------------------------------

PGPError PGPCommitKeyRingChanges( PGPKeySetRef keySet )
{
   PGPCOMMITKEYRINGCHANGES commit =
      (PGPCOMMITKEYRINGCHANGES)GetProcAddress(hSdkLib6,"PGPCommitKeyringChanges");
   assert(commit != NULL);

   return commit(keySet);
}
//---------------------------------------------------------------------------

PGPOptionListRef PGPOEventHandler(PGPContextRef context, PGPEventHandlerProcPtr eventHandler,
   PGPUserValue eventHandlerData)
{
   PGPOEVENTHANDLER handler =
      (PGPOEVENTHANDLER)GetProcAddress(hSdkLib6,"PGPOEventHandler");
   assert(handler != NULL);

   return handler(context, eventHandler, eventHandlerData);
}
//---------------------------------------------------------------------------

PGPError PGPGetPrimaryUserID(PGPKeyRef key, PGPUserIDRef *userID)
{
   PGPGETPRIMARYUSERID getid =
      (PGPGETPRIMARYUSERID)GetProcAddress(hSdkLib6,"PGPGetPrimaryUserID");
   assert(getid != NULL);

   return getid(key, userID);
}
//---------------------------------------------------------------------------

PGPError PGPGetUserIDStringBuffer(PGPUserIDRef userID, PGPUserIDPropName propName,
   PGPSize availLength, char *propString, PGPSize *usedLength)
{
   PGPGETUSERIDSTRINGBUFFER getstr =
      (PGPGETUSERIDSTRINGBUFFER)GetProcAddress(hSdkLib6,"PGPGetUserIDStringBuffer");
   assert(getstr != NULL);

   return getstr(userID, propName, availLength, propString, usedLength);
}
//---------------------------------------------------------------------------

PGPError PGPGetPrimaryUserIDNameBuffer(PGPKeyRef key, PGPSize size, char *userid,
   PGPSize *fullsize)
{
   PGPGETPRIMARYUSERIDNAMEBUFFER getname =
      (PGPGETPRIMARYUSERIDNAMEBUFFER)GetProcAddress(hSdkLib6, "PGPGetPrimaryUserIDNameBuffer");
   assert(getname != NULL);

   return getname(key, size, userid, fullsize);
}
//---------------------------------------------------------------------------

PGPBoolean PGPPassphraseIsValid(PGPKeyRef key, PGPOptionListRef opt1, PGPOptionListRef opt2)
{
   PGPPASSPHRASEISVALID goodpass =
      (PGPPASSPHRASEISVALID)GetProcAddress(hSdkLib6,"PGPPassphraseIsValid");
   assert(goodpass != NULL);

   return goodpass(key, opt1, opt2);
}
//---------------------------------------------------------------------------

time_t PGPGetStdTimeFromPGPTime(PGPTime theTime)
{
   PGPGETSTDTIMEFROMPGPTIME convtime =
      (PGPGETSTDTIMEFROMPGPTIME)GetProcAddress(hSdkLib6,"PGPGetStdTimeFromPGPTime");
   assert(convtime != NULL);

   return convtime(theTime);
}
//---------------------------------------------------------------------------

PGPTime PGPGetTime()
{
   PGPGETTIME gettime = (PGPGETTIME)GetProcAddress(hSdkLib6,"PGPGetTime");
   assert(gettime != NULL);

   return gettime();
}
//---------------------------------------------------------------------------

PGPError PGPGetKeyNumber(PGPKeyRef key, PGPKeyPropName propName, PGPInt32 *propData)
{
   PGPGETKEYNUMBER getnum =
      (PGPGETKEYNUMBER)GetProcAddress(hSdkLib6, "PGPGetKeyNumber");
   assert(getnum != NULL);

   return getnum(key, propName, propData);
}
//---------------------------------------------------------------------------

PGPError PGPGetKeyBoolean(PGPKeyRef key, PGPKeyPropName propName, PGPBoolean *propData)
{
   PGPGETKEYBOOLEAN getbool =
      (PGPGETKEYBOOLEAN)GetProcAddress(hSdkLib6, "PGPGetKeyBoolean");
   assert(getbool != NULL);

   return getbool(key, propName, propData);
}
//---------------------------------------------------------------------------

PGPError PGPCountAdditionalRecipientRequests(PGPKeyRef baseKey, PGPUInt32 *numARRKeys)
{
   PGPCOUNTADDITIONALRECIPIENTREQUESTS countreqs =
      (PGPCOUNTADDITIONALRECIPIENTREQUESTS)GetProcAddress(hSdkLib6,
         "PGPCountAdditionalRecipientRequests");
   assert(countreqs != NULL);

   return countreqs(baseKey, numARRKeys);
}
//---------------------------------------------------------------------------

PGPOptionListRef PGPOInputFile(PGPContextRef context, PGPFileSpecRef fileSpec)
{
   PGPOINPUTFILE infile = (PGPOINPUTFILE)GetProcAddress(hSdkLib6, "PGPOInputFile");
   assert(infile != NULL);

   return infile(context, fileSpec);
}
//---------------------------------------------------------------------------

PGPError PGPImportKeySet(PGPContextRef context, PGPKeySetRef *keySet,
   PGPOptionListRef opt1, PGPOptionListRef opt2, PGPOptionListRef opt3,
   PGPOptionListRef opt4, PGPOptionListRef opt5, PGPOptionListRef opt6)
{
   PGPIMPORTKEYSET impkeys =
      (PGPIMPORTKEYSET)GetProcAddress(hSdkLib6, "PGPImportKeySet");
   assert(impkeys != NULL);

   return impkeys(context, keySet, opt1, opt2, opt3, opt4, opt5, opt6);
}
//---------------------------------------------------------------------------

PGPError PGPKeyIterSubKey(PGPKeyIterRef iter, PGPSubKeyRef *subKey)
{
   PGPKEYITERSUBKEY itersub =
      (PGPKEYITERSUBKEY)GetProcAddress(hSdkLib6, "PGPKeyIterSubKey");
   assert(itersub != NULL);

   return itersub(iter, subKey);
}
//---------------------------------------------------------------------------

PGPError PGPGetKeyIDFromSubKey(PGPSubKeyRef subKey, PGPKeyID *keyID)
{
   PGPGETKEYIDFROMSUBKEY getid =
      (PGPGETKEYIDFROMSUBKEY)GetProcAddress(hSdkLib6, "PGPGetKeyIDFromSubKey");
   assert(getid != NULL);

   return getid(subKey, keyID);
}
//---------------------------------------------------------------------------

PGPError PGPGetSubKeyNumber(PGPSubKeyRef subKey, PGPKeyPropName propName,
   PGPInt32 *propData )
{
   PGPGETSUBKEYNUMBER getnum =
      (PGPGETSUBKEYNUMBER)GetProcAddress(hSdkLib6, "PGPGetSubKeyNumber");
   assert(getnum != NULL);

   return getnum(subKey, propName, propData);
}
//---------------------------------------------------------------------------

PGPError PGPNewEmptyKeySet(PGPKeySetRef baseKeySet, PGPKeySetRef *newKeySet )
{
   PGPNEWEMPTYKEYSET newset =
      (PGPNEWEMPTYKEYSET)GetProcAddress(hSdkLib6, "PGPNewEmptyKeySet");
   assert(newset != NULL);

   return newset(baseKeySet, newKeySet);
}
//---------------------------------------------------------------------------

PGPError PGPNewUserIDEmailFilter(PGPContextRef pgpContext, char const *emailString,
   PGPMatchCriterion match, PGPFilterRef *outFilter )
{
   PGPNEWUSERIDEMAILFILTER filter =
      (PGPNEWUSERIDEMAILFILTER)GetProcAddress(hSdkLib6, "PGPNewUserIDEmailFilter");
   assert(filter != NULL);

   return filter(pgpContext, emailString, match, outFilter);
}
//---------------------------------------------------------------------------

PGPError PGPCountKeys(PGPKeySetRef keySet, PGPUInt32 *numKeys )
{
   PGPCOUNTKEYS count =
      (PGPCOUNTKEYS)GetProcAddress(hSdkLib6, "PGPCountKeys");
   assert(count != NULL);

   return count(keySet, numKeys);
}
//---------------------------------------------------------------------------

PGPUInt32 PGPGlobalRandomPoolMouseMoved()
{
   PGPGLOBALRANDOMPOOLMOUSEMOVED moved =
     (PGPGLOBALRANDOMPOOLMOUSEMOVED)GetProcAddress(hSdkLib6, "PGPGlobalRandomPoolMouseMoved");
   assert(moved != NULL);

   return moved();
}
//---------------------------------------------------------------------------

PGPOptionListRef PGPOAskUserForEntropy(PGPContextRef context, PGPBoolean askUser)
{
   PGPOASKUSERFORENTROPY askuser =
      (PGPOASKUSERFORENTROPY)GetProcAddress(hSdkLib6, "PGPOAskUserForEntropy");
   assert(askuser != NULL);

   return askuser(context, askUser);
}
//---------------------------------------------------------------------------

PGPError PGPContextGetRandomBytes(PGPContextRef context, void *dataBuf, PGPSize availLength)
{
   PGPCONTEXTGETRANDOMBYTES getbytes =
      (PGPCONTEXTGETRANDOMBYTES)GetProcAddress(hSdkLib6, "PGPContextGetRandomBytes");
   assert(getbytes != NULL);

   return getbytes(context, dataBuf, availLength);
}
//---------------------------------------------------------------------------






