//---------------------------------------------------------------------------
#include <vcl\vcl.h>
#pragma hdrstop
#include <math.h>
#include <assert.h>
#include "PgpSdk8Calls.h"

HANDLE hSdkLib8;
PGPContextRef Context8;
PGPError err;
//---------------------------------------------------------------------------
//  PGP utility
//---------------------------------------------------------------------------

bool __fastcall IsPgpError(PGPError err, const AnsiString &errmsg)
{
   if (IsPGPError(err))
   {
      char *buf = new char[256];

      PGPGetErrorString(err, 255, buf);
         Application->MessageBox(
         AnsiString(errmsg+"\nError: ("+AnsiString(err)+") "+AnsiString(buf)).c_str(),
        "PGP Error",
         MB_OK
      );

      delete [] buf;
      return true;
   }
   return false;
}

//---------------------------------------------------------------------------
//  PGP Callbacks
//---------------------------------------------------------------------------

PGPError DecodeEventHandler(PGPContextRef context, PGPEvent *event,
   PGPUserValue userValue)
{
   PGPError err;
   PGPSize useridsize;
   char useridstr[kPGPMaxUserIDSize];
   PGPKeyDBObjRef sigKey = kInvalidPGPKeyDBObjRef;
   PGPEventSignatureData *sig = &event->data.signatureData;
   Tdecinfo8 *decinfo = (Tdecinfo8*)userValue;
   AnsiString status, signer, sigtime;

   switch(event->type)
   {
      case kPGPEvent_DecryptionEvent:
      {
         decinfo->decrypts++;
         break;
      }
      case kPGPEvent_SignatureEvent:
      {
         bool known = false;

         char keyidstr[kPGPMaxKeyIDStringSize];
         err = PGPGetKeyIDString(&sig->signingKeyID, kPGPKeyIDString_Abbreviated, keyidstr);
         if (IsPGPError(err))
            goto addsig;

         err = PGPFindKeyByKeyID(decinfo->keyDB, &sig->signingKeyID, &sigKey);
         if (IsPGPError(err))
            goto addsig;

         err = PGPGetPrimaryUserIDName(sigKey, useridstr, kPGPMaxKeyIDStringSize- 1, &useridsize);
         if (IsPGPError(err))
            goto addsig;

         known = true;

      addsig:

         if (known) signer = AnsiString(useridstr) + " (" + AnsiString(keyidstr) + ")";
         else signer = "Unknown, Key ID = " + AnsiString(keyidstr);

         sigtime = TimeT2SigTime(PGPGetStdTimeFromPGPTime(sig->creationTime));

         if (sig->verified) status = "good"; else
         if (signer.Pos("Unknown")==1) status = "unknown"; else
            status = "bad";

         if (decinfo->siginfos->Count) decinfo->siginfos->Add("");
         decinfo->siginfos->Add("*** PGP Signature Status: " + status);
         decinfo->siginfos->Add("*** Signer: " + signer);
         decinfo->siginfos->Add("*** Signed: " + sigtime);
         decinfo->siginfos->Add("*** Verified: " + FormatDateTime("m/dd/yy h:nn:ss ampm", Now()));
         decinfo->sigs++;

         break;
      }
   }

   return kPGPError_NoErr;
}
//---------------------------------------------------------------------------

AnsiString __fastcall TimeT2SigTime(time_t thetime)
{
   //Thu Oct 29 2:40:37 2001 (ctime)
   //10/29/01 2:40:37 PM     (sigtime)
   AnsiString mt,dy,yr,hr,mn,ampm="AM",timestr=ctime(&thetime);
   AnsiString months = "~~~JanFebMarAprMayJunJulAugSepOctNovDec";
   AnsiString mthstr = timestr.SubString(5, 3);
   mt = IntToStr(months.Pos(mthstr) / 3);
   dy = timestr.SubString(9, 2);
   yr = timestr.SubString(23, 2);
   mn = timestr.SubString(15, 5);
   hr = timestr.SubString(12, 2);
   while (hr.Length() && hr[1] == '0') hr = hr.SubString(2, 2);
   int h = hr.ToIntDef(-1);
   if (h > 12) { h -= 12; ampm = "PM"; }

   return mt + "/" + dy + "/" + yr + " " + AnsiString(h) + ":" + mn + " " + ampm;
}

//---------------------------------------------------------------------------
//  PGP SDK 3.0 Functions
//---------------------------------------------------------------------------

PGPError PGPsdkInit(PGPFlags Options)
{
   PGPSDKINITV8 sdkinit = (PGPSDKINITV8)GetProcAddress(hSdkLib8, "PGPsdkInit");
   if (sdkinit == NULL) return (PGPUInt32)-1;

   return sdkinit(Options);
}
//---------------------------------------------------------------------------

PGPError PGPsdkCleanup()
{
   PGPSDKCLEANUP sdkcleanup = (PGPSDKCLEANUP)GetProcAddress(hSdkLib8, "PGPsdkCleanup");
   assert(sdkcleanup != NULL);

   return sdkcleanup();
}
//---------------------------------------------------------------------------

PGPUInt32 PGPGetPGPsdkVersion()
{
   PGPGETPGPSDKVERSION sdkversion =
      (PGPGETPGPSDKVERSION)GetProcAddress(hSdkLib8,"PGPGetPGPsdkVersion");
   assert(sdkversion != NULL);

   return  sdkversion();
}
//---------------------------------------------------------------------------

PGPUInt32 PGPGetPGPsdkAPIVersion()
{
   PGPGETPGPSDKAPIVERSION apiversion =
      (PGPGETPGPSDKAPIVERSION)GetProcAddress(hSdkLib8,"PGPGetPGPsdkAPIVersion");
   assert(apiversion != NULL);

   return  apiversion();
}
//---------------------------------------------------------------------------

PGPError PGPOpenKeyDBFile(PGPContextRef context,
   PGPOpenKeyDBFileOptions options, PGPFileSpecRef pubKeyFileSpec,
   PGPFileSpecRef secKeyFileSpec,
   PGPKeyDBRef *keyDBRef)
{
   PGPOPENKEYDBFILE opendb =
      (PGPOPENKEYDBFILE)GetProcAddress(hSdkLib8, "PGPOpenKeyDBFile");
   assert(opendb != NULL);

   return opendb(context, options, pubKeyFileSpec, secKeyFileSpec, keyDBRef);
}
//---------------------------------------------------------------------------

PGPError PGPNewKeySet(PGPKeyDBRef keyDB, PGPKeySetRef *keySet)
{
   PGPNEWKEYSET newset = (PGPNEWKEYSET)GetProcAddress(hSdkLib8, "PGPNewKeySet");
   assert(newset != NULL);

   return newset(keyDB, keySet);
}
//---------------------------------------------------------------------------

PGPError PGPNewEmptyKeySet(PGPKeyDBRef keyDB, PGPKeySetRef *keySet)
{
   PGPNEWEMPTYKEYSET emptyset =
      (PGPNEWEMPTYKEYSET)GetProcAddress(hSdkLib8, "PGPNewEmptyKeySet");
   assert(emptyset != NULL);

   return emptyset(keyDB, keySet);
}
//---------------------------------------------------------------------------

PGPError PGPFreeFileSpec(PGPFileSpecRef fileRef)
{
   PGPFREEFILESPEC freespec =
      (PGPFREEFILESPEC)GetProcAddress(hSdkLib8,"PGPFreeFileSpec");
   assert(freespec != NULL);

   return freespec(fileRef);
}
//---------------------------------------------------------------------------

PGPError PGPGetErrorString(PGPError theError, PGPSize bufferSize,
   char *theString)
{
   PGPGETERRORSTRING geterrstr =
      (PGPGETERRORSTRING)GetProcAddress(hSdkLib8, "PGPGetErrorString");
   assert(geterrstr != NULL);

   return geterrstr(theError, bufferSize, theString);
}
//---------------------------------------------------------------------------

PGPError PGPNewContext(PGPUInt32 clientAPIVersion,
   PGPContextRef *newContext)
{
   PGPNEWCONTEXT newcontext = (PGPNEWCONTEXT)GetProcAddress(hSdkLib8,"PGPNewContext");
   assert(newcontext != NULL);

   return newcontext(clientAPIVersion, newContext);
}
//---------------------------------------------------------------------------

PGPError PGPFreeContext(PGPContextRef Context)
{
   PGPFREECONTEXT pgpfreecontext;
   pgpfreecontext = (PGPFREECONTEXT)GetProcAddress(hSdkLib8,"PGPFreeContext");
   assert(pgpfreecontext != NULL);

   return pgpfreecontext(&Context);
}
//---------------------------------------------------------------------------

PGPError PGPGetFullPathFromFileSpec(PGPFileSpecRef fileRef,
   char **fullPathPtr)
{
   PGPGETFULLPATHFROMFILESPEC fullpath =
     (PGPGETFULLPATHFROMFILESPEC)GetProcAddress(hSdkLib8, "PGPGetFullPathFromFileSpec");
   assert(fullpath != NULL);

   return fullpath(fileRef, fullPathPtr);
}
//---------------------------------------------------------------------------

PGPOptionListRef PGPOExportKeySet(PGPContextRef context, PGPKeySetRef keys)
{
   PGPOEXPORTKEYSET exportset =
      (PGPOEXPORTKEYSET)GetProcAddress(hSdkLib8, "PGPOExportKeySet");
   assert(exportset != NULL);

   return exportset(context, keys);
}
//---------------------------------------------------------------------------

PGPOptionListRef PGPOExportFormat(PGPContextRef context,
   PGPExportFormat exportFormat)
{
   PGPOEXPORTFORMAT exportfmt =
     (PGPOEXPORTFORMAT)GetProcAddress(hSdkLib8, "PGPOExportFormat");

   return exportfmt(context, exportFormat);
}
//---------------------------------------------------------------------------

PGPOptionListRef PGPOExportPrivateKeys(PGPContextRef context,
   PGPBoolean exportKeys)
{
   PGPOEXPORTPRIVATEKEYS exportprivs =
      (PGPOEXPORTPRIVATEKEYS)GetProcAddress(hSdkLib8, "PGPOExportPrivateKeys");
   assert (exportprivs != NULL);

   return exportprivs(context, exportKeys);
}
//---------------------------------------------------------------------------

PGPOptionListRef PGPOVersionString(PGPContextRef context,
   char const *version)
{
   PGPOVERSIONSTRING verstr =
      (PGPOVERSIONSTRING)GetProcAddress(hSdkLib8, "PGPOVersionString");
   assert(verstr != NULL);

   return verstr(context, version);
}
//---------------------------------------------------------------------------

PGPOptionListRef PGPOLastOption(PGPContextRef context)
{
   PGPOLASTOPTION lastopt = (PGPOLASTOPTION)GetProcAddress(hSdkLib8, "PGPOLastOption");
   assert(lastopt != NULL);

   return lastopt(context);
}
//---------------------------------------------------------------------------

PGPOptionListRef PGPOAllocatedOutputBuffer(PGPContextRef context,
   void **buffer, PGPSize maximumBufferSize, PGPSize *actualBufferSize)
{
   PGPOALLOCATEDOUTPUTBUFFER allbuf =
      (PGPOALLOCATEDOUTPUTBUFFER)GetProcAddress(hSdkLib8, "PGPOAllocatedOutputBuffer");
   assert(allbuf != NULL);

   return allbuf(context, buffer, maximumBufferSize, actualBufferSize);
}
//---------------------------------------------------------------------------

PGPOptionListRef PGPORecursivelyDecode(PGPContextRef context,
   PGPBoolean recurse)
{
   PGPORECURSIVELYDECODE recur =
      (PGPORECURSIVELYDECODE)GetProcAddress(hSdkLib8, "PGPORecursivelyDecode");
   assert(recur != NULL);

   return recur(context,recurse);
}
//---------------------------------------------------------------------------

PGPOptionListRef PGPOPassphrase(PGPContextRef context,
   const char *passphrase)
{
   PGPOPASSPHRASE phrase = (PGPOPASSPHRASE)GetProcAddress(hSdkLib8, "PGPOPassphrase");
   assert(phrase != NULL);

   return phrase(context, passphrase);
}
//---------------------------------------------------------------------------

PGPError PGPFreeKeySet(PGPKeySetRef keys)
{
   PGPFREEKEYSET freeset = (PGPFREEKEYSET)GetProcAddress(hSdkLib8, "PGPFreeKeySet");
   assert(freeset != NULL);

   return freeset(keys);
}
//---------------------------------------------------------------------------

PGPError PGPFreeData(void *allocation)
{
   PGPFREEDATA freedat = (PGPFREEDATA)GetProcAddress(hSdkLib8, "PGPFreeData");
   assert(freedat != NULL);

   return freedat(allocation);
}
//---------------------------------------------------------------------------

PGPError PGPNewKeyDisabledFilter(PGPContextRef context,
   PGPBoolean disabled, PGPFilterRef *outFilter)
{
   PGPNEWKEYDISABLEDFILTER filter =
      (PGPNEWKEYDISABLEDFILTER)GetProcAddress(hSdkLib8, "PGPNewKeyDisabledFilter");
   assert(filter != NULL);

   return filter(context, disabled, outFilter);
}
//---------------------------------------------------------------------------

PGPError PGPNewKeyRevokedFilter(PGPContextRef context,
   PGPBoolean revoked, PGPFilterRef *outFilter)
{
   PGPNEWKEYREVOKEDFILTER filter =
      (PGPNEWKEYREVOKEDFILTER)GetProcAddress(hSdkLib8, "PGPNewKeyRevokedFilter");
   assert(filter != NULL);

   return filter(context, revoked, outFilter);
}
//---------------------------------------------------------------------------

PGPError PGPNewKeyDBObjDataFilter(PGPContextRef context, PGPKeyDBObjProperty whichProperty,
   const void *matchData, PGPSize matchDataSize, PGPMatchCriterion matchCriteria,
   PGPFilterRef *outFilter)
{
   PGPNEWKEYDBOBJDATAFILTER  filter =
      (PGPNEWKEYDBOBJDATAFILTER)GetProcAddress(hSdkLib8, "PGPNewKeyDBObjDataFilter");
   assert(filter != NULL);

   return filter(context, whichProperty, matchData, matchDataSize, matchCriteria, outFilter);
}
//---------------------------------------------------------------------------

PGPError PGPFilterKeySet(PGPKeySetRef origSet, PGPFilterRef filter,
      PGPKeySetRef *resultSet)
{
   PGPFILTERKEYSET filterset =
      (PGPFILTERKEYSET)GetProcAddress(hSdkLib8,"PGPFilterKeySet");
   assert(filterset != NULL);

   return filterset(origSet, filter, resultSet);
}
//---------------------------------------------------------------------------

PGPError PGPOrderKeySet(PGPKeySetRef src, PGPKeyOrdering order, PGPBoolean revorder,
      PGPKeyListRef *outRef )
{
   PGPORDERKEYSET orderset =
      (PGPORDERKEYSET)GetProcAddress(hSdkLib8,"PGPOrderKeySet");
   assert(orderset != NULL);

   return orderset(src, order, revorder, outRef);
}
//---------------------------------------------------------------------------

PGPError PGPNewKeyIter(PGPKeyListRef keys, PGPKeyIterRef *outRef)
{
   PGPNEWKEYITER iter =
      (PGPNEWKEYITER)GetProcAddress(hSdkLib8,"PGPNewKeyIter");
   assert(iter != NULL);

   return iter(keys, outRef);
}
//---------------------------------------------------------------------------

PGPError PGPGetKeyID(PGPKeyDBObjRef key, PGPKeyID *id)
{
   PGPGETKEYID getid =
      (PGPGETKEYID)GetProcAddress(hSdkLib8,"PGPGetKeyID");
   assert(getid != NULL);

   return getid(key, id);
}
//---------------------------------------------------------------------------

PGPError PGPGetKeyIDString(PGPKeyID const *ref, PGPKeyIDStringType type,
      char outString[kPGPMaxKeyIDStringSize])
{
   PGPGETKEYIDSTRING getstr =
      (PGPGETKEYIDSTRING)GetProcAddress(hSdkLib8,"PGPGetKeyIDString");
   assert(getstr != NULL);

   return getstr(ref, type, outString);
}
//---------------------------------------------------------------------------

PGPError PGPFreeKeyIter(PGPKeyIterRef iter)
{
   PGPFREEKEYITER freeiter =
      (PGPFREEKEYITER)GetProcAddress(hSdkLib8,"PGPFreeKeyIter");
   assert(freeiter != NULL);

   return freeiter(iter);
}
//---------------------------------------------------------------------------

PGPError PGPFreeKeyList(PGPKeyListRef keys)
{
   PGPFREEKEYLIST freelist = (PGPFREEKEYLIST)GetProcAddress(hSdkLib8,"PGPFreeKeyList");
   assert(freelist != NULL);

   return freelist(keys);
}
//---------------------------------------------------------------------------

PGPError PGPFreeFilter(PGPFilterRef filter)
{
   PGPFREEFILTER freefilter = (PGPFREEFILTER)GetProcAddress(hSdkLib8,"PGPFreeFilter");
   assert(freefilter != NULL);

   return freefilter(filter);
}
//---------------------------------------------------------------------------

PGPError PGPNewFileSpecFromFullPath(PGPContextRef context,
   const char *path, PGPFileSpecRef *ref)
{
   PGPNEWFILESPECFROMFULLPATH getspec =
      (PGPNEWFILESPECFROMFULLPATH)GetProcAddress(hSdkLib8,"PGPNewFileSpecFromFullPath");
   assert(getspec != NULL);

   return getspec(context,path,ref);
}
//---------------------------------------------------------------------------

PGPOptionListRef PGPOInputBuffer( PGPContextRef context,
   void const *buffer, PGPSize bufferSize)
{
   PGPOINPUTBUFFER inbuf = (PGPOINPUTBUFFER)GetProcAddress(hSdkLib8,"PGPOInputBuffer");
   assert(inbuf != NULL);

   return inbuf(context, buffer, bufferSize);
}
//---------------------------------------------------------------------------

PGPOptionListRef PGPOKeySetRef(PGPContextRef context, PGPKeySetRef keysetRef)
{
   PGPOKEYSETREF setref = (PGPOKEYSETREF)GetProcAddress(hSdkLib8,"PGPOKeySetRef");
   assert(setref != NULL);

   return setref(context, keysetRef);
}
//---------------------------------------------------------------------------

PGPError PGPDecode(PGPContextRef context, PGPOptionListRef opt1,
   PGPOptionListRef opt2, PGPOptionListRef opt3, PGPOptionListRef opt4,
   PGPOptionListRef opt5, PGPOptionListRef opt6, PGPOptionListRef opt7,
   PGPOptionListRef opt8, PGPOptionListRef opt9)
{
   PGPDECODE decode = (PGPDECODE)GetProcAddress(hSdkLib8,"PGPDecode");
   assert(decode != NULL);

   return decode(context, opt1, opt2, opt3, opt4, opt5, opt6, opt7, opt8, opt9);
}
//---------------------------------------------------------------------------

PGPError PGPEncode(PGPContextRef context, PGPOptionListRef opt1,
   PGPOptionListRef opt2, PGPOptionListRef opt3, PGPOptionListRef opt4,
   PGPOptionListRef opt5, PGPOptionListRef opt6, PGPOptionListRef opt7,
   PGPOptionListRef opt8, PGPOptionListRef opt9, PGPOptionListRef opt10,
   PGPOptionListRef opt11, PGPOptionListRef opt12, PGPOptionListRef opt13,
   PGPOptionListRef opt14, PGPOptionListRef opt15,
   PGPOptionListRef opt16)
{
   PGPENCODE encode = (PGPENCODE)GetProcAddress(hSdkLib8,"PGPEncode");
   assert(encode != NULL);

   return encode(context, opt1, opt2, opt3, opt4, opt5, opt6, opt7, opt8,
        opt9, opt10, opt11, opt12, opt13, opt14, opt15, opt16);
}
//---------------------------------------------------------------------------

PGPOptionListRef PGPOCompression(PGPContextRef context,
      PGPBoolean isCompressed )
{
   PGPOCOMPRESSION comp = (PGPOCOMPRESSION)GetProcAddress(hSdkLib8,"PGPOCompression");
   assert(comp != NULL);

   return comp(context, isCompressed);
}
//---------------------------------------------------------------------------

PGPOptionListRef PGPOClearSign(PGPContextRef context, PGPBoolean clearsign)
{
   PGPOCLEARSIGN sign = (PGPOCLEARSIGN)GetProcAddress(hSdkLib8,"PGPOClearSign");
   assert(sign != NULL);

   return sign(context, clearsign);
}
//---------------------------------------------------------------------------

PGPError PGPAddKeys(PGPKeySetRef keysToAdd, PGPKeySetRef keySet)
{
   PGPADDKEYS add = (PGPADDKEYS)GetProcAddress(hSdkLib8,"PGPAddKeys");
   assert(add != NULL);

   return add(keysToAdd, keySet);
}
//---------------------------------------------------------------------------

PGPOptionListRef PGPOArmorOutput(PGPContextRef Context,
   PGPBoolean armorOutput)
{
   PGPOARMOROUTPUT armor =
      (PGPOARMOROUTPUT)GetProcAddress(hSdkLib8,"PGPOArmorOutput");
   assert(armor != NULL);

   return armor(Context,armorOutput);
}
//---------------------------------------------------------------------------

PGPOptionListRef PGPOEncryptToKeySet(PGPContextRef Context,
   PGPKeySetRef keySet )
{
   PGPOENCRYPTTOKEYSET encset =
      (PGPOENCRYPTTOKEYSET)GetProcAddress(hSdkLib8,"PGPOEncryptToKeySet");
   assert(encset != NULL);

   return encset(Context,keySet);
}
//---------------------------------------------------------------------------

PGPOptionListRef PGPOConventionalEncrypt(PGPContextRef Context,
      PGPOptionListRef opt1, PGPOptionListRef opt2, PGPOptionListRef opt3)
{
   PGPOCONVENTIONALENCRYPT enccon =
      (PGPOCONVENTIONALENCRYPT)GetProcAddress(hSdkLib8, "PGPOConventionalEncrypt");
   assert(enccon != NULL);

   return enccon(Context, opt1, opt2, opt3);
}
//---------------------------------------------------------------------------

PGPOptionListRef PGPOOutputLineEndType(PGPContextRef context,
   PGPLineEndType lineEndType )
{
   PGPOOUTPUTLINEENDTYPE lend =
      (PGPOOUTPUTLINEENDTYPE)GetProcAddress(hSdkLib8, "PGPOOutputLineEndType");
   assert(lend != NULL);

   return lend(context, lineEndType);
}
//---------------------------------------------------------------------------

PGPOptionListRef PGPOForYourEyesOnly(PGPContextRef pgpContext,
   PGPBoolean forYourEyesOnly)
{
   PGPOFORYOUREYESONLY eyesonly =
      (PGPOFORYOUREYESONLY)GetProcAddress(hSdkLib8,"PGPOForYourEyesOnly");
   assert(eyesonly != NULL);

   return eyesonly(pgpContext,forYourEyesOnly);
}
//---------------------------------------------------------------------------

PGPOptionListRef PGPOHashAlgorithm(PGPContextRef pgpContext,
   PGPHashAlgorithm algID )
{
   PGPOHASHALGORITHM hashalg =
      (PGPOHASHALGORITHM)GetProcAddress(hSdkLib8,"PGPOHashAlgorithm");
   assert(hashalg != NULL);

   return hashalg(pgpContext,algID);
}
//---------------------------------------------------------------------------

PGPOptionListRef PGPODataIsASCII(PGPContextRef context, PGPBoolean dataIsASCII)
{
   PGPODDATAISSCII ascii =
      (PGPODDATAISSCII)GetProcAddress(hSdkLib8,"PGPODataIsASCII");
   assert(ascii != NULL);

   return ascii(context, dataIsASCII);
}
//---------------------------------------------------------------------------

PGPError PGPCommitKeyRingChanges( PGPKeySetRef keySet )
{
   PGPCOMMITKEYRINGCHANGES commit =
      (PGPCOMMITKEYRINGCHANGES)GetProcAddress(hSdkLib8,"PGPCommitKeyringChanges");
   assert(commit != NULL);

   return commit(keySet);
}
//---------------------------------------------------------------------------

PGPOptionListRef PGPOEventHandler(PGPContextRef context,
       PGPEventHandlerProcPtr eventHandler, PGPUserValue eventHandlerData)
{
   PGPOEVENTHANDLER handler =
      (PGPOEVENTHANDLER)GetProcAddress(hSdkLib8,"PGPOEventHandler");
   assert(handler != NULL);

   return handler(context, eventHandler, eventHandlerData);
}
//---------------------------------------------------------------------------

time_t PGPGetStdTimeFromPGPTime(PGPTime theTime)
{
   PGPGETSTDTIMEFROMPGPTIME convtime =
      (PGPGETSTDTIMEFROMPGPTIME)GetProcAddress(hSdkLib8,"PGPGetStdTimeFromPGPTime");
   assert(convtime != NULL);

   return convtime(theTime);
}
//---------------------------------------------------------------------------

PGPTime PGPGetTime()
{
   PGPGETTIME gettime = (PGPGETTIME)GetProcAddress(hSdkLib8,"PGPGetTime");
   assert(gettime != NULL);

   return gettime();
}
//---------------------------------------------------------------------------

PGPError PGPNewKeyDBObjBooleanFilter(PGPContextRef context,
   PGPKeyDBObjProperty whichProperty, PGPBoolean match, PGPFilterRef *outFilter )
{
   PGPNEWKEYDBOBJBOOLEANFILTER filter =
      (PGPNEWKEYDBOBJBOOLEANFILTER)GetProcAddress(hSdkLib8, "PGPNewKeyDBObjBooleanFilter");
   assert(filter != NULL);

   return filter(context, whichProperty, match, outFilter);
}
//---------------------------------------------------------------------------

PGPError PGPFilterKeyDB(PGPKeyDBRef keyDB, PGPFilterRef filter, PGPKeySetRef *resultSet)
{
   PGPFILTERKEYDB filterdb =
      (PGPFILTERKEYDB)GetProcAddress(hSdkLib8, "PGPFilterKeyDB");
   assert(filterdb != NULL);

   return filterdb(keyDB, filter, resultSet);
}
//---------------------------------------------------------------------------

PGPError PGPFreeKeyDB(PGPKeyDBRef keyDB)
{
   PGPFREEKEYDB pgpfreekeydb;
   pgpfreekeydb = (PGPFREEKEYDB)GetProcAddress(hSdkLib8, "PGPFreeKeyDB");
   assert(pgpfreekeydb != NULL);

   return pgpfreekeydb(keyDB);
}
//---------------------------------------------------------------------------

PGPError PGPExport(PGPContextRef context, PGPOptionListRef opt1, PGPOptionListRef opt2,
   PGPOptionListRef opt3, PGPOptionListRef opt4, PGPOptionListRef opt5,
   PGPOptionListRef opt6)
{
   PGPEXPORT Export = (PGPEXPORT)GetProcAddress(hSdkLib8, "PGPExport");
   assert(Export != NULL);

   return Export(context, opt1, opt2, opt3, opt4, opt5, opt6);
}
//---------------------------------------------------------------------------

PGPError PGPImport(PGPContextRef context, PGPKeyDBRef *keyDB, PGPOptionListRef opt1,
   PGPOptionListRef opt2, PGPOptionListRef opt3, PGPOptionListRef opt4,
   PGPOptionListRef opt5, PGPOptionListRef opt6)
{
   PGPIMPORT import = (PGPIMPORT)GetProcAddress(hSdkLib8, "PGPImport");
   assert(import != NULL);

   return import(context, keyDB, opt1, opt2, opt3, opt4, opt5, opt6);
}
//---------------------------------------------------------------------------

PGPError PGPKeyIterNextKeyDBObj(PGPKeyIterRef iter, PGPKeyDBObjType objectType,
   PGPKeyDBObjRef *keyDBObj)
{
   PGPKEYITERNEXTKEYDBOBJ pgpkeyiternextkeydbobj;
   pgpkeyiternextkeydbobj = (PGPKEYITERNEXTKEYDBOBJ)GetProcAddress(hSdkLib8,
      "PGPKeyIterNextKeyDBObj");
   assert(pgpkeyiternextkeydbobj != NULL);

   return pgpkeyiternextkeydbobj(iter, objectType, keyDBObj);
}
//---------------------------------------------------------------------------

PGPOptionListRef PGPOKeyDBRef(PGPContextRef context, PGPKeyDBRef keyDB )
{
   PGPOKEYDBREF dbref = (PGPOKEYDBREF)GetProcAddress(hSdkLib8, "PGPOKeyDBRef");
   assert(dbref != NULL);

   return dbref(context, keyDB);
}
//---------------------------------------------------------------------------

PGPError PGPNewKeyIDFromString(const char *string, PGPPublicKeyAlgorithm pkalg,
   PGPKeyID *keyID )
{
   PGPNEWKEYIDFROMSTRING newkeyid =
      (PGPNEWKEYIDFROMSTRING)GetProcAddress(hSdkLib8, "PGPNewKeyIDFromString");
   assert(newkeyid != NULL);

   return newkeyid(string, pkalg, keyID);
}
//---------------------------------------------------------------------------

PGPError PGPFindKeyByKeyID(PGPKeyDBRef keyDB, const PGPKeyID *keyID,
   PGPKeyDBObjRef *keyRef )
{
   PGPFINDKEYBYKEYID findkey =
      (PGPFINDKEYBYKEYID)GetProcAddress(hSdkLib8, "PGPFindKeyByKeyID");
   assert(findkey != NULL);

   return findkey(keyDB, keyID, keyRef);
}
//---------------------------------------------------------------------------

PGPBoolean PGPPassphraseIsValid(PGPKeyDBObjRef key, PGPOptionListRef opt1,
   PGPOptionListRef opt2, PGPOptionListRef opt3, PGPOptionListRef opt4)
{
   PGPPASSPHRASEISVALID isvalid =
      (PGPPASSPHRASEISVALID)GetProcAddress(hSdkLib8, "PGPPassphraseIsValid");
   assert(isvalid != NULL);

   return isvalid(key, opt1, opt2, opt3, opt4);
}
//---------------------------------------------------------------------------

PGPError PGPGetKeyDBObjNumericProperty(PGPKeyDBObjRef key, PGPKeyDBObjProperty whichProperty,
   PGPInt32 *prop )
{
   PGPGETKEYDBOBJNUMERICPROPERTY getprop =
      (PGPGETKEYDBOBJNUMERICPROPERTY)GetProcAddress(hSdkLib8, "PGPGetKeyDBObjNumericProperty");
   assert(getprop != NULL);

   return getprop(key, whichProperty, prop);
}
//---------------------------------------------------------------------------

PGPError PGPGetKeyDBObjTimeProperty(PGPKeyDBObjRef key, PGPKeyDBObjProperty whichProperty,
   PGPTime *prop)
{
   PGPGETKEYDBOBJTIMEPROPERTY pgpgetkeydbobjtimeproperty =
      (PGPGETKEYDBOBJTIMEPROPERTY)GetProcAddress(hSdkLib8, "PGPGetKeyDBObjTimeProperty");
   assert(pgpgetkeydbobjtimeproperty != NULL);

   return pgpgetkeydbobjtimeproperty(key, whichProperty, prop);
}
//---------------------------------------------------------------------------

PGPError PGPGetKeyDBObjBooleanProperty(PGPKeyDBObjRef key,
   PGPKeyDBObjProperty whichProperty, PGPBoolean *prop)
{
   PGPGETKEYDBOBJBOOLEANPROPERTY pgpgetkeydbobjbooleanproperty =
      (PGPGETKEYDBOBJBOOLEANPROPERTY)GetProcAddress(hSdkLib8, "PGPGetKeyDBObjBooleanProperty");
   assert(pgpgetkeydbobjbooleanproperty != NULL);

   return pgpgetkeydbobjbooleanproperty(key, whichProperty, prop);
}
//---------------------------------------------------------------------------

PGPError PGPCountAdditionalRecipientRequests(PGPKeyDBObjRef baseKey,
   PGPUInt32 *numKeys )
{
   PGPCOUNTADDITIONALRECIPIENTREQUESTS pgpcountadditionalrcipientrquests =
      (PGPCOUNTADDITIONALRECIPIENTREQUESTS)GetProcAddress(hSdkLib8,
         "PGPCountAdditionalRecipientRequests");
   assert(pgpcountadditionalrcipientrquests != NULL);

   return pgpcountadditionalrcipientrquests(baseKey, numKeys);
}
//---------------------------------------------------------------------------

PGPError PGPNewOneKeySet(PGPKeyDBObjRef obj, PGPKeySetRef *keySet)
{
   PGPNEWONEKEYSET pgpnewonekeyset =
      (PGPNEWONEKEYSET)GetProcAddress(hSdkLib8, "PGPNewOneKeySet");
   assert(pgpnewonekeyset != NULL);

   return pgpnewonekeyset(obj, keySet);
}
//---------------------------------------------------------------------------

PGPOptionListRef PGPOSignWithKey(PGPContextRef context, PGPKeyDBObjRef sigKey,
   PGPOptionListRef opt1, PGPOptionListRef opt2, PGPOptionListRef opt3,
   PGPOptionListRef opt4)
{
   PGPOSIGNWITHKEY signer = (PGPOSIGNWITHKEY)GetProcAddress(hSdkLib8, "PGPOSignWithKey");
   assert(signer != NULL);

   return signer(context, sigKey, opt1, opt2, opt3, opt4);
}
//---------------------------------------------------------------------------

PGPError PGPAddKey(PGPKeyDBObjRef key, PGPKeySetRef keySet )
{
   PGPADDKEY addkey = (PGPADDKEY)GetProcAddress(hSdkLib8, "PGPAddKey");
   assert(addkey != NULL);

   return addkey(key, keySet);
}
//---------------------------------------------------------------------------

PGPError PGPNewKeyIterFromKeyDB(PGPKeyDBRef keyDB, PGPKeyIterRef *iter)
{
   PGPNEWKEYITERFROMKEYDB pgpnewkeyiterfromkeydb;
   pgpnewkeyiterfromkeydb = (PGPNEWKEYITERFROMKEYDB)GetProcAddress(hSdkLib8,
      "PGPNewKeyIterFromKeyDB");
   assert(pgpnewkeyiterfromkeydb != NULL);

   return pgpnewkeyiterfromkeydb(keyDB, iter);
}
//---------------------------------------------------------------------------

PGPError PGPNewKeyIterFromKeySet(PGPKeySetRef keySet, PGPKeyIterRef *iter)
{
   PGPNEWKEYITERFROMKEYSET newiter =
      (PGPNEWKEYITERFROMKEYSET)GetProcAddress(hSdkLib8, "PGPNewKeyIterFromKeySet");
   assert(newiter != NULL);

   return newiter(keySet, iter);
}
//---------------------------------------------------------------------------

PGPOptionListRef PGPOInputFile(PGPContextRef context, PGPFileSpecRef fileSpec)
{
   PGPOINPUTFILE inputf = (PGPOINPUTFILE)GetProcAddress(hSdkLib8, "PGPOInputFile");
   assert(inputf != NULL);

   return inputf(context, fileSpec);
}
//---------------------------------------------------------------------------

PGPError PGPGetPrimaryUserIDName(PGPKeyDBObjRef key,
   void *buffer, PGPSize bufferSize, PGPSize *dataSize)
{
   PGPGETPRIMARYUSERIDNAME getname =
     (PGPGETPRIMARYUSERIDNAME)GetProcAddress(hSdkLib8, "PGPGetPrimaryUserIDName");
   assert(getname != NULL);

   return getname(key, buffer, bufferSize, dataSize);
}
//---------------------------------------------------------------------------

PGPError PGPCopyKeys( PGPKeySetRef keySet, PGPKeyDBRef destKeyDB,
   PGPKeySetRef *destKeySet )
{
   PGPCOPYKEYS copy = (PGPCOPYKEYS)GetProcAddress(hSdkLib8, "PGPCopyKeys");
   assert(copy != NULL);

   return copy(keySet, destKeyDB, destKeySet);
}
//---------------------------------------------------------------------------

PGPError PGPGetKeyIDAlgorithm(const PGPKeyID *keyID, PGPPublicKeyAlgorithm *pkalg )
{
   PGPGETKEYIDALGORITHM getalgo =
      (PGPGETKEYIDALGORITHM)GetProcAddress(hSdkLib8, "PGPGetKeyIDAlgorithm");
   assert(getalgo != NULL);

   return getalgo(keyID, pkalg);
}
//---------------------------------------------------------------------------

PGPError PGPNewOptionList(PGPContextRef context, PGPOptionListRef *outList)
{
   PGPNEWOPTIONLIST newlist =
      (PGPNEWOPTIONLIST)GetProcAddress(hSdkLib8, "PGPNewOptionList");
   assert(newlist != NULL);

   return newlist(context, outList);
}
//---------------------------------------------------------------------------

PGPError PGPAppendOptionList(PGPOptionListRef outList, PGPOptionListRef opt1,
   PGPOptionListRef opt2, PGPOptionListRef opt3, PGPOptionListRef opt4,
   PGPOptionListRef opt5, PGPOptionListRef opt6,
   PGPOptionListRef opt7, PGPOptionListRef opt8)
{
   PGPAPPENDOPTIONLIST append =
      (PGPAPPENDOPTIONLIST)GetProcAddress(hSdkLib8, "PGPAppendOptionList");
   assert(append != NULL);

   return append(outList, opt1, opt2, opt3, opt4, opt5, opt6, opt7, opt8);
}
//---------------------------------------------------------------------------

PGPError PGPFreeOptionList(PGPOptionListRef optionList)
{
   PGPFREEOPTIONLIST freelist =
      (PGPFREEOPTIONLIST)GetProcAddress(hSdkLib8, "PGPFreeOptionList");
   assert(freelist != NULL);

   return freelist(optionList);
}
//---------------------------------------------------------------------------

PGPUInt32 PGPGlobalRandomPoolMouseMoved()
{
   PGPGLOBALRANDOMPOOLMOUSEMOVED moved =
     (PGPGLOBALRANDOMPOOLMOUSEMOVED)GetProcAddress(hSdkLib8, "PGPGlobalRandomPoolMouseMoved");
   assert(moved != NULL);

   return moved();
}
//---------------------------------------------------------------------------

PGPOptionListRef PGPOAskUserForEntropy(PGPContextRef context, PGPBoolean askUser)
{
   PGPOASKUSERFORENTROPY askuser =
      (PGPOASKUSERFORENTROPY)GetProcAddress(hSdkLib8, "PGPOAskUserForEntropy");
   assert(askuser != NULL);

   return askuser(context, askUser);
}
//---------------------------------------------------------------------------

PGPError PGPContextGetRandomBytes(PGPContextRef context, void *dataBuf, PGPSize availLength)
{
   PGPCONTEXTGETRANDOMBYTES getbytes =
      (PGPCONTEXTGETRANDOMBYTES)GetProcAddress(hSdkLib8, "PGPContextGetRandomBytes");
   assert(getbytes != NULL);

   return getbytes(context, dataBuf, availLength);
}
//---------------------------------------------------------------------------






