//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop
#include <inifiles.hpp>

#include "global2.h"
#include "ProxyDlgs.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TProxyDlg *ProxyDlg;
//---------------------------------------------------------------------------

AnsiString __fastcall OpenProxyDialog(TForm *Owner, AnsiString Proxy)
{
   if (SecureModeMsg(Owner))
      return "";

   ProxyDlg = new TProxyDlg(Owner, Proxy);
   ProxyDlg->ShowModal();
   Proxy = "";

   if (ProxyDlg->ModalResult == mrOk)
      Proxy = ProxyDlg->ProxyNameStr;

   delete ProxyDlg;
   return Proxy;
}

//---------------------------------------------------------------------------
// ProxyDlg
//---------------------------------------------------------------------------

__fastcall TProxyDlg::TProxyDlg(TComponent* Owner)
        : TForm(Owner)
{
}
//---------------------------------------------------------------------------

__fastcall TProxyDlg::TProxyDlg(TComponent* Owner, const AnsiString &Proxy)
        : TForm(Owner)
{
   OrgData = new TStringList();
   NewData = new TStringList();

   TIniFile *ini = new TIniFile(AppIni());
   ProxyHideTyping->Checked = ini->ReadBool(ProxyIni, "hidetyping", true);
   ProxyPassword->PasswordChar = ProxyHideTyping->Checked ? '*' : '\0';
   AnsiString DlgsFontStr = ini->ReadString(ProxyIni, QSDIALOGFONT, QSDIALOGFONTDEF);
   TColor color = (TColor)ini->ReadInteger(ProxyIni, QSWINDOWCOLOR, QSWINDOWCOLORDEF);
   delete ini;

   StrToFont(DlgsFontStr, Font);
   SocksVerBox->Color = color;
   ProxyName->Color = color;
   ProxyHost->Color = color;

   ProxyPassword->Color = Color;
   ProxyUserID->Color = Color;

   if (!Proxy.IsEmpty())
   {
      Caption = " Proxy: " + GetTok(0, Proxy, ProxyDiv);
      Global->DB->GetProxy(Proxy, OrgData);
      SetData(OrgData);
      isnew = false;
   }
   else
   {
      Caption = " New Proxy";
      GetData(OrgData);
      isnew = true;
   }

   ProxyNameStr = ProxyIndexStr(OrgData);
}
//---------------------------------------------------------------------------

__fastcall TProxyDlg::~TProxyDlg()
{
   delete NewData;
   delete OrgData;
}
//---------------------------------------------------------------------------

void __fastcall TProxyDlg::FormCloseQuery(TObject *, bool &CanClose)
{
   if (ProxyName->Text.IsEmpty()) ProxyName->Text = ProxyHost->Text;

   GetData(NewData);

   if (NewData->Text != OrgData->Text)
   {
      if (SecureModeMsg(this))
      {
         CanClose = false;
         return;
      }

      if (ModalResult == mrOk)
      {
         if (ProxyHost->Text.IsEmpty())
         {
            MsgBox(this, "The field 'Host' is required", MT_INFO, MB_OK);
            CanClose = false;
            return;
         }

         if (!HTTPBox->Checked && !POP3Box->Checked && !SMTPBox->Checked && !NNTPBox->Checked && !FTPBox->Checked)
         {
            MsgBox(this, "At least 1 proxy type must be selected", MT_INFO, MB_OK);
            CanClose = false;
            return;
         }

         if (ProxyAuthBox->Checked && (ProxyUserID->Text.IsEmpty() || ProxyPassword->Text.IsEmpty()))
         {
            MsgBox(this,
              "You have indicated the server requires authentication. Both User ID    \n"
              "and Password are required.",
            MT_INFO, MB_OK);
            CanClose = false;
            return;
         }

         if (isnew)
         {
            TStringList *proxylist = new TStringList();
            Global->DB->GetProxies(proxylist);
            bool exists = proxylist->IndexOf(ProxyIndexStr(NewData)) >= 0;
            delete proxylist;

            if (exists && MsgBox(this, "This proxy already exists. Overwrite it?",
               MT_INFO, MB_OKCANCEL) == IDCANCEL)
            {
               CanClose = false;
               return;
            }
         }
      }

      if (ModalResult == mrCancel)
      {
         switch (MsgBox(this, "Your proxy profile is modified. Save your changes?",
            MT_INFO, MB_YESNOCANCEL))
         {
            case IDCANCEL: CanClose = false;
            case IDNO: return;
         }
      }

      Global->DB->DelProxy(ProxyNameStr);
      ProxyNameStr = ProxyIndexStr(NewData);
      Global->DB->PutProxy(ProxyNameStr, NewData);
   }
}
//---------------------------------------------------------------------------

void __fastcall TProxyDlg::SetData(TStrings *Data)
{
   assert(Data != NULL);

   ProxyName->Text = Data->Values["name"];
   ProxyHost->Text = Data->Values["host"];
   AnsiString socks = Data->Values["socksver"];

   //if (socks == "No socks" || socks == "----" || socks == "")
   //socks = "<none>";

   SocksVerBox->ItemIndex = SocksVerBox->Items->IndexOf(socks);

   HTTPBox->Checked = Data->Values["http"] == "1";
   HTTPPort->Text   = Data->Values["httpport"].ToIntDef(80);

   POP3Box->Checked = Data->Values["pop3"] == "1";
   POP3Port->Text   = Data->Values["pop3port"].ToIntDef(110);

   SMTPBox->Checked = Data->Values["smtp"] == "1";
   SMTPPort->Text   = Data->Values["smtpport"].ToIntDef(25);

   NNTPBox->Checked = Data->Values["nntp"] == "1";
   NNTPPort->Text   = Data->Values["nntpport"].ToIntDef(119);

   FTPBox->Checked = Data->Values["ftp"] == "1";
   FTPPort->Text   = Data->Values["ftpport"].ToIntDef(21);

   TORBox->Checked = Data->Values["tor"] == "1";

   //TORSignOn->Text = Data->Values["torsignon"];
   //if (TORSignOn->Text.IsEmpty())
   //   TORSignOn->Text = "nowhere.invalid";
   //TORBoxClick(NULL);

   ProxyAuthBox->Checked = Data->Values["auth"] == "1";
   ProxyHideTyping->Checked = Data->Values["hidetyping"] == "1";
   ProxyPassword->Text = Data->Values["password"];
   ProxyUserID->Text = Data->Values["userid"] ;

   if (ProxyAuthBox->Checked)
   {
      ProxyAuthBoxClick(NULL);
   }
}
//---------------------------------------------------------------------------

void __fastcall TProxyDlg::GetData(TStrings *Data)
{
   assert(Data != NULL);

   Data->Values["name"] = ProxyName->Text;
   Data->Values["host"] = ProxyHost->Text;
   Data->Values["socksver"] = SocksVerBox->Text;

   Data->Values["http"]       = IntToStr(HTTPBox->Checked);
   Data->Values["httpport"]   = HTTPPort->Text;
   Data->Values["pop3"]       = IntToStr(POP3Box->Checked);
   Data->Values["pop3port"]   = POP3Port->Text;
   Data->Values["smtp"]       = IntToStr(SMTPBox->Checked);
   Data->Values["smtpport"]   = SMTPPort->Text;
   Data->Values["nntp"]       = IntToStr(NNTPBox->Checked);
   Data->Values["nntpport"]   = NNTPPort->Text;
   Data->Values["ftp" ]       = IntToStr(FTPBox->Checked);
   Data->Values["ftpport"]    = FTPPort->Text;
   Data->Values["tor"]        = IntToStr(TORBox->Checked);

   //if (TORSignOn->Text.IsEmpty())
   //   TORSignOn->Text = "nowhere.invalid";
   //Data->Values["torsignon"]  = TORSignOn->Text;

   Data->Values["auth"]       = IntToStr(ProxyAuthBox->Checked);
   Data->Values["password"]   = ProxyPassword->Text;
   Data->Values["hidetyping"] = IntToStr(ProxyHideTyping->Checked);
   Data->Values["userid"]     = ProxyUserID->Text;
}
//---------------------------------------------------------------------------

void __fastcall TProxyDlg::ProxyHideTypingClick(TObject *)
{
   TIniFile *ini = new TIniFile(AppIni());
   ini->WriteBool("proxy", "hidetyping", ProxyHideTyping->Checked);
   ProxyPassword->PasswordChar = ProxyHideTyping->Checked ? '*' : '\0';
   delete ini;
}
//---------------------------------------------------------------------------

void __fastcall TProxyDlg::ProxyAuthBoxClick(TObject *)
{
   bool checked = ProxyAuthBox->Checked;
   Label5->Enabled = checked;
   ProxyUserID->Enabled = checked;
   Label4->Enabled = checked;
   ProxyPassword->Enabled = checked;
   ProxyHideTyping->Enabled = checked;

   if (checked)
   {
      ProxyPassword->Color = ProxyName->Color;
      ProxyUserID->Color = ProxyName->Color;
   }
   else
   {
      ProxyPassword->Color = Color;
      ProxyUserID->Color = Color;
   }

   ProxyPassword->Enabled = checked;
   ProxyUserID->Enabled = checked;
}
//---------------------------------------------------------------------------

void __fastcall TProxyDlg::EnablePort(bool enable, TEdit *port)
{
   if (enable) port->Color = ProxyName->Color;
   else port->ParentColor = true;
   port->Enabled = enable;
}
//---------------------------------------------------------------------------

void __fastcall TProxyDlg::EnableSignOn(bool enable, TEdit *signon)
{
   // obsolete
   //if (enable) signon->Color = ProxyName->Color;
   //else signon->ParentColor = true;
   //Label7->Enabled = enable;
   //signon->Enabled = enable;
}
//---------------------------------------------------------------------------
#pragma argsused
void __fastcall TProxyDlg::ProxyBoxClick(TObject *Sender)
{
   TCheckBox *box = dynamic_cast<TCheckBox*>(Sender);
   if (box)
   {
      bool checked = box->Checked;
      AnsiString type = SansAmpersand(box->Caption).LowerCase();
      if (type == "http") EnablePort(checked, HTTPPort); else
      if (type == "pop3") EnablePort(checked, POP3Port); else
      if (type == "nntp") EnablePort(checked, NNTPPort); else
      if (type == "ftp" ) EnablePort(checked, FTPPort);  else
      if (type == "smtp")
      {
         /*if (SocksVerBox->Text != "<none>")
         {
            if (!checked)
               EnableSignOn(false, TORSignOn);
            else
               EnableSignOn(true, TORSignOn);

         }
         else
         {
            EnableSignOn(false, TORSignOn);
         }*/

         EnablePort(checked, SMTPPort);
      }
   }
}
//---------------------------------------------------------------------------

void __fastcall TProxyDlg::ProxyHostExit(TObject *Sender)
{
   if (ProxyName->Text.IsEmpty())
   {
      ProxyName->Text = ProxyHost->Text;
   }
}
//---------------------------------------------------------------------------

void __fastcall TProxyDlg::SocksVerBoxChange(TObject *Sender)
{
   /*if (SocksVerBox->Text != "<none>")
   {
      if (SMTPBox->Checked)
         EnableSignOn(true, TORSignOn);
   }
   else
   {
      EnableSignOn(false, TORSignOn);
   }*/
}
//---------------------------------------------------------------------------


