/*
*  BackupCpyDlgs.cpp
*  Copyright  1999-2005 by Richard Christman
*  ALL RIGHTS RESERVED
*
*  This source file is released for peer review purposes only. It is not public domain. 
*  Redistribution either in part or whole is forbidden. The sole source for QuickSilver 
*  distribution currently is http://quicksilvermail.net . You have permission to compile
*  this source for your own use.
*
*  Richard
*  --
*  R.Christman
*  quicksilver@quicksilvermail.net
*  http://quicksilvermail.net
*
*/
//---------------------------------------------------------------------------
#include <vcl\vcl.h>
#pragma hdrstop

#include "BackupCpyDlgs.h"
#include "wipeDlgs.h"
#include "Global.h"
//---------------------------------------------------------------------------
#pragma resource "*.dfm"
TBackupCpyDlg *BackupCpyDlg;
TCopyThread *CopyThread;
TZapThread *ZapThread;
TBKFlashThread *BKFlashThread;
//---------------------------------------------------------------------------
// Flash Thread
//---------------------------------------------------------------------------

__fastcall TBKFlashThread::TBKFlashThread(bool CreateSuspended)
        : TThread(CreateSuspended)
{
}
//---------------------------------------------------------------------------

__fastcall TBKFlashThread::TBKFlashThread(TPanel *Display)
        : TThread(false)
{
   display = Display;
   message = display->Caption;
 //FreeOnTerminate = true;
}
//---------------------------------------------------------------------------

void __fastcall TBKFlashThread::Execute()
{
   unsigned now, Ticks = GetTickCount();
   while (!Terminated)
   {
      now = GetTickCount();
      if (now < Ticks) Ticks = now;

      if (now >= Ticks + 300)
      {
         Synchronize(FlashMessage);
         Ticks = GetTickCount();
         MessageBeep(0);
      }
   }
}
//---------------------------------------------------------------------------

void __fastcall TBKFlashThread::FlashMessage()
{
   display->Caption = (display->Caption == "" ? message : AnsiString(""));
}

//---------------------------------------------------------------------------
// ZapThread
//---------------------------------------------------------------------------
void  __fastcall DoThreadZap(AnsiString Filename, int Zapcount)
{
   ZapThread = new TZapThread(true);
   ZapThread->filename = Filename;
   ZapThread->zaps = Zapcount;
   ZapThread->Resume();
}
//---------------------------------------------------------------------------
__fastcall TZapThread::TZapThread(bool CreateSuspended)
	: TThread(CreateSuspended)
{
   FreeOnTerminate = true;
}
//---------------------------------------------------------------------------
void __fastcall TZapThread::Execute()
{
   zapfile(filename,zaps);
}
//---------------------------------------------------------------------------
// CopyThread
//---------------------------------------------------------------------------
void  __fastcall DoThreadCopy(TBackupCpyDlg *Owner, AnsiString Filename, AnsiString Copyname)
{
   CopyThread = new TCopyThread(true);
   CopyThread->owner = Owner;
   CopyThread->filename = Filename;
   CopyThread->copyname = Copyname;
   CopyThread->Resume();
}
//---------------------------------------------------------------------------
__fastcall TCopyThread::TCopyThread(bool CreateSuspended)
	: TThread(CreateSuspended)
{
   FreeOnTerminate = true;
}
//---------------------------------------------------------------------------
void __fastcall TCopyThread::Execute()
{
   if (!CopyFile(filename.c_str(),copyname.c_str(),false))
      errorstr = GetLastErrorStr();
}
//---------------------------------------------------------------------------
void __fastcall TCopyThread::Terminate(TObject *Sender)
{
   owner->errorstr = errorstr;
   if (errorstr.IsEmpty()) owner->ModalResult = mrOk;
   else owner->ModalResult = mrCancel;
}
//---------------------------------------------------------------------------
// Backup Restore Routines
//---------------------------------------------------------------------------
bool __fastcall DoBackupDatabase(TForm *Owner, AnsiString Path, AnsiString dbName,
   int Copies, int Passes, bool Verify)
{
   BackupCpyDlg = new TBackupCpyDlg(Owner);
   bool ok = BackupCpyDlg->ExecuteBackup(Path, dbName, Copies, Passes, Verify);
   delete BackupCpyDlg;
   return ok;
}
//---------------------------------------------------------------------------
bool __fastcall DoRestoreDatabase(TForm *Owner, AnsiString dbName, AnsiString BakFile)
{
   BackupCpyDlg = new TBackupCpyDlg(Owner);
   bool ok = BackupCpyDlg->ExecuteRestore(dbName,BakFile);
   delete BackupCpyDlg;
   return ok;
}
//---------------------------------------------------------------------------
__fastcall TBackupCpyDlg::TBackupCpyDlg(TComponent* Owner)
	: TForm(Owner)
{
   CenterDialog(this,Owner);
   Display->Caption = "";
   isbackup = true;
}
//---------------------------------------------------------------------------
bool __fastcall TBackupCpyDlg::ExecuteBackup(AnsiString Path, AnsiString Name,
   int Copies, int Passes, bool Verify)
{
   path = Path;
   name = Name;
   copies = Copies;
   passes = Passes;
   verify = Verify;
   isbackup = true;
   return ShowModal()==mrOk;
}
//---------------------------------------------------------------------------

bool __fastcall TBackupCpyDlg::ExecuteRestore(AnsiString Name, AnsiString BakFile)
{
   name = Name;
   bakfile = BakFile;
//sm(name+"\n"+bakfile);
   isbackup = false;
   return ShowModal()==mrOk;
}
//---------------------------------------------------------------------------

void __fastcall TBackupCpyDlg::Timer1Timer(TObject *Sender)
{
   bool ok;
   Timer1->Enabled = false;

   if (isbackup) ok = BackupDatabase();
   else ok = RestoreDatabase();

   ModalResult = ok ? mrOk : mrCancel;
}
//---------------------------------------------------------------------------

bool __fastcall TBackupCpyDlg::RestoreDatabase()
{
   StartFlashMessage("...restoring...");
   if (!copyfile(bakfile.c_str(), name.c_str()))
   {
      StopFlashMessage();
      return false;
   }
   StopFlashMessage();
   return true;
}
//---------------------------------------------------------------------------

void __fastcall TBackupCpyDlg::Timer2Timer(TObject *Sender)
{
   Display->Caption = Display->Caption == "" ? caption : s("");
   Application->ProcessMessages();
}
//---------------------------------------------------------------------------

void __fastcall TBackupCpyDlg::StartFlashMessage(AnsiString msg)
{
   caption = msg;
   Display->Caption = msg;
   Snooze(200);
   starttick = GetTickCount();
   Timer2->Enabled = true;
}
//---------------------------------------------------------------------------

void __fastcall TBackupCpyDlg::StopFlashMessage()
{
   while (GetTickCount() < starttick + 200)
      Application->ProcessMessages();

   Timer2->Enabled = false;
   Display->Caption == "";
}
//---------------------------------------------------------------------------

bool __fastcall TBackupCpyDlg::copyfile(const AnsiString &ExistingName,
   const AnsiString &NewName)
{
   bool ok = true;
   int bytestoread = 1024000;
   int bytesread = bytestoread;

   bool exists = FileExists(NewName);
   char *buffer = new char[bytestoread];
   TFileStream *orgfile  = new TFileStream(ExistingName, fmOpenRead);
   TFileStream *newfile = new TFileStream(NewName, (unsigned short)(exists ? fmOpenWrite : fmCreate));

   try
   {
      while (bytesread == bytestoread)
      {
         bytesread = orgfile->Read(buffer, bytestoread);
         newfile->Write(buffer, bytesread);
         Application->ProcessMessages();
      }
   }
   catch(Exception &e)
   {
      MsgDlg(this,
         s("The following exception was encountered while copying the file:\n") +
         s(e.Message),
         s("Backup Failed!"),
         MB_OK
      );
      ok = false;
   }

   delete newfile;
   delete orgfile;
   delete buffer;

   return ok;
}
//---------------------------------------------------------------------------

bool __fastcall TBackupCpyDlg::comparefiles(const AnsiString &File1, const AnsiString File2)
{
   bool ok = true;
   int bytestoread = 1024000;
   int bytesread2, bytesread1 = bytestoread;

   char *buffer1 = new char[bytestoread];
   char *buffer2 = new char[bytestoread];
   TFileStream *orgfile = new TFileStream(File1, fmOpenRead);
   TFileStream *newfile = new TFileStream(File2, fmOpenRead);

   try
   {
      while (bytesread1 == bytestoread)
      {
         bytesread1 = orgfile->Read(buffer1, bytestoread);
         bytesread2 = newfile->Read(buffer2, bytestoread);
         if (bytesread1 != bytesread2 || memcmp(buffer1, buffer2, bytesread1) != 0)
         {
            MsgDlg(this,
               s("Backup verification failed! Please try your backup again."),
               s("Verify Error"),
               MB_OK
            );
            ok = false;
            break;
         }
      }
   }
   catch(Exception &e)
   {
      MsgDlg(this,
         s("The following exception was encountered while comparing files:\n") +
         s(e.Message),
         s("Operation Failed!"),
         MB_OK
      );
      ok = false;
   }

   delete newfile;
   delete orgfile;
   delete buffer2;
   delete buffer1;

   return ok;
}
//---------------------------------------------------------------------------

/*bool __fastcall TBackupCpyDlg::comparefiles(const AnsiString &File1, const AnsiString File2)
{
   if (!FileExists(File1))
   {
      MsgDlg(this,
         "The data file " + File1 + " doesn't exist.",
         "Verify Error",
         MB_OK
      );
      return false;
   }

   if (!FileExists(File2))
   {
      MsgDlg(this,
         "The backup file " + File2 + " doesn't exist.",
         "Verify Error",
         MB_OK
      );
      return false;
   }

   DWORD dwSize, dwSize1, dwSize2, dwPageSize, dwNumPages;
   SYSTEM_INFO sysinfo;
   HANDLE hFile1, hFile2, hMap1, hMap2;
   LPSTR lpFile1, lpFile2;

   // get pagesize we'll use up to one extra
   GetSystemInfo(&sysinfo);
   dwPageSize = sysinfo.dwPageSize;

   // open file1 to compare
   if ((hFile1 = CreateFile(File1.c_str(),
      GENERIC_READ, 0, NULL, OPEN_EXISTING,
      FILE_ATTRIBUTE_NORMAL|FILE_FLAG_SEQUENTIAL_SCAN, NULL)) == INVALID_HANDLE_VALUE)
   {
      MsgDlg(this,
         "Failed to open data file " + File1 + " for verify.",
         "Verify Error",
         MB_OK
      );
      return false;
   }

   // open file2 to compare
   if ((hFile2 = CreateFile(File2.c_str(),
      GENERIC_READ, 0, NULL, OPEN_EXISTING,
      FILE_ATTRIBUTE_NORMAL|FILE_FLAG_SEQUENTIAL_SCAN, NULL)) == INVALID_HANDLE_VALUE)
   {
      MsgDlg(this,
         "Failed to open backup file " + File2 + " for verify.",
         "Verify Error",
         MB_OK
      );
      return false;
   }

   // if the data file is empty, impossible
   if ((dwSize1 = GetFileSize(hFile1, NULL)) == 0)
   {
      MsgDlg(this,
         "The data file " + File1 + " is empty!",
         "Verify Error",
         MB_OK
      );
      CloseHandle(hFile1);
      CloseHandle(hFile2);
      return false;
   }

   // if the backup file is empty verify fails
   if ((dwSize2 = GetFileSize(hFile2, NULL)) == 0)
   {
      MsgDlg(this,
         "The backup file " + File2 + " is empty!",
         "Verify Error",
         MB_OK
      );
      CloseHandle(hFile1);
      CloseHandle(hFile2);
      return false;
   }

   if (dwSize2 != dwSize2)
   {
      MsgDlg(this,
         "Backup file failed compare!",
         "Verify Error",
         MB_OK
      );
      CloseHandle(hFile1);
      CloseHandle(hFile2);
      return false;
   }

   // get pages required
   dwNumPages = (dwSize1 + dwPageSize - 1) / dwPageSize;
   dwSize = dwNumPages * dwPageSize;

   // create a file mapping object for  file1
   if ((hMap1 = CreateFileMapping(hFile1, NULL, PAGE_READONLY, 0, 0, NULL)) == 0)
   {
      MsgDlg(this,
         "Failed to create mapping device for " + File1,
         "Verify Error",
         MB_OK
      );
      CloseHandle(hFile1);
      CloseHandle(hFile2);
      return false;
   }

   // create a file mapping object for  file2
   if ((hMap2 = CreateFileMapping(hFile2, NULL, PAGE_READONLY, 0, 0, NULL)) == 0)
   {
      MsgDlg(this,
         "Failed to create mapping device for " + File2,
         "Verify Error",
         MB_OK
      );
      CloseHandle(hFile1);
      CloseHandle(hFile2);
      CloseHandle(hMap1);
      return false;
   }

   // map the file1 into view
   if ((lpFile1 = (char*)MapViewOfFile(hMap1, FILE_MAP_READ, 0, 0, 0)) == NULL)
   {
      MsgDlg(this,
         "Failed to map view of " + File1,
         "Verify Error",
         MB_OK
      );
      CloseHandle(hFile1);
      CloseHandle(hFile2);
      CloseHandle(hMap1);
      CloseHandle(hMap2);
      return false;
   }

   // map the file2 into view
   if ((lpFile2 = (char*)MapViewOfFile(hMap2, FILE_MAP_READ, 0, 0, 0)) == NULL)
   {
      MsgDlg(this,
         "Failed to map view of " + File2,
         "Verify Error",
         MB_OK
      );

      CloseHandle(hFile1);
      CloseHandle(hFile2);
      CloseHandle(hMap1);
      CloseHandle(hMap2);
      return false;
   }

   bool ok = true;

   for (unsigned count = 0; count < dwSize; count += 1024000)
   {
      int bytescheck = dwSize >= count + 1024000 ? 1024000 : dwSize - count;

      if (memcmp(lpFile1 + count, lpFile2 + count, bytescheck))
      {
         MsgDlg(this,
            "Backup file failed compare!",
            "Verify Error",
            MB_OK
         );
         ok = false;
         break;
      }
   }

   UnmapViewOfFile(lpFile1);
   UnmapViewOfFile(lpFile2);

   CloseHandle(hMap1);
   CloseHandle(hMap2);

   CloseHandle(hFile1);
   CloseHandle(hFile2);

   return ok;
}*/
//---------------------------------------------------------------------------

bool __fastcall TBackupCpyDlg::BackupDatabase()
{
   StartFlashMessage("...copying...");
   AnsiString bakfile=AddBS(path)+ExtractFileName(name),tmpfile=bak(bakfile,1)+".tmp";
   if (!copyfile(name.c_str(), tmpfile.c_str()))
   {
      StopFlashMessage();
      return false;
   }
   StopFlashMessage();

   if (verify)
   {
      StartFlashMessage("...verifying...");
      if (!comparefiles(name.c_str(), tmpfile.c_str()))
      {
         StopFlashMessage();
         return false;
      }
      StopFlashMessage();
   }

   // wipe file
   if (FileExists(bak(bakfile,copies)))
   {
      if (passes)
      {
         StartFlashMessage("...wiping...");
         zapfile(bak(bakfile, copies), passes);
         StopFlashMessage();
      }
      else
         DeleteFile(bak(bakfile, copies).c_str());
   }

   // rename each backup copy then
   for (int i = copies; i >= 1; i--)
      if (FileExists(bak(bakfile, i - 1)))
          rename(bak(bakfile, i - 1).c_str(), bak(bakfile, i).c_str());
   rename(tmpfile.c_str(), bak(bakfile, 1).c_str());

   return true;
}
//---------------------------------------------------------------------------

/*bool __fastcall TBackupCpyDlg::BackupDatabase()
{
   Snooze(200);
   FlashMessage("...copying...");
   AnsiString bakfile=AddBS(path)+ExtractFileName(name),tmpfile=bak(bakfile,1)+".tmp";
   if (!CopyFile(name.c_str(), tmpfile.c_str(), false))
   {
      BingBox(this,
         s("System Error: "+GetLastErrorStr())+
         s("File: "+bakfile),
         "Copy Failed",
         MB_OK
      );

      return false;
   }

   if (verify)
   {
      FlashMessage("...verifying...");
      TMemoryStream *Org = new TMemoryStream();
      TMemoryStream *Bak = new TMemoryStream();
      Bak->LoadFromFile(tmpfile);
      Org->LoadFromFile(name);
      bool flawed = Org->Size != Bak->Size || memcmp(Org->Memory, Bak->Memory, Org->Size) !=0;
      delete Org;
      delete Bak;

      if (flawed)
      {
         BingBox(this,
            s("Your database has been copied, but failed the verify test.\n")+
            s("I'll remove the file. Please try your backup again."),
            s("File Verify Error"),MB_OK);
         if (passes) wipefiles(tmpfile,passes);
         else DeleteFile(tmpfile.c_str());
         return false;
      }
   }

   // rename each backup copy then
   if (FileExists(bak(bakfile,copies)))
   {
      if (passes)
      {
         FlashMessage("...wiping...");
         zapfile(bak(bakfile,copies),passes);
      }
      else DeleteFile(bak(bakfile,copies).c_str());
   }
   for (int i=copies; i>=1; i--)
      if (FileExists(bak(bakfile,i-1)))
          rename(bak(bakfile,i-1).c_str(),bak(bakfile,i).c_str());
   rename(tmpfile.c_str(),bak(bakfile,1).c_str());

   Hide(); Snooze(300);
   return true;
}*/
//---------------------------------------------------------------------------

void __fastcall TBackupCpyDlg::FormShow(TObject *Sender)
{
   Timer1->Enabled = true;
}
//---------------------------------------------------------------------------
