/*
*  BackupDlgs.cpp
*  Copyright  1999-2005 by Richard Christman
*  ALL RIGHTS RESERVED
*
*  This source file is released for peer review purposes only. It is not public domain. 
*  Redistribution either in part or whole is forbidden. The sole source for QuickSilver 
*  distribution currently is http://quicksilvermail.net . You have permission to compile
*  this source for your own use.
*
*  Richard
*  --
*  R.Christman
*  quicksilver@quicksilvermail.net
*  http://quicksilvermail.net
*
*/
//---------------------------------------------------------------------------
#include <vcl\vcl.h>
#include <math.h>
#pragma hdrstop

#include "BackupDlgs.h"
#include "BackupDirDlgs.h"
#include "BackupCpyDlgs.h"
#include "FmxUtils.h"
#include "WipeDlgs.h"
#include "HelpDlgs.h"
#include "Fileman.h"
#include "Global.h"
//---------------------------------------------------------------------------
#pragma resource "*.dfm"
TBackupDlg *BackupDlg;
//---------------------------------------------------------------------------

bool __fastcall BackupQuickSilverData(TForm *Owner, bool SecureMode)
{
   BackupDlg = new TBackupDlg(Owner,SecureMode);
   BackupDlg->ShowModal();
   bool ok = BackupDlg->ModalResult==mrOk;
   delete BackupDlg;
   return ok;
}
//---------------------------------------------------------------------------

bool __fastcall BackupQuickSilverDataAuto(TForm *Owner)
{
   bool securemode = !FileExists(AppMainDat);
   BackupDlg = new TBackupDlg(Owner, securemode);

   if (securemode) BackupDlg->PgpDB->Checked = true;
   else BackupDlg->DbcDB->Checked = true;

   bool ok = BackupDlg->BackupDatabase();
   if (!ok)
   {
      BackupDlg->ShowModal();
      ok = BackupDlg->ModalResult == mrOk;
   }

   delete BackupDlg;
   return ok;
}

//---------------------------------------------------------------------------
// TBackupDlg
//---------------------------------------------------------------------------

__fastcall TBackupDlg::TBackupDlg(TComponent* Owner)
	: TForm(Owner)
{
}
//---------------------------------------------------------------------------

__fastcall TBackupDlg::TBackupDlg(TComponent* Owner, bool SecureMode)
	: TForm(Owner)
{
   securemode = SecureMode; CenterDialog(this,Owner);
   if (!FileExists(AppPgp)) PgpDB->Checked = PgpDB->Enabled = false;

   AnsiString db,bakinfo;
   bakinfo = GetAppIni("BackupPathName",ActiveDB()+"|"+ExtractFilePath(ActiveDB())+"|1|7|1|0");

   db                     = GetToken(0,bakinfo,"|");
   BackupPath->Text       = GetToken(1,bakinfo,"|");
   BackupCopies->Text     = GetToken(2,bakinfo,"|").ToIntDef(1);
   WipePasses->Text       = GetToken(3,bakinfo,"|").ToIntDef(7);
   VerifyBox->Checked     = GetToken(4, bakinfo, "|") != "0";
   AutoBackupBox->Checked = GetToken(5, bakinfo, "|") == "1";

   if (!DirectoryExists(BackupPath->Text)) BackupPath->Text = AppDir;
   PageControl1->ActivePage = TabSheet1;
}
//---------------------------------------------------------------------------

AnsiString __fastcall TBackupDlg::ActiveDB()
{
   return s(securemode?AppPgp:AppDat).LowerCase();
}
//---------------------------------------------------------------------------

AnsiString __fastcall TBackupDlg::ChosenDB()
{
   if (PgpDB->Checked) return AppPgp.LowerCase();
   else return AppDat.LowerCase();
}
//---------------------------------------------------------------------------

bool __fastcall TBackupDlg::BackupDatabase()
{
   AnsiString cap,bakfile;
   if (!DirectoryExists(BackupPath->Text))
   {
      BingBox(this,
         "The directory :  "+BackupPath->Text+"\ndoesn't exist!",
         "Directory Error",MB_OK);
      return false;
   }
 //if (ConfirmBox(this,"Backup database now?",MB_OKCANCEL)==IDCANCEL)
 //   return false;

   Snooze(100);
   bool ok = DoBackupDatabase(this,BackupPath->Text, ChosenDB(), BackupCopies->Text.ToIntDef(1),
      WipePasses->Text.ToIntDef(7), VerifyBox->Checked);

   PgpDB->Enabled = FileExists(AppPgp);
   Snooze(100);

   if (ok)
   {
      Hide(); Snooze(100);
      ((TForm*)Owner)->Perform(WM_SHOWSTATUS,0,(LPARAM)"Your database is saved and verified!");
      ModalResult = mrOk;
      MessageBeep(0);
   }
   return ok;
}
//---------------------------------------------------------------------------

bool __fastcall TBackupDlg::RestoreDatabase()
{
   int idx = ListBox1->ItemIndex;
   if (idx<0)
   {
      BingBox(this,
         "You have not selected a file to restore!",
         "Filename Error",MB_OK);
      return false;
   }

   AnsiString restfile = AddBS(DirectoryListBox1->Directory) + GetToken(0, ListBox1->Items->Strings[idx], "|").LowerCase();
   AnsiString restext = ExtractFileExt(restfile), justfile = ExtractFileName(restfile);

   if (justfile.Pos("qs-bak-") != 1 || (restext != ".dbc" && restext != ".pgp"))
   {
      BingBox(this,
         "The file you have selected: "+restfile+"\nis not a QuickSilver datafile",
         "Filename Error", MB_OK);
      return false;
   }

   if (!FileExists(restfile))
   {
      BingBox(this,
         "The file you have selected: " + restfile + "\ndoes not exist!",
         "Filename Error", MB_OK);
      return false;
   }

   if (ConfirmBox(this, "Restore database now?", MB_OKCANCEL) == IDCANCEL)
      return false;

   Snooze(100);
   bool ok = DoRestoreDatabase(this, AppDir + "QS" + restext, restfile);
   Snooze(100);

   if (ok)
   {
      Hide(); Snooze(100);
    ((TForm*)Owner)->Perform(WM_SHOWSTATUS, 0, (LPARAM)"Your database is restored!");
      ModalResult = mrOk;
      MessageBeep(0);
   }
   return ok;
}
//---------------------------------------------------------------------------

void __fastcall TBackupDlg::OkBtnClick(TObject *Sender)
{
   if (OkBtn->Caption=="Backup") BackupDatabase();
   else RestoreDatabase();
}
//---------------------------------------------------------------------------

void __fastcall TBackupDlg::FormCloseQuery(TObject *Sender, bool &CanClose)
{
   if (ModalResult==mrOk)
   {
      int copies = BackupCopies->Text.ToIntDef(1);
      if (AutoBackupBox->Checked && copies < 2)
      {
         BingBox(this,
            "You have auto-backup enabled. You need to specify at    \nleast 2 copies. More is better.",
            "Information", MB_OK);
         CanClose = false;
         return;
      }
      PutAppIni("BackupPathName", ChosenDB() +
         "|" + BackupPath->Text +
         "|" + BackupCopies->Text.ToIntDef(1) +
         "|" + WipePasses->Text.ToIntDef(7) +
         "|" + (VerifyBox->Checked ? "1" : "0") +
         "|" + (AutoBackupBox->Checked ? "1" : "0"));
   }
}
//---------------------------------------------------------------------------

void __fastcall TBackupDlg::Timer1Timer(TObject *Sender)
{
   if (Caption.IsEmpty()) Caption = (OkBtn->Caption=="Backup"?"Saving...":"Restoring...");
   else Caption = "";
   AppProcessMessages();
}
//---------------------------------------------------------------------------

void __fastcall TBackupDlg::CancelBtnClick(TObject *Sender)
{
   if (CancelBtn->Caption=="Close") ModalResult = mrOk;
   else ModalResult = mrCancel;
}
//---------------------------------------------------------------------------

void __fastcall TBackupDlg::BrowseBtnClick(TObject *Sender)
{
   AnsiString path=GetBackupDirectory(this,BackupPath->Text);
   if (!path.IsEmpty()) BackupPath->Text = path;
}
//---------------------------------------------------------------------------

void __fastcall TBackupDlg::PageControl1Change(TObject *Sender)
{
   if (PageControl1->ActivePage==TabSheet1) OkBtn->Caption = "Backup";
   else
   {
      OkBtn->Caption = "Restore";
      DirectoryListBox1->Directory = BackupPath->Text;
      UpdateFileList();
   }
}
//---------------------------------------------------------------------------

void __fastcall TBackupDlg::UpdateFileList()
{
   TSearchRec sr;
   AnsiString line;
   ListBox1->Clear();
   int done = FindFirst(DirectoryListBox1->Directory+"\\qs-bak-*.*",faAnyFile,sr);
   while (!done)
   {
      if (sr.Name != "." && sr.Name != "..")
      {
         line = sr.Name+"|"+sr.Size+"|"+sr.Time;
         ListBox1->Items->Add(line);
      }
      done = FindNext(sr);
   }
}
//---------------------------------------------------------------------------

void __fastcall TBackupDlg::PopupMenu1Popup(TObject *Sender)
{
   PopDeleteItem->Enabled = ListBox1->ItemIndex>=0;
}
//---------------------------------------------------------------------------

void __fastcall TBackupDlg::PopDeleteItemClick(TObject *Sender)
{
   int idx = ListBox1->ItemIndex;
   if (idx>=0)
   {
      if (ConfirmBox(this,
         s("You are deleting a backup of your QS database.\n")+
         s("Are you sure?"),MB_OKCANCEL)==IDCANCEL) return;

      int passes=WipePasses->Text.ToIntDef(7);
      AnsiString filename = DirectoryListBox1->Directory+"\\"+
         GetToken(0,ListBox1->Items->Strings[idx],"|");

      if (passes) wipefiles(filename,passes);
      else DeleteFile(filename.c_str());
      UpdateFileList();
   }
}
//---------------------------------------------------------------------------

void __fastcall TBackupDlg::FileListBox1DblClick(TObject *Sender)
{
   OkBtn->Click();
}
//---------------------------------------------------------------------------

void __fastcall TBackupDlg::HelpBtnClick(TObject *Sender)
{
   OpenHelp("Backups","Making And Restoring Backups",NULL,this);
}
//---------------------------------------------------------------------------

void __fastcall TBackupDlg::DirectoryListBox1Change(TObject *Sender)
{
   UpdateFileList();
}
//---------------------------------------------------------------------------

void __fastcall TBackupDlg::ListBox1DrawItem(TWinControl *Control, int Index,
	TRect &Rect, TOwnerDrawState State)
{
   TCanvas *canvas = ListBox1->Canvas;
   AnsiString line = ListBox1->Items->Strings[Index];
   AnsiString sizestr = GetToken(1,line,"|").ToIntDef(0)/1000;
   if (sizestr.Length()>3) sizestr.Insert(",",sizestr.Length()-2);
   sizestr += "K";
   TDateTime date = FileDateToDateTime(GetToken(2,line,"|").ToIntDef(0));
   int sizepos = Rect.Left+112-canvas->TextWidth(sizestr);
   canvas->FillRect(Rect);
   canvas->TextOut(Rect.Left+2,Rect.Top,GetToken(0,line,"|"));
   canvas->TextOut(sizepos,Rect.Top,sizestr);
   canvas->TextOut(Rect.Left+118,Rect.Top,FormatDateTime("mm/dd/yy hh:nn am/pm",date));
}
//---------------------------------------------------------------------------

void __fastcall TBackupDlg::Timer2Timer(TObject *Sender)
{
   Timer2->Enabled = false;
   if (BackupDatabase())
      Close();
}
//---------------------------------------------------------------------------
