/*
*  HashCashDlgs.cpp
*  Copyright  1999-2005 by Richard Christman
*  ALL RIGHTS RESERVED
*
*  This source file is released for peer review purposes only. It is not public domain. 
*  Redistribution either in part or whole is forbidden. The sole source for QuickSilver 
*  distribution currently is http://quicksilvermail.net . You have permission to compile
*  this source for your own use.
*
*  Richard
*  --
*  R.Christman
*  quicksilver@quicksilvermail.net
*  http://quicksilvermail.net
*
*/
//---------------------------------------------------------------------------
#include <vcl\vcl.h>
#pragma hdrstop

#include "Global.h"
#include "HashCashDlgs.h"
#include "HashCashM2NDlgs.h"
#include "fileman.h"
#include "wipedlgs.h"
#include "helpdlgs.h"
//---------------------------------------------------------------------------
#pragma resource "*.dfm"
THashCashDlg *HashCashDlg;
//---------------------------------------------------------------------------

bool __fastcall OpenHashCashManager(TForm *Owner)
{
   HashCashDlg = new THashCashDlg(Owner);
   HashCashDlg->ShowModal();
   bool ok = HashCashDlg->ModalResult ==  mrOk;
   delete HashCashDlg;
   return ok;
}
//---------------------------------------------------------------------------

__fastcall THashCashDlg::THashCashDlg(TComponent* Owner)
        : TForm(Owner)
{
   CenterDialog(this, Owner);
   TStringList *m2n = new TStringList();
   GetAppData(m2n, HashCashM2N);

   if (m2n->Count)
   {
      for (int i = 0; i < m2n->Count; i++)
      {
         AnsiString item = m2n->Strings[i];
         TListItem *listitem = ListView->Items->Add();
         listitem->Caption = GetToken(hcMail2news,    item, "\255");
         listitem->SubItems->Add(GetToken(hcResource, item, "\255"));
         listitem->SubItems->Add(GetToken(hcValidity, item, "\255"));
         listitem->SubItems->Add(GetToken(hcDBName,   item, "\255"));
      }
   }

   delete m2n;
   modified = false;
}
//---------------------------------------------------------------------------

void __fastcall THashCashDlg::Save()
{
   TStringList *m2n = new TStringList();

   for (int i = 0; i < ListView->Items->Count; i++)
   {
      TListItem *item = ListView->Items->Item[i];
      m2n->Add(
         item->Caption + "\255" +
         item->SubItems->Strings[0] + "\255" +
         item->SubItems->Strings[1] + "\255" +
         item->SubItems->Strings[2]
      );
   }

   PutAppData(HashCashM2N, m2n);

   delete m2n;
}
//---------------------------------------------------------------------------

void __fastcall THashCashDlg::FormCloseQuery(TObject *Sender, bool &CanClose)
{
   if (modified)
   {
      if (ModalResult == mrCancel)
      {
         switch(MsgBoxHwnd(Handle,
            "Your data is modified. Save your changes?",
            "Confirm", MB_YESNOCANCEL))
         {
            case IDCANCEL: CanClose = false;
            case IDNO: return;
         }
      }
      Save();
   }
}
//---------------------------------------------------------------------------

void __fastcall THashCashDlg::AddBtnClick(TObject *Sender)
{
   int validity;
   AnsiString gateway, resource, dbname;

   if (AddHashCashM2N(this, gateway, resource, validity, dbname))
   {
      TListItem *item = ListView->Items->Add();
      item->Caption = gateway;
      item->SubItems->Add(resource);
      item->SubItems->Add(validity);
      item->SubItems->Add(dbname);
      modified = true;
   }
}
//---------------------------------------------------------------------------

void __fastcall THashCashDlg::ModBtnClick(TObject *Sender)
{
   assert(ListView->Selected != NULL && ListView->Selected->SubItems->Count == 3);

   int validity;
   AnsiString gateway, dbname, resource = ListView->Selected->SubItems->Strings[0];
   validity = ListView->Selected->SubItems->Strings[1].ToIntDef(-1);
   dbname = ListView->Selected->SubItems->Strings[2];
   gateway = ListView->Selected->Caption;

   if (ModHashCashM2N(this, gateway, resource, validity, dbname))
   {
      ListView->Selected->SubItems->Strings[0] = resource;
      ListView->Selected->SubItems->Strings[1] = validity;
      ListView->Selected->SubItems->Strings[2] = dbname;
      modified = true;
   }
}
//---------------------------------------------------------------------------

void __fastcall THashCashDlg::DelBtnClick(TObject *Sender)
{
   assert(ListView->Selected != NULL);
   if (MsgBoxHwnd(Handle, "Delete the current mail2news gateway?   ",
      "Confirm", MB_OKCANCEL) == IDOK)
   {
      ListView->Selected->Delete();
      modified = true;
   }
}
//---------------------------------------------------------------------------

void __fastcall THashCashDlg::ListViewClick(TObject *Sender)
{
   ModBtn->Enabled = ListView->Selected;
   DelBtn->Enabled = ModBtn->Enabled;
}

//---------------------------------------------------------------------------
// various hashcash functions used for adding hashcash to message
//---------------------------------------------------------------------------

bool __fastcall AddHashCashToHeaders(TStrings *Message, const AnsiString &GateWayStr)
{
   AnsiString token, gate, resource, dbname;

   dbname = GetToken(hcDBName, GateWayStr, "\255");
   if (!FileExists(dbname))
   {
      MsgBoxHwnd(Application->MainForm->Handle,
         "The token file '" + dbname + "' doesn't exist!   "
         "I'm canceling the send.",
         "Information", MB_OK);
      return false;
   }

   int validity = GetToken(hcValidity, GateWayStr, "\255").ToIntDef(0);
   resource = GetToken(hcResource, GateWayStr, "\255");
   gate = GetToken(hcMail2news, GateWayStr, "\255");

   bool ok = false;
   TStringList *tokens = new TStringList();

   tokens->LoadFromFile(dbname);
   AnsiString tokensstr = tokens->CommaText;

   token = hashcashgettoken(tokens, GateWayStr, validity);
   if (tokens->CommaText != tokensstr)
   {
      zapfile(dbname, 10); // wipe file to eliminate used tokens
      tokens->SaveToFile(dbname);
   }

   if (!token.IsEmpty())
   {
      hashcashaddheader(Message, HashCashHdr + token);
      ok = true;
   }

   delete tokens;
   return ok;
}
//---------------------------------------------------------------------------

bool __fastcall hashcashrecipient(TStrings *recips, const AnsiString &gate)
{
   return recips->IndexOf(GetToken(0, gate, "\255")) >= 0;
}
//---------------------------------------------------------------------------

bool __fastcall hashcashexistingtoken(TStrings *tokens, const AnsiString &gate)
{
   bool ok = false;
   AnsiString resource = GetToken(hcResource, gate, "\255");

   for (int i = 0; i < tokens->Count; i++)
   {
      if (tokens->Strings[i].Pos(":" + resource + ":") > 0)
      {
         tokens->Delete(i);
         ok = true;
         break;
      }
   }

   return ok;
}
//---------------------------------------------------------------------------

void __fastcall hashcashgrabtokens(TStrings *message, TStrings *tokens)
{
   for (int i = 0; i < message->Count; i++)
   {
      AnsiString line = message->Strings[i].Trim();
      if (line.IsEmpty())
         break;

      if (line.Pos(HashCashHdr) == 1)
         tokens->Add(line);
   }
}
//---------------------------------------------------------------------------

void __fastcall hashcashaddheader(TStrings *message, const AnsiString &header)
{
   for (int i = 0; i < message->Count; i++)
   {
      if (message->Strings[i].Trim().IsEmpty())
      {
         message->Insert(i, header);
         break;
      }
   }
}
//---------------------------------------------------------------------------

TDateTime __fastcall hashcashtokendate(const AnsiString &token, int validperiod)
{
   unsigned short yr, mt, dy;
   AnsiString datestr = GetToken(2, token, ":");

   yr = (unsigned short)datestr.SubString(1, 2).ToIntDef(0);
   mt = (unsigned short)datestr.SubString(3, 2).ToIntDef(0);
   dy = (unsigned short)datestr.SubString(5, 2).ToIntDef(0);
   yr +=(unsigned short)2000;

   TDateTime date(yr, mt, dy);
   date += validperiod;

   return date;
}
//---------------------------------------------------------------------------

TDateTime __fastcall getcurrentGMTdate()
{
   struct tm *gmt;
   struct _TIME_ZONE_INFORMATION tzinfo;
   time_t t = time(NULL);

   GetTimeZoneInformation(&tzinfo);
   t += tzinfo.Bias * 60; // converts to gmt
   gmt = localtime(&t);   // no need for gmtime()

   TDateTime date(
      (unsigned short)(gmt->tm_year + 1900),
      (unsigned short)(gmt->tm_mon + 1),
      (unsigned short)(gmt->tm_mday)
   );

   return date;
}
//---------------------------------------------------------------------------

AnsiString __fastcall hashcashgettoken(TStrings *alltokens, const AnsiString &gateway,
   int validity)
{
   AnsiString token, mail2news, resource;
   mail2news = GetToken(0, gateway, "\255");
   resource = GetToken(1, gateway, "\255");

   TStringList *tokens = new TStringList();

   // get the tokens for this resource only
   for (int i = 0; i < alltokens->Count; i++)
   {
      token = alltokens->Strings[i];
      if (token.Pos(":" + resource + ":") > 0)
         tokens->Add(token);
   }

   // get rid of expired and version 0 tokens
   int i = 0; while (i < tokens->Count)
   {
      token = tokens->Strings[i];
      if (GetToken(0, token, ":") == "0")
      {
         MsgBoxHwnd(Application->MainForm->Handle,
            "I have encountered a version 0 token: " + token + "   \n"
            "The gateways no longer support this version. I'm going to delete it. You need to update   \n"
            "your hashcash program in order to create version 1 tokens.",
            "Information", MB_OK);
         alltokens->Delete(alltokens->IndexOf(token));
         tokens->Delete(i);
         continue;
      }

    //sm(AnsiString(hashcashtokendate(token, validity))+"\n"+AnsiString(getcurrentGMTdate()));
      if (hashcashtokendate(token, validity) < getcurrentGMTdate())
      {
         MsgBoxHwnd(Application->MainForm->Handle,
            "I have encountered an expired token: " + token + "    \n"
            "I'm going to delete it",
            "Information", MB_OK);
         alltokens->Delete(alltokens->IndexOf(token));
         tokens->Delete(i);
         continue;
      }

      i++;
   }

   // report no more tokens
   if (tokens->Count == 0)
   {
      MsgBoxHwnd(Application->MainForm->Handle,
         "You need a hashcash token for: " + mail2news + " and there    \n"
         "are none. I'm canceling the send.",
         "HashCash Error", MB_OK);
      delete tokens;
      return "";
   }

   token = tokens->Strings[0];
   alltokens->Delete(alltokens->IndexOf(token));
   tokens->Delete(0);

   // report minimum tokens available
   if (tokens->Count < 3)
   {
      MsgBoxHwnd(Application->MainForm->Handle,
         "You have only " + IntToStr(tokens->Count) + " token(s) for " + mail2news + "   \n"
         "remaining after this one.",
         "Information", MB_OK
      );
   }

   delete tokens;
   return token;
}
//---------------------------------------------------------------------------

void __fastcall THashCashDlg::HelpBtnClick(TObject *Sender)
{
   OpenHelp("Hashcash","How do I used HashCash for mail2news gateways?","");
}
//---------------------------------------------------------------------------
