/*
*  PgpKeyImpDlgs.cpp
*  Copyright  1999-2005 by Richard Christman
*  ALL RIGHTS RESERVED
*
*  This source file is released for peer review purposes only. It is not public domain. 
*  Redistribution either in part or whole is forbidden. The sole source for QuickSilver 
*  distribution currently is http://quicksilvermail.net . You have permission to compile
*  this source for your own use.
*
*  Richard
*  --
*  R.Christman
*  quicksilver@quicksilvermail.net
*  http://quicksilvermail.net
*
*/
//---------------------------------------------------------------------------
#include <vcl\vcl.h>
#pragma hdrstop
#include <clipbrd.hpp>

#include "PgpKeyImpDlgs.h"
#include "PgpFiles.h"
#include "view.h"
//---------------------------------------------------------------------------
#pragma resource "*.dfm"
TPgpKeyImpDlg *PgpKeyImpDlg;
const AnsiString PgpNoKeysFoundAtAll = "...NO KEYS FOUND AT ALL!";
const AnsiString PgpLoading = "Loading...";
//---------------------------------------------------------------------------
bool __fastcall ImportPGPKeys(TForm *Owner, TPGPbuffer *ImpKeys, AnsiString &ImportDir)
{
   PgpKeyImpDlg = new TPgpKeyImpDlg(Owner,ImpKeys,ImportDir);
   PgpKeyImpDlg->ShowModal();
   bool ok = PgpKeyImpDlg->ModalResult==mrOk && ImpKeys->count;
   delete PgpKeyImpDlg;
   ImpKeys->rewind();
   return ok;
}
//---------------------------------------------------------------------------
__fastcall TPgpKeyImpDlg::TPgpKeyImpDlg(TComponent* Owner)
  : TForm(Owner)
{
}
//---------------------------------------------------------------------------
__fastcall TPgpKeyImpDlg::TPgpKeyImpDlg(TComponent* Owner, TPGPbuffer *ImpKeys,
   AnsiString &ImportDir) : TForm(Owner)
{
   CenterDialog(this);
   imports = ImpKeys; imports->clear();
   keys = new TPGPbuffer();
   secrets = new TPGPbuffer();
   publics = new TPGPbuffer();

   impdir = &ImportDir;
   OpenDialog->InitialDir = *impdir;
   OpenDialog->Filter = "PGP import files (*.asc)|*.asc|All files (*.*)|*.*";
   OpenDialog->DefaultExt = ".asc";
   OpenDialog->FileName = "";//"*.asc";
   ComboBox1->Items->Text = "Import Personal PGP Keys";
   ComboBox1->ItemIndex = 0;
   cap = Caption;
}
//---------------------------------------------------------------------------
__fastcall TPgpKeyImpDlg::~TPgpKeyImpDlg()
{
   delete publics;
   delete secrets;
   delete keys;
}
//---------------------------------------------------------------------------
TPGPbuffer* __fastcall TPgpKeyImpDlg::IsPublic(TPGPbuffer *key)
{
   TPGPbuffer *pub=NULL;
   AnsiString id8,keyid8=key->idtext8;
   for (int i=0; i<ListBox1->Items->Count; i++)
   {
      id8 = ListBox1->Items->Strings[i].SubString(3,8);
      if (id8==keyid8)
      {
         pub=dynamic_cast<TPGPbuffer*>(ListBox1->Items->Objects[i]);
         break;
      }
   }
   return pub;
}
//---------------------------------------------------------------------------
void __fastcall TPgpKeyImpDlg::ImportKeys(AnsiString Name, TStrings *AsciiBlock)
{
   AnsiString cap=Caption;
   Caption = "";
   keys->clear();
   ListBox1->Clear();
   bool gotkeys=false;

   ImportBtn->Enabled = false;
   AnsiString line,orglines = AsciiBlock->Text;

   SetRedraw(Panel1,0);
   keys->import(AsciiBlock);
   keys->rewind();

   int newcnt = keys->count;
   if (newcnt)
   {
      TPGPbuffer *key = new TPGPbuffer();

      keys->rewind();
      while (key->copykeyfrom(keys, REWIND))
      {
         if (PublicKeyPacket(key->CTB))
         {
            line = "0x" + key->idtext8 + "  " + key->firstuid;
            if (ListBox1->Items->IndexOf(line) < 0)
                ListBox1->Items->Add(line);
            publics->copyfrom(key);
         }
         key->clear();
      }

      keys->rewind();
      while (key->copykeyfrom(keys, REWIND))
      {
         if (SecretKeyPacket(key->CTB))
         {
            line = "0x" + key->idtext8 + "  " + key->firstuid;
            if (ListBox1->Items->IndexOf(line) >= 0)
               secrets->copyfrom(key);
         }

         key->clear();
      }

      // select the entire list
      for (int i=0; i<ListBox1->Items->Count; i++)
         ListBox1->Selected[i] = true;

      ListBox1->TopIndex = 0;
      ImportBtn->Enabled = true;
      gotkeys = true;
      delete key;
   }
   else
   {
      ListBox1->Items->Add(Name+PgpNoKeysFoundAtAll);
      ListBox1->Items->Add("Done");
   }
   while (flash<4) AppProcessMessages();
   Timer1->Enabled = false;

   AppProcessMessages();
   SetRedraw(Panel1,1);
   Caption = cap;
   if (gotkeys)
   {
      int count = ListBox1->Items->Count;
      Caption = cap+" ..."+s(count)+" key"+(count!=1?"s":"");
   }
   MyBeep(MB_OK);
}
//---------------------------------------------------------------------------
void __fastcall TPgpKeyImpDlg::ClipBtnClick(TObject *Sender)
{
   if (MsgBoxHwnd(Handle,"Import keys from the clipboard?","Confirm",MB_OKCANCEL)==IDOK)
   {
      ListBox1->Clear();
      ListBox1->BringToFront();
      RichEdit1->Clear();
      TStringList *clip = new TStringList();
      clip->Text = Clipboard()->AsText; //PasteFromClipboard(clip);

      flash=0; Timer1->Enabled = true;
      ImportKeys("Clipboard",clip);
      Timer1->Enabled = false;
      ActiveControl = Panel1;

      if (ListBox1->Items->Count &&
          ListBox1->Items->Strings[0].Pos(PgpNoKeysFoundAtAll)>0)
      {
         RichEdit1->Lines->Add("Clipboard"+PgpNoKeysFoundAtAll);
         RichEdit1->BringToFront();
         RichEdit1->Lines->Add("");
         RichEdit1->Lines->Add("Clipboard Contents-------------------------------------");
         RichEdit1->Lines->AddStrings(clip);
      }
      delete clip;
   }
}
//---------------------------------------------------------------------------
void __fastcall TPgpKeyImpDlg::FileBtnClick(TObject *Sender)
{
   if (OpenDialog->Execute())
   {
      ListBox1->Clear();
      RichEdit1->Clear();
      ListBox1->BringToFront();

      flash=0; Timer1->Enabled = true;
      TStringList *keys = new TStringList();
      keys->LoadFromFile(OpenDialog->FileName);
      ImportKeys(ExtractFileName(OpenDialog->FileName),keys);
      Timer1->Enabled = false;

      if (ListBox1->Items->Count &&
          ListBox1->Items->Strings[0].Pos(PgpNoKeysFoundAtAll)>0)
      {
         RichEdit1->Lines->Add(OpenDialog->FileName+PgpNoKeysFoundAtAll);
         RichEdit1->BringToFront();
         RichEdit1->Lines->Add("");
         RichEdit1->Lines->Add(OpenDialog->FileName+" Contents-----------------------------------------------");
         RichEdit1->Lines->AddStrings(keys);
      }

     *impdir = ExtractFilePath(OpenDialog->FileName);
      OpenDialog->InitialDir = *impdir;
      ActiveControl = Panel1;
      delete keys;
   }
}
//---------------------------------------------------------------------------
void __fastcall TPgpKeyImpDlg::Timer1Timer(TObject *Sender)
{
   if (Caption.IsEmpty()) Caption = PgpLoading;
   else Caption = "";
   flash++;
}
//---------------------------------------------------------------------------
void __fastcall TPgpKeyImpDlg::AddTrust(TPGPbuffer *key, bool ispair)
{
   TPGPbuffer *pkt=new TPGPbuffer(),*newkey=new TPGPbuffer();
   bool addingtrust=false;
   key->rewind();
   while (pkt->copypacketfrom(key,REWIND))
   {
      newkey->copyfrom(pkt);
      if (PublicKeyPacket(pkt->CTB)) addingtrust = true; else
      if (SecretKeyPacket(pkt->CTB)) addingtrust = false;
      if (addingtrust)
      {
         if (PublicKeyPacket(pkt->CTB))
         {
            byte trust[3] = { 176, 1, ispair ? (byte)135 : (byte)0 };
            newkey->Write(trust,3);
         }
         if (UserIdPacket(pkt->CTB))
         {
            byte trust[3] = { 176, 1, ispair ? (byte)3 : (byte)0 };
            newkey->Write(trust,3);
         }
         if (SignaturePacket(pkt->CTB))
         {
            byte trust[3] = { 176, 1, ispair ? (byte)103 : (byte)0 };
            newkey->Write(trust,3);
         }
      }
      pkt->clear();
   }
   key->clear();
   key->copyfrom(newkey);
   delete newkey;
   delete pkt;
}
//---------------------------------------------------------------------------
void __fastcall TPgpKeyImpDlg::ImportBtnClick(TObject *Sender)
{
   AnsiString idtext8,line;
   if (MsgBoxHwnd(Handle, "Import selected keys now?", "Confirm", MB_OKCANCEL) == IDOK)
   {
      imports->clear();
      TPGPbuffer *key = new TPGPbuffer();
      for (int i = 0; i < ListBox1->Items->Count; i++)
      {
         if (ListBox1->Selected[i])
         {
            line = ListBox1->Items->Strings[i];
            int p = line.Pos("0x");
            idtext8 = line.SubString(p+2,8);

            if (publics->get(idtext8,key))
            {
               AddTrust(key, secrets->keyexists(key));
               imports->copyfrom(key);

               if (secrets->get(idtext8,key))
                  imports->copyfrom(key);
            }
         }
         key->clear();
      }
      delete key;
      imports->rewind();
      ModalResult = mrOk;
   }
}
/*void __fastcall TPgpKeyImptDlg::ImportBtnClick(TObject *Sender)
{
   if (MsgBoxHwnd(Handle,"Import selected keys now?","Confirm",MB_OKCANCEL)==IDOK)
   {
      imports->clear();
      for (int i=0; i<ListBox1->Items->Count; i++)
      {
         if (ListBox1->Selected[i])
         {  // public keys are in individual buffers in items array
            // some only public, some with trailing privates
            TPGPbuffer *key=(TPGPbuffer*)ListBox1->Items->Objects[i];
            TPGPbuffer *pkt=new TPGPbuffer();
            bool ispair=key->ispair,addingtrust=false;
            key->rewind();
            while (pkt->copypacketfrom(key,REWIND))
            {
               if (PublicKeyPacket(pkt->CTB)) addingtrust = true; else
               if (SecretKeyPacket(pkt->CTB)) addingtrust = false;
               if (addingtrust)
               {
                  if (PublicKeyPacket(pkt->CTB))
                  {
                     byte trust[3] = { 176, 1, ispair ? (byte)135 : (byte)0 };
                     imports->Write(trust,3);
                  }
                  else
                  {
                     byte trust[3] = { 176, 1, 0 };
                     imports->Write(trust,3);
                  }
               }
               imports->copyfrom(pkt);
               pkt->clear();
            }
         }
      }
      ModalResult = mrOk;
      imports->rewind();
   }
}*/
//---------------------------------------------------------------------------
