/*
*  Mixmaster.cpp
*  Copyright  1999-2006 by Richard Christman
*  ALL RIGHTS RESERVED
*
*  This source file is released for peer review purposes only. It is not public domain. 
*  Redistribution either in part or whole is forbidden. The sole source for QuickSilver 
*  distribution currently is http://quicksilvermail.net . You have permission to compile
*  this source for your own use.
*
*  Richard
*  --
*  R.Christman
*  quicksilver@quicksilvermail.net
*  http://quicksilvermail.net
*
*/
#define _WINSOCKAPI_
//---------------------------------------------------------------------------
#include <vcl\vcl.h>
#include <vcl\registry.hpp>
#pragma hdrstop

#include <assert.h>
#include "Mixmaster.h"
#include "global.h"   // for AppDir
#include "wipedlgs.h"
#include "fileman.h"
#include "headman.h"
#include "pgp.h"

AnsiString mretbuf;
HINSTANCE hMixLib=NULL;
HINSTANCE hSSLLib=NULL;
//---------------------------------------------------------------------------
AnsiString __fastcall MixPublicRing()
{
   AnsiString name=MixDir()+"pubring.mix";
   if (!FileExists(name))
   {
      MsgBox("Unable to locate mixmaster public keyring!","Mixmaster Error",MB_OK);
      return "";
   }
   return name;
}
//---------------------------------------------------------------------------
AnsiString __fastcall MixProblemRing()
{
   AnsiString name=MixDir()+"pubring.mix";
   if (!FileExists(name))
   {
      MsgBox("Unable to locate mixmaster public keyring!","Mixmaster Error",MB_OK);
      return "";
   }
   return name;
}
//---------------------------------------------------------------------------
int __fastcall data_encrypt(char **data, long &datlen, char *password, int paslen)
{
   int result;
   BUFFER *dat = buf_new(), *pas = buf_new();
   dat->sensitive = true; pas->sensitive = true;
   buf_append(pas,(byte*)password,paslen);
   buf_append(dat,(byte*)*data,datlen);
   if ((result = pgp_encrypt(PGP_CONVENTIONAL|PGP_NOARMOR,dat,pas,NULL,NULL,NULL,NULL))==0)
   {
      delete *data; *data = new char[dat->length+1];
      // copy uncompressed data to data variable
      memcpy(*data,dat->data,dat->length+1);
      // set new length
      datlen = dat->length;
   }
   buf_free(pas); buf_free(dat);
   return result;
}
//---------------------------------------------------------------------------
int __fastcall data_decrypt(char **data, long &datlen, char *password, int paslen)
{
   int result;
   BUFFER *dat = buf_new(), *pas = buf_new();
   dat->sensitive = true; pas->sensitive = true;
   buf_append(pas,(byte*)password,paslen);
   buf_append(dat,(byte*)*data,datlen);
   if ((result = pgp_decrypt(dat,pas,NULL,NULL,NULL))==0)
   {
      delete *data; *data = new char[dat->length+1];
      // copy uncompressed data to data variable
      memcpy(*data,dat->data,dat->length+1);
      // set new length
      datlen = dat->length;
   }
   buf_free(pas); buf_free(dat);
   return result;
}
//---------------------------------------------------------------------------
bool __fastcall MixGetShortNames(TStrings *names)
{
   names->Clear();
   AnsiString line;
   TStringList *lines = new TStringList();
   if (FileExists(mtyp2txt)) lines->LoadFromFile(mtyp2txt);
   for (int i=3; i<lines->Count; i++)
      if (!(line=lines->Strings[i]).IsEmpty()) names->Add(GetToken(0,line," "));
   delete lines;
   return names->Count;
}
//---------------------------------------------------------------------------
/*AnsiString __fastcall MixGetShortNamesCommaText()
{
   TStringList *names = new TStringList();
   MixGetShortNames(names);
   AnsiString commatext;
   for (int i=0; i<names->Count; i++)
      commatext += names->Strings[i]+(i<names->Count-1?",":"");
   delete names;
   return commatext;
}*/
//---------------------------------------------------------------------------
bool __fastcall MixKeyDescOK(AnsiString &line)
{
   int p1=line.Pos(" "),p2=line.Pos("@"),p3=line.Pos(".");
   return p1>0 && p2>0 && p3>0 && p2>p1 && p3>p2;
}
//---------------------------------------------------------------------------
bool __fastcall MixKeyDefinately(TStrings* keys)
{
   if (keys->Count<15) return false;
   return MixKeyDescOK(keys->Strings[0]) &&                   // descrip ok
      keys->Strings[2]==BeginMixKey &&                        // begin key ok
      GetParam(2,keys->Strings[0]," ")==keys->Strings[3] &&   // keyid ok
      keys->Strings[4]=="258" &&                              // size ok
      keys->Strings[14]==EndMixKey;                           // end key ok
}
//---------------------------------------------------------------------------
bool __fastcall MixGetNextKey(AnsiString &keydesc, TStrings *key, TStrings *keys)
{
   key->Clear();
   while (keys->Count && !MixKeyDefinately(keys)) keys->Delete(0);
   if (!keys->Count) return false;

   int pos; AnsiString line=keys->Strings[0]; keys->Delete(0); keys->Delete(0);
   while ((pos=line.Pos("  "))>0) line.Delete(pos,1);

   keydesc = GetToken(0,line," ")+","+GetToken(1,line," ")+","+GetToken(2,line," ")+","+
      GetToken(3,line," ")+","+GetToken(4,line," ");

   key->Add(line);
   key->Add("");
   for (int i=0; i<13; i++)
   {
      key->Add(keys->Strings[0]);
      keys->Delete(0);
   }
   return true;
}
//---------------------------------------------------------------------------
bool __fastcall MixRingLineOk(AnsiString &line)
{
   int fld=0;
   bool ok=true;
   AnsiString data;
   while (ok && !(data=GetParam(fld,line," ")).IsEmpty())
   {
      if (fld==1 && data.Pos("@")<1) ok = false;
      if (fld==3 && data.Pos(".")<1) ok = false;
      fld++;
   }
   if (fld!=4 && fld!=5) ok = false;
   return ok;
}
//---------------------------------------------------------------------------
bool __fastcall MixListLineOk(AnsiString &line)
{  //0         1         2         3         4
   //012345678901234567890123456789012345678901234
   //"austria        ##********#*    11:02 100.00%"
   int l=line.Length(); if (l<44) return false;
   return line[l-10]==':' && line[l-3]=='.' && line[l]=='%';
}
//---------------------------------------------------------------------------
bool __fastcall InType2(AnsiString &remailer, TStrings *list)
{
   bool ok = false; if (!list) return ok;

   for (int i=0; i<list->Count; i++)
   {
      if (list->Strings[i]==remailer || list->Strings[i]=="#"+remailer)
      {
         ok = true;
         break;
      }
   }
   return ok;
}
//---------------------------------------------------------------------------
bool __fastcall MixGenType2()
{
   AnsiString md=MixDir(),oldtype2;
   TStringList *type2 = new TStringList();
   if (FileExists(md+MTyp2Txt)) type2->LoadFromFile(md+MTyp2Txt);
   TStringList *mring = new TStringList();
   if (FileExists(md+MRingTxt)) mring->LoadFromFile(md+MRingTxt);
   oldtype2 = type2->Text;
   MixGenType2(type2,mring,save);
   delete mring;
   delete type2;
   return true;
}
//---------------------------------------------------------------------------
bool __fastcall MixGenType2(TStrings *type2, TStrings *mring, bool SaveToDisk)
{
   AnsiString shrt,line;
   if (!type2 || !mring) return false;
   TStringList *ring = new TStringList();
   // determine if mring is mixkeylist or raw doc
   if (mring->Count && dynamic_cast<TStrings*>(mring->Objects[0])!=NULL) // is mixkeylist
      ring->AddStrings(mring);
   else
   {
      for (int i=0; i<mring->Count; i++)
      {
         line = mring->Strings[i];
         if (MixRingLineOk(line))
         {
            //int pos = line.Pos(" ");
            //line.Insert(s("           ").SubString(1,11-pos),pos);
            ring->Add(line);
         }
      }
   }
   // store disabled lines--delete all
   int i=0; while (i<type2->Count)
   {
      if (type2->Strings[i].Pos("#")!=1) type2->Delete(i);
      else i++;
   }
   // add keys from pubring.mix
   for (i=0; i<ring->Count; i++)
   {
      line = ring->Strings[i];
      if (!InType2(line,type2)) type2->Add(line);
   }
   // finish up
   if (SaveToDisk && type2->Count)
   {
      MixUnSpaceType2(type2);
      SaveToFileBak(type2,mtyp2txt,STATSBAKCOPIES);
   }
   delete ring;
   return true;
}
//---------------------------------------------------------------------------
void __fastcall MixSpaceType2(TStrings *type2)
{
   int i,pos,max=0;
   AnsiString t2line,spc;
   MixUnSpaceType2(type2);
   // find longest name
   for (i=0; i<type2->Count; i++)
      if ((pos=type2->Strings[i].Pos(" "))>max) max = pos;
   // insert space
   for (i=0; i<type2->Count; i++)
   {
      t2line = type2->Strings[i];
      pos = t2line.Pos(" ");
      t2line.Insert(spc.StringOfChar(' ',max-pos),pos);
      type2->Strings[i] = t2line;
   }
}
//---------------------------------------------------------------------------
void __fastcall MixUnSpaceType2(TStrings *type2)
{
   AnsiString t2line; int pos;
   for (int i=0; i<type2->Count; i++)
   {
      t2line = type2->Strings[i];
      while ((pos=t2line.Pos("  "))>0) t2line.Delete(pos,1);
      type2->Strings[i] = t2line;
   }
}
//---------------------------------------------------------------------------
void __fastcall MixGetType2(TStrings *type2)
{
   type2->Clear();
   AnsiString t2line; int pos1,pos2;
   if (FileExists(mtyp2txt)) type2->LoadFromFile(mtyp2txt);
   for (int i=0; i<type2->Count; i++)
   {
      t2line=type2->Strings[i].Trim();
      if ((pos1=t2line.Pos(" "))>0)
      {
         pos2=pos1;
         while (pos2<t2line.Length() && t2line[pos2]==' ') pos2++;
         t2line = t2line.SubString(1,pos1)+t2line.SubString(pos2,128);
         type2->Strings[i] = t2line;
      }
   }
}
//---------------------------------------------------------------------------
// Base mixmaster routines
//---------------------------------------------------------------------------
bool _fastcall HasMixmaster()
{
   return hMixLib>0 && hSSLLib>0;
}
//---------------------------------------------------------------------------
bool __fastcall HasMix()
{
   return HasMixmaster();
}
//---------------------------------------------------------------------------
AnsiString __fastcall MixDir()
{
   AnsiString mixdir = "";
   TRegistry *reg = new TRegistry();
   if (reg->OpenKey(MixKey,false)) mixdir = reg->ReadString("MixDir");
   delete reg;
   if ( mixdir.IsEmpty()) mixdir = getenv("MIXDIR");
   if (!mixdir.IsEmpty() && mixdir[mixdir.Length()]!='\\') mixdir += "\\";
   return mixdir.LowerCase();
}
//---------------------------------------------------------------------------
int buf_readfile(BUFFER *buffer, char *filename)
{
   int err = -1;
   buf_reset(buffer);
   TFileStream *f;
   if (!FileExists(filename))
   {
      MsgBoxHwnd(NULL,
         s("The file "+s(filename)+" does not exist!\n")+
         s("Unable to complete operation."),
        "QS Pgp Error",
         MB_OK
      );
      return err;
   }
   try
   {
      f = new TFileStream(filename,fmOpenRead);
   }
   catch (...)
   {
      MsgBoxHwnd(NULL,
         s("I was unable to open "+s(filename)+" !\n")+
         s("Please make sure it is not in use by another program, and try\n")+
         s("the operation again."),
        "QS Pgp Error",
         MB_OK
      );
      f = NULL;
   }
   if (f)
   {
      char buf[BUFSIZE] = { 0 };
      while (true)
      {
         if (!f->Size) { err = 0; break; }

         int n = f->Read(buf,BUFSIZE); if (n > 0) err = 0;
         buf_append(buffer, (byte*)buf, n);
         if (n < BUFSIZE) break;

         if (buffer->length > BUFFER_MAX)
         {
            MsgBox("Message file too large. Giving up.","Error",MB_OK);
            return(1);
         }
      }
      delete f;
   }
   return err;
}
//---------------------------------------------------------------------------
int buf_writefile(BUFFER *buffer, char *filename, char* mode) // mode = "wb" or "wt"
{
   int err = -2;
   FILE *f = fopen(filename,mode);
   if (f) { err = -1;
      int n = (int)fwrite(buffer->data, 1, buffer->length, f)==buffer->length ? 0:-1;
      if (n>0) err=0;
      fclose(f);
   }
   return err;
}
//---------------------------------------------------------------------------
void __fastcall RAND_bytes(unsigned char *buf, int num)
{
   RAND_BYTES randbytes;
   if ((randbytes = (RAND_BYTES)GetProcAddress(hSSLLib,"RAND_bytes"))!=NULL)
      randbytes(buf,num);
}
//---------------------------------------------------------------------------
int __fastcall mix_init(char *mixdir)
{
   int ret; MIX_INIT mixinit;
   if ((mixinit = (MIX_INIT)GetProcAddress(hMixLib,"mix_init"))!=NULL)
      ret = (mixinit)(mixdir);
   return ret;
}
//---------------------------------------------------------------------------
void __fastcall mix_exit()
{
   MIX_EXIT mixexit;
   if ((mixexit = (MIX_EXIT)GetProcAddress(hMixLib,"mix_exit"))!=NULL)
      (mixexit)();
}
//---------------------------------------------------------------------------
void __fastcall MixShutDown()
{
   mix_exit();
}
//---------------------------------------------------------------------------
BUFFER* __fastcall buf_new(void)
{
   BUFFER* ret = NULL; BUF_NEW bufnew;
   if ((bufnew = (BUF_NEW)GetProcAddress(hMixLib,"buf_new"))!=NULL)
      ret = (bufnew)();
   return ret;
}
//---------------------------------------------------------------------------
int __fastcall buf_free(BUFFER *buf)
{
   int ret = NULL; BUF_FREE buffree;
   if ((buffree = (BUF_FREE)GetProcAddress(hMixLib,"buf_free"))!=NULL)
      ret = (buffree)(buf);
   return ret;
}
//---------------------------------------------------------------------------
int __fastcall buf_reset(BUFFER *buf)
{
   int ret = NULL; BUF_RESET bufreset;
   if ((bufreset = (BUF_RESET)GetProcAddress(hMixLib,"buf_reset"))!=NULL)
      ret = (bufreset)(buf);
   return ret;
}
//---------------------------------------------------------------------------
int __fastcall buf_append(BUFFER *buffer, byte *mess, int len)
{
   int ret = NULL; BUF_APPEND bufappend;
   if ((bufappend = (BUF_APPEND)GetProcAddress(hMixLib,"buf_append"))!=NULL)
      ret = (bufappend)(buffer,mess,len);
   return ret;
}
//---------------------------------------------------------------------------
int __fastcall buf_sets(BUFFER *buffer, char *s)
{
   buf_clear(buffer);
   return (buf_appends(buffer, s));
}
//---------------------------------------------------------------------------
void __fastcall buf_move(BUFFER *dest, BUFFER *src)
{
   BUF_MOVE bufmove;
   if ((bufmove = (BUF_MOVE)GetProcAddress(hMixLib,"buf_move"))!=NULL)
       (bufmove)(dest,src);
}
//---------------------------------------------------------------------------
int __fastcall mix_encrypt(int type, BUFFER *message, char *chain, int numcopies,
   BUFFER *feedback)
{
   int ret=NULL; MIX_ENCRYPT mixencrypt;
   if ((mixencrypt = (MIX_ENCRYPT)GetProcAddress(hMixLib,"mix_encrypt"))!=NULL)
      ret = (mixencrypt)(type,message,chain,numcopies,feedback);
   return ret;
}
//---------------------------------------------------------------------------
int __fastcall mix_send()
{
   int ret = NULL; MIX_SEND mixsend;
   if ((mixsend = (MIX_SEND)GetProcAddress(hMixLib,"mix_send"))!=NULL)
      ret = (mixsend)();
   return ret;
}
//---------------------------------------------------------------------------
int __fastcall pgp_mailenc(int mode, BUFFER *message, char *sigid,
   BUFFER *pass, char *pubring, char *secring)
{
   int ret = NULL; PGP_MAILENC_BUF pgpmailenc;
   if ((pgpmailenc = (PGP_MAILENC_BUF)GetProcAddress(hMixLib,"pgp_mailenc"))!=NULL)
      ret = (pgpmailenc)(mode,message,sigid,pass,pubring,secring);
   return ret;
}
//---------------------------------------------------------------------------
int __fastcall pgp_encrypt(int mode, BUFFER *message, BUFFER *encr, BUFFER *sigid,
		BUFFER *pass, char *pubring, char *secring)
{
   int ret = NULL; PGP_ENCRYPT_BUF pgpencrypt;
   if ((pgpencrypt = (PGP_ENCRYPT_BUF)GetProcAddress(hMixLib,"pgp_encrypt"))!=NULL)
      ret = (pgpencrypt)(mode,message,encr,sigid,pass,pubring,secring);
   return ret;
}
//---------------------------------------------------------------------------
int __fastcall pgp_decrypt(BUFFER *message, BUFFER *pass, BUFFER *sig, char *pubring,
		char *secring)
{
   int ret = NULL; PGP_DECRYPT pgpdecrypt;
   if ((pgpdecrypt = (PGP_DECRYPT)GetProcAddress(hMixLib,"pgp_decrypt"))!=NULL)
      ret = (pgpdecrypt)(message,pass,sig,pubring,secring);
   return ret;
}
//---------------------------------------------------------------------------
int __fastcall rnd_add(byte *b, long len)
{
   int ret = NULL; RND_ADD rndadd;
   if ((rndadd = (RND_ADD)GetProcAddress(hMixLib,"rnd_add"))!=NULL)
      ret = (rndadd)(b,len);
   return ret;
}
//---------------------------------------------------------------------------
void __fastcall rnd_time()
{
   RND_TIME rndtime;
   if ((rndtime = (RND_TIME)GetProcAddress(hMixLib,"rnd_time"))!=NULL)
      (rndtime)();
}
//---------------------------------------------------------------------------
int __fastcall rnd_mouse(UINT i, WPARAM w, LPARAM l)
{
   int ret = NULL; RND_MOUSE rndmouse;
   if ((rndmouse = (RND_MOUSE)GetProcAddress(hMixLib,"rnd_mouse"))!=NULL)
      ret = (rndmouse)(i,w,l);
   return ret;
}
//---------------------------------------------------------------------------
int __fastcall buf_read(BUFFER *outmsg, FILE *infile)
{
   int ret = NULL; BUF_READ bufread;
   if ((bufread = (BUF_READ)GetProcAddress(hMixLib,"buf_read"))!=NULL)
      ret = (bufread)(outmsg,infile);
   return ret;
}
//---------------------------------------------------------------------------
int __fastcall buf_write(BUFFER *buffer, FILE *out)
{
   int ret = NULL; BUF_WRITE bufwrite;
   if ((bufwrite = (BUF_WRITE)GetProcAddress(hMixLib,"buf_write"))!=NULL)
      ret = (bufwrite)(buffer,out);
   return ret;
}
//---------------------------------------------------------------------------
int __fastcall buf_getline(BUFFER *buffer, BUFFER *line)
{
   int ret = NULL; BUF_GETLINE bufgetline;
   if ((bufgetline = (BUF_GETLINE)GetProcAddress(hMixLib,"buf_getline"))!=NULL)
      ret = (bufgetline)(buffer,line);
   return ret;
}
//---------------------------------------------------------------------------
int __fastcall buf_getheader(BUFFER *buffer, BUFFER *field, BUFFER *content)
{
   int ret = NULL; BUF_GETHEADER bufgetheader;
   if ((bufgetheader = (BUF_GETHEADER)GetProcAddress(hMixLib,"buf_getheader"))!=NULL)
      ret = (bufgetheader)(buffer,field,content);
   return ret;
}
//---------------------------------------------------------------------------
int __fastcall buf_prepare(BUFFER *buffer, int size)
{
   int ret = NULL; BUF_PREPARE bufprepare;
   if ((bufprepare = (BUF_PREPARE)GetProcAddress(hMixLib,"buf_prepare"))!=NULL)
      ret = (bufprepare)(buffer,size);
   return ret;
}
//---------------------------------------------------------------------------
int __fastcall buf_appendc(BUFFER *to, byte b)
{
   int ret = NULL; BUF_APPENDC bufappendc;
   if ((bufappendc = (BUF_APPENDC)GetProcAddress(hMixLib,"buf_appendc"))!=NULL)
      ret = (bufappendc)(to,b);
   return ret;
}
//---------------------------------------------------------------------------
int __fastcall buf_appendi(BUFFER *to, int i)
{
   int ret = NULL; BUF_APPENDI bufappendi;
   if ((bufappendi = (BUF_APPENDI)GetProcAddress(hMixLib,"buf_appendi"))!=NULL)
      ret = (bufappendi)(to,i);
   return ret;
}
//---------------------------------------------------------------------------
int __fastcall buf_appendl(BUFFER *to, long l)
{
   int ret = NULL; BUF_APPENDL bufappendl;
   if ((bufappendl = (BUF_APPENDL)GetProcAddress(hMixLib,"buf_appendl"))!=NULL)
      ret = (bufappendl)(to,l);
   return ret;
}
//---------------------------------------------------------------------------
int __fastcall buf_cat(BUFFER *to, BUFFER *from)
{
   int ret = NULL; BUF_CAT bufcat;
   if ((bufcat = (BUF_CAT)GetProcAddress(hMixLib,"buf_cat"))!=NULL)
      ret = (bufcat)(to,from);
   return ret;
}
//---------------------------------------------------------------------------
int __fastcall buf_clear(BUFFER *buffer)
{
   int ret = NULL; BUF_CLEAR bufclear;
   if ((bufclear = (BUF_CLEAR)GetProcAddress(hMixLib,"buf_clear"))!=NULL)
      ret = (bufclear)(buffer);
   return ret;
}
//---------------------------------------------------------------------------
int __fastcall buf_appends(BUFFER *buffer, char *s)
{
   int ret = NULL; BUF_APPENDS bufappends;
   if ((bufappends = (BUF_APPENDS)GetProcAddress(hMixLib,"buf_appends"))!=NULL)
      ret = (bufappends)(buffer,s);
   return ret;
}
//---------------------------------------------------------------------------
int __fastcall buf_get(BUFFER *buffer, BUFFER *to, int n)
{
  buf_clear(to);
  if (buffer->length - buffer->ptr < n)
    return (-1);
  buf_append(to, buffer->data + buffer->ptr, n);
  buffer->ptr += n;
  return (0);
}
//---------------------------------------------------------------------------
int __fastcall buf_rest(BUFFER *to, BUFFER *from)
{
   int ret = NULL; BUF_REST bufrest;
   if ((bufrest = (BUF_REST)GetProcAddress(hMixLib,"buf_rest"))!=NULL)
      ret = (bufrest)(to,from);
   return ret;
}
//---------------------------------------------------------------------------
int __fastcall buf_nl(BUFFER *b)
{
   return (buf_append(b,(byte*)"\n", 1));
}
//---------------------------------------------------------------------------
int __fastcall buf_set(BUFFER *to, BUFFER *from)
{
   int ret = NULL; BUF_SET bufset;
   if ((bufset = (BUF_SET)GetProcAddress(hMixLib,"buf_set"))!=NULL)
      ret = (bufset)(to,from);
   return ret;
}
//---------------------------------------------------------------------------
int __fastcall decode(BUFFER *in, BUFFER *out)
{
   int ret = NULL; DECODE decode;
   if ((decode = (DECODE)GetProcAddress(hMixLib,"decode"))!=NULL)
      ret = (decode)(in,out);
   return ret;
}
//---------------------------------------------------------------------------
int __fastcall encode(BUFFER *in, int linelen)
{
   int ret = NULL; ENCODE encode;
   if ((encode = (ENCODE)GetProcAddress(hMixLib,"encode"))!=NULL)
      ret = (encode)(in,linelen);
   return ret;
}
//---------------------------------------------------------------------------
int __fastcall strifind(const char *string, const char *keyword)
{
   int ret = NULL; STRIFIND strfindi;
   if ((strfindi = (STRIFIND)GetProcAddress(hMixLib,"strifind"))!=NULL)
      ret = (strfindi)(string,keyword);
   return ret;
}
//---------------------------------------------------------------------------
int __fastcall buf_getc(BUFFER *buffer)
{
  if (buffer->ptr == buffer->length) return (-1);
  else return (buffer->data[buffer->ptr++]);
}
//---------------------------------------------------------------------------
int __fastcall buf_geti(BUFFER *b)
{
  int i;
  if (b->ptr + 2 > b->length) return (-1);
  i  = buf_getc(b) << 8;
  i += buf_getc(b);
  return (i);
}
//---------------------------------------------------------------------------
long __fastcall buf_getl(BUFFER *b)
{
  long l;
  if (b->ptr + 4 > b->length) return (-1);
  l  = buf_getc(b) << 24;
  l += buf_getc(b) << 16;
  l += buf_getc(b) << 8;
  l += buf_getc(b);
  return (l);
}
//---------------------------------------------------------------------------
int __fastcall buf_rewind(BUFFER *buffer)
{
  buffer->ptr = 0;
  return (0);
}
//---------------------------------------------------------------------------
int __fastcall buf_eq(BUFFER *b1, BUFFER *b2)
{
//  sanity_check(b1);
//  sanity_check(b2);
  if (b1->length != b2->length) return (0);
  if (b1->length > 0 && memcmp(b1->data, b2->data, b1->length) != 0) return (0);
  return (1);
}
//---------------------------------------------------------------------------
int mixfile(char *path, const char *name)
{
  assert(path != NULL && name != NULL);
  if (name[0] == '/' || MixDir().IsEmpty() == NULL)
    strncpy(path, name, PATHMAX);
  else {
    strncpy(path, MixDir().c_str(), PATHMAX);
    strncat(path, name, PATHMAX);
  }
  return (0);
}
//---------------------------------------------------------------------------
FILE *mix_openfile(const char *name, const char *a)
{
  char path[PATHMAX];

  mixfile(path, name);
  return (fopen(path, a));
}
//---------------------------------------------------------------------------






