//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "Global2.h"
#include "BarRemsDlgs.h"
#include "BarEditDlgs.h"
#include "images256s.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TBarRemsDlg *BarRemsDlg;
const int fopen = 0;
const int fclosed = 1;
const int check = 3;
const int nocheck = 2;
const bool existing = 1;
//---------------------------------------------------------------------------

bool __fastcall ManageBarredRemailers(TQSStatsWin4 *Owner, const AnsiString &FileName)
{
   BarRemsDlg = new TBarRemsDlg(Owner, FileName);
   StrToFont(Owner->DlgsFontStr, BarRemsDlg->Font);
   StrToFont(Owner->WinsFontStr, BarRemsDlg->TreeView->Font);
   BarRemsDlg->TreeView->Color = Owner->WindowColor;
   BarRemsDlg->TreeView->Images = Images256->Barred;
   BarRemsDlg->ShowModal();

   bool modified = BarRemsDlg->Modified;
   delete BarRemsDlg;
   return modified;
}

//---------------------------------------------------------------------------
// TBarRemsDlg
//---------------------------------------------------------------------------

__fastcall TBarRemsDlg::TBarRemsDlg(TComponent* Owner)
   : TForm(Owner)
{
}
//---------------------------------------------------------------------------

__fastcall TBarRemsDlg::TBarRemsDlg(TComponent* Owner, const AnsiString &FileName)
   : TForm(Owner)
{
   assert(Owner != NULL && FileName != "");

   BarFile = FileName;

   BarredLast = new TStringList();
   BarredRemailers = new TStringList();

   if (FileExists(BarFile))
      BarredRemailers->LoadFromFile(BarFile);

   OrgBars = BarredRemailers->Text;

   int i = 0; while (i < BarredRemailers->Count)
   {
      AnsiString remailer = BarredRemailers->Strings[i];

      if (GetTok(1, remailer, ',').Trim().LowerCase() == "last")
      {
         BarredLast->Add(GetTok(0, remailer, ','));
         BarredRemailers->Delete(i);
         continue;
      }

      i++;
   }

   Modified = false;
   LoadTreeView();
}

//---------------------------------------------------------------------------

__fastcall TBarRemsDlg::~TBarRemsDlg()
{
   delete AllRemailers;
   delete BarredRemailers;
   delete BarredLast;
}
//---------------------------------------------------------------------------

void __fastcall TBarRemsDlg::FormCloseQuery(TObject *, bool &CanClose)
{
   GetBarredFromTreeView(BarredRemailers);

   if (BarredRemailers->Text != OrgBars && ModalResult == mrOk)
   {
      if (MsgBox(this,
         "Newly barred remailers will have their entries removed from\n"
         "corresponding stats page when the Stats Window closes. This        \n"
         "will occur even if you have downloaded pages and choose not\n"
         "to save them. Keys are not removed. Stats entries are\n"
         "sufficient to bar remailers.",
         mtInformation, MB_OKCANCEL) == IDCANCEL)
      {
         CanClose = false;
         return;
      }

      BarredRemailers->SaveToFile(BarFile);
      Modified = true;
   }
}
//---------------------------------------------------------------------------

void __fastcall TBarRemsDlg::GetBarredFromTreeView(TStrings *Barred)
{
   Barred->Clear();
   for (int i = 1; i < TreeView->Items->Count; i++)
   {
      TTreeNode *node = TreeView->Items->Item[i];
      if (IsChecked(node))
      {
         if (node->Parent == mnode)
         {
            BarredRemailers->Add(node->Text);
         }
      }
   }
}
//---------------------------------------------------------------------------

void __fastcall TBarRemsDlg::LoadTreeView()
{
   AnsiString rem;
   TTreeNode *node;
   TStringList *rems = new TStringList();
   rems->Duplicates = dupIgnore;
   rems->Sorted = true;

   if (TreeView->Items->Count)
   {
      TreeView->Items->Clear();
      Snooze(200);
   }

   GetMixmasterRemailers(rems);
   rems->AddStrings(BarredRemailers);

   rems->Sorted = false;
   for (int i = 0; i < BarredLast->Count; i++)
   {
      AnsiString bar = BarredLast->Strings[i];
      rems->Add(bar);

      int idx = rems->IndexOf(bar);
      rems->Strings[idx] = bar + ",last";
   }

   mnode = TreeView->Items->AddChild(NULL, "Barred Remailers");
   mnode->SelectedIndex = fopen;
   mnode->ImageIndex = fopen;

   for (int i = 0; i < rems->Count; i++)
   {
      rem = rems->Strings[i];
      node = TreeView->Items->AddChild(mnode, rem);

      rem = GetTok(0, rem, ',');
      int barred = BarredRemailers->IndexOf(rem) >= 0 || BarredLast->IndexOf(rem) >= 0;
      node->SelectedIndex = barred ? check : nocheck;
      node->ImageIndex = node->SelectedIndex;
   }

   TreeView->FullExpand();
   mnode->MakeVisible();
   TreeView->Selected = NULL;
   delete rems;
}
//---------------------------------------------------------------------------

/*void __fastcall TBarRemsDlg::GetMixmasterRemailers(TStrings *mixers)
{
   assert(mixers != NULL);

   AnsiString line, remailer, name;
   mixers->Clear();

   TStringList *mlist = new TStringList();
   AnsiString mlistorg = MixDir() + "mlist.txt.org";
   if (FileExists(mlistorg)) mlist->LoadFromFile(mlistorg);

   for (int i = 0; i < mlist->Count; i++)
   {
      line = mlist->Strings[i];
      if (line.Pos("%") > 0)
      {
         mixers->Add(GetTok(0, line, ' '));
      }
   }

   delete mlist;
}*/
//---------------------------------------------------------------------------

void __fastcall TBarRemsDlg::GetMixmasterRemailers(TStrings *mixers)
{
   assert(mixers != NULL);

   AnsiString line, remailer, name;
   mixers->Clear();

   TStringList *mlist = new TStringList();
   AnsiString mlisttxt = MixDir() + "mlist.txt";
   if (FileExists(mlisttxt)) mlist->LoadFromFile(mlisttxt);

   for (int i = 0; i < mlist->Count; i++)
   {
      line = mlist->Strings[i];
      if (line.Pos("%") > 0)
      {
         mixers->Add(GetTok(0, line, ' '));
      }
   }

   delete mlist;
}
//---------------------------------------------------------------------------

bool __fastcall TBarRemsDlg::NodeRemailerExists(TTreeNode *Node)
{
   assert(Node != NULL);
   return Node->Data != NULL;
}
//---------------------------------------------------------------------------

bool __fastcall TBarRemsDlg::IsChecked(TTreeNode *node)
{
   assert(node != NULL);
   return node->ImageIndex == check;
}
//---------------------------------------------------------------------------
#pragma argsused
void __fastcall TBarRemsDlg::TreeViewMouseDown(TObject *Sender,
      TMouseButton Button, TShiftState Shift, int X, int Y)
{
   if (TreeView->GetHitTestInfoAt(X, Y).Contains(htOnIcon))
   {
      TTreeNode *node = TreeView->GetNodeAt(X, Y);

      if (node != TreeView->Items->Item[0])
      {
         if (node->ImageIndex == nocheck) node->ImageIndex = check;
         else node->ImageIndex = nocheck;
         node->SelectedIndex = node->ImageIndex;
      }
   }
}
//---------------------------------------------------------------------------
#pragma argsused
void __fastcall TBarRemsDlg::TreeViewCustomDrawItem(TCustomTreeView *Sender,
   TTreeNode *Node, TCustomDrawState State, bool &DefaultDraw)
{
   TRect NodeRect;
   int Indent = TreeView->Indent;
   TCanvas *Canvas = TreeView->Canvas;

   if (Node->Level == 0)
   {
    //Canvas->Font->Style = TFontStyles() << fsBold;
      Node->ImageIndex = Node->Expanded ? fopen : fclosed;
      Node->SelectedIndex = Node->ImageIndex;
   }
   else
   {
      if (Node->ImageIndex == check) Canvas->Font->Style = TFontStyles() << fsStrikeOut;
   }

   NodeRect = Node->DisplayRect(true);
   if (State.Contains(cdsSelected))
   {
      NodeRect.Left -= 3;
      NodeRect.Right += 0;
      TreeView->Canvas->FillRect(NodeRect);
      Canvas->Font->Color = clHighlightText;
   }

   NodeRect = Node->DisplayRect(false);
   NodeRect.Left = NodeRect.Left + (Node->Level * Indent);

   int cy = NodeRect.Top + (NodeRect.Bottom - NodeRect.Top) / 2;
   Images256->Barred->Draw(Canvas, NodeRect.Left, cy-TreeView->Images->Height / 2,
      Node->ImageIndex, true);

   NodeRect = Node->DisplayRect(true);
   if (Node->Level == 0)
   {
      NodeRect.Right = NodeRect.Left + Canvas->TextWidth(Node->Text + " ");
      Canvas->TextRect(NodeRect, NodeRect.Left,  NodeRect.Top, Node->Text);
   }
   else
   {
      NodeRect.Right = NodeRect.Left + Canvas->TextWidth(Node->Text + " ");
      Canvas->TextRect(NodeRect, NodeRect.Left,  NodeRect.Top, Node->Text);
   }

   DefaultDraw = false;
}
//---------------------------------------------------------------------------

void __fastcall TBarRemsDlg::EditSrcBtnClick(TObject *)
{
   GetBarredFromTreeView(BarredRemailers);
   if (EditBarredSource(this, BarredRemailers))
   {
      LoadTreeView();
   }
}
//---------------------------------------------------------------------------

void __fastcall TBarRemsDlg::PopBarRemailerLastItem(
      TObject *Sender)
{
   TListItem *item = ListView->Selected;
   assert(item != NULL);

   AnsiString rem = item->Caption;
   if (rem.Pos(",last") > 0)
   {
      AnsiString name = GetTok(0, rem, ',');
      BarredRemailers->Add(name);
      //BarredLast->Delete(BarredLast->IndexOf(name));
      item->Caption = name;
   }
   else
   {
      //BarredRemailers->Delete(BarredRemailers->IndexOf(rem));
      BarredLast->Add(rem);
      item->Caption = rem + ",last";
   }
}
//---------------------------------------------------------------------------

void __fastcall TBarRemsDlg::PopupMenu1Popup(TObject *Sender)
{
   PopBarredLastItem->Enabled =
      TreeView->Selected || TreeView->Selected->Level > 0;            
}
//---------------------------------------------------------------------------

