//---------------------------------------------------------------------------
#pragma hdrstop

#include "Global2.h"
#include "RemailerDocs.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
const int DaysTooOld = 3;
//---------------------------------------------------------------------------

//__fastcall TRemailerDoc::TRemailerDoc()
//   : TObject()
//{
//}
//---------------------------------------------------------------------------

__fastcall TRemailerDoc::TRemailerDoc(const AnsiString &source, TRichEdit *Doc)
   : TObject()
{
   ok = true;
   Name = "generic";
   Source = source;
   Lines = Doc->Lines;
   ed = Doc;
}
//---------------------------------------------------------------------------

__fastcall TRemailerDoc::~TRemailerDoc()
{
   delete ed;
}
//---------------------------------------------------------------------------

bool __fastcall TRemailerDoc::IsValid()
{
   return true;
}
//---------------------------------------------------------------------------

bool __fastcall TRemailerDoc::IsVersion2Stats()
{
   return false;
}
//---------------------------------------------------------------------------

bool __fastcall TRemailerDoc::IsTooOld()
{
   return false;
}

//---------------------------------------------------------------------------
// mlist.txt
//---------------------------------------------------------------------------

__fastcall TMixMListTxt::TMixMListTxt(const AnsiString &source, TRichEdit *doc)
   : TRemailerDoc(source, doc)
{
   Name = "mlist.txt";

   if (!IsVersion2Stats())
   {
    // reverse the page layout -- stats to top, all else after
      TrimNulls(Lines);
      Lines->Add("");

      while (Lines->Strings[0].Pos("Last update: ") == 0)
      {
         Lines->Add(Lines->Strings[0]);
         Lines->Delete(0);
      }
   }
}
//---------------------------------------------------------------------------

bool __fastcall TMixMListTxt::IsValid()
{
   if (IsVersion2Stats())
       return false;

   bool valid = false;
   for (int i = 0; i < Lines->Count; i++)
   {
      AnsiString line = Lines->Strings[i].Trim().LowerCase();
      valid = line.Pos("mixmaster ") == 1;
      if (!valid) continue;

      valid = line.Pos(" history ") > 1;
      if (valid) valid = line.Pos(" latency ") > 1;
      if (valid) valid = line.Pos(" uptime") > 1;
      if (valid) break;
   }
   return valid;
}
//---------------------------------------------------------------------------

bool __fastcall TMixMListTxt::IsVersion2Stats()
{
   return Lines->Text.Pos("Stats-Version: 2.0") > 0;
}
//---------------------------------------------------------------------------

bool __fastcall TMixMListTxt::IsTooOld()
{
 //Last update: Wed 28 Jul 2004 06:35:09 GMT
   bool old = false;
   for (int i = 0; i < Lines->Count; i++)
   {
      AnsiString line = Lines->Strings[i];

      if (line.Pos("Last Update: ") == 1)
      {
         Word d = (Word)GetTok(3, line, ' ').ToIntDef(1);
         Word m = (Word)GetMonthFromStr3(GetTok(4, line, ' '));
         Word y = (Word)GetTok(5, line, ' ').ToIntDef(1);

         TDateTime page;
         if (TryEncodeDate(y, m, d, page))
         {
            if ((int)Date() - (int)page > 3)
               old = true;
         }
         break;
      }
   }

   return old;
}

//---------------------------------------------------------------------------
// pubring.mix
//---------------------------------------------------------------------------

__fastcall TMixPubringMix::TMixPubringMix(const AnsiString &source, TRichEdit *doc)
   : TRemailerDoc(source, doc)
{
   Name = "pubring.mix";
}
//---------------------------------------------------------------------------

bool __fastcall TMixPubringMix::IsValid()
{
   return Lines->IndexOf("-----Begin Mix Key-----") >= 0;
}
//---------------------------------------------------------------------------

bool __fastcall TMixPubringMix::IsTooOld()
{
   return false;
}

//---------------------------------------------------------------------------
// type2.list
//---------------------------------------------------------------------------

__fastcall TMixType2List::TMixType2List(const AnsiString &source, TRichEdit *doc)
   : TRemailerDoc(source, doc)
{
   Name = "type2.list";
}
//---------------------------------------------------------------------------

bool __fastcall TMixType2List::IsValid()
{
   bool ok = true;
   TrimNulls(Lines);
   for (int i = 0; i < Lines->Count; i++)
   {
      if (GetTok(1, Lines->Strings[i], ' ').Pos("@") == 0)
      {
         ok = false;
         break;
      }
   }
   return ok;
}
//---------------------------------------------------------------------------

bool __fastcall TMixType2List::IsTooOld()
{
   return false;
}

//---------------------------------------------------------------------------
// rlist.txt
//---------------------------------------------------------------------------

__fastcall TCpkRListTxt::TCpkRListTxt(const AnsiString &source, TRichEdit *doc)
   : TRemailerDoc(source, doc)
{
   Name = "rlist.txt";

   if (!IsVersion2Stats())
   {
    // reverse the page layout -- stats to top, all else after
      TrimNulls(Lines);
      Lines->Add("");

      while (Lines->Strings[0].Pos("Last update: ") == 0)
      {
         Lines->Add(Lines->Strings[0]);
         Lines->Delete(0);
      }
   }
}
//---------------------------------------------------------------------------

bool __fastcall TCpkRListTxt::IsValid()
{
   if (IsVersion2Stats())
       return false;

   bool valid = false;
   for (int i = 0; i < Lines->Count; i++)
   {
      AnsiString line = Lines->Strings[i].Trim().LowerCase();
      valid = line.Pos("remailer ") == 1;
      if (!valid) continue;

      valid = line.Pos(" email ") > 1;
      if (valid) valid = line.Pos(" address ") > 1;
      if (valid) valid = line.Pos(" history ") > 1;
      if (valid) valid = line.Pos(" latency ") > 1;
      if (valid) valid = line.Pos(" uptime") > 1;
      if (valid) break;
   }
   return valid;
}
//---------------------------------------------------------------------------

bool __fastcall TCpkRListTxt::IsVersion2Stats()
{
   return Lines->Text.Pos("Stats-Version: 2.0") > 0;
}
//---------------------------------------------------------------------------

bool __fastcall TCpkRListTxt::IsTooOld()
{
 //Last update: Wed 28 Jul 2004 07:06:59 GMT
   bool old = false;
   for (int i = 0; i < Lines->Count; i++)
   {
      AnsiString line = Lines->Strings[i];

      if (line.Pos("Last Update: ") == 1)
      {
         Word d = (Word)GetTok(3, line, ' ').ToIntDef(1);
         Word m = (Word)GetMonthFromStr3(GetTok(4, line, ' '));
         Word y = (Word)GetTok(5, line, ' ').ToIntDef(1);

         TDateTime page;
         if (TryEncodeDate(y, m, d, page))
         {
            if ((int)Date() - (int)page > 3)
               old = true;
         }
         break;
      }
   }

   return old;
}

//---------------------------------------------------------------------------
// pubring.asc
//---------------------------------------------------------------------------

__fastcall TCpkPubringAsc::TCpkPubringAsc(const AnsiString &source, TRichEdit *doc)
   : TRemailerDoc(source, doc)
{
   Name = "pubring.asc";
}
//---------------------------------------------------------------------------

bool __fastcall TCpkPubringAsc::IsValid()
{
   return Lines->IndexOf("-----BEGIN PGP PUBLIC KEY BLOCK-----") >= 0;
}
//---------------------------------------------------------------------------

bool __fastcall TCpkPubringAsc::IsTooOld()
{
   return false;
}

//---------------------------------------------------------------------------
// advisory.txt
//---------------------------------------------------------------------------

__fastcall TQSAdvisoryTxt::TQSAdvisoryTxt(const AnsiString &source, TRichEdit *doc)
   : TRemailerDoc(source, doc)
{
   Name = AdvisoryFile;
}
//---------------------------------------------------------------------------

bool __fastcall TQSAdvisoryTxt::IsTooOld()
{
   return false;
}
//---------------------------------------------------------------------------

bool __fastcall TQSAdvisoryTxt::IsValid()
{
   return Lines->IndexOf("quicksilver user advisory") >= 0;
}

//---------------------------------------------------------------------------
// statsurls.txt
//---------------------------------------------------------------------------

__fastcall TQSUrlsTxt::TQSUrlsTxt(const AnsiString &source, TRichEdit *doc)
   : TRemailerDoc(source, doc)
{
   Name = URLFile;
}

//---------------------------------------------------------------------------

bool __fastcall TQSUrlsTxt::IsTooOld()
{
   return false;
}

//---------------------------------------------------------------------------

bool __fastcall TQSUrlsTxt::IsValid()
{
   return Lines->IndexOf("quicksilver urls") >= 0;
}

//---------------------------------------------------------------------------
// nymservers.asc
//---------------------------------------------------------------------------

__fastcall TQSNymServerAsc::TQSNymServerAsc(const AnsiString &source, TRichEdit *doc)
   : TRemailerDoc(source, doc)
{
   Name = NymserverFile;
}

//---------------------------------------------------------------------------

bool __fastcall TQSNymServerAsc::IsTooOld()
{
   return false;
}

//---------------------------------------------------------------------------

bool __fastcall TQSNymServerAsc::IsValid()
{
   return Lines->IndexOf("-----BEGIN PGP PUBLIC KEY BLOCK-----") >= 0;
}

//---------------------------------------------------------------------------
// mail2news.txt
//---------------------------------------------------------------------------

__fastcall TQSMail2newsTxt::TQSMail2newsTxt(const AnsiString &source, TRichEdit *doc)
   : TRemailerDoc(source, doc)
{
   Name = Mail2newsFile;
}

//---------------------------------------------------------------------------

bool __fastcall TQSMail2newsTxt::IsTooOld()
{
   return false;
}

//---------------------------------------------------------------------------

bool __fastcall TQSMail2newsTxt::IsValid()
{
   return Lines->Text.Pos("mail2news@") >= 0;
}

//---------------------------------------------------------------------------
// fromheader.txt
//---------------------------------------------------------------------------

/*__fastcall TQSFromHeadTxt::TQSFromHeadTxt(const AnsiString &source, TRichEdit *doc)
   : TRemailerDoc(source, doc)
{
   Name = "fromheader.txt";
}
//---------------------------------------------------------------------------

bool __fastcall TQSFromHeadTxt::IsTooOld()
{
   return false;
}

//---------------------------------------------------------------------------

bool __fastcall TQSFromHeadTxt::IsValid()
{
   return true;
}*/
//---------------------------------------------------------------------------

