//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop
#include <registry.hpp>
#include <assert.h>

#include "Global2.h"
#include "StatsWins.h"
#include "URLDlgs.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "SHDocVw_OCX"
#pragma resource "*.dfm"
TStatsWin *StatsWin;
//---------------------------------------------------------------------------

void __fastcall GetRemailerPage(TForm *Owner, const AnsiString &Source,
   const AnsiString &Page, bool AutoLoad)
{
   StatsWin = new TStatsWin(Owner, Source, Page, AutoLoad);
   StatsWin->Show();
}

//---------------------------------------------------------------------------
// TStatsWin
//---------------------------------------------------------------------------

__fastcall TStatsWin::TStatsWin(TComponent* Owner)
        : TForm(Owner)
{
}
//---------------------------------------------------------------------------

__fastcall TStatsWin::TStatsWin(TComponent* Owner, const AnsiString &Source,
   const AnsiString &Page, bool AutoLoad) : TForm(Owner)
{
   if (!GetMixmasterDirectory(MixmasterDir))
   {
      MsgDlg(this,
        "Mixmaster is not installed on this computer! Without it, I can't continue!",
         mtError, TMsgDlgButtons() << mbOK
      );
      Application->ShowMainForm = false;
      PostMessage(Handle, WM_QUIT, 0, 0);
      return;
   }

   TIniFile *ini = new TIniFile(AppIni());
   CascadeBox->Checked = ini->ReadBool("Main", "Cascade", true);
   delete ini;

   URLs = new TStringList();
   URLFileName = MixmasterDir + "QuickSilverURLs.txt";

   if (FileExists(URLFileName))
      URLs->LoadFromFile(URLFileName);

   LoadPages();
   LoadSources();
   LoadURLs(Page);

   PageBox->ItemIndex = PageBox->Items->IndexOf(Page);
   SourceBox->ItemIndex = SourceBox->Items->IndexOf(Source);
   URLBox->ItemIndex = URLBox->Items->IndexOf(GetURL(Source, Page));

   if (AutoLoad) Timer1->Enabled = true;

   Cap = Caption;
}
//---------------------------------------------------------------------------

__fastcall TStatsWin::~TStatsWin()
{
   delete URLs;
}
//---------------------------------------------------------------------------
#pragma argsused
void __fastcall TStatsWin::FormClose(TObject *Sender, TCloseAction &Action)
{
   Action = caFree;
}
//---------------------------------------------------------------------------
#pragma argsused
void __fastcall TStatsWin::Timer1Timer(TObject *Sender)
{
   Timer1->Enabled = false;
}
//---------------------------------------------------------------------------
#pragma argsused
void __fastcall TStatsWin::Exit1Click(TObject *Sender)
{
   Close();
}
//---------------------------------------------------------------------------

bool __fastcall TStatsWin::GetMixmasterDirectory(AnsiString &Directory)
{
   //no assert, directory can be empty coming in

   TRegistry *reg = new TRegistry();
   if (!reg->OpenKey("Software\\Mixmaster", false))
   {
      delete reg;
      return false;
   }
   Directory = AddBackSlash(reg->ReadString("MixDir"));
   delete reg;

   return true;
}
//---------------------------------------------------------------------------
#pragma argsused
void __fastcall TStatsWin::BrowserNavigateError(TObject *Sender,
      LPDISPATCH pDisp, Variant *URL, Variant *Frame, Variant *StatusCode,
      VARIANT_BOOL *Cancel)
{
   GotError = true;
}
//---------------------------------------------------------------------------
void __fastcall TStatsWin::LoadSources()
{
   for (int i = 0; i < URLs->Count; i++)
   {
      AnsiString line = URLs->Strings[i];
      if (line.Pos("[") == 1)
         SourceBox->Items->Add(line.SubString(2, line.Length() - 2));
   }
}
//---------------------------------------------------------------------------

void __fastcall TStatsWin::LoadURLs(AnsiString Page)
{
   assert(Page != "");

   URLBox->Clear();
   for (int i = 0; i < URLs->Count; i++)
   {
      AnsiString line = URLs->Strings[i].Trim().LowerCase();
      if (line.Pos(Page) == 1)
         URLBox->Items->Add(line.SubString(Page.Length()+1, line.Length()).Trim());
   }
}
//---------------------------------------------------------------------------

void __fastcall TStatsWin::LoadPages()
{
   PageBox->Items->Add("mlist.txt");
   PageBox->Items->Add("pubring.mix");
   PageBox->Items->Add("type2.list");
   PageBox->Items->Add("rlist.txt");
   PageBox->Items->Add("pubring.asc");
}
//---------------------------------------------------------------------------

AnsiString __fastcall TStatsWin::GetURL(AnsiString Source, AnsiString Page)
{
   assert(Source != "" && Page != "");

   AnsiString line;
   Page = Page.LowerCase();
   Source = Source.LowerCase();
   int i = 0; while (i < URLs->Count)
   {
      line = URLs->Strings[i].Trim().LowerCase();
      if (line == "[" + Source + "]")
      {
         i++; while (i < URLs->Count)
         {
            line = URLs->Strings[i].Trim().LowerCase();
            if (line.Pos(Page) == 1)
               return line.SubString(Page.Length() + 1, line.Length()).Trim();

            i++;
         }
      }
      i++;
   }
   return "";
}
//---------------------------------------------------------------------------

AnsiString __fastcall TStatsWin::GetSource(AnsiString URL)
{
   assert(URL != "");

   URL = URL.LowerCase();
   AnsiString line, source;
   int i = 0; while (i < URLs->Count)
   {
      line = URLs->Strings[i];
      if (line.Pos("[") == 1)
      {
         source = line.SubString(2, line.Length() - 2).LowerCase().Trim();
         i++; continue;
      }

      if (line.Pos(URL) > 0)
         return source;

      i++;
   }
   return "";
}
//---------------------------------------------------------------------------
#pragma argsused
void __fastcall TStatsWin::URLBoxChange(TObject *Sender)
{
   SourceBox->ItemIndex = SourceBox->Items->IndexOf(GetSource(URLBox->Text));
}
//---------------------------------------------------------------------------
#pragma argsused
void __fastcall TStatsWin::SourceBoxChange(TObject *Sender)
{
   AnsiString URL = GetURL(SourceBox->Text, PageBox->Text);
   int index = URLBox->Items->IndexOf(URL);
   assert(index >= 0);

   URLBox->Items->Delete(index);
   URLBox->Items->Insert(0, URL);
   URLBox->ItemIndex = 0;
}
//---------------------------------------------------------------------------
#pragma argsused
void __fastcall TStatsWin::PageBoxChange(TObject *Sender)
{
   LoadURLs(PageBox->Text);
   AnsiString URL = GetURL(SourceBox->Text, PageBox->Text);
   int index = URLBox->Items->IndexOf(URL);
   assert(index >= 0);

   URLBox->Items->Delete(index);
   URLBox->Items->Insert(0, URL);
   URLBox->ItemIndex = 0;
}
//---------------------------------------------------------------------------
#pragma argsused
void __fastcall TStatsWin::BrowserDocumentComplete(TObject *Sender,
      LPDISPATCH pDisp, Variant *URL)
{
   Caption = Cap;
   SaveBtn->Enabled = !GotError;
}
//---------------------------------------------------------------------------

void __fastcall TStatsWin::FileOpenItemClick(TObject *Sender)
{
   GotError = false;
   Caption = "Loading...";
   SaveBtn->Enabled = false;
   Browser->Navigate(WideString(URLBox->Text));
}
//---------------------------------------------------------------------------

void __fastcall TStatsWin::CascadeBoxClick(TObject *Sender)
{
   TIniFile *ini = new TIniFile(AppIni());
   ini->WriteBool("Main", "Cascade", CascadeBox->Checked);
   delete ini;
}
//---------------------------------------------------------------------------

void __fastcall TStatsWin::FileURLManItemClick(TObject *Sender)
{
   if (OpenURLManager(this, URLFileName))
   {
      // reload in window
   }
}
//---------------------------------------------------------------------------

void __fastcall TStatsWin::FileMenuClick(TObject *Sender)
{
   FileCascadeErrorItem->Checked = CascadeBox->Checked;
}
//---------------------------------------------------------------------------

void __fastcall TStatsWin::FileCascadeErrorItemClick(TObject *Sender)
{
   CascadeBox->Checked = !CascadeBox->Checked;
}
//---------------------------------------------------------------------------

