//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop
#include <registry.hpp>
#include <time.h>

#include "Global2.h";
#include "StatsWins4.h"
#include "QSstatsVer.h"
#include "AboutDlgs2.h"
#include "BarRemsDlgs2.h"
#include "UpdateSelectDlgs.h"
#include "StatsOptionsDlgs.h"
#include "RearrangeDlgs.h"
#include "NewMixKeyWins.h"
#include "URLDlgs.h"
#include "HTTPmods.h"
#include "QSDBs.h"
#include "Images256s.h"
#include "PrntSelPagesDlgs.h"
#include "HistoryWins.h"
#include "..\htmlhelp\htmlhelp.h"
#include "ChkAddKeysDlgs.h"

const int HTTP_START = 0;
const int HTTP_BOGUSPAGE = -6000;
const AnsiString AppHelpFile = ChangeFileExt(Application->ExeName, ".chm");
HWND HelpWin = 0;
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TQSStatsWin4 *QSStatsWin4;
const AnsiString FailedChecks = "*****Failed Error Checks*****";

//---------------------------------------------------------------------------
// TStatsWin4
//---------------------------------------------------------------------------

__fastcall TQSStatsWin4::TQSStatsWin4(TComponent* Owner)
        : TForm(Owner)
{
   Color = clBtnFace;
   SetCurrentDir(AppDir());

   MixmasterDir = MixDir();
   if (MixmasterDir.IsEmpty())
   {
      PostMessage(HWND_BROADCAST, WM_QSAPPMESSAGE, QSMSGID_STATSUPDATED, StatsUpdateFailed);
      ShowMessage(
         "Mixmaster is not installed on this computer! Without it, I cannot continue!"
      );
      Application->ShowMainForm = false;
      PostMessage(Handle, WM_QUIT, 0, 0);
      return;
   }

   if (!FileExists(AppDat()) && FileExists(AppPGP()))
   {
      ShowMessage(
         "I find your QS database is encrypted. You need    \n"
         "to start QS and decrypt the file.");
      Application->ShowMainForm = false;
      PostMessage(Handle, WM_QUIT, 0, 0);
      return;
   }

   AnsiString appdat = AppDir() + "qs.dbc";
   if (!FileExists(appdat) && FileExists(ChangeFileExt(appdat, ".pgp")))
   {
      ShowMessage(
         "QSstats requires use of the QS database. Your database is   \n"
         "encrypted. You need to start QS to decrypt it! Then you can  \n"
         "restart QSstats.");
      PostMessage(Handle, WM_QUIT, 0, 0);
      return;
   }

   if (!InitGlobal(this))
   {
      Application->ShowMainForm = false;
      PostMessage(HWND_BROADCAST, WM_QSAPPMESSAGE, QSMSGID_STATSUPDATED, StatsUpdateFailed);
      PostMessage(Handle, WM_QUIT, 0, 0);
      return;
   }

   AutoRun = false;
   AutoRunFiles = NULL;
   if (ParamCount() > 0 && ParamStr(1).LowerCase().Pos("version") > 0)
   {
      TIniFile *ini = new TIniFile(AppIni());
      ini->WriteString(StatsIni, "version", ReleaseVersion);
      Application->ShowMainForm = false;
      PostMessage(Handle, WM_QUIT, 0, 0);
      delete ini;
      return;
   }

   for (int i = 1; i <= ParamCount(); i++)
   {
      AnsiString param = ParamStr(i).LowerCase();

      if (param.Pos("hidden") > 0)
      {
         Hidden = true;
         continue;
      }

      if (param.Pos("hide") > 0)
      {
         Hidden = true;
         continue;
      }

      if (param.Pos("auto") > 0)
      {
         AutoRun = true;
         continue;
      }

      if (param == "pubring.asc")
      {
         if (!AutoRunFiles)
            AutoRunFiles = new TStringList();
         AutoRunFiles->Add(param);
      }

      if (param == "rlist.txt")
      {
         if (!AutoRunFiles)
            AutoRunFiles = new TStringList();
         AutoRunFiles->Add(param);
      }
   }

   if (Hidden && !AutoRun)
      Hidden = false;

   Caption = AppName + " v" + ReleaseVersion;
   Application->OnMessage = AppMessage;

   Images256 = new TImages256(this);
   ToolBar->DisabledImages = Images256->StatsDisabled;
   ToolBar->HotImages = Images256->StatsHot;
   ToolBar->Images = Images256->Stats;

   Proxy = new TStringList();
   Proxies = new TStringList();
   LoadIni();

   OpenDialog->InitialDir = MixDir();
   ProgressBar->BringToFront();
   ProgressBar->Position = 0;

   mlistorg = new TStringList();
   rlistorg = new TStringList();
   newbars = new TStringList();

   ChildX = Left;
   ChildY = Top;

   KeyringsMenu->Visible = false;
   KeyringBtn->Visible = false;
   KeyringSep->Visible = false;

   SourceBox->Text = GetSource();

   if (FileExists(AppHelpFile))
      LoadHtmlHelp();

   if (FileExists(PathName("status.txt")))
      GetLocalDocument(PathName("status.txt"));

   //if (TabSet->Font->Size == 8)
   //   TabSet->TabWidth = 80;
   //else
   //   TabSet->TabWidth = 92;

   if (AutoRun)
   {
      if (Hidden)
         Application->ShowMainForm = false;
      else
      {
         if (TabSet->Tabs->Count > 0)
            TabEditor(0)->Clear();
      }
      Timer3->Enabled = true;
      return;
   }

   if (AutoLoad)
   {
      Timer2->Enabled = true;
      return;
   }
}
//---------------------------------------------------------------------------

__fastcall TQSStatsWin4::~TQSStatsWin4()
{
   ShutDownGlobal();

   delete AutoRunFiles;
   delete newbars;
   delete rlistorg;
   delete mlistorg;
   delete Proxies;
   delete Proxy;
}
//---------------------------------------------------------------------------

void __fastcall TQSStatsWin4::FormCloseQuery(TObject*, bool &CanClose)
{
   if (Working())
   {
      MessageBeep(MB_OK);
      CanClose = false;
      return;
   }

   if (!CheckSave())
   {
      CanClose = false;
      return;
   }

 //AnsiString orgsource = GetSource();
   if (GetSource() != SourceBox->Text)
   {
      switch (MsgBox(this,
        "Your stats source has been changed. Do you wish to keep the current source?     ",
         MT_CONFIRM, MB_YESNOCANCEL))
      {
         case IDCANCEL:
            CanClose = false;
            return;
         case IDYES:
            MoveSource(SourceBox->Text, 0);
            break;
      }
   }

   RemoveBarredRemailersFromStatsFiles();
   UnloadHtmlHelp();
   SaveIni();

   Hide();
   ClearTabs();
}
//---------------------------------------------------------------------------

/*bool __fastcall TQSStatsWin4::SecureMode()
{
   if (!FileExists(AppDat()) && FileExists(AppPGP()))
   {
      MsgBox(this, "Your QuickSilver database is encrypted! You need    \n"
         "to start QS and decrypt it.", MT_INFO, MB_OK);
      return true;
   }
   return false;
}*/
//---------------------------------------------------------------------------

void __fastcall TQSStatsWin4::LoadIni()
{
   TIniFile *ini = new TIniFile(AppIni());

   BlankView->Color = (TColor)ini->ReadInteger(StatsIni, QSWINDOWCOLOR, QSWINDOWCOLORDEF);
   StrToFont(ini->ReadString(StatsIni, QSSTATSFONT, QSSTATSFONTDEF), BlankView->Font);

   ShowToolBar(ini->ReadBool(StatsIni, "showtoolbar", true));
   StatusBar->Visible = ini->ReadBool(StatsIni, "showstatusbar", true);

   AnsiString ticked = ini->ReadString(StatsIni, "pages", "");
   UpdateMlistTxtItem->Checked = ticked.Pos("mlist.txt") > 0;
   UpdatePubringMixItem->Checked = ticked.Pos("pubring.mix") > 0;
   UpdateRlistTxtItem->Checked = ticked.Pos("rlist.txt") > 0;
   UpdatePubringAscItem->Checked = ticked.Pos("pubring.asc") > 0;
   UpdateNymserverAscItem->Checked = ticked.Pos(NymserverFile) > 0;
   UpdateMail2newsTxtItem->Checked = ticked.Pos(Mail2newsFile) > 0;
   UpdateAdvisoryTxtItem->Checked = ticked.Pos(AdvisoryFile) > 0;
   UpdateURLsTxtItem->Checked = ticked.Pos(URLFile) > 0;

   WinsFontStr = ini->ReadString(StatsIni, QSWINDOWFONT, QSEDITORFONTDEF);
   EditFontStr = ini->ReadString(StatsIni, QSEDITORFONT, QSEDITORFONTDEF);
   DlgsFontStr = ini->ReadString(StatsIni, QSDIALOGFONT, QSDIALOGFONTDEF);
   WindowColor = (TColor)ini->ReadInteger(StatsIni, QSWINDOWCOLOR, QSWINDOWCOLORDEF);

   StrToFont(DlgsFontStr, Font);
   StatusBar->Font->Assign(Font);
   SourceBox->Color = WindowColor;
   ProxyBox->Color = WindowColor;
   StrToFont(WinsFontStr, SourceBox->Font);
   StrToFont(WinsFontStr, ProxyBox->Font);

   AutoLoad = false; //ini->ReadBool(StatsIni, "autoload", true);
   BarNewRemailers = ini->ReadBool(StatsIni, "barnew", false);
   OkBtn->Visible = ini->ReadBool(StatsIni, "exitleft", false);
   TB3->Visible = OkBtn->Visible;

   AnsiString winpos = ini->ReadString(StatsIni, "window", "");
   WindowState = (TWindowState)GetTok(0, winpos, ',').ToIntDef(wsNormal);
   Position = poDesigned; //GetTok(1, winpos, ',') == "1" ? poScreenCenter : poDesigned;

   Left   = GetTok(2, winpos, ',').ToIntDef(0);
   Top    = GetTok(3, winpos, ',').ToIntDef(0);
   Width  = GetTok(4, winpos, ',').ToIntDef(Width);
   Height = GetTok(5, winpos, ',').ToIntDef(Height);

   LoadSourceConfig(ini, GetSource());
   GetProxies(ini);

   delete ini;
}
//---------------------------------------------------------------------------

void __fastcall TQSStatsWin4::GetProxies(TIniFile *ini)
{
   if (SecureModeMsg(this))
      return;
      
   TStringList *proxies = new TStringList();

   Global->DB->GetProxies(proxies);
   ProxyBox->Items->Clear();
   ProxyBox->Sorted = true;
   Proxies->Clear();
   Proxies->Sorted = true;

   for (int i = 0; i < proxies->Count; i++)
   {
      AnsiString line = proxies->Strings[i];
      int x; while ((x = line.Pos("\255")) > 0)
         line[x] = '/';

      if (GetTok(ProxyHTTP, line, ProxyDiv) != " ")
      {
         ProxyBox->Items->Add(GetTok(0, line, ','));
         Proxies->Add(line);
      }
   }

   delete proxies;

   Proxies->Sorted = false;
   ProxyBox->Sorted = false;
   ProxyBox->Items->Insert(0, "<none>");
   Proxies->Insert(0, "<none>");

   AnsiString last = ini->ReadString(StatsIni, "lastproxy", "<none>");
   int index = ProxyBox->Items->IndexOf(last);
   if (index < 0)
   {
      index = 0;
      MsgBox(this,
        "The proxy you have been using for stats     \nretrieval--" + last +
        "--is no \nlonger present!",
         MT_WARN, MB_OK
      );
   }
   ProxyBox->ItemIndex = index;
   ProxyBoxChange(NULL);
}
//---------------------------------------------------------------------------

void __fastcall TQSStatsWin4::ShowToolBar(bool Show)
{
   CoolBar->Visible = Show;
   SourceBox->Visible = Show;
   ProxyBox->Visible = Show;
   StaticText3->Visible = Show;
   StaticText4->Visible = Show;
}
//---------------------------------------------------------------------------

void __fastcall TQSStatsWin4::SaveIni()
{
   TIniFile *ini = new TIniFile(AppIni());

   int state = WindowState;
   AnsiString winpos, winorg = ini->ReadString(StatsIni, "window", "");

   winpos = IntToStr(state) + "," + IntToStr(Position);

   winpos += "," + (state == wsNormal ? IntToStr(Left  ) : GetTok(2, winorg, ','));
   winpos += "," + (state == wsNormal ? IntToStr(Top   ) : GetTok(3, winorg, ','));
   winpos += "," + (state == wsNormal ? IntToStr(Width ) : GetTok(3, winorg, ','));
   winpos += "," + (state == wsNormal ? IntToStr(Height) : GetTok(3, winorg, ','));

   ini->WriteString(StatsIni, "window", winpos);
   ini->WriteString(StatsIni, "lastproxy", ProxyBox->Text);

   ini->WriteInteger(StatsIni, QSWINDOWCOLOR, WindowColor);
   ini->WriteString(StatsIni, QSWINDOWFONT, WinsFontStr);
   ini->WriteString(StatsIni, QSEDITORFONT, EditFontStr);
   ini->WriteString(StatsIni, QSDIALOGFONT, DlgsFontStr);

   TStringList *ticked = new TStringList();
   if (UpdateMlistTxtItem->Checked) ticked->Add("mlist.txt");
   if (UpdatePubringMixItem->Checked) ticked->Add("pubring.mix");
   if (UpdateRlistTxtItem->Checked) ticked->Add("rlist.txt");
   if (UpdatePubringAscItem->Checked) ticked->Add("pubring.asc");
   if (UpdateNymserverAscItem->Checked) ticked->Add(NymserverFile);
   if (UpdateMail2newsTxtItem->Checked) ticked->Add(Mail2newsFile);
   if (UpdateAdvisoryTxtItem->Checked) ticked->Add(AdvisoryFile);
   if (UpdateURLsTxtItem->Checked) ticked->Add(URLFile);
   ini->WriteString(StatsIni, "pages", ticked->CommaText);
   delete ticked;

   ini->WriteBool(StatsIni, "showstatusbar", StatusBar->Visible);
   ini->WriteBool(StatsIni, "showtoolbar", CoolBar->Visible);

   delete ini;
}
//---------------------------------------------------------------------------

void __fastcall TQSStatsWin4::AppMessage(tagMSG &Msg, bool &Handled)
{
   if (Msg.message == WM_QSAPPMESSAGE)
   {
      switch (Msg.wParam)
      {
         case QSMSGID_SHUTDOWN:
            Close();
            Handled = true;
            return;
         case QSMSGID_LOADPROXIES:
            Timer5->Enabled = true;
            Handled = true;
            return;
      }
   }
}
//---------------------------------------------------------------------------

void __fastcall TQSStatsWin4::Timer5Timer(TObject *)
{
   Timer5->Enabled = false;

   if (SecureModeMsg(this))
      return;
      
   TStringList *proxies = new TStringList();

   AnsiString proxy = ProxyBox->Text;
   Global->DB->GetProxies(proxies);
   ProxyBox->Items->Clear();
   ProxyBox->Sorted = true;
   Proxies->Clear();
   Proxies->Sorted = true;

   for (int i = 0; i < proxies->Count; i++)
   {
      AnsiString line = proxies->Strings[i];
      if (GetTok(ProxyHTTP, line, ProxyDiv) != " ")
      {
         ProxyBox->Items->Add(GetTok(0, line, ','));
         Proxies->Add(line);
      }
   }

   delete proxies;

   Proxies->Sorted = false;
   ProxyBox->Sorted = false;
   ProxyBox->Items->Insert(0, "<none>");
   Proxies->Insert(0, "<none>");

   int index = ProxyBox->Items->IndexOf(proxy);
   if (index < 0)
   {
      Show();
      ProxyBox->ItemIndex = 0;
      MsgBox(this,
        "The proxy you have been using for stats     \nretrieval--" + proxy +
        "--has been delete!",
         MT_WARN, MB_OK
      );
      return;
   }

   ProxyBox->ItemIndex = index;
}
//---------------------------------------------------------------------------

void __fastcall TQSStatsWin4::ProxyBoxChange(TObject *)
{
   if (SecureModeMsg(this))
      return;

   if (ProxyBox->Text == "<none>")
   {
      Proxy->Clear();
      return;
   }

   Global->DB->GetProxy(Proxies->Strings[ProxyBox->ItemIndex], Proxy);
}
//---------------------------------------------------------------------------

bool __fastcall TQSStatsWin4::Working()
{
   return CancelBtn->Enabled;
}
//---------------------------------------------------------------------------

void __fastcall TQSStatsWin4::DisplayStatus(const AnsiString &Msg)
{
   if (Panel3->Visible) Label1->Caption = Msg;
   else StatusBar->SimpleText = " " + Msg;
}
//---------------------------------------------------------------------------

void __fastcall TQSStatsWin4::MoveSource(const AnsiString &Source, int Position)
{
   TStringList *AllURLs = new TStringList();
   TStringList *SourceURLs = new TStringList();
   if (FileExists(PathName(URLFile)))
      AllURLs->LoadFromFile(PathName(URLFile));

   int i;
   AnsiString line;
   bool GotURLs = false;
   i = -1; while (++i < AllURLs->Count)
   {
      line = AllURLs->Strings[i];
      if (line == "[" + Source + "]")
      {
         SourceURLs->Add(line);
         AllURLs->Delete(i);

         while (i < AllURLs->Count)
         {
            line =  AllURLs->Strings[i];

            if (line.Pos("[") == 1)
            {
               GotURLs = true;
               break;
            }

            SourceURLs->Add(line);
            AllURLs->Delete(i);
         }

         if (GotURLs)
            break;
      }
      else i++;
   }

   int srcpos = -1;
   i = -1; while (++i < AllURLs->Count)
   {
      line = AllURLs->Strings[i];
      if (line.Pos("[") == 1)
      {
         srcpos++;
         if (srcpos == Position)
         {
            for (int x = 0; x < SourceURLs->Count; x++)
            {
               AllURLs->Insert(i + x, SourceURLs->Strings[x]);
            }
            break;
         }
      }
   }

   try {
      AllURLs->SaveToFile(PathName(URLFile));
   }
   catch (...)
   {
      MsgBox(this,
        "I was unable to save " + URLFile + ". Possibly you don't have file permissions.     ",
         MT_ERROR, MB_OK);
   }

   delete SourceURLs;
   delete AllURLs;
}
//---------------------------------------------------------------------------

AnsiString __fastcall TQSStatsWin4::GetLastSource()
{
   return Global->DB->GetIni("statslastsource", "");
}
//---------------------------------------------------------------------------

void __fastcall TQSStatsWin4::PutLastSource(const AnsiString &Source)
{
   Global->DB->PutIni("statslastsource", Source);
}
//---------------------------------------------------------------------------

void __fastcall TQSStatsWin4::SourceBoxChange(TObject*)
{
   if (MsgBox(this, "Update remailer pages now?     ", MT_CONFIRM, MB_OKCANCEL) == IDCANCEL)
      return;

   DisplayStatus("");
   TStringList *pages = new TStringList();
   TStringList *enabled = new TStringList();

   LoadPAGEs(SourceBox->Text, pages);

   int i = 0; while (i < pages->Count)
   {
      if (!GetPageEnabled(pages->Strings[i]))
         pages->Delete(i);
      else
         i++;
   }

   if (!CheckAddKeyrings(this, pages, GetLastSource(), SourceBox->Text))
   {
      delete enabled;
      delete pages;
      return;
   }

   UpdatePagesSource(SourceBox->Text, pages);
   PutLastSource(SourceBox->Text);

   delete enabled;
   delete pages;
}
//---------------------------------------------------------------------------

void __fastcall TQSStatsWin4::UpdateMenuClick(TObject *)
{
   TStringList *pages = new TStringList();

   LoadPAGEs(SourceBox->Text, pages);
   UpdateMlistTxtItem->Enabled = pages->IndexOf("mlist.txt") >= 0;
   UpdatePubringMixItem->Enabled = pages->IndexOf("pubring.mix") >= 0;
   UpdateRlistTxtItem->Enabled = pages->IndexOf("rlist.txt") >= 0;
   UpdatePubringAscItem->Enabled = pages->IndexOf("pubring.asc") >= 0;

   LoadPAGEs("quicksilver", pages);
   UpdateNymserverAscItem->Enabled = pages->IndexOf(NymserverFile) >= 0;
   UpdateMail2newsTxtItem->Enabled = pages->IndexOf(Mail2newsFile) >= 0;
   UpdateAdvisoryTxtItem->Enabled = pages->IndexOf(AdvisoryFile) >= 0;

   int c = TabSet->Tabs->Count;
   UpdateCloseAllItem->Enabled = c > 0;
   UpdatePrintItem->Enabled = c > 0;
   UpdateSaveItem->Enabled = SaveBtn->Enabled;

   delete pages;
}
//---------------------------------------------------------------------------

void __fastcall TQSStatsWin4::ViewMenuClick(TObject *)
{
   ViewToolBarItem->Checked = CoolBar->Visible;
   ViewStatusBarItem->Checked = StatusBar->Visible;

   ViewMlistTxtItem->Enabled = FileExists(PathName("mlist.txt"));
   ViewPubringMixItem->Enabled = FileExists(PathName("pubring.mix"));
   ViewRlistTxtItem->Enabled = FileExists(PathName("rlist.txt"));
   ViewPubringAscItem->Enabled = FileExists(PathName("pubring.asc"));
   ViewNymserverAscItem->Enabled = FileExists(PathName(NymserverFile));
   ViewMail2newsTxtItem->Enabled = FileExists(PathName(Mail2newsFile));
   ViewAdvisoryTxtItem->Enabled = FileExists(PathName(AdvisoryFile));

   ViewAllPagesItem->Enabled =
      ViewMlistTxtItem->Enabled ||
      ViewPubringMixItem->Enabled ||
      ViewRlistTxtItem->Enabled ||
      ViewPubringAscItem->Enabled ||
      ViewNymserverAscItem->Enabled ||
      ViewAdvisoryTxtItem->Enabled ||
      ViewURLsTxtItem->Enabled;
}
//---------------------------------------------------------------------------

void __fastcall TQSStatsWin4::SourceMenuItemClick(TObject *Sender)
{
   TMenuItem *item = dynamic_cast<TMenuItem*>(Sender);
   SourceBox->ItemIndex = SourceBox->Items->IndexOf(SansAmpersand(item->Caption));
   SourceBoxChange(NULL);
}
//---------------------------------------------------------------------------

void __fastcall TQSStatsWin4::ProxyMenuItemClick(TObject *Sender)
{
   TMenuItem *item = dynamic_cast<TMenuItem*>(Sender);
   for (int i = 0; i < ProxyBox->Items->Count; i++)
   {
      if (GetTok(0, ProxyBox->Items->Strings[i], ',') == SansAmpersand(item->Caption))
      {
         ProxyBox->ItemIndex = i;
         break;
      }
   }
   ProxyBoxChange(NULL);
}
//---------------------------------------------------------------------------

void __fastcall TQSStatsWin4::ToolsMenuClick(TObject *)
{
   ToolsBarRemailerItem->Visible = true;
   ToolsProxyItem->Enabled = FileExists(AppDir() + "qsproxy.exe");
}
//---------------------------------------------------------------------------

void __fastcall TQSStatsWin4::LoadSourceConfig(TIniFile *ini, const AnsiString &Source)
{
   assert(ini != NULL);

   SourceBox->Clear();
   TStringList *urlfile = new TStringList();

   if (FileExists(PathName(URLFile)))
      urlfile->LoadFromFile(PathName(URLFile));
   for (int i = 0; i < urlfile->Count; i++)
   {
      AnsiString line = urlfile->Strings[i];
      if (line.Pos("[") == 1 && line.LowerCase().Pos("[quicksilver]") != 1)
         SourceBox->Items->Add(line.SubString(2, line.Pos("]") - 2));
   }

   delete urlfile;

   int index = SourceBox->Items->IndexOf(Source);
   if (index < 0 || index >= SourceBox->Items->Count)
      index = 0;

   SourceBox->ItemIndex = index;
}
//---------------------------------------------------------------------------

int __fastcall TQSStatsWin4::PageNo(const AnsiString &Page)
{
   int i = 0; while (i < TabSet->Tabs->Count)
   {
      if (TabSet->Tabs->Strings[i] == Page)
         return i;
      i++;
   }
   return -1;
}
//---------------------------------------------------------------------------

void __fastcall TQSStatsWin4::GetPage(const AnsiString &Page, TStrings *Text)
{
   Text->Clear();
   int i = 0; while (i < TabSet->Tabs->Count)
   {
      if (TabSet->Tabs->Strings[i] == Page)
      {
         TRemailerDoc* Doc = TabRemailerDoc(i);
         Text->AddStrings(Doc->Lines);
         return;
      }
      i++;
   }
}
//---------------------------------------------------------------------------

int __fastcall TQSStatsWin4::GetPagesEnabled(TStrings *Pages)
{
   int i = 0; while (i < UpdateMenu->Count)
   {
      TMenuItem *item = UpdateMenu->Items[i];
      if (item->Checked)
         Pages->Add(SansAmpersand(item->Caption));
      i++;
   }
   return Pages->Count;
}
//---------------------------------------------------------------------------

bool __fastcall TQSStatsWin4::GetPageEnabled(const AnsiString &Page)
{
   assert(Page != "");

   int i = 0; while (i < UpdateMenu->Count)
   {
      TMenuItem *item = UpdateMenu->Items[i];
      if (SansAmpersand(item->Caption) == Page)
         return item->Checked;

      i++;
   }
   return false;
}
//---------------------------------------------------------------------------

void __fastcall TQSStatsWin4::SetPageEnabled(const AnsiString &Page, bool Enabled)
{
   assert(Page != "");
   int i = 0; while (i < UpdateMenu->Count)
   {
      TMenuItem *item = UpdateMenu->Items[i];
      if (SansAmpersand(item->Caption) == Page)
      {
         item->Checked = Enabled;
         TIniFile *ini = new TIniFile(AppIni());
         TStringList *pages = new TStringList();

         pages->CommaText = ini->ReadString(StatsIni, "pages", "");
         pages->Values[Page] = IntToStr(Enabled);
         ini->WriteString(StatsIni, "pages", pages->CommaText);

         delete pages;
         delete ini;
         return;
      }
      i++;
   }
}
//---------------------------------------------------------------------------

void __fastcall TQSStatsWin4::FileExitClick(TObject *)
{
   Close();
}
//---------------------------------------------------------------------------

AnsiString __fastcall TQSStatsWin4::GetSource(int index)
{
   TStringList *urls = new TStringList();
   if (FileExists(PathName(URLFile)))
      urls->LoadFromFile(PathName(URLFile));

   int sidx = -1;
   AnsiString source;
   int i = 0; while (i < urls->Count)
   {
      AnsiString url = urls->Strings[i].Trim();
      if (url.Pos("[") == 1)
      {
         if (++sidx == index)
         {
            source = url.SubString(2, url.Length() - 2);
            break;
         }
      }

      i++;
   }
   delete urls;

   return source;
}
//---------------------------------------------------------------------------

void __fastcall TQSStatsWin4::LoadURLs(const AnsiString &Source, TStrings *URLs)
{
   URLs->Clear();

   TStringList *urlfile = new TStringList();
   if (FileExists(PathName(URLFile)))
      urlfile->LoadFromFile(PathName(URLFile));

   int i = 0; while (i < urlfile->Count)
   {
      AnsiString URL = urlfile->Strings[i].Trim();
      if (URL.Pos("[" + Source + "]") == 1)
      {
         while (++i < urlfile->Count)
         {
            AnsiString line = urlfile->Strings[i].Trim();
            if (line.IsEmpty() || line.Pos("[") == 1)
               break;

            if (line.Pos(" ") > 0)
               URLs->Add(line);
         }
      }
      i++;
   }

   delete urlfile;
}
//---------------------------------------------------------------------------

void __fastcall TQSStatsWin4::LoadPAGEs(const AnsiString &Source, TStrings *Pages)
{
   AnsiString page;
   LoadURLs(Source, Pages);
   TrimNulls(Pages);

   for (int i = 0; i < Pages->Count; i++)
   {
      page = Pages->Strings[i];
      int pos = page.Pos(" ");
      assert(pos > 0);

      Pages->Strings[i] = page.SubString(1, pos - 1);
   }

   TrimNulls(Pages);
}
//---------------------------------------------------------------------------

void __fastcall TQSStatsWin4::LoadPAGENames(const AnsiString &Source, TStrings *Pages)
{
   assert(Source != "" && Pages != NULL);

   LoadPAGEs(Source, Pages);
   int i = 0; while (i < Pages->Count)
   {
      if (GetPageEnabled(Pages->Strings[i])) i++;
      else Pages->Delete(i);
   }

   TrimNulls(Pages);
}
//---------------------------------------------------------------------------

TRemailerDoc* __fastcall TQSStatsWin4::TabRemailerDoc(int index)
{
   assert(index >= 0 && index < TabSet->Tabs->Count);
   TRemailerDoc *Doc = dynamic_cast<TRemailerDoc*>(TabSet->Tabs->Objects[index]);
   assert(Doc != NULL);
   return Doc;
}
//---------------------------------------------------------------------------

TRemailerDoc* __fastcall TQSStatsWin4::NewRemailerDoc(const AnsiString &Source,
   AnsiString Name, TStrings *Lines, bool Status)
{
   assert(Name != "" && Lines != NULL);
   AnsiString name = Name.LowerCase();
   if (name == "mlist.txt")
   {
      mlistorg->Clear();
      mlistorg->AddStrings(Lines);
      RemoveBarredRemailers("mlist.txt", Lines);
   }

   if (name == "rlist.txt")
   {
      rlistorg->Clear();
      rlistorg->AddStrings(Lines);
      RemoveBarredRemailers("rlist.txt", Lines);
   }

   if (name == "pubring.mix")
   {
      rlistorg->Clear();
      rlistorg->AddStrings(Lines);
      RemoveBarredRemailers("pubring.mix", Lines);
   }

   //GetNewRemailers
   TRichEdit *ed = new TRichEdit((void*)NULL);
   ed->Left = -2000;
   ed->Visible = false;
   ed->Parent = TabSet;
   ed->Width = 1600;
   ed->Align = alClient;
   ed->PlainText = true;
   StrToFont(WinsFontStr, ed->Font);
   ed->Color = WindowColor;
   ed->WordWrap = name == AdvisoryFile; // || name == "advisory.html";
   ed->ScrollBars = ssBoth;
   ed->HideSelection = true;
   if (Lines) ed->Lines->AddStrings(Lines);
   ed->SelStart = 0;
   ed->Perform(EM_SCROLLCARET, 0, 0);

   TRemailerDoc *Doc;
   if (name == "mlist.txt")   Doc = new TMixMListTxt(Source,    ed); else
   if (name == "pubring.mix") Doc = new TMixPubringMix(Source,  ed); else
   if (name == "type2.list")  Doc = new TMixType2List(Source,   ed); else
   if (name == "rlist.txt")   Doc = new TCpkRListTxt(Source,    ed); else
   if (name == "pubring.asc") Doc = new TCpkPubringAsc(Source,  ed); else
   if (name == AdvisoryFile)  Doc = new TQSAdvisoryTxt(Source,  ed); else
   if (name == URLFile)       Doc = new TQSUrlsTxt(Source,      ed); else
   if (name == NymserverFile) Doc = new TQSNymServerAsc(Source, ed); else
   if (name == Mail2newsFile) Doc = new TQSMail2newsTxt(Source, ed);
   else
   {
      Doc = new TRemailerDoc(Source, ed);
      Doc->Name = name;
   }
   Doc->ok = Status;

   if (Doc->ok && !Doc->IsValid())
   {
      ed->Lines->Insert(0, "# " + FailedChecks);
      ed->Lines->Insert(1, "#");
      if ((Doc->Name == "mlist.txt" || Doc->Name == "rlist.txt") && Doc->IsVersion2Stats())
         ed->Lines->Insert(1, "# This is a version 2 stats page. QS requires version 1.");
   }

   ed->Modified = true;
   ed->ReadOnly = true;
   ed->SendToBack();
   ed->Visible = true;
   Snooze(20);

   return Doc;
}

//---------------------------------------------------------------------------

TRemailerDoc* __fastcall TQSStatsWin4::NewLocalRemailerDoc(const AnsiString &Source,
   const AnsiString &FileName, TStrings *Lines)
{
   assert(FileName != "" && Lines != NULL);
   AnsiString name = ExtractFileName(FileName).LowerCase();
   TRichEdit *ed = new TRichEdit((void*)NULL);
   ed->Left = -2000;
   ed->Visible = false;
   ed->Parent = TabSet;
   ed->Width = 1600;
   ed->Align = alClient;
   ed->PlainText = true;
   StrToFont(WinsFontStr, ed->Font);
   ed->Color = WindowColor;
   ed->WordWrap = name == "advisory.txt" || name == "advisory.html";
   ed->ScrollBars = ssBoth;
   ed->HideSelection = true;
   if (Lines) ed->Lines->AddStrings(Lines);
   ed->SelStart = 0;
   ed->Perform(EM_SCROLLCARET, 0, 0);

   TRemailerDoc *Doc;
   if (name == "mlist.txt")    Doc = new TMixMListTxt(Source,    ed); else
   if (name == "pubring.mix")  Doc = new TMixPubringMix(Source,  ed); else
   if (name == "type2.list")   Doc = new TMixType2List(Source,   ed); else
   if (name == "rlist.txt")    Doc = new TCpkRListTxt(Source,    ed); else
   if (name == "pubring.asc")  Doc = new TCpkPubringAsc(Source,  ed); else
   if (name == "advisory.txt") Doc = new TQSAdvisoryTxt(Source,  ed); else
   if (name == "urls.txt")     Doc = new TQSUrlsTxt(Source,      ed); else
   if (name == NymserverFile)  Doc = new TQSNymServerAsc(Source, ed); else
   if (name == Mail2newsFile)  Doc = new TQSMail2newsTxt(Source, ed);
   else
   {
      Doc = new TRemailerDoc(Source, ed);
      Doc->Name = name;
   }

   ed->Modified = false;
   ed->ReadOnly = true;
   ed->SendToBack();
   ed->Visible = true;
   //Snooze(20);

   return Doc;
}
//---------------------------------------------------------------------------

bool __fastcall TQSStatsWin4::GetLocalDocument(const AnsiString &DocName)
{
   assert(DocName != "");

   AnsiString name = ExtractFileName(DocName);
   // already open?
   bool exists = false;
   for (int i = 0; i < TabSet->Tabs->Count; i++)
   {
      TRemailerDoc *Doc = TabRemailerDoc(i);
      if (Doc->Source == LocalFile && Doc->Name == name)
      {
         DisplayStatus(Doc->Source + ": " + Doc->Name);
         Doc->ed->BringToFront();
         TabSet->TabIndex = i;
         exists = true;
      }
   }

   if (exists) return true;

   TStringList *doc = new TStringList();

   AnsiString FileName = DocName;
   if (FileExists(FileName))
      doc->LoadFromFile(FileName);
   TrimNulls(doc);

   AnsiString tabname = name.LowerCase();
   if (tabname == "status.txt")
      tabname = "status";

   TabSet->Tabs->AddObject(tabname, NewLocalRemailerDoc(LocalFile, name, doc));

   DisplayStatus(LocalFile + ": " + name);
   PrintBtn->Enabled = true;

   delete doc;

   return true;
}
//---------------------------------------------------------------------------

int __fastcall TQSStatsWin4::GetLocalDocumentTab(const AnsiString &DocName)
{
   assert(DocName != "");

   AnsiString name = ExtractFileName(DocName);
   // already open?
   bool exists = false;
   for (int i = 0; i < TabSet->Tabs->Count; i++)
   {
      TRemailerDoc *Doc = TabRemailerDoc(i);
      if (Doc->Source == LocalFile && Doc->Name == name)
      {
         DisplayStatus(Doc->Source + ": " + Doc->Name);
         Doc->ed->BringToFront();
         TabSet->TabIndex = i;
         exists = true;
      }
   }

   if (exists)
      return true;

   TStringList *doc = new TStringList();

   AnsiString FileName = DocName;
   if (FileExists(FileName))
      doc->LoadFromFile(FileName);
   TrimNulls(doc);

   int index = TabSet->Tabs->AddObject(name, NewLocalRemailerDoc(LocalFile, name, doc));

   DisplayStatus(LocalFile + ": " + name);

   delete doc;

   return index;
}
//---------------------------------------------------------------------------

void __fastcall TQSStatsWin4::ClearTabs()
{
   for (int i = 0; i < TabSet->Tabs->Count; i++)
      delete TabRemailerDoc(i);
   TabSet->Tabs->Clear();

   TIniFile *ini = new TIniFile(AppIni());
   LoadSourceConfig(ini, SourceBox->Text);
   delete ini;

   PrintBtn->Enabled = false;
   SaveBtn->Enabled = false;
}
//---------------------------------------------------------------------------

void __fastcall TQSStatsWin4::TabSetChange(TObject *)
{
   TRemailerDoc *Doc = TabRemailerDoc(TabSet->TabIndex);
   Doc->ed->BringToFront();
   DisplayStatus(Doc->Source + ": " + Doc->Name);
}
//---------------------------------------------------------------------------

AnsiString __fastcall TQSStatsWin4::GetURL(const AnsiString &Source, const AnsiString &Page)
{
   AnsiString line, url;
   TStringList *urls = new TStringList();

   LoadURLs(Source, urls);
   for (int i = 0; i < urls->Count; i++)
   {
      AnsiString line = urls->Strings[i];
      if (line.LowerCase().Pos(Page + " ") == 1)
      {
         url = line.SubString(Page.Length() + 1, line.Length()).Trim();
         break;
      }
   }

   if (url.IsEmpty())
   {
      MsgBox(this,
         "I don't have a URL for " + Page + " at " + Source + "!",
          MT_INFO,
          MB_OK);
   }

   delete urls;
   return url;
}
//---------------------------------------------------------------------------

void __fastcall TQSStatsWin4::OpenLocalFiles()
{
   Cancel = false;
   DisplayStatus("Loading...");
   RunBtn->Enabled = false;
   SaveBtn->Enabled = false;
   CancelBtn->Enabled = true;

   if (FileExists(PathName("status.txt")))
      GetLocalDocument(PathName("status.txt"));
   if (FileExists(PathName("mlist.txt")))
      GetLocalDocument(PathName("mlist.txt"));
   if (FileExists(PathName("pubring.mix")))
      GetLocalDocument(PathName("pubring.mix"));
   if (FileExists(PathName("rlist.txt")))
      GetLocalDocument(PathName("rlist.txt"));
   if (FileExists(PathName("pubring.asc")))
      GetLocalDocument(PathName("pubring.asc"));

   if (FileExists(PathName(NymserverFile)))
      GetLocalDocument(PathName(NymserverFile));
   if (FileExists(PathName(Mail2newsFile)))
      GetLocalDocument(PathName(Mail2newsFile));
   if (FileExists(PathName("advisory.txt")))
      GetLocalDocument(PathName("advisory.txt"));

   DisplayStatus("local files loaded");
   CancelBtn->Enabled = false;
   RunBtn->Enabled = true;

   Label1->Show();
}
//---------------------------------------------------------------------------

bool __fastcall TQSStatsWin4::CanCascade(const AnsiString &PageName)
{
   bool juststats, cascade, cancascade;

   TIniFile *ini = new TIniFile(AppIni());
   cascade = ini->ReadBool(StatsIni, "cascade", false);
   juststats = ini->ReadBool(StatsIni, "cascadestatsonly", true);
   delete ini;

   cancascade = cascade;
   if (cancascade) cancascade = !juststats ||
      juststats && (PageName == "mlist.txt" || PageName == "rlist.txt");

   return cancascade;
}
//---------------------------------------------------------------------------

void __fastcall TQSStatsWin4::AddToLast(TStrings *Feedback, const AnsiString &text)
{
   assert(Feedback != NULL);

   int last = Feedback->Count - 1;
   Feedback->Strings[last] = Feedback->Strings[last] + text;
}
//---------------------------------------------------------------------------

TRichEdit* __fastcall TQSStatsWin4::TabEditor(int index)
{
   TRemailerDoc *doc = TabRemailerDoc(index);
   return doc->ed;
}
//---------------------------------------------------------------------------

void __fastcall TQSStatsWin4::SetEditorHideSelection(bool hide)
{
   for (int i = 0; i < TabSet->Tabs->Count; i++)
   {
      TRichEdit *ed = TabEditor(i);
      ed->HideSelection = hide;
   }
}
//---------------------------------------------------------------------------

int __fastcall TQSStatsWin4::GetRemailerPages(const AnsiString &Source,
   TStrings *pages, TRemailerDoc *status, TStrings *report, int timeoutsecs)
{
   assert(pages != NULL && status != NULL);
   TStrings *Feedback = status->Lines;

   AnsiString name, source, nxtsource;
   TStringList *page = new TStringList();
   bool needsmixreport = false;
   bool needscpkreport = false;
   bool cascade = false;
   result = HTTP_START;

   for (int i = 0; i < pages->Count; i++)
   {
      if (Cancel)
         break;

      name = pages->Strings[i];
      if (cascade && (name == "pubring.mix" || name == "pubring.asc"))
         continue;

      nxtsource = Source;

      bool success = false;
      bool checkdate = true;
      int idx = 0;

      result = HTTP_START;
      while (result != HTTP_OK)
      {
         AnsiString URL = GetURL(nxtsource, name);
         if (URL.IsEmpty())
            continue;

         Feedback->Add(URL + "...");

         result = GetRemailerPage(nxtsource, URL, name, page, checkdate, timeoutsecs);
         Snooze(100);

         if (Cancel)
            break;

         bool abort = false;
         success = false;
         switch (result)
         {
            case HTTP_OK:
               if (name == "mlist.txt") needsmixreport = true;
               if (name == "rlist.txt") needscpkreport = true;
               AddToLast(Feedback, "ok");
               success = true;
               break;

            case HTTP_NOTMODIFIED:
               AddToLast(Feedback, "good");
               success = true;
               break;

            case HTTP_ICSEXCEPTION:
               AddToLast(Feedback, "encountered HTTP library exception");
               abort = true;
               break;

            case HTTP_EXCEPTION:
               AddToLast(Feedback, "encountered HTTP exception");
               abort = true;
               break;

            case HTTP_BOGUSPAGE:
               AddToLast(Feedback, "failed error checks");
               if (page->Text.Pos("Stats-Version: 2.0") == 1)
               {
                  Feedback->Add("Wrong stats version. Got version 2. Needed version 1!");
                  Feedback->Add("Click 'Help|noreply.org|all pingers' for all URLs.");
                  Feedback->Add("");
               }
               break;

            case HTTP_TIMEOUT:
               AddToLast(Feedback, "timed out");
               break;

            case HTTP_ERROR:
               AddToLast(Feedback, "http error");
               Feedback->AddStrings(page);
               break;

            default:
               AddToLast(Feedback, "unknown error");
               Feedback->AddStrings(page);
         }

         status->ed->Perform(EM_SCROLLCARET, 0, 0);

         if (success)
            break;

         if (!AutoRun || abort)
            break;

         nxtsource = GetSource(++idx);
         if (nxtsource.LowerCase() == "quicksilver")
            nxtsource = GetSource(++idx);

         if (nxtsource.IsEmpty())
         {
            Feedback->Add("There is something wrong and it is not your stats source!");
            Feedback->Add("Sorry, I failed completely!");
            break;
         }

       //Feedback->Add("trying " + nxtsource + " " + name + "...");
         Feedback->Add("next source...");

         result = HTTP_START;
         checkdate = false;
         cascade = true;
      }

      if (Cancel)
         break;
   }

   //if (!success)
   //   ReportHTTPError(report);

   if (needsmixreport)
      ReportNewMixmasterRemailers(report);

   if (needscpkreport)
      ReportNewCypherpunkRemailers(report);

   TabSet->Enabled = true;
   delete page;

   if (result == HTTP_START)
      result = HTTP_OK;

   PrintBtn->Enabled = true;

   return result;
}
//---------------------------------------------------------------------------

/*int __fastcall TQSStatsWin4::GetRemailerPages(const AnsiString &Source,
   TStrings *pages, TStrings *Feedback, TStrings *report)
{
   assert(pages != NULL && Feedback != NULL);

   AnsiString name, source, nxtsource;
   TStringList *page = new TStringList();
   bool needsmixreport = false;
   bool needscpkreport = false;
   bool cascade = false;
   result = HTTP_START;

   for (int i = 0; i < pages->Count; i++)
   {
      if (Cancel)
         break;

      name = pages->Strings[i];
      if (cascade && (name == "pubring.mix" || name == "pubring.asc"))
         continue;

      AnsiString URL = GetURL(Source, name);
      Feedback->Add(URL + "...");
      nxtsource = Source;

      bool checkdate = true;
      int idx = 0;

      result = HTTP_START;
      while (result != HTTP_OK)
      {
         result = GetRemailerPage(nxtsource, URL, name, page, checkdate);
         if (Cancel)
            break;

         switch (result)
         {
            case HTTP_ICSEXCEPTION:
               AddToLast(Feedback, "encountered HTTP library exception");
               break;
            case HTTP_EXCEPTION:
               AddToLast(Feedback, "encountered HTTP exception");
               break;
            case HTTP_TIMEOUT:
               AddToLast(Feedback, "timed out");
               break;
            default:
               assert(page->Count > 0);
               AddToLast(Feedback, page->Strings[0]);
         }

         if (result == HTTP_OK)
         {
            if (name == "mlist.txt")
               needsmixreport = true;

            if (name == "rlist.txt")
               needscpkreport = true;

            AddToLast(Feedback, "ok");
            break;
         }

         if (result == HTTP_NOTMODIFIED)
         {
            AddToLast(Feedback, "good");
            break;
         }

         if (result == HTTP_BOGUSPAGE)
         {
            AddToLast(Feedback, "failed error checks");
            if (page->Text.Pos("Stats-Version: 2.0") == 1)
            {
               Feedback->Add("Wrong stats version. Got version 2. Needed version 1!");
               Feedback->Add("Click 'Help|noreply.org|all pingers' for all URLs.");
               Feedback->Add("");
            }
            break;
         }

         if (!AutoRun)
            break;

         nxtsource = GetSource(++idx);
         if (nxtsource.IsEmpty())
         {
            Feedback->Add("Failed completely!");
            break;
         }

         Feedback->Add("trying " + nxtsource + " " + name + "...");

         checkdate = false;
         cascade = true;
      }

      if (Cancel)
         break;

   }

   if (needsmixreport)
      ReportNewMixmasterRemailers(report);

   if (needscpkreport)
      ReportNewCypherpunkRemailers(report);

   TabSet->Enabled = true;
   delete page;

   if (result == HTTP_START)
      result = HTTP_OK;

   PrintBtn->Enabled = true;

   return result;
}*/
//---------------------------------------------------------------------------

int __fastcall TQSStatsWin4::GetRemailerPage(const AnsiString &Source, const AnsiString &URL,
   const AnsiString &PageName, TStrings *Page, bool CheckDate, int timeoutsecs)
{
   assert(Source != "" && URL != "" && PageName != "" && Page != NULL);

   TDateTime datetime = 0;
   if (AutoRun && CheckDate && !AutoRunFiles)
      datetime = Global->DB->GetPageDate(PageName);

   result = HTTPGet(this, URL, Page, ProgressBar, Proxy, &Cancel, timeoutsecs, datetime);

   if (result == HTTP_OK)
   {
      TRemailerDoc *Doc = NewRemailerDoc(Source, PageName, Page, true);
      Doc->ModifiedDate = AnsiString((double)datetime);
      if (!Doc->IsValid())
         result = HTTP_BOGUSPAGE;

      TabSet->Tabs->AddObject(PageName, Doc);
   }

   return result;
}
//---------------------------------------------------------------------------

int __fastcall TQSStatsWin4::GetQuickSilverPage(const AnsiString &Source, const AnsiString &URL,
   const AnsiString &PageName, TStrings *Page, int timeoutsecs)
{
   assert(Source != "" && PageName != "" && Page != NULL);

   TDateTime datetime = 0;
   if (AutoRun)
      datetime = Global->DB->GetPageDate(PageName); // getpagedate converts getpagedatestr to tdatetime

   result = HTTPGet(this, URL, Page, ProgressBar, Proxy, &Cancel, timeoutsecs, datetime);

   if (result == HTTP_OK)
   {
      TRemailerDoc *Doc = NewRemailerDoc(Source, PageName, Page, result == HTTP_OK);
      Doc->ModifiedDate = AnsiString((double)datetime);
      if (!Doc->IsValid())
         result = HTTP_BOGUSPAGE;

      TabSet->Tabs->AddObject(PageName, Doc);
   }

   return result;
}
//---------------------------------------------------------------------------

int __fastcall TQSStatsWin4::GetQuickSilverPages(TStrings *Pages, TRemailerDoc *doc, TStrings *Report,
   bool &NewAdvisory, int timeoutsecs)
{
   assert(Pages != NULL && doc != NULL && Report!= NULL);

   TStrings *Feedback = doc->Lines;
   TStringList *page = new TStringList();

   result = HTTP_START;
   AnsiString name, resultmsg;
   bool abort = false;

   for (int i = 0; i < Pages->Count; i++)
   {
      name = Pages->Strings[i];
      AnsiString URL = GetURL(QuikSource, name);
      if (URL.IsEmpty())
         continue;

      Feedback->Add(URL + "...");

      result = GetQuickSilverPage(QuikSource, URL, name, page, timeoutsecs);

      if (Cancel)
         break;

      switch (result)
      {
         case HTTP_OK:
            AddToLast(Feedback, "ok");
            if (name == "advisory.txt")
            {
               if (AutoRun)
               {
                  Report->Add("");
                  Report->AddStrings(page);
                  Report->Delete(Report->Count - 1);
               }
               else
               {
                  TrimNullsAfter(page);

                  AnsiString datetimestr = Global->DB->GetPageDateStr("advisory.txt");
                  AnsiString datemodified = TabRemailerDoc(TabSet->Tabs->Count - 1)->ModifiedDate;
                  if (datetimestr != datemodified)
                     NewAdvisory = true;
               }
            }
            break;

         case HTTP_NOTMODIFIED:
            AddToLast(Feedback, "good");
            break;

         case HTTP_ICSEXCEPTION:
            AddToLast(Feedback, "encountered HTTP library exception");
            abort = true;
            break;

         case HTTP_EXCEPTION:
            AddToLast(Feedback, "encountered HTTP exception");
            abort = true;
            break;

         case HTTP_BOGUSPAGE:
            AddToLast(Feedback, "failed error checks");
            break;

         case HTTP_TIMEOUT:
            AddToLast(Feedback, "timed out");
            break;

         case HTTP_ERROR:
            AddToLast(Feedback, "HTTP error");
            Feedback->AddStrings(page);
            break;

         default:
            AddToLast(Feedback, "error");
            Feedback->AddStrings(page);
      }
      if (abort)
         break;

      doc->ed->Perform(EM_SCROLLCARET, 0, 0);
   }

   delete page;

   if (result == HTTP_START)
      result = HTTP_OK;

   PrintBtn->Enabled = true;

   return result;
}
//---------------------------------------------------------------------------
#pragma argsused // qsource
void __fastcall TQSStatsWin4::UpdatePages(const AnsiString &psource, TStrings *rempages,
   const AnsiString &qsource, TStrings *qicpages)
{
   if (SecureModeMsg(this))
      return;

   Cancel = false;
   DisplayStatus("");
   OkBtn->Enabled = false;
   RunBtn->Enabled = false;
   SaveBtn->Enabled = false;
   CancelBtn->Enabled = true;

   TIniFile *ini = new TIniFile(AppIni());
   int timeoutsecs = ini->ReadInteger(StatsIni, "timeoutseconds", DefHTTPTimeOut);
   delete ini;

   TabSet->Enabled = false;
   if (Visible)
      SetRedraw(this, 0);

   ClearTabs();

   TStringList *doc = new TStringList();
   TRemailerDoc *status = NewRemailerDoc("download", "status.txt", doc, true);

   status->Lines->Add(FormatDateTime("ddd mmm dd hh:nn:ss yyyy", Now()));

   TabSet->Tabs->AddObject("status", status);
   delete doc;

   if (Visible)
      SetRedraw(this, 1);

   DisplayStatus("downloading pages...");

   TStringList *report = new TStringList();
   if (CoolBar->Visible) ProgressBar->Show();

   bool newadvisory = false;

   if (!Cancel)
      GetRemailerPages(psource, rempages, status, report, timeoutsecs);
   if (!Cancel && (result == HTTP_OK || result == HTTP_NOTMODIFIED))
      GetQuickSilverPages(qicpages, status, report, newadvisory, timeoutsecs);

   if (report->Count > 0)
      status->Lines->AddStrings(report);

   ProgressBar->Hide();

   TStrings *Feedback = status->Lines;
   AnsiString text = Feedback->Text.LowerCase();

   Feedback->Add("");
   if (Cancel)
      Feedback->Add("operation cancelled by user");

 //if (result != HTTP_OK && result != HTTP_NOTMODIFIED && result != HTTP_CANCEL)
 //   Feedback->Add("http error");

   DisplayStatus("Update complete!");

   CancelBtn->Enabled = false;
   RunBtn->Enabled = true;
   OkBtn->Enabled = true;

   if (ValidPages())
      SaveBtn->Enabled = true;

   TabSet->Enabled = true;

   if (!Cancel && report->Count > 0 && AutoRun)
   {
      status->Lines->SaveToFile(AppDir() + "report.txt");
    //MessageBeep(MB_OK);

      Exec95(AppDir() + "qsreports.exe " +
         "\"file=" + AppDir() + "report.txt\" " +
         "\"title=QuickSilver Update Report\" " +
         "left=" + IntToStr(Left + 10) + " " +
         "top=" + IntToStr(Top + 10) + " " +
         "height=auto " +
         "width=auto"
      );
   }

   if (newadvisory)
   {
      Snooze(200);
      BringPageToFront("advisory.txt");
   }

   SetEditorHideSelection(false);
 //sm("ok");
   delete report;
}
//---------------------------------------------------------------------------

void __fastcall TQSStatsWin4::UpdatePagesSource(const AnsiString &source, TStrings *rempages)
{
   if (SecureModeMsg(this))
      return;

   Cancel = false;
   DisplayStatus("");
   OkBtn->Enabled = false;
   RunBtn->Enabled = false;
   SaveBtn->Enabled = false;
   CancelBtn->Enabled = true;

   TIniFile *ini = new TIniFile(AppIni());
   int timeoutsecs = ini->ReadInteger(StatsIni, "timeoutseconds", DefHTTPTimeOut);
   delete ini;

   TabSet->Enabled = false;
   if (Visible)
      SetRedraw(this, 0);

   ClearTabs();

   TStringList *doc = new TStringList();
   TRemailerDoc *status = NewRemailerDoc("download", "status.txt", doc, true);

   status->Lines->Add(FormatDateTime("ddd mmm dd hh:nn:ss yyyy", Now()));

   TabSet->Tabs->AddObject("status", status);
   delete doc;

   if (Visible)
      SetRedraw(this, 1);

   DisplayStatus("downloading pages...");
   TStringList *report = new TStringList();

   if (CoolBar->Visible) ProgressBar->Show();
   if (!Cancel)
      GetRemailerPages(source, rempages, status, report, timeoutsecs);
   ProgressBar->Hide();

   TStrings *Feedback = status->Lines;
   AnsiString text = Feedback->Text.LowerCase();

   Feedback->Add("");
   if (Cancel) Feedback->Add("operation cancelled by user");
   DisplayStatus("Update complete!");

   CancelBtn->Enabled = false;
   RunBtn->Enabled = true;
   OkBtn->Enabled = true;

   if (ValidPages())
      SaveBtn->Enabled = true;

   TabSet->Enabled = true;

   if (!Cancel && report->Count > 0 && AutoRun)
   {
      report->SaveToFile(AppDir() + "report.txt");
      MessageBeep(MB_OK);
      Exec95(AppDir() + "qsreports.exe " +
         "\"file=" + AppDir() + "report.txt\" " +
         "\"title=QuickSilver Update Report\" " +
         "winpos=" +
            IntToStr(Left + 25) + "," +
            IntToStr(Top + 25 ) + "," +
            IntToStr(350) + "," +
            IntToStr(400)
      );
   }

   delete report;
}
//---------------------------------------------------------------------------

void __fastcall TQSStatsWin4::FileUpdateItemClick(TObject *)
{
   TStringList *pages = new TStringList();
   TStringList *ppages = new TStringList();
   TStringList *qpages = new TStringList();
   AnsiString PageSource = SourceBox->Text; //GetSource(0);

   LoadPAGENames(PageSource, ppages);
   LoadPAGENames(QuikSource, qpages);

   if (ppages->Count == 0 && qpages->Count == 0)
   {
      MsgBox(this, "No pages marked for update!     ", MT_INFO, MB_OK);
      delete qpages;
      delete ppages;
      delete pages;
      return;
   }

   // get pages in the order we desire
   if (ppages->IndexOf("mlist.txt") >= 0)
      pages->Add("mlist.txt");
   if (ppages->IndexOf("pubring.mix") >= 0)
      pages->Add("pubring.mix");
   if (ppages->IndexOf("rlist.txt") >= 0)
      pages->Add("rlist.txt");
   if (ppages->IndexOf("pubring.asc") >= 0)
      pages->Add("pubring.asc");

   if (!CheckAddKeyrings(this, pages, GetLastSource(), PageSource))
   {
      delete qpages;
      delete ppages;
      delete pages;
      return;
   }

   UpdatePages(PageSource, pages, QuikSource, qpages);
   PutLastSource(PageSource);

   delete qpages;
   delete ppages;
   delete pages;
}
//---------------------------------------------------------------------------

void __fastcall TQSStatsWin4::FileUpdateSelectItemClick(TObject *)
{
   DisplayStatus("");
   TStringList *pages = new TStringList();
   TStringList *ppages = new TStringList();
   TStringList *qpages = new TStringList();
   AnsiString PageSource, QuikSource, page;

   if (GetUpdateSelections(this, PageSource, ppages, QuikSource, qpages,
      PathName(URLFile)))
   {
      // get pages in the order we desire
      if (ppages->IndexOf("mlist.txt") >= 0)
         pages->Add("mlist.txt");
      if (ppages->IndexOf("pubring.mix") >= 0)
         pages->Add("pubring.mix");
      if (ppages->IndexOf("rlist.txt") >= 0)
         pages->Add("rlist.txt");
      if (ppages->IndexOf("pubring.asc") >= 0)
         pages->Add("pubring.asc");

      UpdatePages(PageSource, pages, QuikSource, qpages);
   }

   delete qpages;
   delete ppages;
   delete pages;
}
//---------------------------------------------------------------------------

void __fastcall TQSStatsWin4::FileUpdateAutoRunFiles()
{
   DisplayStatus("");
   AnsiString source, page, line;
   TStringList *pages = new TStringList();

   if (FileExists(PathName(URLFile)))
   {
      TStringList *urls = new TStringList();
      urls->LoadFromFile(PathName(URLFile));
      for (int i = 0; i < urls->Count; i++)
      {
         line = urls->Strings[i];
         if (line.Pos("[") == 1)
         {
            source = GetTok(0, GetTok(1, line, '['), ']');
            break;
         }
      }
   }

   if (source.IsEmpty())
   {
      MsgBox(this,
         "Sorry, I can't locate the default stats source in URLs.txt.   \n"
         "I can't continue with the update!",
          MT_INFO, MB_OK);
      AutoRun = false;
      delete pages;
      return;
   }

   if (!CheckAddKeyrings(this, pages, GetLastSource(), source))
   {
      delete pages;
      return;
   }

   UpdatePages(source, AutoRunFiles, QuikSource, pages);
   PutLastSource(source);

   delete pages;
}
//---------------------------------------------------------------------------

int __fastcall TQSStatsWin4::GetMlistNames(TStrings *mlist, TStringList *names)
{
   names->Clear();
   names->Sorted = true;

   AnsiString name;
   bool saving = false;
   for (int i = 0; i < mlist->Count; i++)
   {
      AnsiString line = mlist->Strings[i].Trim();
      if (line.IsEmpty() && saving)
         break;
      if (line.IsEmpty())
         continue;

      if (line.Pos("----------------") == 1)
      {
         saving = true;
         continue;
      }

      if (saving)
      {
         if (line.Pos("%") == 0)
            break;
            
         name = GetTok(0, line, ' ');
         if (BarNewRemailers)
            newbars->Add(name);

         names->Add(name);
      }
   }
   return names->Count;
}
//---------------------------------------------------------------------------

bool __fastcall TQSStatsWin4::ReportNewMixmasterRemailers(TStrings *Report)
{
   AnsiString name;
   TStringList *section = new TStringList();
   TStringList *mlist = new TStringList();
   TStringList *oldmlist = new TStringList();
   TStringList *newmlist = new TStringList();

   if (FileExists(PathName("mlist.txt")))
      mlist->LoadFromFile(PathName("mlist.txt"));
   GetMlistNames(mlist, oldmlist);

   GetPage("mlist.txt", mlist);
   GetMlistNames(mlist, newmlist);

   AnsiString header = "NEW Mixmaster Remailers:";
   if (BarNewRemailers) header += " (all barred)";

   for (int i = 0; i < newmlist->Count; i++)
   {
      name = newmlist->Strings[i];
      if (oldmlist->IndexOf(name) < 0)
      {
         newbars->Add(name);
         section->Add(name);
      }
   }

   if (section->Count > 0)
   {
      Report->Add("");
      Report->Add(header);
      Report->AddStrings(section);
   }

   section->Clear();

   for (int i = 0; i < oldmlist->Count; i++)
   {
      name = oldmlist->Strings[i];
      if (newmlist->IndexOf(name) < 0)
      {
         section->Add(name);
      }
   }

   if (section->Count > 0)
   {
      Report->Add("");
      Report->Add("DECEASED Mixmaster Remailers:");
      Report->AddStrings(section);
   }

   delete newmlist;
   delete oldmlist;
   delete mlist;
   delete section;
   return Report->Count > 0;
}
//---------------------------------------------------------------------------

bool __fastcall TQSStatsWin4::GetMixmasterRemailersChanged(TStrings *newkeys)
{
   return newkeys->Count > 0;
}
//---------------------------------------------------------------------------

int __fastcall TQSStatsWin4::GetRlistNames(TStrings *rlist, TStringList *names)
{
   names->Clear();
   names->Sorted = true;

   AnsiString name;
   bool saving = false;
   for (int i = 0; i < rlist->Count; i++)
   {
      AnsiString line = rlist->Strings[i].Trim();
      if (line.IsEmpty() && saving) break;
      if (line.IsEmpty()) continue;
      if (line.Pos("%") > 0)
         saving = true;

      if (saving)
      {
         if (line.Pos("%") == 0)
            break;

         name = GetTok(0, line, ' ');
         if (BarNewRemailers)
            newbars->Add(name);

         names->Add(name);
      }
   }
   return names->Count;
}
//---------------------------------------------------------------------------

bool __fastcall TQSStatsWin4::ReportNewCypherpunkRemailers(TStrings *Report)
{
   AnsiString name;
   TStringList *section = new TStringList();
   TStringList *rlist = new TStringList();
   TStringList *oldrlist = new TStringList();
   TStringList *newrlist = new TStringList();

   if (FileExists(PathName("rlist.txt")))
      rlist->LoadFromFile(PathName("rlist.txt"));
   GetMlistNames(rlist, oldrlist);

   GetPage("rlist.txt", rlist);
   GetRlistNames(rlist, newrlist);

   AnsiString header = "NEW Cypherpunk Remailers:";
   if (BarNewRemailers) header += " (all barred)";

   for (int i = 0; i < newrlist->Count; i++)
   {
      name = newrlist->Strings[i];
      if (oldrlist->IndexOf(name) < 0)
      {
         newbars->Add(name);
         section->Add(name);
      }
   }
   if (section->Count > 0)
   {
      Report->Add("");
      Report->Add(header);
      Report->AddStrings(section);
   }

   section->Clear();

   for (int i = 0; i < oldrlist->Count; i++)
   {
      name = oldrlist->Strings[i];
      if (newrlist->IndexOf(name) < 0)
      {
         section->Add(name);
      }
   }

   if (section->Count > 0)
   {
      Report->Add("");
      Report->Add("DECEASED Cypherpunk Remailers:");
      Report->AddStrings(section);
   }

   delete newrlist;
   delete oldrlist;
   delete rlist;

   return Report->Count > 0;
}
//---------------------------------------------------------------------------

bool __fastcall TQSStatsWin4::GetCypherpunkRemailersChanged(TStrings *newkeys)
{
   newkeys->Clear();
   return newkeys->Count > 0;
}
//---------------------------------------------------------------------------
int __fastcall TQSStatsWin4::ValidPages()
{
   int valids = 0;
   for (int i = 1; i < TabSet->Tabs->Count; i++)
   {
      TRemailerDoc *Doc = TabRemailerDoc(i);
      if (!Doc || Doc->Source == LocalFile)
         continue;

      if (Doc->ok && Doc->IsValid())
         valids++;
   }
   return valids;
}
//---------------------------------------------------------------------------

void __fastcall TQSStatsWin4::RunBtnClick(TObject *)
{
   if (!CheckSave())
      return;

   for (int i = 0; i < PopupMenu1->Items->Count; i++)
   {
      // run the default menu item
      TMenuItem *item = PopupMenu1->Items->Items[i];
      if (item->Default == true)
      {
         item->Click();
         break;
      }
   }
}
//---------------------------------------------------------------------------

void __fastcall TQSStatsWin4::CancelBtnClick(TObject *)
{
   Cancel = true;
}
//---------------------------------------------------------------------------

void __fastcall TQSStatsWin4::Options1Click(TObject *)
{
   TColor windowcolor = WindowColor;
   AnsiString winsfontstr, dlgsfontstr, editfontstr;

   OpenOptionsDlg(this, BarNewRemailers, windowcolor, winsfontstr, dlgsfontstr, editfontstr);
   if (windowcolor != WindowColor)
   {
      WindowColor = windowcolor;
      for (int i = 0; i < TabSet->Tabs->Count; i++)
      {
         TabRemailerDoc(i)->ed->Color = WindowColor;
      }
      SourceBox->Color = WindowColor;
      ProxyBox->Color = WindowColor;
   }

   if (winsfontstr != WinsFontStr)
   {
      WinsFontStr = winsfontstr;
      for (int i = 0; i < TabSet->Tabs->Count; i++)
      {
         StrToFont(WinsFontStr, TabRemailerDoc(i)->ed->Font);
      }
      StrToFont(WinsFontStr, SourceBox->Font);
      StrToFont(WinsFontStr, ProxyBox->Font);
   }

   if (dlgsfontstr != DlgsFontStr)
   {
      DlgsFontStr = dlgsfontstr;
      StrToFont(DlgsFontStr, Font);
      StrToFont(DlgsFontStr, StatusBar->Font);
      StrToFont(DlgsFontStr, CoolBar->Font);
   }

   if (editfontstr != EditFontStr);
      EditFontStr = editfontstr;

   if (TabSet->Font->Size < 10)
      TabSet->TabWidth = 80;
   else
      TabSet->TabWidth = 96;
}
//---------------------------------------------------------------------------

void __fastcall TQSStatsWin4::SpeedButton1Click(TObject *)
{
   TStringList *keys = new TStringList();

   GetPage("type2.list", keys);
   while (keys->Count > 10) keys->Delete(0);
   ManageNewMixmasterKeys(this, keys);

   delete keys;
}
//---------------------------------------------------------------------------

void __fastcall TQSStatsWin4::Timer1Timer(TObject *)
{
   //BlankView->Lines->Text += ".";
}
//---------------------------------------------------------------------------

void __fastcall TQSStatsWin4::ViewToolbarItemClick(TObject *)
{
   ShowToolBar(!CoolBar->Visible);
}
//---------------------------------------------------------------------------

void __fastcall TQSStatsWin4::ViewStatusBarItemClick(TObject *)
{
   StatusBar->Visible = !StatusBar->Visible;
}
//---------------------------------------------------------------------------

void __fastcall TQSStatsWin4::ToolsURLManagerItemClick(TObject *)
{
   OpenURLManager(this, PathName(URLFile), Proxy);
}
//---------------------------------------------------------------------------

void __fastcall TQSStatsWin4::RearrangeURLsourcesClick(TObject *)
{
   ReorderURLSources(this, PathName(URLFile));
}
//---------------------------------------------------------------------------

void __fastcall TQSStatsWin4::ToolsBarRemailerItemClick(TObject *)
{
   BarsModified = ManageBarredRemailers2(this, PathName(BarredFile));
}
//---------------------------------------------------------------------------

void __fastcall TQSStatsWin4::UpdateCloseAllItemClick(TObject *)
{
   if (!CheckSave())
      return;

   ClearTabs();
}
//---------------------------------------------------------------------------

void __fastcall TQSStatsWin4::ViewOtherFileItemClick(TObject *)
{
   if (OpenDialog->Execute())
   {
      GetLocalDocument(OpenDialog->FileName);
   }
}
//---------------------------------------------------------------------------

void __fastcall TQSStatsWin4::About1Click(TObject *)
{
   ShowAboutBox(this, AppName + "  v" + ReleaseVersion);
}
//---------------------------------------------------------------------------

void __fastcall TQSStatsWin4::QuickStart1Click(TObject *)
{
   AnsiString readme = AppDir() + "readme.stats.txt";
   if (FileExists(readme))
      Exec95("notepad.exe " + readme);
}
//---------------------------------------------------------------------------

void __fastcall TQSStatsWin4::Timer2Timer(TObject *)
{
   Timer2->Enabled = false;
   OpenLocalFiles();
}
//---------------------------------------------------------------------------

bool __fastcall TQSStatsWin4::CheckSave()
{
   if (SaveBtn->Enabled && ValidPages())
   {
       switch (MsgBox(this,
         "Your remailer pages are not saved. Save them now?     ",
          MT_CONFIRM, MB_YESNOCANCEL))
       {
          case IDCANCEL: return false;
          case IDNO: return true;
       }
       Save();
   }
   return true;
}
//---------------------------------------------------------------------------

void __fastcall TQSStatsWin4::GenType2List(TStrings *pubringmix, AnsiString ModifiedDate)
{
   if (SecureModeMsg(this))
      return;
      
   TStringList *type2 = new TStringList();
   for (int i = 0; i < pubringmix->Count; i++)
   {
      AnsiString line = pubringmix->Strings[i];
      if (GetTok(1, line, ' ').Pos("@") > 0 && GetTok(3, line, ' ').Pos(".") > 0)
         type2->Add(line);
   }
   Global->DB->PutPageDateStr("type2.list", ModifiedDate);
   type2->SaveToFile(MixDir() + "type2.list");
   delete type2;
}
//---------------------------------------------------------------------------

void __fastcall TQSStatsWin4::BundleCypherpunkNymservers()
{
      AnsiString pubring = MixDir() + "pubring.asc";
      AnsiString nymservers = AppDir() + "nymserver.asc";

      TStringList *pubasc = new TStringList();
      TStringList *nymasc = new TStringList();
      if (FileExists(pubring))
         pubasc->LoadFromFile(pubring);
      if (FileExists(nymservers))
         nymasc->LoadFromFile(nymservers);

      while (nymasc->Count > 0 && nymasc->Strings[0].Pos("-----BEGIN PGP PUBLIC") == 0)
         nymasc->Delete(0);

      pubasc->AddStrings(nymasc);
      pubasc->SaveToFile(pubring);

      delete nymasc;
      delete pubasc;
}
//---------------------------------------------------------------------------

void __fastcall TQSStatsWin4::Save()
{
   if (SecureModeMsg(this))
      return;

   bool gotpubringasc = false, gotnymserverasc = false;
   for (int i = 0; i < TabSet->Tabs->Count; i++)
   {
      TRemailerDoc *Doc = TabRemailerDoc(i);
      if (Doc->Source == LocalFile)
         continue;

      if (Doc->ok && Doc->IsValid())
      {
         Global->DB->PutPageDateStr(Doc->Name, Doc->ModifiedDate);

         if (Doc->Name == "pubring.mix")
         {
            GenType2List(Doc->Lines, Doc->ModifiedDate);
         }
         else
         if (Doc->Name == "mlist.txt")
         {
            mlistorg->SaveToFile(PathName(MListOrgFile));
         }
         else
         if (Doc->Name == "rlist.txt")
         {
            rlistorg->SaveToFile(PathName(RListOrgFile));
         }
         else
         if (dynamic_cast<TCpkPubringAsc*>(Doc) != NULL) //Doc->Name == "pubring.asc")
         {
            gotpubringasc = true;
         }
         else
         if (Doc->Name == "nymserver.asc")
         {
            gotnymserverasc = true;
         }
         else
         if (Doc->Name == "advisory.txt")
         {
         }

         Doc->Lines->SaveToFile(PathName(Doc->Name));
      }

      if (BarNewRemailers && newbars->Count > 0)
         SaveNewBarred(newbars);

      SaveBtn->Enabled = false;
   }

   if (gotpubringasc || gotnymserverasc)
   {
      BundleCypherpunkNymservers();
   }
}
//---------------------------------------------------------------------------


void __fastcall TQSStatsWin4::OkBtnClick(TObject *)
{
   if (ValidPages() && SaveBtn->Enabled) Save();
   FileExit->Click();
}
//---------------------------------------------------------------------------

void __fastcall TQSStatsWin4::Timer3Timer(TObject *)
{
   Timer3->Enabled = false;

   if (AutoRunFiles)
   {
      FileUpdateAutoRunFiles();
   }
   else
   {
      FileUpdateItem->Click();
   }

   if (Cancel)
   {
      if (AutoRun)
         PostMessage(HWND_BROADCAST, WM_QSAPPMESSAGE, QSMSGID_STATSUPDATED, StatsUpdateCancel);
      AutoRun = false;
      Visible = true;
      return;
   }

   if (result != HTTP_OK && result != HTTP_NOTMODIFIED)
   {
      if (AutoRun)
         PostMessage(HWND_BROADCAST, WM_QSAPPMESSAGE, QSMSGID_STATSUPDATED, StatsUpdateError);
      AutoRun = false;
      Visible = true;
      return;
   }
   Save();
   Snooze(600);
   Hide();
   Snooze(200);

   PostMessage(HWND_BROADCAST, WM_QSAPPMESSAGE, QSMSGID_STATSUPDATED, StatsUpdateOk);

   Close();
}
//---------------------------------------------------------------------------

void __fastcall TQSStatsWin4::ToolsProxyItemClick(TObject *)
{
   AnsiString ProxyManager = AppDir() + "qsproxy.exe";
   if (FileExists(ProxyManager))
      Exec95(ProxyManager);
}
//---------------------------------------------------------------------------

void __fastcall TQSStatsWin4::KeyringsMenuClick(TObject *)
{
   KeyringsNymserverItem->Enabled  = FileExists(PathName(NymserverFile));
   KeyringsCypherpunkItem->Enabled = FileExists(PathName("pubring.asc"));
   KeyringsMixmasterItem->Enabled  = FileExists(PathName("pubring.mix"));
}
//---------------------------------------------------------------------------

void __fastcall TQSStatsWin4::KeyringsMixmasterItemClick(TObject *)
{
   Exec95(AppDir() + "qsmixkeys.exe \"" + MixmasterDir + "pubring.mix\"");
}
//---------------------------------------------------------------------------

void __fastcall TQSStatsWin4::KeyringsCypherpunkItemClick(TObject *)
{
   Exec95(AppDir() + "qspgpkeys.exe \"" + MixmasterDir + "pubring.asc\" \"" +
      MixmasterDir + "rlist.txt\"");
}
//---------------------------------------------------------------------------

void __fastcall TQSStatsWin4::KeyringsNymserverItemClick(TObject *)
{
   Exec95(AppDir() + "qspgpkeys.exe \"" + PathName(NymserverFile) + "\"" +
      MixmasterDir + PathName(NymserverFile) + "\"");
}
//---------------------------------------------------------------------------

void __fastcall TQSStatsWin4::KeyringBtnClick(TObject *)
{
   for (int i = 0; i < PopupMenu2->Items->Count; i++)
   {
      TMenuItem *item = PopupMenu2->Items->Items[i];
      if (item->Default)
      {
         item->Click();
         break;
      }
   }
}
//---------------------------------------------------------------------------

void __fastcall TQSStatsWin4::PopupMenu3Popup(TObject *)
{
   //PopSaveAndExitItem->Enabled = SaveBtn->Enabled;
   //PopExitItem->Default = !SaveBtn->Enabled;
}
//---------------------------------------------------------------------------

void __fastcall TQSStatsWin4::Timer4Timer(TObject*)
{
   // unused function...
   if (!Panel3->Visible)
   {
      if (StatusBar->SimpleText != "") DisplayStatus("");
      else DisplayStatus(Label1Cap);
   }
   else
   {
      if (Label1->Caption != "") Label1->Caption = "";
      else Label1->Caption = Label1Cap;
   }
}
//---------------------------------------------------------------------------

void __fastcall TQSStatsWin4::FormResize(TObject*)
{
   int left = KeyringSep->Left + KeyringSep->Width - 4;
   ProgressBar->Width = ToolBar->Width - left - 5;
   ProgressBar->Left = left + 12;
}
//---------------------------------------------------------------------------

void __fastcall TQSStatsWin4::Hist1Click(TObject*)
{
   if (FileExists(AppDir() + "qsstats.history.txt"))
      Exec95("notepad.exe \"" + AppDir() + "qsstats.history.txt\"");
}
//---------------------------------------------------------------------------

bool __fastcall TQSStatsWin4::BringPageToFront(const AnsiString &page)
{
   bool ok = false;
   AnsiString docname = SansAmpersand(page);
   for (int i = 0; i < TabSet->Tabs->Count; i++)
   {
      TRemailerDoc *Doc = TabRemailerDoc(i);
      if (Doc->Name == docname)
      {
         Label1->Caption = Doc->Source + ": " + Doc->Name;
         Doc->ed->BringToFront();
         TabSet->TabIndex = i;
         ok = true;
         break;
      }
   }
   return ok;
}
//---------------------------------------------------------------------------

void __fastcall TQSStatsWin4::OpenLocalPage(const AnsiString &docname)
{
   if (PageNo(docname) >= 0)
      return;

   if (FileExists(PathName(docname)))
      GetLocalDocumentTab(PathName(docname));
   Snooze(5);
}
//---------------------------------------------------------------------------

void __fastcall TQSStatsWin4::ViewMenuItemClick(TObject *Sender)
{
   TMenuItem *item = dynamic_cast<TMenuItem*>(Sender);
   if (item)
   {
      AnsiString docname = SansAmpersand(item->Caption);
      if (docname == "Toolbar" || docname == "Status Bar")
         return;

      if (docname.Pos("All") == 1)
      {
         OpenLocalPage("mlist.txt"); Snooze(50);
         OpenLocalPage("pubring.mix"); Snooze(50);
         OpenLocalPage("type2.list"); Snooze(50);
         OpenLocalPage("rlist.txt"); Snooze(50);
         OpenLocalPage("pubring.asc"); Snooze(50);
         return;
      }

      if (BringPageToFront(docname))
         return;

      if (FileExists(PathName(docname)))
         GetLocalDocumentTab(PathName(docname));

      BringPageToFront(docname);
   }
}
//---------------------------------------------------------------------------

AnsiString __fastcall TQSStatsWin4::PathName(AnsiString page)
{
   page = SansAmpersand(page);

   if (page == "mlist.txt" || page == "pubring.mix" || page == "type2.list" ||
       page == "rlist.txt" || page == "pubring.asc")
   {
      return MixDir() + page;
   }

   return AppDir() + page;
}
//---------------------------------------------------------------------------

/*AnsiString __fastcall TQSStatsWin4::PathName(AnsiString page)
{
   page = SansAmpersand(page);

   if (page == NymserverFile || page == AdvisoryFile || page == URLFile ||
       page == BarredFile || page == Mail2newsFile)
   {
      return AppDir() + page;
   }

   return MixDir() + page;
}*/
//---------------------------------------------------------------------------

void __fastcall TQSStatsWin4::UpdatePrintItemClick(TObject *)
{
   PrintSelectedPages(this, TabSet);
}
//---------------------------------------------------------------------------


void __fastcall TQSStatsWin4::FileMenuClick(TObject *)
{
   while (SourceMenu->Count > 0)
      SourceMenu->Delete(0);
   for (int i = 0; i < SourceBox->Items->Count; i++)
   {
      TMenuItem *item = new TMenuItem(SourceMenu);
      item->Caption = SourceBox->Items->Strings[i];
      item->OnClick = SourceMenuItemClick;
      if (i == SourceBox->ItemIndex)
         item->Checked = true;
      item->RadioItem = true;
      SourceMenu->Add(item);
   }

   while (ProxyMenu->Count > 0)
      ProxyMenu->Delete(0);
   for (int i = 0; i < ProxyBox->Items->Count; i++)
   {
      TMenuItem *item = new TMenuItem(ProxyMenu);
      item->Caption = GetTok(0, ProxyBox->Items->Strings[i], ',');
      item->OnClick = ProxyMenuItemClick;
      if (i == ProxyBox->ItemIndex)
         item->Checked = true;
      item->RadioItem = true;
      ProxyMenu->Add(item);
   }
}
//---------------------------------------------------------------------------

void __fastcall TQSStatsWin4::UpdateSaveItemClick(TObject *)
{
   Save();
}

//---------------------------------------------------------------------------
// Barred Remailers
//---------------------------------------------------------------------------

void __fastcall TQSStatsWin4::SaveNewBarred(TStrings *newbars)
{
   assert(newbars != NULL && newbars->Count > 0);
   if (!newbars->Count)
      return;

   TStringList *barred = new TStringList();
   barred->Duplicates = dupIgnore;
   barred->Sorted = true;

   if (FileExists(BarredFile))
      barred->LoadFromFile(BarredFile);
   barred->AddStrings(newbars);

   barred->SaveToFile(BarredFile);
   delete barred;
}
//---------------------------------------------------------------------------

bool __fastcall TQSStatsWin4::MarkMixmasterMiddlemen(TStrings *last, TStrings *mring)
{
   bool modified = false;
   for (int i = 0; i < mring->Count; i++)
   {
      AnsiString remailer, line = mring->Strings[i];
      remailer = GetTok(0, line, ' ');
      int idx = last->IndexOf(remailer);
      if (idx >= 0)
      {
         if (GetTok(4, line, ' ').Pos("M") == 0)
         {
            AnsiString field, newline;
            int x = 0; while ((field = GetTok(x, line, ' ')) != "")
            {
               if (x == 4)
                  field = "M" + field;

               newline += field + " ";

               x++;
            }
            mring->Strings[i] = newline;
            modified = true;
         }
      }
   }
   return modified;
}
//---------------------------------------------------------------------------

void __fastcall TQSStatsWin4::RemoveBarredRemailersFromStatsFiles()
{
   AnsiString rlisttxt, mlisttxt, mringmix;
   TStringList *page = new TStringList();

   mlisttxt = PathName("mlist.txt");
   mringmix = PathName("pubring.mix");
   rlisttxt = PathName("rlist.txt");

   // mlist.txt
   if (FileExists(mlisttxt))
      page->LoadFromFile(mlisttxt);

   RemoveBarredRemailers("mlist.txt", page);
   page->SaveToFile(mlisttxt);

   // pubring.mix
   if (FileExists(mringmix))
      page->LoadFromFile(mringmix);

   if (MarkMiddlemanRemailers("pubring.mix", page))
   {
      page->SaveToFile(mringmix);
      TStringList *type2 = new TStringList();
      for (int i = 0; i < page->Count; i++)
      {
         AnsiString line = page->Strings[i];
         if (line.Pos("@") > 0)
            type2->Add(line);
      }
      type2->SaveToFile(MixDir() + "type2.list");
      delete type2;
   }

   // rlist.txt
   if (FileExists(rlisttxt))
      page->LoadFromFile(rlisttxt);

   RemoveBarredRemailers("rlist.txt", page);
   page->SaveToFile(rlisttxt);

   delete page;
}
//---------------------------------------------------------------------------

bool __fastcall CheckMakeCypherpunkMiddlemen(TStrings *last, TStrings *rlist)
{
   bool modified = false;
   for (int i = 0; i < rlist->Count; i++)
   {
      AnsiString line = rlist->Strings[i];
      if (line.Pos("$remailer{") == 1)
      {
         AnsiString remailer = GetTok(1, line, '\"');
         int idx = last->IndexOf(remailer);
         if (idx >= 0)
         {
            if (line.Pos(" middle") == 0)
            {
               int p = line.Pos(">");
               line.Insert(" middle", p + 1);
               rlist->Strings[i] = line;
               modified = true;
            }
         }
      }
   }
   return modified;
}
//---------------------------------------------------------------------------

bool __fastcall TQSStatsWin4::MarkMiddlemanRemailers(const AnsiString &Page,
   TStrings *ring)
{

   TStringList *last = new TStringList();
   TStringList *barred = new TStringList();
   AnsiString BarFile = PathName(BarredFile);

   if (FileExists(BarFile))
      barred->LoadFromFile(BarFile);

   int i = 0; while (i < barred->Count)
   {
      AnsiString line = barred->Strings[i];
      if (GetTok(1, line, ',').Trim().LowerCase() == "last")
      {
         last->Add(GetTok(0, line, ',').Trim());
         barred->Delete(i);
         continue;
      }
      i++;
   }

   bool ok = MarkMixmasterMiddlemen(last, ring);

   delete barred;
   delete last;
   return ok;
}
//---------------------------------------------------------------------------

void __fastcall TQSStatsWin4::RemoveBarredRemailers(const AnsiString &Page, TStrings *List)
{
   assert(List != NULL);

   TStringList *last = new TStringList();
   TStringList *barred = new TStringList();
   AnsiString BarFile = PathName(BarredFile);

   if (FileExists(BarFile))
      barred->LoadFromFile(BarFile);

   int i = 0; while (i < barred->Count)
   {
      AnsiString line = barred->Strings[i];
      if (GetTok(1, line, ',').Trim().LowerCase() == "last")
      {
         last->Add(GetTok(0, line, ',').Trim());
         barred->Delete(i);
         continue;
      }
      i++;
   }

   if (barred->IndexOf("lcs") < 0)
      barred->Add("lcs");

   i = 0; while (i < List->Count)
   {
      AnsiString line = List->Strings[i];
      if (line.Pos("%") == 0)
      {
         i++;
         continue;
      }

      if (barred->IndexOf(GetTok(0, line, ' ')) >= 0)
         List->Delete(i);
      else i++;
   }

   if (Page == "rlist.txt")
      CheckMakeCypherpunkMiddlemen(last, List);

   if (Page == "pubring.mix")
      MarkMixmasterMiddlemen(last, List);

   delete barred;
   delete last;
}
//---------------------------------------------------------------------------

/*void __fastcall TQSStatsWin4::RemoveBarredRemailers(const AnsiString &Page, TStrings *List)
{
   assert(List != NULL);

   TStringList *barred = new TStringList();
   AnsiString BarFile = PathName(BarredFile);

   if (FileExists(BarFile))
      barred->LoadFromFile(BarFile);

   if (barred->IndexOf("lcs") < 0)
      barred->Add("lcs");

   int i = 0; while (i < List->Count)
   {
      AnsiString line = List->Strings[i];
      if (line.Pos("%") == 0)
      {
         i++;
         continue;
      }

      if (barred->IndexOf(GetTok(0, line, ' ')) >= 0)
         List->Delete(i);
      else i++;
   }

   delete barred;
}*/
//---------------------------------------------------------------------------

void __fastcall TQSStatsWin4::HelpTopicsItemClick(TObject *)
{
   if (!FileExists(AppHelpFile))
   {
      MsgBox(this,
        "Sorry, there is no program help available!\n"
        "The help file " + ExtractFileName(AppHelpFile) + " is missing.",
         MT_INFO, MB_OK);
      return;
   }

   if (HHLib == 0)
   {
      MsgBox(this,
        "Sorry, there is no program help available!\n"
        "HTML help is not installed on this PC.",
         MT_INFO, MB_OK);
      return;
   }

   AnsiString topic = AppHelpFile + ">qshelpwin";
   HelpWin = HTMLHelp(Handle, topic.c_str(), HH_DISPLAY_TOPIC, NULL);
}
//---------------------------------------------------------------------------

void __fastcall TQSStatsWin4::HelpHistoryItemClick(TObject *Sender)
{
   AnsiString history = "qsstats.history.txt";
   StrToFont(WinsFontStr, BlankView->Font);

   if (FileExists(AppDir() + history))
   {
      OpenApplicationHistory(this, BlankView->Font, WindowColor,
         AppDir() + history);
      return;
   }

   if (FileExists(AppDir() + "docs\\" + history))
   {
      OpenApplicationHistory(this, BlankView->Font, SourceBox->Color,
         AppDir() + "docs\\" + history);
      return;
   }
}
//---------------------------------------------------------------------------

void __fastcall TQSStatsWin4::Home1Click(TObject *Sender)
{
   TIniFile *ini = new TIniFile(AppIni());
   AnsiString home = ini->ReadString("noreply", "home",
      "http://www.noreply.org");
   delete ini;

   ShellExecute( Handle, "open", home.c_str(), NULL, NULL, SW_SHOWNORMAL );
}
//---------------------------------------------------------------------------

void __fastcall TQSStatsWin4::Latency1Click(TObject *Sender)
{
   TIniFile *ini = new TIniFile(AppIni());
   AnsiString latency = ini->ReadString("noreply", "latency",
      "http://www.noreply.org/latency/");
   delete ini;

   ShellExecute( Handle, "open", latency.c_str(), NULL, NULL, SW_SHOWNORMAL );
}
//---------------------------------------------------------------------------

void __fastcall TQSStatsWin4::Remsaint1Click(TObject *Sender)
{
   TIniFile *ini = new TIniFile(AppIni());
   AnsiString remsaint = ini->ReadString("noreply", "remsaint",
      "http://www.noreply.org/remsaint.php");
   delete ini;

   ShellExecute( Handle, "open", remsaint.c_str(), NULL, NULL, SW_SHOWNORMAL );
}
//---------------------------------------------------------------------------

void __fastcall TQSStatsWin4::AllPingers1Click(TObject *Sender)
{
   TIniFile *ini = new TIniFile(AppIni());
   AnsiString allpingers = ini->ReadString("noreply", "allpingers",
      "http://www.noreply.org/allpingers/");
   delete ini;

   ShellExecute( Handle, "open", allpingers.c_str(), NULL, NULL, SW_SHOWNORMAL );
}
//---------------------------------------------------------------------------

void __fastcall TQSStatsWin4::FormShow(TObject *Sender)
{
   if (TabSet->Font->Size < 10)
      TabSet->TabWidth = 80;
   else
      TabSet->TabWidth = 96;
}
//---------------------------------------------------------------------------

